#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <ctype.h>

#include "ptrbuf.h"

typedef struct 
{
  char *name;
  int type;
} fitem_t;

static int populate_fs(file_selector_t *fs, char *dn,int hidden);

static char *combine_fn(char *dir, char *file)
{
  char *p;
  int i,L;

  if (!strcmp(file,".")) return strdup(dir);
  if (strcmp(file,".."))
  {
     if (!strcmp(dir,"/")) dir= "";
     p= malloc(strlen(dir)+strlen(file)+2);
     sprintf(p,"%s/%s",dir,file);
     return p;
  }
  p= strdup(dir);
  L= strlen(p);
again:
  if (L==0) goto retroot;
  if (p[L-1]=='/') { p[L-1]= 0; L--; goto again; }

  for(i=L-1;i>0;i--)
    if (p[i]=='/') { p[i]=0; return p; }
retroot:
  free(p);
  return strdup("/");
}

static int has_suffix(const char *k, const char *s)
{
  int Lk, Ls,i;
  Lk= strlen(k);
  Ls= strlen(s);
  if (Lk<Ls) return 0;
  for(i=0;i<Ls;i++)
   if (toupper(k[Lk-1-i])!=toupper(s[Ls-1-i]))
     return 0;
  return 1;
}


static fitem_t *mk_fitem(char *dn, char *fn,char **sfx)
{
  fitem_t *fi;
  int Lfn;
  int i;
  struct stat sb;

  fi= malloc(sizeof(*fi));

  fi->path= combine_fn(dn,fn);
  if (stat(fi->path, &sb))      fi->type= 'U';
  else if (S_ISREG(sb.st_mode)) fi->type= 'F';
  else if (S_ISDIR(sb.st_mode)) fi->type= 'D';
  else                          fi->type= 'U';

  if (sfx && fi->type!='D')
  {
    for(i=0;sfx[i];i++) if (has_suffix(fn,sfx[i])) break;
    if (!sfx[i])
    {
       free(fi->path);
       free(fi);
       return NULL;
    }
  }

  Lfn= strlen(fn);
  if (!strcmp(fn,"..")) fi->name= strdup("<UP>");
  else
  {
    fi->name= malloc(Lfn+2);
    strcpy(fi->name, fn);
    if (fi->type=='D')      strcpy(fi->name+Lfn,"/");
    else if (fi->type=='U') strcpy(fi->name+Lfn, "?");
  }

  return fi;
}

static void destroy_items(file_selector_t *fs)
{
  int i;
  if (!fs->items) return ;
  for(i=0;i<fs->n_items;i++)
    free_item(fs->items[i]);
  free(fs->items);
}

  if (populate_fs(fs, dirname,hidden))
     populate_fs(fs,"/",hidden);

static void fs_destroy(file_selector_t *fs, XEvent *event)
{
  destroy_items(fs);
  free(fs);
}



static int compar_fi(const void *vA, const void *vB)
{
  const fitem_t **fA, **fB, *A, *B;
  const char *types= "DFU";
  const char *tA, *tB;
  fA= (void*) vA; fB= (void*) vB;
  A= *fA; B= *fB;

  if (!strcmp(A->name,"<UP>")) return -1;
  if (!strcmp(B->name,"<UP>")) return 1;
  tA= strchr(types, A->type); if (!tA) tA= types+4;
  tB= strchr(types, B->type); if (!tB) tB= types+4;

  if (tA!=tB) return tA - tB;
  if (A->name[0]=='.' && B->name[0]!='.') return  -1;
  if (A->name[0]!='.' && B->name[0]=='.') return  1;
  return strcmp(A->name, B->name);
}

static int populate_fs(file_selector_t *fs, char *dn,int hidden)
{
  DIR *dir;
  struct dirent *de;
  ptrbuf_t *Bi;
  int y;
  fitem_t **items;
  size_t n_items;
  size_t i;

  dir= opendir(dn);
  if (!dir) return 1;

  Bi= ptrbuf_new(50);
  while((de=readdir(dir)))
  {
     fitem_t *fi;
     if (!strcmp(de->d_name,".")) continue;
     if (strcmp(de->d_name,"..") && !hidden && de->d_name[0]=='.') continue;
     if (!strcmp(dn,"/") && !strcmp(de->d_name,"..")) continue;
     fi= mk_fitem(dn, de->d_name, fs->sfx);
     if (fi) ptrbuf_put(Bi, fi);
  }
  closedir(dir);

  ptrbuf_free(Bi, (void***) &items, &n_items);
  if (n_items==0) return 1;

  fs->selected= -1;

  qsort(items, n_items, sizeof(void*), compar_fi);

  if (fs->title) free(fs->title);
  text_render(fs_font, dn, &fs->title_w, &fs->title);
  fs->title_h= fs_font->height;

  if (fs->dir_name) free(fs->dir_name);
  fs->dir_name= strdup(dn);

  destroy_items(fs);
  fs->items= items;
  fs->n_items= n_items;

  y= 0;
  for(i=0;i<n_items;i++) { items[i]->y= y; y+= items[i]->h + FS_vspace; }

  return 0;
}

