/* This file is an amalgamation of several source files.
   Please look for BEGIN FILE,END FILE and CONTINUE FILE
   strings to browse contents of individual files contained
   here. */
/* BEGIN FILE pbuffer.c */
#include <stdlib.h>
#include <string.h>
/* BEGIN FILE pbuffer.h */
typedef unsigned int color_t;
typedef struct
{
  int x,y;
// ARGB
  color_t color;
} point_t;
#define D_RED(x) ((x)>>16)&0xff)
#define D_GRE(x) ((x)>>8)&0xff)
#define D_BLU(x) ((x)>>0)&0xff)
#define D_ALP(x) ((x)>>24)&0xff)
typedef struct pbuffer pbuffer_t;
pbuffer_t *pbuffer_new();
void       pbuffer_free(pbuffer_t *);
void       pbuffer_clip(pbuffer_t *B,int x, int y, int w, int h);
pbuffer_t *pbuffer_get(pbuffer_t *B, point_t **Rp, unsigned int *Rl);
void       pixel(pbuffer_t* B, int x,int y,color_t color);
/* END FILE pbuffer.h */
/* CONTINUE FILE pbuffer.c */
#define NPO 50
typedef struct pnode
{
  struct pnode *next;
  point_t points[NPO];
  int     npoints;
} pnode_t;
struct pbuffer
{
  pnode_t *first, *last;
};
static pnode_t *pnode_new()
{
  return calloc(1,sizeof(pnode_t));
}
pbuffer_t *pbuffer_new()
{
  pbuffer_t *B;
  B= calloc(1,sizeof(*B));
  B->first= B->last= pnode_new();
  return B;
}
void pbuffer_free(pbuffer_t *B)
{
  pnode_t *P,*N;
  for(P=B->first;P;P=N)
  {
    N= P->next;
    free(P);
  }
  free(B);
}
void       pixel(pbuffer_t* B, int x,int y,color_t color)
{
  point_t po;
  po.x= x;
  po.y= y;
  po.color= color;
  if (B->last->npoints==NPO)
  {
    B->last->next= pnode_new();
    B->last= B->last->next;
  }
  B->last->points[B->last->npoints]= po;
  B->last->npoints++;
}
pbuffer_t* pbuffer_get(pbuffer_t *B, point_t **Rp, unsigned int *Rl)
{
  unsigned int np;
  pnode_t *P;
  point_t *D;
  np= 0;
  for(P=B->first;P;P=P->next)
   np+= P->npoints;
  if (Rl) *Rl= np;
  if (Rp)
  {
    D= malloc(sizeof(point_t)*np);
    np= 0;
    for(P=B->first;P;P=P->next)
    {
      memcpy(D+np, P->points, sizeof(point_t)*P->npoints);
      np+= P->npoints;
    }
    *Rp= D;
  }
  return B;
}
void       pbuffer_clip(pbuffer_t *B,int x, int y, int w, int h)
{
  pnode_t *D, *S;
  int Do;
  D= B->first;
  Do= 0;
  for(S=B->first;S;S=S->next)
  {
    int i;
    for(i=0;i<S->npoints;i++)
      if (S->points[i].x>=x &&
          S->points[i].y>=y &&
          S->points[i].x<(x+w) &&
          S->points[i].y<(y+h))
      {
        D->points[Do]= S->points[i];
        Do++;
        if (Do==NPO) { Do= 0; D= D->next; }
      }
  }
  D->npoints= Do;
  while(D->next)
  {
    S= D->next->next;
    free(D->next);
    D->next= S;
  }
}
/* END FILE pbuffer.c */
/* BEGIN FILE line.c */
/* CONTINUE FILE line.c */
/* BEGIN FILE line.h */
pbuffer_t* line(pbuffer_t *B,unsigned int color,int x0, int y0, int x1, int y1);
/* END FILE line.h */
/* CONTINUE FILE line.c */
pbuffer_t* line(pbuffer_t *B,unsigned int color,int x0, int y0, int x1, int y1)
{
  int dx = abs(x1-x0), sx = x0<x1 ? 1 : -1;
  int dy = abs(y1-y0), sy = y0<y1 ? 1 : -1;
  int err = (dx>dy ? dx : -dy)/2, e2;
  B= B ? B : pbuffer_new();
  for(;;){
    pixel(B,x0,y0,color);
    if (x0==x1 && y0==y1) break;
    e2 = err;
    if (e2 >-dx) { err -= dy; x0 += sx; }
    if (e2 < dy) { err += dx; y0 += sy; }
  }
  return B;
}
/* END FILE line.c */
