#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "gfx.h"
#include "draw.h"

#define THR 1.8

#define NPIX 32

int GA_x, GA_y;

void button(char *name, int action)
{
  gfx_button(GFX_UPDATE, GA_x, GA_y, 80, 30, name, action);
  GA_y+= 40;
}

unsigned int *copy()
{
  unsigned int *T;
  int x,y;
  T= malloc(NPIX*NPIX*sizeof(unsigned int));
  for(x=0;x<NPIX;x++)
    for(y=0;y<NPIX;y++)
      T[x+y*NPIX]= get_pixel(x,y);
  return T;
}

int label(char *title,char *text)
{
  int w,h,r;
  gfx_print(GFX_UPDATE, 1, 0xffffff, 0,GA_x, GA_y+5, title, &w, &h); 
  r= gfx_label(GFX_UPDATE, 1, 0xffffff, 0, GA_x+w+5, GA_y, 150, h+10, text);
  GA_y+= h+10 + 10;
  return r;
}

int entry(char *name)
{
  int w,h,r;
  gfx_print(GFX_UPDATE, 1, 0xffffff, 0,GA_x, GA_y+5, name, &w, &h); 
  r= gfx_text_entry(GFX_UPDATE, GA_x+w+5, GA_y, 120, h+10, "");
  GA_y+= h+10 + 10;
  return r;
}

int color(int e)
{
  char *K;
  char *P;
  int R,G,B;
  K= gfx_text_entry_get(e);
  P= strchr(K,',');
  if (!P)
    return strtol(K, NULL, 16);
  P[0]= 0;

  R= atoi(K)&0xff;
  K= P+1;
  if (K[0]==0) return R<<16;
  P= strchr(K,',');
  if (!P)
  {
    G= atoi(K) & 0xff;
    return (R<<16) | (G<<8);
  }
  P[0]= 0;
  G= atoi(K) & 0xff;
  K= P+1;
  B= atoi(K) & 0xff;
  return (R<<16) | (G<<8) | B;
}

void point(int x,int y, int color)
{
  set_pixel(x,y,color);
  draw();
}

unsigned int BL(int A, double V, int B)
{
  int K;
  K= A*V + (1.0-V)*B;
  if (K<0) K= 0; if (K>255) K= 255;
  return K;
}

#if 0
void line(int x0,int y0,int x1,int y1, int col1, int col2)
{
  int x,y,sigx,sigy,i,len; double V;
  if (abs(x0-x1)>abs(y0-y1)) y1= y0;
                       else  x1= x0;
  len= abs(x1-x0) + abs(y0-y1);
  if (len==0) return ;
  sigx= (x1-x0) /len; 
  sigy= (y1-y0) /len; 
  for(i=0;i<=len;i++)
  {
    x= x0+ sigx*i;
    y= y0+ sigy*i;
  }
  draw();
}
#endif

unsigned int  n_temp;
point_t *p_temp;

static void temprevert()
{
  int i;
  if (p_temp)
  {
    for(i=0;i<n_temp;i++)
      set_pixel(p_temp[i].x, p_temp[i].y, p_temp[i].color);
    free(p_temp);
    p_temp= NULL;
  }
}

static void tempobj(pbuffer_t *B)
{
  int i;
  temprevert();
  pbuffer_free(pbuffer_get(B,&p_temp, &n_temp));
  for(i=0;i<n_temp;i++)
  {
    p_temp[i].color= get_pixel(p_temp[i].x,p_temp[i].y);
    set_pixel(p_temp[i].x, p_temp[i].y, p_temp[i].color ^ 0xffffff);
  }
  draw();
}

static unsigned int colrgb(unsigned int r,unsigned int g,unsigned int b)
{
  return ((r&0xff)<<16) |
         ((g&0xff)<<8) |
         ((b&0xff)<<0);
}

int main(int argc,char** argv)
{
  int i;
  int c1,c2,l1;
  int PX[2], PY[2];
  int np= 0;
  int F;
  int x,y;
  unsigned int *T;
  int ls_x, ls_y;

  pixinit(&GA_x, &GA_y);
  c1= entry("Color1");
  c2= entry("Color2");
  l1= label("Function: ","Point");
  button("Point", GFXEV_LAST+4);
  button("Line", GFXEV_LAST+3);
  label("Move Image","");
  button("Up", GFXEV_LAST+5);
  button("Left", GFXEV_LAST+6);
  button("Rotate", GFXEV_LAST+7);
  button("Flip X", GFXEV_LAST+8);
  button("Flip Y", GFXEV_LAST+9);
  button("Invert", GFXEV_LAST+10);
  F= 'P';

  if (argc>1) { load_image(argv[1]); gfx_text_entry_set(1,0,argv[1]); draw(); }
  while(1)
  {
    gfx_event_t event;
    pixevent(&event);
    switch(event.type)
    {
    case GFXEV_QUIT: goto done;
    case GFXEV_BUTTON_PRESS: 
      if (event.button==3)
      {
        if (F=='L') { np= 0; temprevert(); draw(); break; }
        break;
      }
      PX[np]= event.x; PY[np]= event.y;
      if (F=='P') { point(PX[0], PY[0], color(c1)); np= 0; continue; }
      if (F=='L')
      {
        np++;
        if (np==1) { ls_x= event.x; ls_y= event.y; break; }
        tempobj(line(NULL,0xff,ls_x,ls_y,event.x,event.y));
        if (n_temp)
        {
         unsigned int col1= color(c1), col1r,col1g,col1b;
         unsigned int col2= color(c2), col2r,col2g,col2b;
         col1r= (col1>>16)&0xff; col1g= (col1>>8)&0xff; col1b= (col1>>0)&0xff;
         col2r= (col2>>16)&0xff; col2g= (col2>>8)&0xff; col2b= (col2>>0)&0xff;
         for(i=0;i<n_temp;i++)
         {
            double V;
            V= 1.0 - (double) i / (n_temp-1);
            set_pixel(p_temp[i].x,p_temp[i].y, 
                       colrgb( BL(col1r,V,col2r), 
                               BL(col1g,V,col2g),
                               BL(col1b,V,col2b)));
         }
        }
        np= 0; 
        free(p_temp); p_temp= 0; n_temp= 0;
        draw();
        break;
      }
      break;
     case GFXEV_POINTER_MOVE:
      if (F=='L' && np==1)
        tempobj(line(NULL,0xff,ls_x,ls_y,event.x,event.y));
      break;
     case GFXEV_LAST+4:
       F= 'P'; np=0; gfx_label_set_text(GFX_UPDATE,l1,"Point"); break;
     case GFXEV_LAST+3:
       F= 'L'; np=0; gfx_label_set_text(GFX_UPDATE,l1,"Line"); break;
     case GFXEV_LAST+5:
       for(y=0;y<NPIX-1;y++)
         for(x=0;x<NPIX;x++)
           set_pixel(x,y, get_pixel(x,y+1));
       for(x=0;x<NPIX;x++) set_pixel(x, NPIX-1,0);
       draw();
       break;
     case GFXEV_LAST+6:
       for(x=0;x<NPIX-1;x++)
         for(y=0;y<NPIX;y++)
           set_pixel(x,y, get_pixel(x+1,y));
       for(y=0;y<NPIX;y++) set_pixel(NPIX-1, y,0);
       draw();
       break;
     case GFXEV_LAST+7:
         T= copy();
         for(y=0;y<NPIX;y++)
           for(x=0;x<NPIX;x++)
             set_pixel(NPIX-1-y,x, T[x+y*NPIX]);
         free(T); draw();
         break;
     case GFXEV_LAST+8:
         T= copy();
         for(y=0;y<NPIX;y++)
           for(x=0;x<NPIX;x++)
             set_pixel(NPIX-1-x, y, T[x+y*NPIX]);
         free(T); draw();
         break;
     case GFXEV_LAST+9:
         T= copy();
         for(y=0;y<NPIX;y++)
           for(x=0;x<NPIX;x++)
             set_pixel(x, NPIX-1-y, T[x+y*NPIX]);
         free(T); draw();
         break;
      case GFXEV_LAST+10:
         for(y=0;y<NPIX;y++)
           for(x=0;x<NPIX;x++)
             set_pixel(x, y, get_pixel(x,y) ^ 0xffffff);
         draw();
         break;
    }
  }
done:
  gfx_close();
  return 0;
}

