#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "gfx.h"

#define NPIX 32

int GA_x, GA_y;

void button(char *name, int action)
{
  gfx_button(GFX_UPDATE, GA_x, GA_y, 80, 30, name, action);
  GA_y+= 40;
}

unsigned int *copy()
{
  unsigned int *T;
  int x,y;
  T= malloc(NPIX*NPIX*sizeof(unsigned int));
  for(x=0;x<NPIX;x++)
    for(y=0;y<NPIX;y++)
      T[x+y*NPIX]= get_pixel(x,y);
  return T;
}

int label(char *title,char *text)
{
  int w,h,r;
  gfx_print(GFX_UPDATE, 1, 0xffffff, 0,GA_x, GA_y+5, title, &w, &h); 
  r= gfx_label(GFX_UPDATE, 1, 0xffffff, 0, GA_x+w+5, GA_y, 150, h+10, text);
  GA_y+= h+10 + 10;
  return r;
}

int entry(char *name)
{
  int w,h,r;
  gfx_print(GFX_UPDATE, 1, 0xffffff, 0,GA_x, GA_y+5, name, &w, &h); 
  r= gfx_text_entry(GFX_UPDATE, GA_x+w+5, GA_y, 120, h+10, "");
  GA_y+= h+10 + 10;
  return r;
}

int color(int e)
{
  char *K;
  char *P;
  int R,G,B;
  K= gfx_text_entry_get(e);
  P= strchr(K,',');
  if (!P)
    return strtol(K, NULL, 16);
  P[0]= 0;

  R= atoi(K)&0xff;
  K= P+1;
  if (K[0]==0) return R<<16;
  P= strchr(K,',');
  if (!P)
  {
    G= atoi(K) & 0xff;
    return (R<<16) | (G<<8);
  }
  P[0]= 0;
  G= atoi(K) & 0xff;
  K= P+1;
  B= atoi(K) & 0xff;
  return (R<<16) | (G<<8) | B;
}

void point(int x,int y, int color)
{
  set_pixel(x,y,color);
  draw();
}

unsigned int BL(int A, double V, int B)
{
  int K;
  K= A*V + (1.0-V)*B;
  if (K<0) K= 0; if (K>255) K= 255;
  return K;
}

static unsigned int colrgb(unsigned int r,unsigned int g,unsigned int b)
{
  return ((r&0xff)<<16) |
         ((g&0xff)<<8) |
         ((b&0xff)<<0);
}


void pleft_octant1(int x0,int y0,int dx,int dy,int error,int width)
{
  static int toggle;
  int x,y,threshold,E_diag,E_square,length,p;
  y= y0;
  x= x0;
  threshold = dx - 2*dy;
  E_diag= -2*dx;
  E_square= 2*dy;
  length = width;
  for(p=0;p<length;p++)
  {
     set_pixel(x,y, toggle ? 0x00ff00 : 0xffff00);
     if (error > threshold)
     {
       x= x - 1;
       error = error + E_diag;
     }
     error = error + E_square;
     y= y + 1;
  }
  toggle^=1;
}

void pright_octant1(int x0,int y0,int dx,int dy,int error,int width)
{
  static int toggle;
  int x,y,threshold,E_diag,E_square,length,p;
  y= y0;
  x= x0;
  threshold = dx - 2*dy;
  E_diag= -2*dx;
  E_square= 2*dy;
  length = width;
  error= -error;
  for(p=0;p<length;p++)
  {
     set_pixel(x,y, toggle ? 0x888888 : 0x880000);
     if (error > threshold)
     {
       x= x + 1;
       error = error + E_diag;
     }
     error = error + E_square;
     y= y - 1;
  }
  toggle^=1;
}


#if 0
void thick_octant1(int x0,int y0,int x1,int y1,int width)
{
  int p_error, error, x,y, threshold, E_diag, E_square, length, p, dx, dy;
  dx= x1 - x0;
  dy= y1 - y0;
  p_error= 0;
  error= 0;
  y= y0;
  x= x0;
  threshold = dx - 2*dy;
  E_diag= -2*dx;
  E_square= 2*dy;
  length = dx+1;
  for(p=0;p<length;p++)
  {
    pleft_octant1(x,y, dx, dy, p_error,width);
    if (error > threshold)
    {
      y= y + 1;
      error = error + E_diag;
      if (p_error > threshold)
      {
        p_error= p_error + E_diag;
      }
      p_error= p_error + E_square;
    }
    error = error + E_square;
    x= x + 1;
  }
}
#endif

void perp_octant1(int x0,int y0,int dx,int dy,int einit,double width,int winit)
{
  static int toggle;
  int x,y,threshold,E_diag,E_square,length,p;
  int tk,wthr;
  int error;

  y= y0;
  x= x0;
  threshold = dx - 2*dy;
  E_diag= -2*dx;
  E_square= 2*dy;
  length = width;
  error= einit;
  wthr= 2*width*sqrt(dx*dx+dy*dy);
  tk= dx+dy-winit;
  while(tk<=wthr)
  {
     set_pixel(x,y, toggle ? 0x00ff00 : 0xffff00);
     if (error > threshold)
     {
       x= x - 1;
       error = error + E_diag;
       tk= tk + 2*dy;
     }
     error = error + E_square;
     y= y + 1;
     tk= tk + 2*dx;
  }

  y= y0;
  x= x0;
  error= -einit;
  tk= dx+dy+winit;
  while(tk<=wthr)
  {
     set_pixel(x,y, toggle ? 0x888888 : 0x880000);
     if (error > threshold)
     {
       x= x + 1;
       error = error + E_diag;
       tk= tk + 2*dy;
     }
     error = error + E_square;
     y= y - 1;
     tk= tk + 2*dx;
  }
  toggle^=1;
}

void thick_octant1(int x0,int y0,int x1,int y1,double width)
{
  int p_error, error, x,y, threshold, E_diag, E_square, length, p, dx, dy;
  dx= x1 - x0;
  dy= y1 - y0;
  p_error= 0;
  error= 0;
  y= y0;
  x= x0;
  threshold = dx - 2*dy;
  E_diag= -2*dx;
  E_square= 2*dy;
  length = dx+1;
  for(p=0;p<length;p++)
  {
    perp_octant1(x,y, dx, dy, p_error,width,error);
    if (error > threshold)
    {
      y= y + 1;
      error = error + E_diag;
      if (p_error > threshold)
      {
        perp_octant1(x,y, dx, dy, p_error+E_diag+E_square,width,error);
        p_error= p_error + E_diag;
      }
      p_error= p_error + E_square;
    }
    error = error + E_square;
    x= x + 1;
  }
}

#if 0

void thick_octant1(int x0,int y0,int x1,int y1,int width)
{
  int p_error, error, x,y, threshold, E_diag, E_square, length, p, dx, dy;
  dx= x1 - x0;
  dy= y1 - y0;
  p_error= 0;
  error= 0;
  y= y0;
  x= x0;
  threshold = dx - 2*dy;
  E_diag= -2*dx;
  E_square= 2*dy;
  length = dx+1;
  for(p=0;p<length;p++)
  {
    pleft_octant1(x,y, dx, dy, p_error,width);
    pright_octant1(x,y, dx, dy, p_error,width);
    if (error > threshold)
    {
      y= y + 1;
      error = error + E_diag;
      if (p_error > threshold)
      {
        pleft_octant1(x,y, dx, dy, p_error+E_diag+E_square,width);
        pright_octant1(x,y, dx, dy, p_error+E_diag+E_square,width);
        p_error= p_error + E_diag;
      }
      p_error= p_error + E_square;
    }
    error = error + E_square;
    x= x + 1;
  }
}
#endif

int main(int argc,char** argv)
{
  int x,y;
  int c1;
  int PX[2], PY[2];
  int np= 0;

  pixinit(&GA_x, &GA_y);
  c1= entry("Width");
  button("Clear", GFXEV_LAST+4);

  while(1)
  {
    gfx_event_t event;
    pixevent(&event);
    switch(event.type)
    {
    case GFXEV_QUIT: goto done;
    case GFXEV_BUTTON_PRESS: 
      if (event.button!=1) continue;
      PX[np]= event.x; PY[np]= event.y;
      set_pixel(event.x,event.y,0xff0000);
      draw();
      np++;
      if (np==2)
      {
        int w;
        w= strtod(gfx_text_entry_get(c1),NULL);
        thick_octant1(PX[0],PY[0],PX[1],PY[1], w==0?1:w);
        draw();
        np= 0;
      }
      break;
     case GFXEV_LAST+4:
       np=0; 
       for(x=0;x<NPIX;x++)
        for(y=0;y<NPIX;y++)
          set_pixel(x,y,0);
       draw();
       break;
    }
  }
done:
  gfx_close();
  return 0;
}

