#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "gfx.h"

#include "pngio.h"

typedef struct { unsigned char R,G,B; } pix_t;

void int2comp(unsigned int P, pix_t *C)
{
  C->R= (P>>16)&0xff;
  C->G= (P>>8)&0xff;
  C->B= (P)&0xff;
}

unsigned int comp2int(pix_t C)
{
  return (C.R<<16) | (C.G<<8) | (C.B);
}

unsigned int blend_chan(unsigned int F, double A, unsigned int B)
{
  return  (unsigned int) ((F&0xff)*A + (B&0xff)*(1-A));
}

unsigned int blend(pix_t F, double A, pix_t B, pix_t *R)
{
  R->R= blend_chan(F.R, A, B.R);
  R->G= blend_chan(F.G, A, B.G);
  R->B= blend_chan(F.B, A, B.B);

  return comp2int(*R);
}

double dist(double a,double b)
{
  return sqrt(a*a+b*b);
}

int bf;

double bfunc(int x,int y,int w,int h)
{
  switch(bf)
  {
  default: return 1.0-(double)y/h;
  case 1: return 1.0 - dist(x,y) / dist(w,h);
  case 2: return 1.0 - (double)(x+y) / (w+h);
  }
}

unsigned int* fill_grad(unsigned int start, unsigned int end, int w, int h)
{
  int x,y;
  unsigned int *data;
  pix_t S,E,P;
  unsigned int p;
  

  int2comp(start, &S); int2comp(end, &E);
  data= malloc(w*h*sizeof(unsigned int));
  for(y=0;y<h;y++)
    for(x=0;x<w;x++)
     data[y*w+x]= blend(S,bfunc(x,y,w,h), E,&P);
  return data;
}

int main(int argc,char **argv)
{
  gfx_event_t event;
  unsigned int w,h,*d;
  char *e;
  unsigned int *buffer; int bw,bh;
  
  if (gfx_init(NULL,640,480,"PNG Viewer"))
    return 1;
  e=read_png_image(argv[1],&w,&h,&d);
  if (e)
  {
    printf("png error: %s\n",e);
    gfx_close();
    return 1;
  }
  gfx_put(1, d, w,h, 0,0, 0,0, w,h);
  free(d);
  gfx_print(GFX_UPDATE, 1, 0xffffff, 0, 50, 50, "Hello \tWorld!Ş",NULL,NULL);
  gfx_button(GFX_UPDATE, 10, 400, 90, 30, "Test",GFXEV_LAST+1);
  gfx_label(GFX_UPDATE, 1, 0, 0xededed, 0, 440, 150,30,"This is a label");
  int drawing;
  drawing= 0;
  buffer= NULL;
  while(1)
    if (gfx_event(NULL, &event))
     switch(event.type)
     {
     case GFXEV_LAST+1: printf("Testing!!\n"); break;
     case GFXEV_QUIT: goto done;
     case GFXEV_BUTTON_PRESS:
           if (event.button==3)
           {
             char buf[200];
             bf= (bf+1)%3;
             sprintf(buf,"method: %d   ",bf);
             gfx_print(GFX_UPDATE|GFX_BACKGND, 2, 0xffffff, 0, 10, 10, buf,NULL,NULL);
             printf("%s\n",buf);
             gfx_label_set_text(GFX_UPDATE, 0, buf);
             continue;
           }
           if (event.button==2)
           {
             static int btnok=1;
             btnok ^= 1;
             gfx_button_enable(0, btnok);
             continue;
           }
           if (event.button!=1) continue;
           if (drawing)
           {
             int x1,y1,x2,y2;
             gfx_recsel_end(GFX_ORDER, &x1,&y1,&x2,&y2);
             printf("rectangle (%d,%d) (%d,%d)\n", x1,y1,x2,y2);
             if (x2!=x1 && y2!=y1)
             {
#if 0
               bw= x2-x1+1; bh= y2-y1+1;
               if (buffer) free(buffer);
               buffer= malloc(bw*bh*4);
               gfx_get(buffer, bw, bh, 0,0, x1, y1, bw, bh);
#else
               unsigned int *D, w,h;
               w= x2-x1; h= y2-y1;
               D= fill_grad(0xffffff, 0, w,h);
               gfx_put(1, D, w,h, 0,0, x1, y1, w,h);
               free(D);
#endif
             }
             drawing= 0;
           } else {
             drawing= 1;
             gfx_recsel_begin(event.x,event.y,0,0,-1,-1);
           }
           break;
     }

done:
  gfx_close();
  return 0;
}
