#line 6 "/home/dodo/tmp/gfx/android/basics.u"
/***********************************************************************
 *                                                                     *
 *                          Table of Contents                          *
 *                                                                     *
 ***********************************************************************
  Introduction......................................................##A
  Application Object................................................##B 
  Application Init..................................................##C
    Activity Callback Installation.................................#C.1
    Worker Function................................................#C.2
  System Callbacks..................................................##D
    Application State..............................................#D.1
    System Object Handling.........................................#D.2
    Command Handler................................................#D.3
    Command Loop...................................................#D.4
  Basic Graphics....................................................##E
  Events............................................................##F
  Miscellaneous.....................................................##M
    Definitions....................................................#M.1
    Prototypes.....................................................#M.2
    Unimplemented..................................................#M.3
 ***********************************************************************/

/***********************************************************************
 *##A                                                                  *
 *                        Android Graphics I/O                         *
 *                                                                     *
 ***********************************************************************
  The android system is very weird in the way it handles events. First
  of all, when you make a native application, it's not really this native
  program running directly on the processor. A Java class is loaded first,
  which sets up the environment in which to talk to the rest of the system.
  This Java program then loads some methods from your native shared library
  and calls them. While it's doing these things, some threads are created
  to do various stuff, I don't really understand when and how many. 

  The system provides events thru several channels. There are two main
  methods used. One group of events are 'system events'. These are not
  related to user input etc. but are generated by the system in order to
  manage the application. For example, when the application is sent to
  background, a 'Pause' event is sent. Such system events are not reported
  thru a file descriptor but are processed directly by the Java wrapper
  around the native shared object. The application object exposed to the
  Java interface contains some function pointers, one for each type of
  system event. The Java part calls these function in *SOME* thread when
  the corresponding event occurs. This causes system events to be reported
  asynchronously. Your event loop in the main thread never sees these
  things, they get handled by another thread. In order to make them 
  synchronous, we report these events to the main thread via a pipe from
  the event handler functions.

  Another group of events are related to user input. These get reported
  thru a file descriptor, much like X11 events. The system has a struct
  called a 'looper' which can watch multiple file descriptors for events
  or 'ready to read' status. The looper is actually a simple wrapper 
  around the 'poll' system call. When you're ready to process user input,
  you add the input queue to the looper and then go into a loop, telling
  the looper to report activity on any file descriptor. 

  Fortunately, the looper can also watch regular file descriptors such
  as pipes or sockets. This makes it possible to synchronize system
  events with user events. 

  Here, we start with some boilerplate.
 ***********************************************************************/

#include <jni.h>
#include <stdlib.h>
#include <alloca.h>
#include <stdio.h>
#include <math.h>

#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/resource.h>
#include <poll.h>
#include <pthread.h>
#include <sched.h>
#include <android/configuration.h>
#include <android/looper.h>
#include <android/native_activity.h>
#include <android/log.h>
#include "gfx.h"

#define LOGI(...) \
  ((void)__android_log_print(ANDROID_LOG_INFO, "threaded_app", __VA_ARGS__))
#define LOGE(...) \
  ((void)__android_log_print(ANDROID_LOG_ERROR, "threaded_app", __VA_ARGS__))

#ifndef NDEBUG
#  define LOGV(...) \
  ((void)__android_log_print(ANDROID_LOG_VERBOSE, "threaded_app", __VA_ARGS__))
#else
#  define LOGV(...)  ((void)0)
#endif




/***********************************************************************
 *#M.1                                                                 *
 *                             Definitions                             *
 *                                                                     *
 ***********************************************************************
  The PIPE_* constants indicate which channel is ready for reading.
  They are given as identifiers to the looper thingy.

    PIPE_CMD     indicates that some asynchronous event has happened and
                 the callback has written some event on the command pipe.
    PIPE_EVENT   indicates that user input is available.
    PIPE_SENSOR  will be used for the sensor channel. It's not used yet.
 ***********************************************************************/
#define  PIPE_CMD    1
#define  PIPE_EVENT  2
#define  PIPE_SENSOR 3

/***********************************************************************
  CMD_* constants identify which asynchronous callback has occured.
 ***********************************************************************/
  
enum
{
  CMD_APP_START,
  CMD_APP_STOP,
  CMD_APP_RESUME,
  CMD_APP_PAUSE,
  CMD_WINDOW_DESTROYED,
  CMD_WINDOW_CREATED,
  CMD_WINDOW_RESIZED,
  CMD_WINDOW_REDRAW,
  CMD_WINDOW_CONTENTRECT,
  CMD_SAVE_STATE,
  CMD_FOCUS_CHANGE,
  CMD_LOW_MEMORY,
  CMD_CONFIG_CHANGE,
  CMD_INPUTQ_DESTROYED,
  CMD_INPUTQ_CREATED,
  CMD_QUIT
};

#line 13 "/home/dodo/tmp/gfx/android/../src/event.u"

typedef struct
{
  int  x,y,w,h;
  int  (*func)(void *,gfx_event_t*);
  void *arg;
  int  *enabled;
} click_handler_t;

#line 6 "/home/dodo/tmp/gfx/android/../src/widget.u"

typedef struct
{
  int type;
  int x,y,w,h;
  void (*destroy)(void*);
} gui_widget_t;

enum 
{
#line 21 "/home/dodo/tmp/gfx/android/../src/button.u"

  GFXCLASS_BUTTON,

#line 17 "/home/dodo/tmp/gfx/android/../src/widget.u"
  GFXCLASS_LAST
};

#line 11 "/home/dodo/tmp/gfx/android/../src/button.u"
typedef struct
{
  gui_widget_t widget;
  uint32_t *pic[3];
  int state;  //0 = normal, 1= depressed 2= disabled
  int action;
  int enabled; // this is used to inform the event system
} gui_button_t;

#line 9 "/home/dodo/tmp/gfx/android/../src/label.u"
typedef struct 
{
  int font, fg, bg, x,y,w,h,flags;
} gui_label_t;

#line 11 "/home/dodo/tmp/gfx/android/../src/screen.u"

typedef struct
{
  int n_gui_widget;
  int n_click_handler, click_handler_start;
  click_handler_t release_handler;
  click_handler_t motion_handler;

  uint32_t *backstore;
  uint32_t backw, backh;
} screen_t;

#line 201 "gfx.c"
#define UTF8_DECODER_SCOPE static

/*
UTF-8 Table
8421 8421
110x xxxx   2 bytes    mask= e0  result= c0   min= 80       vmask= 1f
1110 xxxx   3 bytes    mask= f0  result= e0   min= 800      vmask= 0f
1111 0xxx   4 bytes    mask= f8  result= f0   min= 10000    vmask= 07
1111 10xx   5 bytes    mask= fc  result= f8   min= 200000   vmask= 03
1111 110x   6 bytes    mask= fe  result= fc   min= 4000000  vmask= 01

10xx xxxx   cont. byte mask= c0  result= 80   vmask= 3f
*/

UTF8_DECODER_SCOPE
int utf8_decode_string
  (uint8_t *input, size_t ilen, uint32_t *output, size_t *olen)
{
   uint8_t *end,*start;
   static const uint32_t bit31= (1<<31);
   size_t L;
   int have_errors;
   int i, contb;
   uint8_t C;
   uint32_t outc; uint32_t min;

   end= input+ilen;
   L= 0;
   have_errors= 0;

   if (ilen==0) goto done;

again:
   C= input[0];
   if ( (C&0x80) ==0)    { outc= C; input++; goto out; }
   if ( (C&0xe0) ==0xc0) goto seq2; 
   if ( (C&0xf0) ==0xe0) goto seq3;
   if ( (C&0xf8) ==0xf0) goto seq4;

   // Here, we don't care about the seq5 and seq6 stuff.
   // they are invalid anyway, either overlong or greater
   // then the unicode limit. We just treat them as unrecognized
   // bytes and output them.
invalid_byte:
   outc= input[0] | bit31;
   input++;
   have_errors= 1;
   goto out;

seq2: outc= C & 0x1f;  min=    0x80; contb= 1; goto do_cont;
seq3: outc= C & 0x0f;  min=   0x800; contb= 2; goto do_cont;
seq4: outc= C & 0x07;  min= 0x10000; contb= 3; goto do_cont;

do_cont:
   start= input;
   input++;
   for(i=0;i<contb;i++)
   {
     if ((input==end) || ( (input[0]&0xc0) !=0x80)) goto reject;
     outc= (outc << 6) | (input[0]&0x3f);
     input++;
   }

   if (outc<min || outc>0x10ffff) goto reject; 
   if (outc>=0xd800 && outc<=0xdfff) goto reject;
   if (outc==0xffff || outc==0xfffe) goto reject;

out:
   output[L++]= outc;
   if (input<end) goto again;
done:
   *olen= L;
   return have_errors;
reject:
   input= start;
   goto invalid_byte; 
}


static unsigned int font_hash(unsigned int V)
{
  static const struct
  { unsigned short val; unsigned char pos; } htable[109]= {
 {0x20,0}, {0x21,1}, {0x22,2}, {0x23,3}, {0x24,4}, {0x25,5}, {0x26,6},
 {0x27,7}, {0x28,8}, {0x29,9}, {0x2a,10}, {0x2b,11}, {0x2c,12},
 {0x2d,13}, {0x2e,14}, {0x2f,15}, {0x30,16}, {0x31,17}, {0x32,18},
 {0x33,19}, {0x34,20}, {0x35,21}, {0x36,22}, {0x37,23}, {0x38,24},
 {0x39,25}, {0x3a,26}, {0x3b,27}, {0x3c,28}, {0x3d,29}, {0x3e,30},
 {0x3f,31}, {0x40,32}, {0x41,33}, {0x42,34}, {0x43,35}, {0x44,36},
 {0x45,37}, {0x46,38}, {0x47,39}, {0x48,40}, {0x49,41}, {0x4a,42},
 {0x4b,43}, {0x4c,44}, {0x4d,45}, {0x4e,46}, {0x4f,47}, {0x50,48},
 {0x51,49}, {0x52,50}, {0x53,51}, {0x54,52}, {0x55,53}, {0x56,54},
 {0x57,55}, {0x58,56}, {0x59,57}, {0x5a,58}, {0x5b,59}, {0x5c,60},
 {0x5d,61}, {0x5e,62}, {0x5f,63}, {0x60,64}, {0x61,65}, {0x62,66},
 {0x63,67}, {0x64,68}, {0x65,69}, {0x66,70}, {0x67,71}, {0x68,72},
 {0x69,73}, {0x6a,74}, {0x6b,75}, {0x6c,76}, {0x6d,77}, {0x6e,78},
 {0x6f,79}, {0x70,80}, {0x71,81}, {0x72,82}, {0x73,83}, {0x74,84},
 {0x75,85}, {0x76,86}, {0x77,87}, {0x78,88}, {0x79,89}, {0x7a,90},
 {0x7b,91}, {0x7c,92}, {0x7d,93}, {0x7e,94}, {0xc7,95}, {0x15f,106},
 {0x11e,101}, {0xdc,97}, {0x11f,102}, {0x15e,105}, {0xd6,96}, {0x20,107},
 {0xe7,98}, {0x131,104}, {0xf6,99}, {0xfc,100}, {0x130,103}, {0x20,107},
  };
  unsigned int A;
  int p;
  A= V;
  p= 0;
  A-= 0x20;
  if (A<=0x5e) { p+= A; goto done; }
  A-= 0xa7;
  p+= 95;
  A^= A>>4;
  A &= 0x3f;
  A= (A&0xf) | ((A&0xffffffe0)>>1);
  A= (A&0x7) | ((A&0xfffffff0)>>1);
  A-= A>>2;
  A-= 0x0;
  if (A<=0xc) { p+= A; goto done; }
  A-= 0x0;
  p+= 13;
done:
  if (htable[p].val!=V) return 107;
  return htable[p].pos;
}
static 
void font1_render
        (uint32_t *str, size_t L, 
        int *RW, int *RH, int *RA,
        unsigned char **Rdata)
{
  static const uint8_t font_data[]= {

0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x70,0x27,0x03,0xff,0x7e,0x00,0xf9,0x74,0x00,0xef,0x6a,0x00,0xe4,
0x5f,0x00,0xd9,0x54,0x00,0xce,0x49,0x00,0xc4,0x3f,0x00,0xaa,0x2a,0x00,0x00,0x00,0x04,0x7c,
0x1e,0x19,0xf8,0x60,0x00,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x01,0x00,
0x00,0xe6,0x21,0xa0,0x67,0x00,0xef,0x21,0xa7,0x6a,0x00,0xe2,0x12,0x9a,0x5d,0x00,0x9f,0x03,
0x6c,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x41,
0xe0,0x00,0x00,0x68,0xb6,0x00,0x00,0x00,0x00,0x00,0x00,0x44,0xe8,0x00,0x00,0x6c,0xbc,0x00,
0x00,0x00,0x00,0x00,0x00,0x44,0xe8,0x00,0x00,0x6c,0xbc,0x00,0x00,0x00,0x00,0xfc,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x90,0x00,0x13,0x14,0x52,0xe9,0x14,0x14,0x77,0xc1,0x14,
0x14,0x0b,0x00,0x00,0x00,0x44,0xe8,0x00,0x00,0x6c,0xbc,0x00,0x00,0x00,0x00,0x46,0x48,0x78,
0xee,0x48,0x48,0x95,0xcf,0x48,0x48,0x28,0x00,0xd0,0xd4,0xdf,0xfb,0xd4,0xd4,0xe6,0xf4,0xd4,
0xd4,0x77,0x00,0x00,0x00,0x44,0xe8,0x00,0x00,0x6c,0xbc,0x00,0x00,0x00,0x00,0x00,0x00,0x44,
0xe8,0x00,0x00,0x6c,0xbc,0x00,0x00,0x00,0x00,0x00,0x00,0x27,0x86,0x00,0x00,0x3e,0x6c,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x01,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x8c,0x00,0x00,0x00,
0x00,0x2f,0xbf,0xf9,0xfd,0xc2,0x29,0x00,0x10,0xe8,0x95,0x86,0xa1,0x94,0xea,0x17,0x48,0xf8,
0x03,0x70,0x8c,0x00,0x4a,0x01,0x36,0xfe,0x16,0x70,0x8c,0x00,0x00,0x00,0x01,0xb5,0xd7,0xb5,
0x8d,0x00,0x00,0x00,0x00,0x03,0x63,0xcf,0xfb,0xb0,0x33,0x00,0x00,0x00,0x00,0x70,0x9f,0x8f,
0xfa,0x33,0x00,0x00,0x00,0x70,0x8c,0x00,0xc7,0x8b,0x3f,0x52,0x00,0x70,0x8c,0x00,0xbf,0x85,
0x62,0xf5,0x4f,0x71,0x8e,0x5e,0xfc,0x34,0x00,0x6c,0xf1,0xfc,0xfe,0xee,0x5d,0x00,0x00,0x00,
0x06,0x85,0x9c,0x03,0x00,0x00,0x00,0x00,0x00,0x18,0x1e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x70,0x6a,0x0a,
0x00,0x00,0x19,0x40,0x00,0x00,0x03,0xd0,0x93,0xa5,0xba,0x00,0x00,0xb7,0x69,0x00,0x00,0x25,
0xdc,0x00,0x04,0xf6,0x0e,0x54,0xcc,0x02,0x00,0x00,0x06,0xe3,0x60,0x75,0xd0,0x0c,0xe1,0x35,
0x00,0x00,0x00,0x00,0x25,0x96,0x8e,0x19,0x8c,0x98,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x2d,0xe6,0x12,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x01,0xc4,0x62,0x4a,0xe4,0xec,0x6d,0x00,
0x00,0x00,0x00,0x62,0xc6,0x02,0xea,0x31,0x1a,0xee,0x1d,0x00,0x00,0x12,0xe7,0x2f,0x03,0xf3,
0x1a,0x05,0xe3,0x25,0x00,0x00,0x88,0x8b,0x00,0x00,0x6e,0xdd,0xd9,0x93,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x15,0x1b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x18,0x81,0xa3,0x78,0x10,0x00,0x00,0x00,0x00,0x19,0xe6,0xb7,0x79,0xba,
0xdb,0x12,0x00,0x00,0x00,0x85,0xc6,0x00,0x00,0x01,0x6e,0x13,0x00,0x00,0x00,0x9d,0xa2,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0xf1,0x2f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x97,0xfd,0xe0,0x30,0x00,0x00,0x00,0x00,0x00,0x61,0xe9,0x72,0x44,0x0e,0x00,0x00,0x00,0x00,
0x08,0xf1,0x4b,0x00,0x00,0x3d,0x48,0x48,0x48,0x04,0x2d,0xff,0x0f,0x00,0x00,0xb2,0xe1,0xfd,
0xd4,0x0d,0x17,0xfe,0x32,0x00,0x00,0x00,0x80,0xd6,0x00,0x00,0x00,0x9b,0xcb,0x1f,0x00,0x3d,
0xf1,0x6a,0x00,0x00,0x00,0x0a,0x9a,0xfd,0xf0,0xf9,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0x18,
0x31,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0xe6,0x21,0x00,0xef,0x21,
0x00,0xe2,0x12,0x00,0x9f,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x17,0x0f,0x00,0x29,0xfa,0x38,0x00,0xa6,0xa7,0x00,0x0f,0xf7,0x3a,0x00,0x50,0xe8,
0x00,0x00,0x84,0xae,0x00,0x00,0x9f,0x93,0x00,0x00,0xa6,0x8a,0x00,0x00,0x94,0x9d,0x00,0x00,
0x6f,0xc9,0x00,0x00,0x34,0xfb,0x10,0x00,0x00,0xdc,0x6e,0x00,0x00,0x70,0xde,0x07,0x00,0x06,
0x9f,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x1c,0x02,0x00,0x0c,0xe6,0x66,0x00,0x00,0x68,
0xe2,0x02,0x00,0x0c,0xf2,0x45,0x00,0x00,0xac,0x8f,0x00,0x00,0x6e,0xc8,0x00,0x00,0x4e,0xe3,
0x00,0x00,0x42,0xee,0x00,0x00,0x55,0xde,0x00,0x00,0x84,0xb7,0x00,0x00,0xc6,0x7d,0x00,0x28,
0xfe,0x26,0x00,0xa0,0xb7,0x00,0x13,0xa6,0x2b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0xc8,0x01,0x92,0x70,0x00,0x00,0x09,0xda,0x78,0xf2,
0x25,0x00,0x52,0x86,0xad,0xff,0xca,0x83,0x6f,0x59,0x93,0xb7,0xff,0xd1,0x90,0x7a,0x00,0x08,
0xda,0x78,0xf1,0x25,0x00,0x00,0x44,0xcd,0x01,0x96,0x79,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xf8,0x1b,0x00,0x00,0x00,0x00,0x00,0xfc,0x1c,0x00,0x00,0x38,0x90,
0x90,0xfe,0x9c,0x90,0x4a,0x36,0x8c,0x8c,0xfe,0x98,0x8c,0x48,0x00,0x00,0x00,0xfc,0x1c,0x00,
0x00,0x00,0x00,0x00,0xf8,0x1b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x04,0x7c,0x1e,0x19,0xf7,0x73,0x00,0x75,0x36,0x00,0x5f,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x12,0x84,0x84,0x84,0x84,0x84,0x12,0x15,0x98,0x98,0x98,0x98,0x98,0x15,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x04,0x7c,0x1e,0x19,0xf8,0x60,0x00,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x09,0x0a,
0x00,0x00,0x00,0x00,0x00,0x01,0xd5,0x64,0x00,0x00,0x00,0x00,0x00,0x4e,0xe6,0x07,0x00,0x00,
0x00,0x00,0x00,0xc5,0x77,0x00,0x00,0x00,0x00,0x00,0x3c,0xf1,0x0f,0x00,0x00,0x00,0x00,0x00,
0xb3,0x8a,0x00,0x00,0x00,0x00,0x00,0x2b,0xf8,0x1a,0x00,0x00,0x00,0x00,0x00,0xa1,0x9c,0x00,
0x00,0x00,0x00,0x00,0x1d,0xf8,0x27,0x00,0x00,0x00,0x00,0x00,0x8f,0xaf,0x00,0x00,0x00,0x00,
0x00,0x12,0xf4,0x38,0x00,0x00,0x00,0x00,0x00,0x7d,0xc1,0x00,0x00,0x00,0x00,0x00,0x09,0xea,
0x4b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x06,0x6d,0xa3,0x7a,0x0b,0x00,0x00,0x00,0x03,0xbb,0xd3,0x7d,0xcb,0xc4,
0x02,0x00,0x00,0x68,0xe3,0x0f,0x00,0x09,0xe0,0x68,0x00,0x00,0xcd,0x7f,0x00,0x00,0x00,0x75,
0xc7,0x00,0x0d,0xfe,0x38,0x00,0x00,0x00,0x38,0xfd,0x04,0x2c,0xff,0x17,0x00,0x00,0x00,0x1e,
0xff,0x1d,0x31,0xff,0x0a,0x00,0x00,0x00,0x0f,0xff,0x2e,0x19,0xff,0x22,0x00,0x00,0x00,0x24,
0xff,0x1d,0x02,0xf3,0x48,0x00,0x00,0x00,0x55,0xef,0x01,0x00,0xa9,0x9c,0x00,0x00,0x00,0xa4,
0xa7,0x00,0x00,0x32,0xf7,0x49,0x00,0x4f,0xf6,0x30,0x00,0x00,0x00,0x68,0xf8,0xf1,0xf1,0x57,
0x00,0x00,0x00,0x00,0x00,0x12,0x32,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x4a,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0xb5,0xc8,0x00,
0x00,0x00,0x00,0x00,0x00,0x3f,0xc4,0xc8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0xc8,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x74,0xc8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0xc8,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x74,0xc8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0xc8,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x74,0xc8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0xc8,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x74,0xc8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0xc8,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x1a,0x82,0xa6,0x7f,0x15,0x00,0x00,0x00,0x37,0xf0,0xab,0x78,0xc6,0xe0,
0x15,0x00,0x00,0xd3,0x80,0x00,0x00,0x04,0xcf,0x80,0x00,0x00,0x1e,0x0a,0x00,0x00,0x00,0x85,
0xaf,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x9d,0x9b,0x00,0x00,0x00,0x00,0x00,0x00,0x3b,0xf3,
0x2f,0x00,0x00,0x00,0x00,0x01,0x70,0xf2,0x57,0x00,0x00,0x00,0x00,0x09,0xb1,0xd4,0x33,0x00,
0x00,0x00,0x00,0x03,0xba,0xbe,0x0c,0x00,0x00,0x00,0x00,0x00,0x74,0xde,0x0d,0x00,0x00,0x00,
0x00,0x00,0x00,0xe1,0x7c,0x1c,0x1c,0x1c,0x1c,0x1a,0x00,0x12,0xff,0xff,0xff,0xff,0xff,0xff,
0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x1a,0x87,0xab,0x7d,0x0f,0x00,0x00,0x00,0x1e,0xe8,0xa6,0x6e,0xc4,0xd7,
0x0d,0x00,0x00,0x1c,0x66,0x00,0x00,0x07,0xdd,0x6d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb5,
0x8b,0x00,0x00,0x00,0x00,0x00,0x00,0x25,0xf3,0x47,0x00,0x00,0x00,0x00,0x34,0xb7,0xf2,0x90,
0x00,0x00,0x00,0x00,0x00,0x15,0x52,0x98,0xeb,0x32,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x83,
0xcc,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0xfe,0x0c,0x00,0x28,0x11,0x00,0x00,0x00,0x4f,
0xf6,0x04,0x02,0xcd,0xbb,0x17,0x00,0x33,0xdf,0x90,0x00,0x00,0x17,0xb7,0xfb,0xf0,0xfa,0x88,
0x05,0x00,0x00,0x00,0x00,0x1b,0x2f,0x09,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x41,0x48,0x00,0x00,0x00,0x00,0x00,0x00,0x1d,0xf6,0xc0,
0x00,0x00,0x00,0x00,0x00,0x00,0xa7,0xea,0xc0,0x00,0x00,0x00,0x00,0x00,0x3a,0xe8,0x7e,0xc0,
0x00,0x00,0x00,0x00,0x01,0xca,0x65,0x70,0xc0,0x00,0x00,0x00,0x00,0x5f,0xd1,0x02,0x70,0xc0,
0x00,0x00,0x00,0x0b,0xe5,0x42,0x00,0x70,0xc0,0x00,0x00,0x00,0x84,0xce,0x38,0x38,0x8f,0xcd,
0x34,0x0c,0x15,0xdb,0xe4,0xe4,0xe4,0xf1,0xf9,0xe4,0x35,0x00,0x00,0x00,0x00,0x00,0x78,0xc0,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x78,0xc0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x78,0xc0,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x46,0x60,0x60,0x60,0x60,0x34,0x00,0x00,0x00,0xd4,0xcc,0xb8,0xb8,0xb8,
0x64,0x00,0x00,0x01,0xf6,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x1c,0xfe,0x0a,0x00,0x00,0x00,
0x00,0x00,0x00,0x40,0xe1,0x2e,0x53,0x3b,0x02,0x00,0x00,0x00,0x5e,0xfe,0xea,0xb7,0xe3,0xcf,
0x19,0x00,0x00,0x01,0x47,0x08,0x00,0x08,0xba,0xb8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x31,
0xfe,0x15,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0xff,0x2c,0x00,0x8c,0x5d,0x00,0x00,0x00,0x3b,
0xfb,0x0f,0x00,0x6b,0xed,0x39,0x00,0x1a,0xcb,0xa1,0x00,0x00,0x00,0x7f,0xf7,0xef,0xfe,0xae,
0x0d,0x00,0x00,0x00,0x00,0x0d,0x31,0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x5b,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x8e,0xa8,0x00,
0x00,0x00,0x00,0x00,0x00,0x26,0xf3,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0xb3,0x8a,0x00,0x00,
0x00,0x00,0x00,0x00,0x45,0xeb,0x0f,0x00,0x00,0x00,0x00,0x00,0x04,0xd5,0xcd,0xa7,0x83,0x17,
0x00,0x00,0x00,0x64,0xff,0xb9,0x7f,0xb7,0xed,0x2a,0x00,0x00,0xc7,0x9e,0x00,0x00,0x00,0x97,
0xba,0x00,0x00,0xf4,0x3e,0x00,0x00,0x00,0x2e,0xfc,0x02,0x00,0xe0,0x56,0x00,0x00,0x00,0x4b,
0xeb,0x00,0x00,0x78,0xdf,0x34,0x01,0x34,0xdc,0x76,0x00,0x00,0x02,0x91,0xfc,0xfc,0xfc,0x92,
0x03,0x00,0x00,0x00,0x00,0x12,0x33,0x13,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x0c,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x23,0x19,0xc8,0xc8,0xc8,0xc8,0xc8,0xdb,
0xfd,0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0x95,0xb8,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0xf0,
0x56,0x00,0x00,0x00,0x00,0x00,0x00,0x5f,0xed,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0xc4,0x93,
0x00,0x00,0x00,0x00,0x00,0x00,0x29,0xff,0x31,0x00,0x00,0x00,0x00,0x00,0x00,0x8e,0xcf,0x00,
0x00,0x00,0x00,0x00,0x00,0x07,0xec,0x6e,0x00,0x00,0x00,0x00,0x00,0x00,0x58,0xf9,0x13,0x00,
0x00,0x00,0x00,0x00,0x00,0xbd,0xaa,0x00,0x00,0x00,0x00,0x00,0x00,0x23,0xfe,0x48,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x14,0x81,0xa9,0x8b,0x21,0x00,0x00,0x00,0x17,0xe6,0xb3,0x71,0xa0,0xf0,
0x2e,0x00,0x00,0x7f,0xc8,0x00,0x00,0x00,0xa2,0xa6,0x00,0x00,0x99,0xa5,0x00,0x00,0x00,0x82,
0xbd,0x00,0x00,0x48,0xf1,0x33,0x00,0x20,0xdf,0x71,0x00,0x00,0x00,0x8c,0xff,0xed,0xfe,0xb0,
0x02,0x00,0x00,0x51,0xec,0x78,0x42,0x6a,0xe6,0x6c,0x00,0x04,0xeb,0x5b,0x00,0x00,0x00,0x31,
0xfb,0x21,0x25,0xff,0x16,0x00,0x00,0x00,0x00,0xe4,0x58,0x10,0xf9,0x40,0x00,0x00,0x00,0x16,
0xfd,0x39,0x00,0x98,0xd9,0x2a,0x00,0x1c,0xc0,0xc5,0x00,0x00,0x06,0x99,0xfc,0xee,0xfe,0xb1,
0x15,0x00,0x00,0x00,0x00,0x10,0x33,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x08,0x69,0x9b,0x83,0x1f,0x00,0x00,0x00,0x12,0xd2,0xd4,0x90,0xb7,0xf5,
0x3a,0x00,0x00,0x90,0xbf,0x05,0x00,0x00,0x7e,0xd6,0x00,0x00,0xd3,0x5b,0x00,0x00,0x00,0x12,
0xff,0x1b,0x00,0xc3,0x72,0x00,0x00,0x00,0x2c,0xff,0x0f,0x00,0x60,0xec,0x38,0x00,0x16,0xc3,
0xca,0x00,0x00,0x00,0x82,0xf9,0xeb,0xfd,0xff,0x55,0x00,0x00,0x00,0x00,0x15,0x40,0xa1,0xc4,
0x00,0x00,0x00,0x00,0x00,0x00,0x1a,0xf4,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x9f,0xab,0x00,
0x00,0x00,0x00,0x00,0x00,0x30,0xf8,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0xbc,0x92,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x7c,0x1e,0x19,0xf8,0x60,0x00,0x0b,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x7c,0x1e,0x19,0xf8,0x60,0x00,0x0b,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x7c,0x1e,0x19,
0xf8,0x60,0x00,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x7c,0x1e,0x19,
0xf7,0x73,0x00,0x75,0x36,0x00,0x5f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x6d,0x93,0x05,0x00,0x00,0x00,0x5b,0xf8,0x3b,0x00,0x00,0x00,
0x3a,0xf7,0x5c,0x00,0x00,0x00,0x21,0xea,0x83,0x00,0x00,0x00,0x0f,0xd5,0xa9,0x01,0x00,0x00,
0x00,0x15,0xdf,0x95,0x00,0x00,0x00,0x00,0x00,0x2d,0xf3,0x6b,0x00,0x00,0x00,0x00,0x00,0x4e,
0xf9,0x44,0x00,0x00,0x00,0x00,0x00,0x77,0xef,0x27,0x00,0x00,0x00,0x00,0x00,0x94,0xaa,0x05,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x5e,0xc4,0xc4,0xc4,0xc4,0xc4,0x96,0x28,0x54,0x54,0x54,0x54,0x54,0x40,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x70,0xe8,0xe8,0xe8,0xe8,0xe8,0xb1,0x19,0x34,0x34,0x34,0x34,0x34,0x27,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0x9c,0x5d,0x00,0x00,0x00,
0x00,0x00,0x51,0xf9,0x45,0x00,0x00,0x00,0x00,0x00,0x77,0xef,0x28,0x00,0x00,0x00,0x00,0x00,
0x9f,0xdc,0x13,0x00,0x00,0x00,0x00,0x06,0xc1,0xc2,0x06,0x00,0x00,0x00,0x01,0xae,0xce,0x0a,
0x00,0x00,0x00,0x84,0xe9,0x1d,0x00,0x00,0x00,0x58,0xf8,0x3b,0x00,0x00,0x00,0x35,0xf6,0x62,
0x00,0x00,0x00,0x0c,0xbd,0x87,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,
0x63,0xa3,0x98,0x4b,0x00,0x00,0x01,0xaf,0xde,0x79,0x84,0xec,0x8c,0x00,0x4d,0xe6,0x13,0x00,
0x00,0x37,0xff,0x21,0x30,0x4c,0x00,0x00,0x00,0x01,0xfb,0x40,0x00,0x00,0x00,0x00,0x00,0x37,
0xf9,0x16,0x00,0x00,0x00,0x00,0x36,0xe9,0x77,0x00,0x00,0x00,0x00,0x35,0xf2,0x69,0x00,0x00,
0x00,0x00,0x00,0xaa,0xa0,0x00,0x00,0x00,0x00,0x00,0x00,0xbf,0x7c,0x00,0x00,0x00,0x00,0x00,
0x00,0x24,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x5c,0x45,0x00,0x00,0x00,0x00,0x00,0x00,0xc7,
0xac,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x09,0x5f,0x95,0xa8,0x89,0x3a,0x00,0x00,0x00,0x00,0x00,0x00,0x45,0xe6,0xad,0x67,0x57,
0x7c,0xdf,0xb9,0x14,0x00,0x00,0x00,0x46,0xe6,0x38,0x00,0x00,0x00,0x00,0x08,0x8d,0xca,0x0b,
0x00,0x08,0xe7,0x3b,0x00,0x00,0x34,0x52,0x13,0x00,0x00,0x9f,0x98,0x00,0x5e,0xb0,0x00,0x00,
0x97,0xda,0xad,0xe7,0x2b,0x00,0x14,0xf2,0x0a,0x93,0x68,0x00,0x35,0xda,0x08,0x00,0x6a,0xac,
0x00,0x00,0xd2,0x30,0xa4,0x57,0x00,0x57,0xae,0x00,0x00,0x33,0xe0,0x01,0x02,0xe5,0x1d,0x88,
0x80,0x00,0x16,0xe9,0x48,0x16,0xa9,0xff,0x86,0x93,0xbc,0x00,0x3d,0xdf,0x02,0x00,0x3d,0xca,
0xea,0x9a,0x28,0x99,0x94,0x11,0x00,0x00,0xbb,0x91,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x17,0xd6,0x99,0x15,0x00,0x00,0x01,0x54,0x4a,0x00,0x00,0x00,0x00,0x00,0x0d,
0x92,0xf1,0xcc,0xbe,0xe7,0xd6,0x4b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x36,0x42,0x24,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x27,0x45,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xa9,0xf0,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x0e,
0xf6,0xeb,0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x61,0xd1,0x7f,0xb0,0x00,0x00,0x00,0x00,0x00,
0x00,0xbd,0x78,0x24,0xf9,0x12,0x00,0x00,0x00,0x00,0x1c,0xfd,0x20,0x00,0xc9,0x69,0x00,0x00,
0x00,0x00,0x76,0xc8,0x00,0x00,0x6e,0xc5,0x00,0x00,0x00,0x00,0xd2,0xf1,0xdc,0xdc,0xe4,0xff,
0x21,0x00,0x00,0x2e,0xfe,0x43,0x34,0x34,0x34,0xd2,0x7d,0x00,0x00,0x8a,0xc5,0x00,0x00,0x00,
0x00,0x7b,0xd8,0x00,0x02,0xe3,0x6f,0x00,0x00,0x00,0x00,0x27,0xff,0x35,0x42,0xfe,0x1c,0x00,
0x00,0x00,0x00,0x00,0xd2,0x91,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x60,0x60,0x60,0x50,0x2d,0x00,0x00,
0x00,0x0c,0xff,0xc3,0xb8,0xc7,0xf3,0xca,0x13,0x00,0x0c,0xff,0x30,0x00,0x00,0x0d,0xc2,0xa2,
0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x63,0xda,0x00,0x0c,0xff,0x30,0x00,0x00,0x02,0xad,0xa3,
0x00,0x0c,0xff,0xab,0x98,0xa5,0xdb,0xb5,0x13,0x00,0x0c,0xff,0x94,0x7c,0x7d,0x99,0xe8,0x75,
0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x25,0xf7,0x30,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xc4,
0x73,0x0c,0xff,0x30,0x00,0x00,0x00,0x06,0xe8,0x60,0x0c,0xff,0x46,0x1c,0x1d,0x3b,0xb5,0xe7,
0x0f,0x0c,0xff,0xff,0xff,0xfe,0xeb,0xae,0x26,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x31,
0x8d,0xa7,0x86,0x26,0x00,0x00,0x00,0x00,0x6f,0xfa,0xa1,0x74,0xa8,0xf6,0x48,0x00,0x00,0x37,
0xfa,0x4f,0x00,0x00,0x00,0x71,0xb6,0x00,0x00,0xb0,0xb5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x08,0xf8,0x57,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x24,0xff,0x2d,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x31,0xff,0x1d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1b,0xff,0x3a,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x02,0xe6,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8c,0xe1,0x08,
0x00,0x00,0x00,0x1f,0x76,0x04,0x00,0x0f,0xe0,0xb4,0x18,0x00,0x23,0xcb,0xaf,0x00,0x00,0x00,
0x1f,0xb8,0xfc,0xe9,0xfd,0xa8,0x0d,0x00,0x00,0x00,0x00,0x00,0x1c,0x33,0x12,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x60,
0x60,0x60,0x5a,0x33,0x00,0x00,0x00,0x00,0x0c,0xff,0xc5,0xb8,0xc9,0xf6,0xc6,0x1e,0x00,0x00,
0x0c,0xff,0x30,0x00,0x00,0x17,0xbb,0xd6,0x07,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x0f,0xec,
0x6d,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x90,0xc4,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,
0x00,0x5a,0xf3,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x4c,0xfe,0x00,0x0c,0xff,0x30,0x00,
0x00,0x00,0x00,0x5d,0xe9,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x9b,0xb1,0x00,0x0c,0xff,
0x30,0x00,0x00,0x00,0x1a,0xf6,0x4d,0x00,0x0c,0xff,0x46,0x1c,0x25,0x59,0xdb,0xa8,0x00,0x00,
0x0c,0xff,0xff,0xff,0xfb,0xd9,0x76,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x60,0x60,0x60,
0x60,0x60,0x60,0x60,0x0d,0x0c,0xff,0xc5,0xb8,0xb8,0xb8,0xb8,0xb8,0x19,0x0c,0xff,0x30,0x00,
0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,
0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x81,0x64,0x64,0x64,0x64,0x51,0x00,0x0c,0xff,0xc5,0xb8,
0xb8,0xb8,0xb8,0x95,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,
0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x46,0x1c,
0x1c,0x1c,0x1c,0x1c,0x03,0x0c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x24,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x60,0x60,0x60,
0x60,0x60,0x60,0x60,0x0d,0x0c,0xff,0xc5,0xb8,0xb8,0xb8,0xb8,0xb8,0x19,0x0c,0xff,0x30,0x00,
0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,
0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x81,0x64,0x64,0x64,0x64,0x51,0x00,0x0c,0xff,0xc5,0xb8,
0xb8,0xb8,0xb8,0x95,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,
0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,
0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2a,0x8a,0xa9,0x8f,0x31,0x00,0x00,0x00,0x00,0x00,0x60,
0xfa,0xa7,0x74,0x9e,0xf8,0x5a,0x00,0x00,0x00,0x29,0xf7,0x5f,0x00,0x00,0x00,0x53,0x53,0x00,
0x00,0x00,0xa6,0xc1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0xf4,0x5d,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x23,0xff,0x2e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x32,0xff,
0x1d,0x00,0x00,0x49,0x70,0x70,0x70,0x47,0x00,0x1c,0xff,0x3b,0x00,0x00,0x6e,0xa8,0xa8,0xdc,
0x9b,0x00,0x00,0xe3,0x7f,0x00,0x00,0x00,0x00,0x00,0xaf,0x7a,0x00,0x00,0x82,0xe5,0x0b,0x00,
0x00,0x00,0x12,0xf3,0x2d,0x00,0x00,0x0d,0xda,0xb7,0x18,0x00,0x19,0xbc,0xa7,0x00,0x00,0x00,
0x00,0x1c,0xb6,0xfb,0xe9,0xfd,0xa2,0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x19,0x36,0x19,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x04,0x60,0x12,0x00,0x00,0x00,0x00,0x3f,0x37,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xa8,0x94,
0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xa8,0x94,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xa8,0x94,
0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xa8,0x94,0x0c,0xff,0x7e,0x60,0x60,0x60,0x60,0xc9,0x94,
0x0c,0xff,0xc8,0xbc,0xbc,0xbc,0xbc,0xe8,0x94,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xa8,0x94,
0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xa8,0x94,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xa8,0x94,
0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xa8,0x94,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0xa8,0x94,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x2a,0x00,0xcc,0x70,
0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,
0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x13,0x60,0x03,0x00,0x00,0x00,
0x00,0x00,0x00,0x34,0xff,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x34,0xff,0x08,0x00,0x00,0x00,
0x00,0x00,0x00,0x34,0xff,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x34,0xff,0x08,0x00,0x00,0x00,
0x00,0x00,0x00,0x34,0xff,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x34,0xff,0x08,0x04,0x2f,0x00,
0x00,0x00,0x00,0x34,0xff,0x07,0x0b,0xff,0x13,0x00,0x00,0x00,0x47,0xf9,0x00,0x00,0xd3,0x65,
0x00,0x00,0x00,0x92,0xc5,0x00,0x00,0x62,0xec,0x3f,0x03,0x58,0xfa,0x53,0x00,0x00,0x00,0x84,
0xfc,0xfd,0xf9,0x77,0x00,0x00,0x00,0x00,0x00,0x10,0x33,0x10,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x60,0x12,0x00,0x00,0x00,0x00,0x10,
0x5f,0x21,0x0c,0xff,0x30,0x00,0x00,0x00,0x0f,0xca,0xa6,0x03,0x0c,0xff,0x30,0x00,0x00,0x0e,
0xc8,0xa7,0x03,0x00,0x0c,0xff,0x30,0x00,0x0d,0xc6,0xa8,0x03,0x00,0x00,0x0c,0xff,0x30,0x0c,
0xc4,0xa9,0x04,0x00,0x00,0x00,0x0c,0xff,0x3b,0xc3,0xbd,0x04,0x00,0x00,0x00,0x00,0x0c,0xff,
0xde,0xca,0xf1,0x2a,0x00,0x00,0x00,0x00,0x0c,0xff,0xa5,0x03,0x8f,0xdc,0x12,0x00,0x00,0x00,
0x0c,0xff,0x30,0x00,0x04,0xbf,0xbe,0x04,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x15,0xe1,0x96,
0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x33,0xf7,0x6a,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,
0x00,0x5f,0xfb,0x42,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x60,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,
0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,
0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,
0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,
0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,
0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x46,0x1c,0x1c,0x1c,0x1c,0x1c,0x04,0x0c,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x28,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x04,0x60,0x42,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x60,0x28,0x0c,0xff,0xec,0x05,
0x00,0x00,0x00,0x00,0x00,0x83,0xff,0x6c,0x0c,0xff,0xf8,0x50,0x00,0x00,0x00,0x00,0x01,0xdf,
0xf8,0x6c,0x0c,0xff,0xad,0xaf,0x00,0x00,0x00,0x00,0x40,0xe5,0xc6,0x6c,0x0c,0xff,0x51,0xf9,
0x13,0x00,0x00,0x00,0x9e,0x8c,0xc4,0x6c,0x0c,0xff,0x24,0xd0,0x6c,0x00,0x00,0x0a,0xf2,0x2f,
0xc4,0x6c,0x0c,0xff,0x24,0x74,0xca,0x00,0x00,0x5b,0xd3,0x00,0xc4,0x6c,0x0c,0xff,0x24,0x1a,
0xfd,0x29,0x00,0xba,0x77,0x00,0xc4,0x6c,0x0c,0xff,0x24,0x00,0xbc,0x88,0x1b,0xfb,0x1d,0x00,
0xc4,0x6c,0x0c,0xff,0x24,0x00,0x5f,0xe3,0x7a,0xbe,0x00,0x00,0xc4,0x6c,0x0c,0xff,0x24,0x00,
0x0d,0xf6,0xf3,0x62,0x00,0x00,0xc4,0x6c,0x0c,0xff,0x24,0x00,0x00,0x96,0xdb,0x0e,0x00,0x00,
0xc4,0x6c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x04,0x60,0x18,0x00,0x00,0x00,0x00,0x07,0x60,0x0f,0x0c,0xff,0xb2,0x00,0x00,0x00,0x00,0x14,
0xff,0x28,0x0c,0xff,0xf9,0x55,0x00,0x00,0x00,0x14,0xff,0x28,0x0c,0xff,0x80,0xe7,0x11,0x00,
0x00,0x14,0xff,0x28,0x0c,0xff,0x30,0xb0,0x9c,0x00,0x00,0x14,0xff,0x28,0x0c,0xff,0x30,0x1c,
0xf1,0x40,0x00,0x14,0xff,0x28,0x0c,0xff,0x30,0x00,0x6f,0xda,0x08,0x14,0xff,0x28,0x0c,0xff,
0x30,0x00,0x03,0xcb,0x86,0x14,0xff,0x28,0x0c,0xff,0x30,0x00,0x00,0x31,0xf7,0x42,0xff,0x28,
0x0c,0xff,0x30,0x00,0x00,0x00,0x8d,0xd9,0xff,0x28,0x0c,0xff,0x30,0x00,0x00,0x00,0x0b,0xe1,
0xff,0x28,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x4c,0xff,0x28,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x3f,0x96,0xa8,0x7d,0x17,0x00,0x00,0x00,0x00,0x88,0xf4,0x8f,0x73,
0xbb,0xec,0x2e,0x00,0x00,0x41,0xf6,0x33,0x00,0x00,0x00,0x94,0xcf,0x04,0x00,0xc0,0x8e,0x00,
0x00,0x00,0x00,0x0b,0xef,0x50,0x11,0xfc,0x3d,0x00,0x00,0x00,0x00,0x00,0xa7,0xa0,0x35,0xff,
0x13,0x00,0x00,0x00,0x00,0x00,0x85,0xc5,0x40,0xff,0x0a,0x00,0x00,0x00,0x00,0x00,0x75,0xd4,
0x27,0xff,0x22,0x00,0x00,0x00,0x00,0x00,0x88,0xbd,0x04,0xed,0x56,0x00,0x00,0x00,0x00,0x00,
0xc3,0x83,0x00,0x97,0xbf,0x00,0x00,0x00,0x00,0x30,0xfb,0x2b,0x00,0x14,0xea,0x94,0x0b,0x00,
0x31,0xdb,0x92,0x00,0x00,0x00,0x2e,0xd0,0xf9,0xec,0xf9,0x90,0x05,0x00,0x00,0x00,0x00,0x00,
0x23,0x34,0x0f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x04,0x60,0x60,0x5f,0x4c,0x24,0x00,0x00,0x00,0x0c,0xff,0xc5,0xb8,0xcb,0xf5,0xba,0x14,0x00,
0x0c,0xff,0x30,0x00,0x00,0x11,0xbb,0xb4,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x42,0xfe,0x07,
0x0c,0xff,0x30,0x00,0x00,0x00,0x5c,0xf3,0x02,0x0c,0xff,0x39,0x0c,0x1e,0x59,0xe8,0x86,0x00,
0x0c,0xff,0xff,0xff,0xff,0xd8,0x71,0x02,0x00,0x0c,0xff,0x3d,0x0f,0x05,0x00,0x00,0x00,0x00,
0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,
0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3d,0x95,0xa9,0x7c,0x11,0x00,0x00,
0x00,0x00,0x00,0x81,0xf2,0x91,0x74,0xbb,0xeb,0x2e,0x00,0x00,0x00,0x3a,0xf6,0x32,0x00,0x00,
0x00,0x8e,0xde,0x06,0x00,0x00,0xb9,0x9b,0x00,0x00,0x00,0x00,0x07,0xe6,0x62,0x00,0x0f,0xfa,
0x40,0x00,0x00,0x00,0x00,0x00,0x99,0xae,0x00,0x33,0xff,0x1b,0x00,0x00,0x00,0x00,0x00,0x75,
0xd4,0x00,0x40,0xff,0x0b,0x00,0x00,0x00,0x00,0x00,0x64,0xe6,0x00,0x27,0xff,0x23,0x00,0x00,
0x00,0x00,0x00,0x74,0xd2,0x00,0x04,0xef,0x5b,0x00,0x00,0x01,0x3d,0x00,0xa7,0x9c,0x00,0x00,
0x97,0xc7,0x01,0x00,0x2a,0xf3,0x9b,0xf6,0x40,0x00,0x00,0x19,0xe6,0x9e,0x0e,0x00,0x53,0xff,
0xef,0x23,0x00,0x00,0x00,0x29,0xc4,0xfa,0xed,0xfc,0x92,0x9c,0xcf,0x01,0x00,0x00,0x00,0x00,
0x18,0x35,0x14,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x04,0x60,0x60,0x5f,0x49,0x20,0x00,0x00,0x00,0x0c,0xff,0xc5,0xb8,0xcb,
0xf4,0xb3,0x19,0x00,0x0c,0xff,0x30,0x00,0x00,0x12,0xbd,0xbe,0x00,0x0c,0xff,0x30,0x00,0x00,
0x00,0x46,0xff,0x09,0x0c,0xff,0x30,0x00,0x00,0x00,0x57,0xf2,0x01,0x0c,0xff,0x39,0x0c,0x1d,
0x52,0xe7,0x80,0x00,0x0c,0xff,0xff,0xff,0xff,0xdb,0x6e,0x00,0x00,0x0c,0xff,0x3d,0x13,0xd1,
0x9c,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x2f,0xf9,0x48,0x00,0x00,0x0c,0xff,0x30,0x00,0x00,
0x7c,0xe3,0x0f,0x00,0x0c,0xff,0x30,0x00,0x00,0x03,0xca,0x9e,0x00,0x0c,0xff,0x30,0x00,0x00,
0x00,0x28,0xf6,0x4a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x45,0x94,0xa4,0x76,0x11,0x00,0x00,0x00,0x7d,0xf3,0x8f,0x77,
0xc0,0xe3,0x22,0x00,0x04,0xf6,0x59,0x00,0x00,0x00,0x94,0x72,0x00,0x10,0xff,0x3a,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xc4,0xc6,0x1e,0x00,0x00,0x00,0x00,0x00,0x00,0x1b,0xbd,0xfa,0xa9,
0x3a,0x00,0x00,0x00,0x00,0x00,0x00,0x36,0xa0,0xfb,0xa6,0x09,0x00,0x00,0x00,0x00,0x00,0x00,
0x2c,0xde,0x9c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45,0xf9,0x03,0x50,0xa5,0x01,0x00,0x00,
0x00,0x37,0xfe,0x07,0x1f,0xf0,0x9e,0x0e,0x00,0x2b,0xcf,0xb8,0x00,0x00,0x39,0xd7,0xfc,0xef,
0xfe,0xb5,0x15,0x00,0x00,0x00,0x02,0x23,0x34,0x15,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1e,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x15,
0x39,0xb8,0xb8,0xb8,0xe1,0xe7,0xb8,0xb8,0xb8,0x28,0x00,0x00,0x00,0x00,0x94,0xa8,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x94,0xa8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x94,0xa8,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x94,0xa8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x94,0xa8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x94,0xa8,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x94,0xa8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x94,0xa8,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x94,0xa8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x94,0xa8,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x60,0x15,0x00,0x00,0x00,0x00,0x0f,
0x60,0x0a,0x0c,0xff,0x38,0x00,0x00,0x00,0x00,0x28,0xff,0x1c,0x0c,0xff,0x38,0x00,0x00,0x00,
0x00,0x28,0xff,0x1c,0x0c,0xff,0x38,0x00,0x00,0x00,0x00,0x28,0xff,0x1c,0x0c,0xff,0x38,0x00,
0x00,0x00,0x00,0x28,0xff,0x1c,0x0c,0xff,0x38,0x00,0x00,0x00,0x00,0x28,0xff,0x1c,0x0b,0xff,
0x38,0x00,0x00,0x00,0x00,0x28,0xff,0x1b,0x02,0xfc,0x44,0x00,0x00,0x00,0x00,0x2f,0xff,0x13,
0x00,0xe4,0x63,0x00,0x00,0x00,0x00,0x51,0xf4,0x02,0x00,0x9f,0xb8,0x00,0x00,0x00,0x00,0xa4,
0xb6,0x00,0x00,0x2e,0xf7,0x7d,0x05,0x05,0x73,0xfc,0x3e,0x00,0x00,0x00,0x4a,0xe3,0xf8,0xf8,
0xee,0x5b,0x00,0x00,0x00,0x00,0x00,0x03,0x26,0x30,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1e,0x5f,0x01,0x00,0x00,0x00,
0x00,0x06,0x60,0x13,0x1c,0xff,0x36,0x00,0x00,0x00,0x00,0x47,0xf6,0x09,0x00,0xcd,0x83,0x00,
0x00,0x00,0x00,0x94,0xb2,0x00,0x00,0x7f,0xd0,0x00,0x00,0x00,0x00,0xe0,0x65,0x00,0x00,0x30,
0xff,0x1d,0x00,0x00,0x2d,0xfe,0x19,0x00,0x00,0x00,0xe1,0x69,0x00,0x00,0x7a,0xca,0x00,0x00,
0x00,0x00,0x94,0xb6,0x00,0x00,0xc7,0x7d,0x00,0x00,0x00,0x00,0x45,0xf7,0x0b,0x16,0xfd,0x2f,
0x00,0x00,0x00,0x00,0x05,0xf1,0x4f,0x60,0xe1,0x00,0x00,0x00,0x00,0x00,0x00,0xa8,0x9c,0xad,
0x95,0x00,0x00,0x00,0x00,0x00,0x00,0x5a,0xe8,0xf3,0x47,0x00,0x00,0x00,0x00,0x00,0x00,0x10,
0xfb,0xf3,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x1e,0x59,0x00,0x00,0x00,0x0c,0x18,0x00,0x00,0x00,0x29,0x4e,0x35,0xfe,
0x09,0x00,0x00,0xa6,0xff,0x25,0x00,0x00,0x88,0xb4,0x0d,0xff,0x2c,0x00,0x00,0xde,0xeb,0x5d,
0x00,0x00,0xad,0x8b,0x00,0xe5,0x51,0x00,0x17,0xfb,0x89,0x96,0x00,0x00,0xd2,0x63,0x00,0xbd,
0x76,0x00,0x50,0xce,0x47,0xcf,0x00,0x01,0xf6,0x3b,0x00,0x95,0x9b,0x00,0x88,0x97,0x11,0xfb,
0x0b,0x1c,0xff,0x12,0x00,0x6d,0xc0,0x00,0xc1,0x60,0x00,0xd9,0x40,0x41,0xea,0x00,0x00,0x45,
0xe5,0x04,0xf5,0x28,0x00,0xa1,0x79,0x66,0xc1,0x00,0x00,0x1d,0xff,0x3d,0xf0,0x01,0x00,0x6a,
0xb1,0x8b,0x99,0x00,0x00,0x01,0xf4,0x9b,0xba,0x00,0x00,0x33,0xea,0xb0,0x70,0x00,0x00,0x00,
0xcd,0xf0,0x82,0x00,0x00,0x04,0xf6,0xf0,0x48,0x00,0x00,0x00,0xa4,0xff,0x4b,0x00,0x00,0x00,
0xc4,0xff,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x1d,0x60,0x0a,0x00,0x00,0x00,0x00,0x30,0x57,0x00,0x06,0xd6,0x85,0x00,0x00,0x00,
0x08,0xdf,0x7b,0x00,0x00,0x3c,0xf7,0x25,0x00,0x00,0x7e,0xd5,0x05,0x00,0x00,0x00,0x98,0xb7,
0x00,0x1e,0xf4,0x3c,0x00,0x00,0x00,0x00,0x0f,0xe6,0x50,0xab,0x9b,0x00,0x00,0x00,0x00,0x00,
0x00,0x52,0xec,0xe9,0x11,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0xf1,0xaa,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x91,0xd2,0xf8,0x3e,0x00,0x00,0x00,0x00,0x00,0x33,0xf8,0x2e,0x85,0xd7,0x07,
0x00,0x00,0x00,0x03,0xcd,0x8c,0x00,0x0a,0xe0,0x80,0x00,0x00,0x00,0x71,0xe3,0x0c,0x00,0x00,
0x50,0xf9,0x28,0x00,0x1c,0xf3,0x52,0x00,0x00,0x00,0x00,0xb5,0xc1,0x01,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x1b,0x5e,0x03,0x00,0x00,0x00,0x00,0x3d,0x3f,0x06,0xd6,0x6c,0x00,0x00,0x00,0x19,0xf1,0x3c,
0x00,0x3c,0xed,0x14,0x00,0x00,0xa5,0x99,0x00,0x00,0x00,0x9b,0x9c,0x00,0x3d,0xe6,0x10,0x00,
0x00,0x00,0x11,0xe7,0x39,0xd1,0x56,0x00,0x00,0x00,0x00,0x00,0x5a,0xf1,0xb5,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xf1,0x4d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf0,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xf0,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf0,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xf0,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf0,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x5e,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x31,0x00,0xb5,0xb8,0xb8,
0xb8,0xb8,0xb8,0xda,0xf2,0x1d,0x00,0x00,0x00,0x00,0x00,0x00,0x0b,0xdd,0x68,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x93,0xbe,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0xf3,0x22,0x00,0x00,
0x00,0x00,0x00,0x00,0x0a,0xdc,0x6f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x91,0xc4,0x02,0x00,
0x00,0x00,0x00,0x00,0x00,0x3c,0xf5,0x26,0x00,0x00,0x00,0x00,0x00,0x00,0x09,0xdb,0x76,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x8f,0xca,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x3b,0xfb,0x47,
0x1c,0x1c,0x1c,0x1c,0x1c,0x14,0x09,0xd9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbc,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xec,0xf4,0xf4,0x1a,0x00,0xf8,0x50,0x24,0x03,0x00,0xf8,0x34,0x00,
0x00,0x00,0xf8,0x34,0x00,0x00,0x00,0xf8,0x34,0x00,0x00,0x00,0xf8,0x34,0x00,0x00,0x00,0xf8,
0x34,0x00,0x00,0x00,0xf8,0x34,0x00,0x00,0x00,0xf8,0x34,0x00,0x00,0x00,0xf8,0x34,0x00,0x00,
0x00,0xf8,0x34,0x00,0x00,0x00,0xf8,0x7a,0x58,0x09,0x00,0xba,0xc0,0xc0,0x15,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x03,0x10,0x02,0x00,0x00,0x00,0x00,0x00,0x0b,0xed,0x62,0x00,0x00,0x00,0x00,0x00,0x00,0x82,
0xd7,0x01,0x00,0x00,0x00,0x00,0x00,0x14,0xf5,0x4f,0x00,0x00,0x00,0x00,0x00,0x00,0x93,0xc6,
0x00,0x00,0x00,0x00,0x00,0x00,0x20,0xfb,0x3d,0x00,0x00,0x00,0x00,0x00,0x00,0xa4,0xb3,0x00,
0x00,0x00,0x00,0x00,0x00,0x2d,0xfd,0x2b,0x00,0x00,0x00,0x00,0x00,0x00,0xb5,0xa1,0x00,0x00,
0x00,0x00,0x00,0x00,0x3e,0xfa,0x1d,0x00,0x00,0x00,0x00,0x00,0x00,0xc7,0x8e,0x00,0x00,0x00,
0x00,0x00,0x00,0x4f,0xf3,0x11,0x00,0x00,0x00,0x00,0x00,0x01,0xd6,0x7c,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf0,0xf4,0xf4,0x16,0x00,
0x23,0x31,0xff,0x18,0x00,0x00,0x10,0xff,0x18,0x00,0x00,0x10,0xff,0x18,0x00,0x00,0x10,0xff,
0x18,0x00,0x00,0x10,0xff,0x18,0x00,0x00,0x10,0xff,0x18,0x00,0x00,0x10,0xff,0x18,0x00,0x00,
0x10,0xff,0x18,0x00,0x00,0x10,0xff,0x18,0x00,0x00,0x10,0xff,0x18,0x00,0x5a,0x66,0xff,0x18,
0x00,0xbd,0xc0,0xc0,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x12,0xa5,0x57,0x00,
0x00,0x00,0x03,0xba,0xb9,0xf1,0x37,0x00,0x00,0x8d,0xd5,0x0a,0x6d,0xe6,0x1a,0x00,0x00,0x00,
0x00,0x00,0x04,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x4c,0x4c,0x4c,
0x4c,0x4c,0x4c,0x4c,0x4c,0x0f,0x41,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0x2a,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x08,0x0e,0x00,0x00,0x67,0xe2,0x2a,0x00,0x00,0x6e,0xe6,0x3c,0x00,0x00,0x2c,0x67,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x75,0x5f,0x0b,0x00,0x00,
0x00,0x58,0xd9,0x9a,0xbd,0xd9,0x0f,0x00,0x00,0x01,0x04,0x00,0x00,0xaa,0x82,0x00,0x00,0x01,
0x5b,0x90,0x65,0x64,0xb5,0x00,0x00,0xa8,0xe2,0x83,0xa4,0xde,0xc4,0x00,0x30,0xfb,0x1e,0x00,
0x00,0x90,0xc4,0x00,0x4b,0xe9,0x00,0x00,0x00,0x5e,0xc4,0x00,0x1a,0xf8,0x61,0x00,0x0f,0xc4,
0xc4,0x00,0x00,0x5f,0xf5,0xe3,0xdf,0x88,0xc4,0x00,0x00,0x00,0x0c,0x30,0x0c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x5a,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0x00,
0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0x35,0x5a,0x92,0x6a,0x06,0x00,0x00,0xfc,
0xaa,0xbe,0x87,0xe6,0xb3,0x00,0x00,0xfc,0xbe,0x02,0x00,0x2a,0xfd,0x31,0x00,0xfc,0x54,0x00,
0x00,0x00,0xbe,0x79,0x00,0xfc,0x31,0x00,0x00,0x00,0x9d,0x96,0x00,0xfc,0x3b,0x00,0x00,0x00,
0xa8,0x8a,0x00,0xfc,0x74,0x00,0x00,0x01,0xdc,0x62,0x00,0xfc,0xda,0x24,0x00,0x71,0xf6,0x10,
0x00,0xfc,0x6e,0xeb,0xe7,0xf6,0x59,0x00,0x00,0x00,0x00,0x0d,0x32,0x10,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x15,0x69,0x7f,0x31,0x00,0x00,0x32,0xec,0xb8,0x98,0xf2,0x57,0x00,0xd5,0x89,0x00,0x00,
0x41,0x4d,0x37,0xf7,0x0c,0x00,0x00,0x00,0x00,0x5d,0xd7,0x00,0x00,0x00,0x00,0x00,0x56,0xe2,
0x00,0x00,0x00,0x00,0x00,0x20,0xfd,0x23,0x00,0x00,0x01,0x00,0x00,0xa9,0xbc,0x0c,0x01,0x88,
0x88,0x00,0x0d,0xb4,0xf7,0xe7,0xd4,0x22,0x00,0x00,0x00,0x1f,0x2e,0x03,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x5c,0x11,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0x30,0x00,0x00,0x55,0x92,0x6c,0x07,0xff,0x30,0x00,0x7f,0xf6,0x8f,0xa7,0xa5,0xff,0x30,
0x0c,0xf3,0x57,0x00,0x00,0x91,0xff,0x30,0x49,0xee,0x02,0x00,0x00,0x25,0xff,0x30,0x63,0xd0,
0x00,0x00,0x00,0x04,0xfd,0x30,0x59,0xdb,0x00,0x00,0x00,0x0c,0xff,0x30,0x32,0xfa,0x15,0x00,
0x00,0x44,0xff,0x30,0x00,0xd3,0x9f,0x02,0x0f,0xbd,0xff,0x30,0x00,0x35,0xe9,0xed,0xf1,0x5e,
0xff,0x30,0x00,0x00,0x08,0x33,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x1d,0x76,0x7b,0x23,0x00,0x00,0x00,0x34,0xf2,0xab,0xa4,0xf4,0x3a,0x00,0x00,0xc9,
0x9b,0x00,0x00,0x7a,0xc1,0x00,0x18,0xff,0x2e,0x00,0x00,0x10,0xfe,0x12,0x35,0xff,0xbb,0xb8,
0xb8,0xb8,0xfb,0x31,0x2e,0xff,0x63,0x54,0x54,0x54,0x54,0x13,0x09,0xf8,0x49,0x00,0x00,0x00,
0x00,0x00,0x00,0x9d,0xd0,0x0c,0x01,0x6a,0x99,0x00,0x00,0x0e,0xbd,0xf0,0xe7,0xdd,0x39,0x00,
0x00,0x00,0x00,0x26,0x2c,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x01,0x57,0x81,0x1e,0x00,0x00,0x88,0xc4,0xa5,0x3e,0x00,0x00,0xe5,0x46,
0x00,0x00,0x01,0x64,0xf6,0x8d,0x4e,0x00,0x02,0xac,0xfa,0xc1,0x86,0x00,0x00,0x00,0xf0,0x40,
0x00,0x00,0x00,0x00,0xf0,0x40,0x00,0x00,0x00,0x00,0xf0,0x40,0x00,0x00,0x00,0x00,0xf0,0x40,
0x00,0x00,0x00,0x00,0xf0,0x40,0x00,0x00,0x00,0x00,0xf0,0x40,0x00,0x00,0x00,0x00,0xf0,0x40,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0x92,0x6c,0x07,0x64,0x12,
0x00,0x7f,0xf6,0x8f,0xa7,0xa5,0xff,0x30,0x0c,0xf3,0x57,0x00,0x00,0x91,0xff,0x30,0x49,0xee,
0x02,0x00,0x00,0x25,0xff,0x30,0x63,0xd0,0x00,0x00,0x00,0x04,0xfd,0x30,0x5d,0xdb,0x00,0x00,
0x00,0x0c,0xff,0x30,0x33,0xfa,0x15,0x00,0x00,0x44,0xff,0x30,0x01,0xd5,0x9f,0x02,0x0f,0xbe,
0xff,0x30,0x00,0x31,0xe4,0xed,0xf0,0x5d,0xff,0x30,0x07,0x20,0x08,0x33,0x15,0x0b,0xff,0x26,
0x0e,0xf4,0x32,0x00,0x00,0x60,0xe2,0x02,0x00,0x5e,0xee,0xa1,0xab,0xf1,0x44,0x00,0x00,0x00,
0x27,0x75,0x6a,0x1d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x5e,0x13,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0x00,
0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0x34,0x59,0x92,0x6b,0x05,0x00,0x00,0xfc,
0xb1,0xb0,0x81,0xe7,0xa7,0x00,0x00,0xfc,0xb2,0x00,0x00,0x34,0xff,0x28,0x00,0xfc,0x5c,0x00,
0x00,0x00,0xda,0x60,0x00,0xfc,0x3d,0x00,0x00,0x00,0xbb,0x74,0x00,0xfc,0x34,0x00,0x00,0x00,
0xb4,0x78,0x00,0xfc,0x34,0x00,0x00,0x00,0xb4,0x78,0x00,0xfc,0x34,0x00,0x00,0x00,0xb4,0x78,
0x00,0xfc,0x34,0x00,0x00,0x00,0xb4,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0e,
0x01,0x00,0xde,0x64,0x00,0x2d,0x0b,0x00,0x52,0x23,0x00,0xd4,0x5c,0x00,0xd4,0x5c,0x00,0xd4,
0x5c,0x00,0xd4,0x5c,0x00,0xd4,0x5c,0x00,0xd4,0x5c,0x00,0xd4,0x5c,0x00,0xd4,0x5c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x09,0x00,0x00,0x8f,0xaf,0x00,0x00,
0x17,0x20,0x00,0x00,0x35,0x41,0x00,0x00,0x88,0xa8,0x00,0x00,0x88,0xa8,0x00,0x00,0x88,0xa8,
0x00,0x00,0x88,0xa8,0x00,0x00,0x88,0xa8,0x00,0x00,0x88,0xa8,0x00,0x00,0x88,0xa8,0x00,0x00,
0x88,0xa8,0x00,0x00,0x88,0xa8,0x02,0x00,0x98,0x91,0xa0,0xc9,0xe3,0x28,0x19,0x4d,0x10,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5a,0x12,0x00,
0x00,0x00,0x00,0x00,0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0x34,0x00,0x00,0x00,
0x00,0x00,0x00,0xfc,0x34,0x00,0x00,0x4d,0x3a,0x00,0x00,0xfc,0x34,0x00,0x59,0xe4,0x1b,0x00,
0x00,0xfc,0x34,0x36,0xed,0x33,0x00,0x00,0x00,0xfc,0x4f,0xe5,0x54,0x00,0x00,0x00,0x00,0xfc,
0xea,0xed,0x15,0x00,0x00,0x00,0x00,0xfc,0xa0,0x99,0xb8,0x01,0x00,0x00,0x00,0xfc,0x34,0x09,
0xd3,0x79,0x00,0x00,0x00,0xfc,0x34,0x00,0x2a,0xf1,0x3c,0x00,0x00,0xfc,0x34,0x00,0x00,0x62,
0xe3,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x21,0x00,
0x00,0xd4,0x5c,0x00,0x00,0xd4,0x5c,0x00,0x00,0xd4,0x5c,0x00,0x00,0xd4,0x5c,0x00,0x00,0xd4,
0x5c,0x00,0x00,0xd4,0x5c,0x00,0x00,0xd4,0x5c,0x00,0x00,0xd4,0x5c,0x00,0x00,0xd4,0x5c,0x00,
0x00,0xce,0x6a,0x00,0x00,0x7a,0xf7,0x3b,0x00,0x00,0x20,0x10,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x62,0x17,0x6e,0x8e,0x36,0x00,0x32,0x8d,0x6e,0x05,0x00,0x00,0xfc,0xb2,0x93,0xad,0xf8,
0x53,0xce,0x93,0xeb,0x98,0x00,0x00,0xfc,0x92,0x00,0x00,0xd2,0xf2,0x18,0x00,0x56,0xf7,0x05,
0x00,0xfc,0x4d,0x00,0x00,0x97,0xc6,0x00,0x00,0x1c,0xff,0x1d,0x00,0xfc,0x38,0x00,0x00,0x86,
0xb0,0x00,0x00,0x0b,0xff,0x28,0x00,0xfc,0x34,0x00,0x00,0x84,0xac,0x00,0x00,0x08,0xff,0x28,
0x00,0xfc,0x34,0x00,0x00,0x84,0xac,0x00,0x00,0x08,0xff,0x28,0x00,0xfc,0x34,0x00,0x00,0x84,
0xac,0x00,0x00,0x08,0xff,0x28,0x00,0xfc,0x34,0x00,0x00,0x84,0xac,0x00,0x00,0x08,0xff,0x28,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x62,0x15,0x59,
0x92,0x6b,0x05,0x00,0x00,0xfc,0xb1,0xb0,0x81,0xe7,0xa7,0x00,0x00,0xfc,0xb2,0x00,0x00,0x34,
0xff,0x28,0x00,0xfc,0x5c,0x00,0x00,0x00,0xda,0x60,0x00,0xfc,0x3d,0x00,0x00,0x00,0xbb,0x74,
0x00,0xfc,0x34,0x00,0x00,0x00,0xb4,0x78,0x00,0xfc,0x34,0x00,0x00,0x00,0xb4,0x78,0x00,0xfc,
0x34,0x00,0x00,0x00,0xb4,0x78,0x00,0xfc,0x34,0x00,0x00,0x00,0xb4,0x78,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1b,0x72,0x7f,0x3e,
0x00,0x00,0x00,0x2f,0xf0,0xb0,0x92,0xeb,0x83,0x00,0x00,0xb6,0x8c,0x00,0x00,0x27,0xf8,0x2a,
0x0a,0xfe,0x2c,0x00,0x00,0x00,0xc0,0x79,0x29,0xff,0x0a,0x00,0x00,0x00,0x9f,0x98,0x25,0xff,
0x11,0x00,0x00,0x00,0xa6,0x91,0x05,0xf3,0x44,0x00,0x00,0x00,0xd9,0x65,0x00,0x91,0xc7,0x0c,
0x00,0x6a,0xef,0x12,0x00,0x09,0xae,0xf1,0xe0,0xe8,0x42,0x00,0x00,0x00,0x00,0x1e,0x2f,0x08,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x62,0x15,0x5a,0x92,0x6a,0x06,0x00,
0x00,0xfc,0xaa,0xbe,0x87,0xe6,0xb3,0x00,0x00,0xfc,0xbe,0x02,0x00,0x2a,0xfd,0x31,0x00,0xfc,
0x54,0x00,0x00,0x00,0xbe,0x79,0x00,0xfc,0x31,0x00,0x00,0x00,0x9d,0x96,0x00,0xfc,0x3b,0x00,
0x00,0x00,0xa8,0x8a,0x00,0xfc,0x74,0x00,0x00,0x01,0xdc,0x62,0x00,0xfc,0xda,0x24,0x00,0x71,
0xf6,0x10,0x00,0xfc,0x6e,0xeb,0xe7,0xf6,0x59,0x00,0x00,0xfc,0x34,0x0d,0x32,0x10,0x00,0x00,
0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0x56,
0x11,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,
0x92,0x6c,0x07,0x64,0x12,0x00,0x00,0x7f,0xf6,0x8f,0xa7,0xa5,0xff,0x30,0x00,0x0c,0xf3,0x57,
0x00,0x00,0x91,0xff,0x30,0x00,0x49,0xee,0x02,0x00,0x00,0x25,0xff,0x30,0x00,0x63,0xd0,0x00,
0x00,0x00,0x04,0xfd,0x30,0x00,0x59,0xdb,0x00,0x00,0x00,0x0c,0xff,0x30,0x00,0x32,0xfa,0x15,
0x00,0x00,0x44,0xff,0x30,0x00,0x00,0xd6,0x9f,0x02,0x0f,0xbd,0xff,0x30,0x00,0x00,0x37,0xea,
0xed,0xf2,0x5d,0xff,0x30,0x00,0x00,0x00,0x09,0x33,0x15,0x00,0xff,0x30,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xfd,0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xca,0xc0,0x0f,0x00,0x00,0x00,
0x00,0x00,0x00,0x16,0x6e,0x0e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x62,0x15,0x59,0x92,0x52,
0x00,0xfc,0xb1,0xb0,0x88,0x3d,0x00,0xfc,0xb2,0x00,0x00,0x00,0x00,0xfc,0x5c,0x00,0x00,0x00,
0x00,0xfc,0x3d,0x00,0x00,0x00,0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0xfc,0x34,0x00,0x00,0x00,
0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0xfc,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x55,0x89,0x71,
0x14,0x00,0x00,0xa2,0xd9,0x83,0xb5,0xe7,0x1b,0x02,0xfa,0x33,0x00,0x00,0x52,0x18,0x00,0xd8,
0x90,0x08,0x00,0x00,0x00,0x00,0x2c,0xcc,0xef,0x98,0x22,0x00,0x00,0x00,0x00,0x34,0xa1,0xf3,
0x2c,0x00,0x00,0x00,0x00,0x00,0xbc,0x83,0x11,0xd3,0x3a,0x00,0x10,0xdc,0x61,0x00,0x69,0xf3,
0xdc,0xf3,0xa1,0x03,0x00,0x00,0x0e,0x32,0x19,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x08,0x58,0x08,0x00,0x00,0x18,0xff,0x18,0x00,0x14,0x72,0xff,0x72,0x3e,
0x22,0xb3,0xff,0xb3,0x6b,0x00,0x18,0xff,0x18,0x00,0x00,0x18,0xff,0x18,0x00,0x00,0x18,0xff,
0x18,0x00,0x00,0x18,0xff,0x18,0x00,0x00,0x18,0xff,0x18,0x00,0x00,0x10,0xff,0x2b,0x00,0x00,
0x00,0xc4,0xeb,0x6b,0x00,0x00,0x03,0x2d,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x64,0x0c,0x00,0x00,0x00,0x4e,0x28,0x10,0xff,
0x20,0x00,0x00,0x00,0xc8,0x68,0x10,0xff,0x20,0x00,0x00,0x00,0xc8,0x68,0x10,0xff,0x20,0x00,
0x00,0x00,0xc8,0x68,0x0f,0xff,0x20,0x00,0x00,0x00,0xc8,0x68,0x08,0xff,0x2d,0x00,0x00,0x00,
0xdf,0x68,0x00,0xe8,0x5a,0x00,0x00,0x0d,0xfc,0x68,0x00,0x9b,0xcf,0x0d,0x02,0x92,0xf4,0x68,
0x00,0x16,0xd3,0xf2,0xe9,0x95,0xc8,0x68,0x00,0x00,0x03,0x30,0x1f,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x1d,0x5f,0x00,0x00,0x00,0x00,0x49,0x31,0x14,0xfb,0x2c,0x00,
0x00,0x06,0xf0,0x41,0x00,0xb8,0x7e,0x00,0x00,0x4b,0xe6,0x02,0x00,0x61,0xcf,0x00,0x00,0x9f,
0x90,0x00,0x00,0x10,0xf9,0x21,0x04,0xed,0x38,0x00,0x00,0x00,0xb4,0x73,0x47,0xdf,0x00,0x00,
0x00,0x00,0x5d,0xc4,0x9b,0x88,0x00,0x00,0x00,0x00,0x0e,0xf6,0xef,0x30,0x00,0x00,0x00,0x00,
0x00,0xaf,0xd7,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x29,0x4d,0x00,0x00,0x0e,0x54,0x07,0x00,0x00,0x55,0x21,
0x3c,0xee,0x01,0x00,0x57,0xff,0x42,0x00,0x0a,0xfb,0x26,0x04,0xf3,0x2c,0x00,0x99,0xdc,0x84,
0x00,0x41,0xe1,0x00,0x00,0xb3,0x68,0x00,0xda,0x5b,0xc6,0x00,0x7d,0x9d,0x00,0x00,0x6f,0xa3,
0x1d,0xdd,0x05,0xef,0x0c,0xb8,0x58,0x00,0x00,0x2b,0xdf,0x5f,0x9a,0x00,0xb7,0x4c,0xef,0x15,
0x00,0x00,0x00,0xe6,0xbb,0x57,0x00,0x75,0xbb,0xcf,0x00,0x00,0x00,0x00,0xa3,0xfe,0x15,0x00,
0x33,0xff,0x8a,0x00,0x00,0x00,0x00,0x5e,0xd1,0x00,0x00,0x02,0xee,0x46,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x2c,0x56,0x00,0x00,0x00,0x45,0x3d,0x18,0xf2,0x3e,0x00,0x1b,0xf5,0x37,0x00,0x75,0xc7,
0x00,0x9b,0xa0,0x00,0x00,0x06,0xd9,0x7a,0xee,0x18,0x00,0x00,0x00,0x51,0xff,0x7e,0x00,0x00,
0x00,0x00,0x8b,0xe5,0xbf,0x00,0x00,0x00,0x26,0xf3,0x27,0xdf,0x55,0x00,0x00,0xb5,0x94,0x00,
0x5a,0xe1,0x09,0x4a,0xf5,0x18,0x00,0x01,0xcc,0x81,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x06,0x64,0x0c,0x00,0x00,0x00,0x4e,0x28,0x10,0xff,0x20,0x00,0x00,0x00,
0xc8,0x68,0x10,0xff,0x20,0x00,0x00,0x00,0xc8,0x68,0x10,0xff,0x20,0x00,0x00,0x00,0xc8,0x68,
0x0f,0xff,0x20,0x00,0x00,0x00,0xc8,0x68,0x08,0xff,0x2d,0x00,0x00,0x00,0xdf,0x68,0x00,0xe8,
0x5a,0x00,0x00,0x0d,0xfc,0x68,0x00,0x9b,0xcf,0x0d,0x02,0x92,0xf4,0x68,0x00,0x16,0xd3,0xf2,
0xe9,0x95,0xc8,0x68,0x00,0x00,0x03,0x30,0x1f,0x00,0xd6,0x5e,0x00,0x22,0x31,0x00,0x00,0x35,
0xf5,0x1e,0x00,0x62,0xf2,0xab,0xa2,0xf0,0x63,0x00,0x00,0x00,0x25,0x73,0x6d,0x24,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x60,0x64,0x64,0x64,0x64,0x5f,0x01,0x00,0xb2,0xb8,0xb8,0xb8,0xfc,0x8d,0x00,
0x00,0x00,0x00,0x00,0x5a,0xe4,0x0c,0x00,0x00,0x00,0x00,0x0e,0xe6,0x56,0x00,0x00,0x00,0x00,
0x00,0x91,0xba,0x00,0x00,0x00,0x00,0x00,0x2f,0xf5,0x26,0x00,0x00,0x00,0x00,0x01,0xc6,0x83,
0x00,0x00,0x00,0x00,0x00,0x63,0xe9,0x25,0x1c,0x1c,0x18,0x00,0x12,0xeb,0xff,0xff,0xff,0xff,
0xdc,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x35,0xcc,0xe6,0x00,0x00,0x00,0xb2,0x9a,0x1a,0x00,0x00,0x00,
0xc2,0x57,0x00,0x00,0x00,0x00,0xcd,0x4b,0x00,0x00,0x00,0x02,0xf1,0x28,0x00,0x00,0x00,0x5f,
0xe7,0x01,0x00,0x00,0x0c,0xc9,0xa7,0x00,0x00,0x00,0x00,0x25,0xfb,0x0e,0x00,0x00,0x00,0x00,
0xe1,0x2e,0x00,0x00,0x00,0x00,0xcb,0x47,0x00,0x00,0x00,0x00,0xbf,0x58,0x00,0x00,0x00,0x00,
0xa7,0xb5,0x44,0x00,0x00,0x00,0x20,0xa5,0xc3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x01,0x5c,0x08,0x04,0xff,0x18,0x04,0xff,0x18,0x04,0xff,0x18,0x04,0xff,
0x18,0x04,0xff,0x18,0x04,0xff,0x18,0x04,0xff,0x18,0x04,0xff,0x18,0x04,0xff,0x18,0x04,0xff,
0x18,0x04,0xff,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xe2,0xcc,0x37,0x00,0x00,0x00,0x19,0x9a,0xb3,0x00,0x00,0x00,0x00,
0x57,0xc2,0x00,0x00,0x00,0x00,0x43,0xd0,0x00,0x00,0x00,0x00,0x28,0xf0,0x02,0x00,0x00,0x00,
0x03,0xe9,0x59,0x00,0x00,0x00,0x00,0xa2,0xc6,0x0a,0x00,0x00,0x08,0xf9,0x29,0x00,0x00,0x00,
0x37,0xe4,0x00,0x00,0x00,0x00,0x50,0xcb,0x00,0x00,0x00,0x00,0x59,0xc2,0x00,0x00,0x00,0x4f,
0xbd,0xa9,0x00,0x00,0x00,0xb4,0x99,0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x9a,0x74,0x00,0x00,0x10,0x76,0x82,
0xb6,0xbd,0xb1,0x72,0x04,0x00,0x02,0x6d,0x74,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x31,0x8d,0xa7,0x86,0x26,0x00,0x00,
0x00,0x00,0x6f,0xfa,0xa1,0x74,0xa8,0xf6,0x48,0x00,0x00,0x37,0xfa,0x4f,0x00,0x00,0x00,0x71,
0xb6,0x00,0x00,0xb0,0xb5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0xf8,0x57,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x24,0xff,0x2d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x31,0xff,0x1d,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x1b,0xff,0x3a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xe6,
0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8c,0xe1,0x08,0x00,0x00,0x00,0x1f,0x76,0x04,
0x00,0x0f,0xe0,0xb4,0x18,0x00,0x23,0xcb,0xaf,0x00,0x00,0x00,0x1f,0xb8,0xfc,0xe9,0xfd,0xa8,
0x0d,0x00,0x00,0x00,0x00,0x00,0x1c,0x6f,0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,
0xd1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xdf,0x88,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x37,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x39,0xc0,0x03,0x00,0x7d,0x85,0x00,0x00,
0x00,0x00,0x1c,0x74,0x00,0x00,0x45,0x46,0x00,0x00,0x00,0x00,0x00,0x3f,0x96,0xa8,0x7d,0x17,
0x00,0x00,0x00,0x00,0x88,0xf4,0x8f,0x73,0xbb,0xec,0x2e,0x00,0x00,0x41,0xf6,0x33,0x00,0x00,
0x00,0x94,0xcf,0x04,0x00,0xc0,0x8e,0x00,0x00,0x00,0x00,0x0b,0xef,0x50,0x11,0xfc,0x3d,0x00,
0x00,0x00,0x00,0x00,0xa7,0xa0,0x35,0xff,0x13,0x00,0x00,0x00,0x00,0x00,0x85,0xc5,0x40,0xff,
0x0a,0x00,0x00,0x00,0x00,0x00,0x75,0xd4,0x27,0xff,0x22,0x00,0x00,0x00,0x00,0x00,0x88,0xbd,
0x04,0xed,0x56,0x00,0x00,0x00,0x00,0x00,0xc3,0x83,0x00,0x97,0xbf,0x00,0x00,0x00,0x00,0x30,
0xfb,0x2b,0x00,0x14,0xea,0x94,0x0b,0x00,0x31,0xdb,0x92,0x00,0x00,0x00,0x2e,0xd0,0xf9,0xec,
0xf9,0x90,0x05,0x00,0x00,0x00,0x00,0x00,0x23,0x34,0x0f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x39,0xc0,0x03,0x00,0x7d,0x85,
0x00,0x00,0x00,0x00,0x1c,0x74,0x00,0x00,0x45,0x46,0x00,0x00,0x04,0x60,0x15,0x00,0x00,0x00,
0x00,0x0f,0x60,0x0a,0x0c,0xff,0x38,0x00,0x00,0x00,0x00,0x28,0xff,0x1c,0x0c,0xff,0x38,0x00,
0x00,0x00,0x00,0x28,0xff,0x1c,0x0c,0xff,0x38,0x00,0x00,0x00,0x00,0x28,0xff,0x1c,0x0c,0xff,
0x38,0x00,0x00,0x00,0x00,0x28,0xff,0x1c,0x0c,0xff,0x38,0x00,0x00,0x00,0x00,0x28,0xff,0x1c,
0x0b,0xff,0x38,0x00,0x00,0x00,0x00,0x28,0xff,0x1b,0x02,0xfc,0x44,0x00,0x00,0x00,0x00,0x2f,
0xff,0x13,0x00,0xe4,0x63,0x00,0x00,0x00,0x00,0x51,0xf4,0x02,0x00,0x9f,0xb8,0x00,0x00,0x00,
0x00,0xa4,0xb6,0x00,0x00,0x2e,0xf7,0x7d,0x05,0x05,0x73,0xfc,0x3e,0x00,0x00,0x00,0x4a,0xe3,
0xf8,0xf8,0xee,0x5b,0x00,0x00,0x00,0x00,0x00,0x03,0x26,0x30,0x06,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x15,0x69,0x7f,0x31,
0x00,0x00,0x00,0x32,0xec,0xb8,0x98,0xf2,0x57,0x00,0x00,0xd5,0x89,0x00,0x00,0x41,0x4d,0x00,
0x37,0xf7,0x0c,0x00,0x00,0x00,0x00,0x00,0x5d,0xd7,0x00,0x00,0x00,0x00,0x00,0x00,0x56,0xe2,
0x00,0x00,0x00,0x00,0x00,0x00,0x20,0xfd,0x23,0x00,0x00,0x01,0x00,0x00,0x00,0xa9,0xbc,0x0c,
0x01,0x88,0x88,0x00,0x00,0x0d,0xb4,0xf7,0xe7,0xd4,0x22,0x00,0x00,0x00,0x00,0x5b,0x70,0x03,
0x00,0x00,0x00,0x00,0x00,0x74,0xd1,0x00,0x00,0x00,0x00,0x00,0x02,0xdf,0x88,0x00,0x00,0x00,
0x00,0x00,0x00,0x37,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x39,0xc0,0x03,0x00,0x7d,
0x85,0x00,0x00,0x1c,0x74,0x00,0x00,0x45,0x46,0x00,0x00,0x00,0x1b,0x72,0x7f,0x3e,0x00,0x00,
0x00,0x2f,0xf0,0xb0,0x92,0xeb,0x83,0x00,0x00,0xb6,0x8c,0x00,0x00,0x27,0xf8,0x2a,0x0a,0xfe,
0x2c,0x00,0x00,0x00,0xc0,0x79,0x29,0xff,0x0a,0x00,0x00,0x00,0x9f,0x98,0x25,0xff,0x11,0x00,
0x00,0x00,0xa6,0x91,0x05,0xf3,0x44,0x00,0x00,0x00,0xd9,0x65,0x00,0x91,0xc7,0x0c,0x00,0x6a,
0xef,0x12,0x00,0x09,0xae,0xf1,0xe0,0xe8,0x42,0x00,0x00,0x00,0x00,0x1e,0x2f,0x08,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x39,0xc0,0x03,0x00,0x7d,0x85,0x00,
0x00,0x1c,0x74,0x00,0x00,0x45,0x46,0x00,0x06,0x64,0x0c,0x00,0x00,0x00,0x4e,0x28,0x10,0xff,
0x20,0x00,0x00,0x00,0xc8,0x68,0x10,0xff,0x20,0x00,0x00,0x00,0xc8,0x68,0x10,0xff,0x20,0x00,
0x00,0x00,0xc8,0x68,0x0f,0xff,0x20,0x00,0x00,0x00,0xc8,0x68,0x08,0xff,0x2d,0x00,0x00,0x00,
0xdf,0x68,0x00,0xe8,0x5a,0x00,0x00,0x0d,0xfc,0x68,0x00,0x9b,0xcf,0x0d,0x02,0x92,0xf4,0x68,
0x00,0x16,0xd3,0xf2,0xe9,0x95,0xc8,0x68,0x00,0x00,0x03,0x30,0x1f,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x13,
0x2e,0x00,0x00,0x16,0x2b,0x00,0x00,0x00,0x00,0x00,0x0e,0xd3,0x1d,0x07,0xb7,0x45,0x00,0x00,
0x00,0x00,0x00,0x00,0x5c,0xf4,0xf3,0x99,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2a,0x99,0xc3,
0x8f,0x31,0x00,0x00,0x00,0x00,0x00,0x60,0xfa,0xa7,0x74,0x9e,0xf8,0x5a,0x00,0x00,0x00,0x29,
0xf7,0x5f,0x00,0x00,0x00,0x53,0x53,0x00,0x00,0x00,0xa6,0xc1,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x06,0xf4,0x5d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x23,0xff,0x2e,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x32,0xff,0x1d,0x00,0x00,0x49,0x70,0x70,0x70,0x47,0x00,0x1c,
0xff,0x3b,0x00,0x00,0x6e,0xa8,0xa8,0xdc,0x9b,0x00,0x00,0xe3,0x7f,0x00,0x00,0x00,0x00,0x00,
0xaf,0x7a,0x00,0x00,0x82,0xe5,0x0b,0x00,0x00,0x00,0x12,0xf3,0x2d,0x00,0x00,0x0d,0xda,0xb7,
0x18,0x00,0x19,0xbc,0xa7,0x00,0x00,0x00,0x00,0x1c,0xb6,0xfb,0xe9,0xfd,0xa2,0x0d,0x00,0x00,
0x00,0x00,0x00,0x00,0x19,0x36,0x19,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x13,0x2e,0x00,0x00,0x16,0x2b,0x00,0x00,0x0e,0xd3,0x1d,0x07,0xb7,
0x45,0x00,0x00,0x00,0x5c,0xf4,0xf3,0x99,0x00,0x00,0x00,0x00,0x55,0xa1,0x87,0x07,0x64,0x12,
0x00,0x7f,0xf6,0x8f,0xa7,0xa5,0xff,0x30,0x0c,0xf3,0x57,0x00,0x00,0x91,0xff,0x30,0x49,0xee,
0x02,0x00,0x00,0x25,0xff,0x30,0x63,0xd0,0x00,0x00,0x00,0x04,0xfd,0x30,0x5d,0xdb,0x00,0x00,
0x00,0x0c,0xff,0x30,0x33,0xfa,0x15,0x00,0x00,0x44,0xff,0x30,0x01,0xd5,0x9f,0x02,0x0f,0xbe,
0xff,0x30,0x00,0x31,0xe4,0xed,0xf0,0x5d,0xff,0x30,0x07,0x20,0x08,0x33,0x15,0x0b,0xff,0x26,
0x0e,0xf4,0x32,0x00,0x00,0x60,0xe2,0x02,0x00,0x5e,0xee,0xa1,0xab,0xf1,0x44,0x00,0x00,0x00,
0x27,0x75,0x6a,0x1d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x39,0xc0,0x03,0x1c,0x74,0x00,
0x00,0x4c,0x2a,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,
0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,0x00,0xcc,0x70,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5a,0x1c,
0x00,0xe8,0x48,0x00,0xe8,0x48,0x00,0xe8,0x48,0x00,0xe8,0x48,0x00,0xe8,0x48,0x00,0xe8,0x48,
0x00,0xe8,0x48,0x00,0xe8,0x48,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x45,0x94,0xa4,0x76,0x11,0x00,0x00,0x00,0x7d,0xf3,0x8f,0x77,0xc0,0xe3,0x21,0x00,
0x04,0xf6,0x59,0x00,0x00,0x00,0x94,0x71,0x00,0x11,0xff,0x3b,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xc3,0xc6,0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0x1a,0xbe,0xfa,0xa9,0x3c,0x00,0x00,0x00,
0x00,0x00,0x00,0x37,0xa0,0xfa,0xa9,0x09,0x00,0x00,0x00,0x00,0x00,0x00,0x2b,0xdd,0x9d,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x45,0xf8,0x03,0x50,0xa5,0x01,0x00,0x00,0x00,0x37,0xfe,0x07,
0x1f,0xf0,0x9e,0x0e,0x00,0x2b,0xcf,0xb9,0x00,0x00,0x37,0xd5,0xfc,0xef,0xfe,0xb3,0x14,0x00,
0x00,0x00,0x02,0x6b,0xa7,0x13,0x00,0x00,0x00,0x00,0x00,0x00,0x5b,0xee,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xd5,0x73,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x5d,0x8a,0x6d,
0x0f,0x00,0x00,0xb4,0xd4,0x83,0xbc,0xdf,0x13,0x0d,0xff,0x29,0x00,0x02,0x54,0x13,0x01,0xe2,
0x89,0x06,0x00,0x00,0x00,0x00,0x32,0xd2,0xed,0x93,0x1e,0x00,0x00,0x00,0x01,0x3a,0xa9,0xef,
0x22,0x00,0x00,0x00,0x00,0x00,0xca,0x75,0x1a,0xd4,0x2e,0x00,0x16,0xe7,0x51,0x00,0x76,0xf5,
0xdb,0xf4,0x94,0x01,0x00,0x00,0x14,0xc5,0x3f,0x00,0x00,0x00,0x00,0x00,0xc8,0x83,0x00,0x00,
0x00,0x00,0x44,0xdf,0x25,0x00,0x00,0x00,0x00,0x08,0x0c,0x00,0x00,0x00,
  };
  static const struct {
    uint32_t ds;
    uint16_t width;
  } glinfo[]= {

{0,5}, {100,3}, {160,5}, {260,12}, 
{500,8}, {660,11}, {880,10}, {1080,3}, 
{1140,4}, {1220,4}, {1300,7}, {1440,7}, 
{1580,3}, {1640,7}, {1780,3}, {1840,8}, 
{2000,9}, {2180,9}, {2360,9}, {2540,9}, 
{2720,9}, {2900,9}, {3080,9}, {3260,9}, 
{3440,9}, {3620,9}, {3800,3}, {3860,3}, 
{3920,7}, {4060,7}, {4200,7}, {4340,8}, 
{4500,13}, {4760,10}, {4960,9}, {5140,10}, 
{5340,10}, {5540,9}, {5720,9}, {5900,11}, 
{6120,9}, {6300,3}, {6360,9}, {6540,10}, 
{6740,9}, {6920,12}, {7160,10}, {7360,10}, 
{7560,9}, {7740,11}, {7960,9}, {8140,9}, 
{8320,10}, {8520,10}, {8720,10}, {8920,12}, 
{9160,10}, {9360,9}, {9540,10}, {9740,5}, 
{9840,8}, {10000,5}, {10100,7}, {10240,10}, 
{10440,4}, {10520,8}, {10680,8}, {10840,7}, 
{10980,8}, {11140,8}, {11300,6}, {11420,8}, 
{11580,8}, {11740,3}, {11800,4}, {11880,8}, 
{12040,4}, {12120,12}, {12360,8}, {12520,8}, 
{12680,8}, {12840,9}, {13020,6}, {13140,7}, 
{13280,5}, {13380,8}, {13540,8}, {13700,11}, 
{13920,7}, {14060,8}, {14220,8}, {14380,6}, 
{14500,3}, {14560,6}, {14680,6}, {14800,10}, 
{15000,10}, {15200,10}, {15400,8}, {15560,8}, 
{15720,8}, {15880,11}, {16100,8}, {16260,3}, 
{16320,3}, {16380,9}, {16560,7}, 
  };
  // static const int nmap= sizeof(glmap)/sizeof(glmap[0]);
  int i,j,k,e,x,y,sX,eX;
  int W, X, *Xpos;
  char buf[20];
  int *map, *gl;
  int ngl;
  unsigned int C;
  int glyph_width;
  const unsigned char *glyph_data;
  unsigned char *data;

  map= alloca(L*sizeof(int));

  e= 0;
  ngl= 0;
  for(i=0;i<L;i++)
  {
    C= str[i];
    
    if (C&(1<<31)) { ngl+=2; e= 1; continue; }
    map[i]= font_hash(C);
    if (map[i]!=107) { ngl++; continue; }
    sprintf(buf,"%x",C); ngl+= strlen(buf); e= 1;
  }

  if (e) goto fix_encoding_errors;
  else gl= map;

encoding_errors_fixed:

  W= 0;
  for(i=0;i<ngl;i++) W+= glinfo[gl[i]].width;

  if (Rdata) data= malloc(W*20);
  X= 0;
  if (e && Rdata) Xpos= alloca(sizeof(int)*(ngl+1));
  for(i=0;i<ngl;i++)
  {
    if (e && Rdata) Xpos[i]= X;
    glyph_width= glinfo[gl[i]].width;
    if (Rdata) {
      glyph_data= font_data + glinfo[gl[i]].ds;
      for(y=0;y<20;y++)
         for(x=0;x<glyph_width;x++)
            data[y*W+X+x]= glyph_data[y*glyph_width+x];
    }
    X+= glyph_width;
  }
  if (e && Rdata) Xpos[i]= X;
  if (RW) *RW= W;
  if (RA) *RA= 16;
  if (RH) *RH= 20;
  if (Rdata) *Rdata= data;

  if (!e || !Rdata) return ;

  j= 0;
  for(i=0;i<L;i++)
  {
    if (str[i]&(1<<31))
    {
      sX= Xpos[j]; eX= Xpos[j+2];
      for(x=sX;x<eX;x++)
        for(y=0;y<20;y++)
          data[y*W + x] ^= 0xff;
      j+= 2;
      continue;
    }
    if (map[i]==107)
    {
      sprintf(buf,"%x",str[i]);
      sX= Xpos[j]; eX= Xpos[j+strlen(buf)];
      for(x=sX;x<eX;x++) {
        data[ 0*W + x]= 0xff;
        data[ (20-1)*W + x]= 0xff;
      }
      for(y=0;y<20;y++)
      {
        data[y*W+sX] = 0xff;
        data[y*W+eX-1] = 0xff;
      }
      j+= strlen(buf);
      continue;
    }
    j++;
  }
  return ;


fix_encoding_errors:
  j= 0;
  gl= alloca(sizeof(int)*ngl);
  for(i=0;i<L;i++)
  {
    if (str[i]&(1<<31))
    {
      sprintf(buf,"%02x", str[i]&0xff);
      for(k=0;buf[k];k++)
         gl[j++]= font_hash(buf[k]);
      continue;
    }
    if (map[i]==107)
    {
      sprintf(buf,"%x", str[i]);
      for(k=0;buf[k];k++)
         gl[j++]= font_hash(buf[k]);
      continue;
    }
    gl[j++]= map[i];
  }
  goto encoding_errors_fixed;
}

static 
void font2_render
        (uint32_t *str, size_t L, 
        int *RW, int *RH, int *RA,
        unsigned char **Rdata)
{
  static const uint8_t font_data[]= {

0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xc4,0xd1,0x07,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,
0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xdb,0xe3,0x0b,0x00,
0x6c,0xcb,0x4c,0x00,0xf3,0xff,0xca,0x00,0x91,0xf3,0x6d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd3,0xe4,0x15,0xdd,0xdc,0x00,0x00,0xff,0xff,
0x28,0xff,0xff,0x00,0x00,0xff,0xff,0x28,0xff,0xff,0x00,0x00,0xe8,0xe8,0x1a,0xe6,0xe3,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x14,0xff,0xff,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xff,
0xff,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0x00,0x00,0xec,0xec,0xed,0xff,0xff,0xec,0xff,0xff,0xed,0xec,0xec,0x00,0x00,0x00,
0x00,0x14,0xff,0xff,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xff,0xff,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0x00,0xec,0xec,0xed,0xff,0xff,0xec,0xff,0xff,0xed,0xec,0xec,
0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x14,
0xff,0xff,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0xff,0xff,
0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x2c,0xfc,0x00,0x00,0x00,0x00,0x00,0x39,0xa8,0xd8,0xff,0xc7,0x48,0x00,0x00,0x69,0xff,
0xff,0xff,0xff,0xff,0xfc,0x3f,0x00,0xeb,0xff,0x84,0x43,0xfc,0x83,0x8d,0x2b,0x00,0xe7,0xff,
0x3b,0x2c,0xfc,0x00,0x00,0x00,0x00,0x8b,0xff,0xf8,0xc4,0xfc,0x02,0x00,0x00,0x00,0x06,0x8a,
0xfa,0xff,0xff,0xde,0x51,0x00,0x00,0x00,0x00,0x0e,0x6c,0xff,0xff,0xfe,0x57,0x00,0x00,0x00,
0x00,0x2c,0xfc,0x71,0xff,0xd1,0x06,0x60,0xa9,0x10,0x2c,0xfc,0x1f,0xff,0xf5,0x03,0xd6,0xff,
0xc9,0x62,0xfc,0x82,0xff,0xd5,0x00,0x2b,0xec,0xff,0xff,0xff,0xff,0xff,0x60,0x00,0x00,0x17,
0x95,0xdd,0xff,0xcb,0x5d,0x00,0x00,0x00,0x00,0x00,0x2c,0xfc,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1e,0xba,0xf3,0xc0,0x23,
0x00,0x06,0x81,0x17,0x00,0x00,0x00,0xbc,0xff,0xfd,0xff,0xcc,0x00,0x6f,0xc5,0x00,0x00,0x00,
0x00,0xf3,0xff,0x43,0xff,0xff,0x15,0xea,0x3e,0x00,0x00,0x00,0x00,0xbf,0xff,0xfc,0xff,0xd1,
0x82,0xb6,0x00,0x00,0x00,0x00,0x00,0x20,0xbc,0xf4,0xc3,0x3e,0xf2,0x30,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x94,0xa7,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x23,0xf5,
0x3c,0xb4,0xf3,0xc5,0x27,0x00,0x00,0x00,0x00,0x00,0xa7,0x98,0xa9,0xff,0xda,0xff,0xcf,0x00,
0x00,0x00,0x00,0x31,0xf4,0x1a,0xe0,0xff,0x3a,0xff,0xff,0x08,0x00,0x00,0x00,0xb9,0x89,0x00,
0xab,0xff,0xfd,0xff,0xd2,0x00,0x00,0x00,0x0f,0x7b,0x11,0x00,0x18,0xb6,0xf3,0xc7,0x29,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x9a,0xe9,0xee,0x8f,0x04,0x00,0x00,0x00,
0x00,0x00,0xad,0xff,0xfb,0xfe,0xff,0x82,0x00,0x00,0x00,0x00,0x06,0xfa,0xff,0x26,0x33,0x69,
0x12,0x00,0x00,0x00,0x00,0x03,0xf6,0xff,0x3d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,
0xff,0xff,0xff,0x18,0x00,0x00,0x00,0x00,0x00,0x0e,0xbf,0xff,0xf9,0xec,0x16,0x00,0x00,0x00,
0x00,0x00,0x95,0xff,0xb0,0x09,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe5,0xff,0x2f,0x00,0xff,
0xff,0xff,0xff,0xff,0x00,0x00,0xf8,0xff,0x23,0x00,0xec,0xf6,0xff,0xf8,0xec,0x00,0x00,0xcb,
0xff,0x9f,0x06,0x0c,0xc1,0xff,0x8a,0x00,0x00,0x00,0x43,0xfb,0xff,0xf9,0xfc,0xff,0xee,0x20,
0x00,0x00,0x00,0x00,0x3c,0xbd,0xf4,0xea,0xae,0x29,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xd3,0xe4,0x0b,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,
0xe8,0xe8,0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x41,0xfe,0xe6,0x0e,0x00,0x02,0xd9,
0xff,0x5c,0x00,0x00,0x4c,0xff,0xe4,0x04,0x00,0x00,0x9c,0xff,0x8c,0x00,0x00,0x00,0xd4,0xff,
0x4f,0x00,0x00,0x00,0xef,0xff,0x29,0x00,0x00,0x00,0xfc,0xff,0x19,0x00,0x00,0x00,0xf1,0xff,
0x28,0x00,0x00,0x00,0xd8,0xff,0x4c,0x00,0x00,0x00,0xa5,0xff,0x8c,0x00,0x00,0x00,0x55,0xff,
0xe4,0x04,0x00,0x00,0x04,0xde,0xff,0x5c,0x00,0x00,0x00,0x42,0xfe,0xe6,0x0e,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0xdc,0xff,0x4b,0x00,0x00,0x00,0x51,0xff,
0xe3,0x06,0x00,0x00,0x02,0xdf,0xff,0x5b,0x00,0x00,0x00,0x83,0xff,0xad,0x00,0x00,0x00,0x41,
0xff,0xe6,0x00,0x00,0x00,0x18,0xff,0xfe,0x02,0x00,0x00,0x06,0xff,0xff,0x0f,0x00,0x00,0x17,
0xff,0xfe,0x03,0x00,0x00,0x3e,0xff,0xe7,0x00,0x00,0x00,0x80,0xff,0xb1,0x00,0x00,0x01,0xdb,
0xff,0x5f,0x00,0x00,0x4d,0xff,0xe6,0x08,0x00,0x07,0xdb,0xff,0x4d,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x6c,0x10,
0x0c,0x6c,0x18,0x00,0x00,0x00,0x00,0x70,0xff,0x8b,0x85,0xff,0x7d,0x00,0x00,0x00,0x00,0x06,
0xcc,0xf8,0xf5,0xd3,0x08,0x00,0x00,0x00,0xc9,0xec,0xf4,0xff,0xff,0xf5,0xec,0xc3,0x00,0x00,
0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xdd,0x00,0x00,0x00,0x02,0xc4,0xf9,0xfb,0xbf,0x02,0x00,
0x00,0x00,0x00,0x69,0xff,0x8c,0x93,0xff,0x65,0x00,0x00,0x00,0x00,0x23,0x7c,0x0e,0x10,0x7d,
0x23,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xec,0xec,0xff,0xff,0xed,0xec,0xec,0x00,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x90,0xf3,0x71,0x00,0xf3,0xff,0xc9,0x00,0x5f,0xff,0x85,
0x00,0x2f,0xd3,0x0e,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0x00,
0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x6c,0xcb,0x4c,0x00,0xf3,0xff,0xca,0x00,0x91,0xf3,0x6d,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,
0x7d,0x80,0x1a,0x00,0x00,0x00,0x00,0x00,0x58,0xff,0xe0,0x02,0x00,0x00,0x00,0x00,0x00,0xc7,
0xff,0x77,0x00,0x00,0x00,0x00,0x00,0x36,0xff,0xf7,0x13,0x00,0x00,0x00,0x00,0x00,0xa6,0xff,
0x9f,0x00,0x00,0x00,0x00,0x00,0x1a,0xfa,0xff,0x33,0x00,0x00,0x00,0x00,0x00,0x84,0xff,0xc6,
0x00,0x00,0x00,0x00,0x00,0x08,0xeb,0xff,0x5a,0x00,0x00,0x00,0x00,0x00,0x63,0xff,0xe8,0x06,
0x00,0x00,0x00,0x00,0x00,0xd2,0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x42,0xff,0xfb,0x1a,0x00,
0x00,0x00,0x00,0x00,0xb1,0xff,0xaa,0x00,0x00,0x00,0x00,0x00,0x22,0xfd,0xff,0x3e,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x16,0xa3,0xef,0xf3,0xb3,0x24,0x00,0x00,0x00,0x07,0xd2,0xff,0xfd,0xfb,0xff,0xe4,0x13,
0x00,0x00,0x6a,0xff,0xc2,0x11,0x09,0xa6,0xff,0x83,0x00,0x00,0xbe,0xff,0x48,0x00,0x00,0x30,
0xff,0xd3,0x00,0x00,0xea,0xff,0x21,0x00,0x00,0x0e,0xff,0xfb,0x03,0x00,0xfb,0xff,0x15,0x00,
0x00,0x02,0xff,0xff,0x0f,0x00,0xf4,0xff,0x19,0x00,0x00,0x04,0xff,0xff,0x07,0x00,0xdd,0xff,
0x28,0x00,0x00,0x17,0xff,0xef,0x00,0x00,0xa7,0xff,0x57,0x00,0x00,0x49,0xff,0xb7,0x00,0x00,
0x4e,0xff,0xc9,0x12,0x09,0xb9,0xff,0x5d,0x00,0x00,0x01,0xbb,0xff,0xfd,0xf9,0xff,0xc5,0x04,
0x00,0x00,0x00,0x0e,0x96,0xe9,0xeb,0x9d,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x89,0xff,0xff,0x14,
0x00,0x00,0x00,0x00,0x04,0xa2,0xff,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x03,0x10,0x10,0xff,
0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1e,0x9f,0xe7,0xf4,0xc4,0x50,0x00,0x00,0x00,0x24,0xe7,
0xff,0xfd,0xf7,0xff,0xff,0x61,0x00,0x00,0x85,0xff,0xd0,0x17,0x02,0x84,0xff,0xd5,0x00,0x00,
0x00,0x1c,0x22,0x00,0x00,0x20,0xff,0xfa,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5a,0xff,0xe7,
0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xee,0xff,0x8e,0x00,0x00,0x00,0x00,0x06,0x87,0xfd,0xff,
0xbf,0x09,0x00,0x00,0x00,0x1a,0xcb,0xff,0xff,0x93,0x06,0x00,0x00,0x00,0x0e,0xd5,0xff,0xf1,
0x4d,0x00,0x00,0x00,0x00,0x00,0x87,0xff,0xf6,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xff,
0xfb,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0xf7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x68,0xd9,0xf7,0xdb,0x7e,0x07,0x00,0x00,0x00,0x64,0xff,0xff,0xf6,0xff,0xff,0x9c,0x00,
0x00,0x00,0x4c,0x9d,0x76,0x00,0x5e,0xff,0xf8,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x0b,0xff,
0xfb,0x07,0x00,0x00,0x00,0x00,0x00,0x04,0x7a,0xff,0xbe,0x00,0x00,0x00,0x00,0x00,0x80,0xff,
0xff,0xff,0x71,0x00,0x00,0x00,0x00,0x00,0x76,0xec,0xfc,0xff,0xfe,0x54,0x00,0x00,0x00,0x00,
0x00,0x00,0x0d,0xa1,0xff,0xd5,0x00,0x00,0x00,0x1d,0x00,0x00,0x00,0x23,0xff,0xf7,0x00,0x1d,
0xe6,0xff,0x6f,0x04,0x07,0x92,0xff,0xc7,0x00,0x00,0x93,0xff,0xff,0xf8,0xfb,0xff,0xf7,0x30,
0x00,0x00,0x00,0x68,0xd1,0xf8,0xf0,0xae,0x27,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x93,0xff,
0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x3b,0xfe,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x07,0xd8,
0xfd,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x86,0xff,0x91,0xff,0xff,0x00,0x00,0x00,0x00,0x31,
0xfb,0xd0,0x19,0xff,0xff,0x00,0x00,0x00,0x04,0xce,0xfb,0x32,0x14,0xff,0xff,0x00,0x00,0x00,
0x79,0xff,0x87,0x00,0x14,0xff,0xff,0x00,0x00,0x00,0xf8,0xff,0xf6,0xf4,0xf4,0xff,0xff,0xf4,
0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xff,
0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x14,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xa6,0xff,0xff,0xff,0xff,0xff,0x0c,0x00,0x00,0x00,0xd3,
0xff,0xf0,0xec,0xec,0xec,0x0b,0x00,0x00,0x05,0xfa,0xff,0x1a,0x00,0x00,0x00,0x00,0x00,0x00,
0x2c,0xff,0xe6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x59,0xff,0xf7,0xf6,0xef,0xb4,0x33,0x00,
0x00,0x00,0x60,0xf7,0xff,0xfa,0xfc,0xff,0xf9,0x3a,0x00,0x00,0x00,0x0e,0x4b,0x06,0x0b,0xab,
0xff,0xc7,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1b,0xff,0xfd,0x06,0x08,0x4a,0x76,0x0d,0x00,
0x00,0x16,0xff,0xfd,0x06,0x07,0xea,0xff,0xa2,0x09,0x09,0xa2,0xff,0xc5,0x00,0x00,0x50,0xfb,
0xff,0xfa,0xfb,0xff,0xf9,0x39,0x00,0x00,0x00,0x3d,0xb9,0xee,0xf1,0xb6,0x33,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xa3,0xff,0xdd,0x08,0x00,0x00,0x00,0x00,0x00,0x2c,0xfd,0xff,0x4a,0x00,0x00,
0x00,0x00,0x00,0x00,0xb0,0xff,0xae,0x00,0x00,0x00,0x00,0x00,0x00,0x36,0xff,0xf4,0x1e,0x00,
0x00,0x00,0x00,0x00,0x00,0xbe,0xff,0xfd,0xf3,0xc2,0x3d,0x00,0x00,0x00,0x47,0xff,0xff,0xff,
0xff,0xff,0xfa,0x38,0x00,0x00,0xb0,0xff,0xbe,0x1b,0x1f,0xc1,0xff,0xb9,0x00,0x00,0xe6,0xff,
0x2f,0x00,0x00,0x32,0xff,0xf2,0x00,0x00,0xf6,0xff,0x29,0x00,0x00,0x2a,0xff,0xf3,0x00,0x00,
0xc1,0xff,0xaf,0x0c,0x0d,0xb3,0xff,0xa9,0x00,0x00,0x37,0xf8,0xff,0xfb,0xfc,0xff,0xed,0x1b,
0x00,0x00,0x00,0x31,0xb5,0xf1,0xf0,0xab,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0x86,0x00,0xec,0xec,0xec,0xec,0xec,0xff,0xff,0xf1,0x12,0x00,0x00,0x00,0x00,0x00,
0x4a,0xff,0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0xc9,0xff,0xef,0x10,0x00,0x00,0x00,0x00,
0x00,0x49,0xff,0xff,0x7d,0x00,0x00,0x00,0x00,0x00,0x00,0xc8,0xff,0xed,0x0d,0x00,0x00,0x00,
0x00,0x00,0x47,0xff,0xff,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0xc6,0xff,0xea,0x0b,0x00,0x00,
0x00,0x00,0x00,0x46,0xff,0xff,0x74,0x00,0x00,0x00,0x00,0x00,0x00,0xc5,0xff,0xe7,0x09,0x00,
0x00,0x00,0x00,0x00,0x45,0xff,0xff,0x6f,0x00,0x00,0x00,0x00,0x00,0x00,0xc4,0xff,0xe4,0x08,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x85,0xdd,0xf6,0xda,0x7e,0x07,0x00,0x00,0x00,0xa5,
0xff,0xff,0xf4,0xff,0xff,0x9e,0x00,0x00,0x06,0xf9,0xff,0x3d,0x00,0x3d,0xff,0xf8,0x05,0x00,
0x03,0xf6,0xff,0x38,0x00,0x3a,0xff,0xf8,0x05,0x00,0x00,0x80,0xff,0xff,0xf2,0xff,0xff,0x85,
0x00,0x00,0x09,0xb6,0xff,0xff,0xff,0xff,0xff,0xb6,0x09,0x00,0x8b,0xff,0xca,0x31,0x0a,0x3c,
0xd8,0xff,0x8d,0x00,0xe7,0xff,0x32,0x00,0x00,0x00,0x39,0xff,0xe7,0x00,0xf6,0xff,0x2c,0x00,
0x00,0x00,0x2d,0xff,0xf5,0x00,0xbe,0xff,0xbd,0x1d,0x00,0x1f,0xc0,0xff,0xbd,0x00,0x35,0xf6,
0xff,0xff,0xf4,0xff,0xff,0xf3,0x33,0x00,0x00,0x2d,0xab,0xea,0xfb,0xe6,0x9b,0x21,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x35,0xb4,0xf2,0xf0,0xb4,0x2f,0x00,0x00,0x00,0x35,0xf9,0xff,0xfb,0xfa,0xff,0xf7,0x33,
0x00,0x00,0xcc,0xff,0x95,0x07,0x07,0x94,0xff,0xbd,0x00,0x00,0xf9,0xff,0x22,0x00,0x00,0x22,
0xff,0xf5,0x00,0x00,0xd6,0xff,0x96,0x08,0x06,0x93,0xff,0xe5,0x00,0x00,0x5b,0xff,0xff,0xfb,
0xfa,0xff,0xff,0xaf,0x00,0x00,0x00,0x4a,0xc7,0xf2,0xfe,0xff,0xff,0x44,0x00,0x00,0x00,0x00,
0x00,0x10,0xf0,0xff,0xbd,0x00,0x00,0x00,0x00,0x00,0x00,0x7e,0xff,0xff,0x35,0x00,0x00,0x00,
0x00,0x00,0x0d,0xed,0xff,0xaf,0x00,0x00,0x00,0x00,0x00,0x00,0x78,0xff,0xfd,0x2b,0x00,0x00,
0x00,0x00,0x00,0x0a,0xea,0xff,0xa3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x91,0xf3,0x6b,0x00,0xf3,0xff,
0xcb,0x00,0x6d,0xcb,0x4f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x6c,0xcb,0x4c,0x00,0xf3,0xff,0xca,0x00,0x91,0xf3,0x6d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x90,0xf3,0x6a,0x00,0xf3,0xff,0xcb,0x00,0x6d,0xcb,0x4f,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x44,0x9b,0x34,0x00,0xef,0xff,
0xc7,0x00,0xac,0xff,0xa6,0x00,0x1c,0xf7,0x2d,0x00,0x12,0x38,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x82,0xff,0xfd,0x40,0x00,0x00,0x38,0xfc,0xff,0x8c,0x00,0x00,0x0c,0xdc,0xff,0xd2,0x07,
0x00,0x00,0x9c,0xff,0xf9,0x2e,0x00,0x00,0x4e,0xff,0xff,0x76,0x00,0x00,0x00,0x41,0xfe,0xff,
0x7a,0x00,0x00,0x00,0x00,0x90,0xff,0xfa,0x30,0x00,0x00,0x00,0x08,0xd7,0xff,0xd3,0x07,0x00,
0x00,0x00,0x34,0xfb,0xff,0x8b,0x00,0x00,0x00,0x00,0x80,0xff,0xfd,0x3d,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,
0xec,0xec,0xec,0xec,0xec,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x36,0xfb,0xff,0x9d,0x00,0x00,0x00,0x00,
0x7e,0xff,0xff,0x4f,0x00,0x00,0x00,0x03,0xc6,0xff,0xea,0x16,0x00,0x00,0x00,0x22,0xf2,0xff,
0xb2,0x00,0x00,0x00,0x00,0x60,0xff,0xff,0x64,0x00,0x00,0x00,0x6b,0xff,0xff,0x52,0x00,0x00,
0x26,0xf5,0xff,0xa4,0x00,0x00,0x04,0xca,0xff,0xe5,0x11,0x00,0x00,0x81,0xff,0xff,0x48,0x00,
0x00,0x36,0xfb,0xff,0x9a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x9c,0xe4,0xf6,0xcd,0x67,0x01,0x00,
0x07,0xcf,0xff,0xff,0xf6,0xff,0xff,0x8b,0x00,0x4e,0xff,0xf7,0x33,0x00,0x54,0xff,0xf4,0x02,
0x03,0x23,0x36,0x00,0x00,0x1c,0xff,0xfa,0x05,0x00,0x00,0x00,0x00,0x15,0xc2,0xff,0xb1,0x00,
0x00,0x00,0x00,0x3b,0xe8,0xff,0xc5,0x13,0x00,0x00,0x00,0x00,0xe2,0xff,0x97,0x04,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0x19,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x07,0x00,0x00,0x00,
0x00,0x00,0x00,0x6c,0xcb,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xf3,0xff,0xca,0x00,0x00,0x00,
0x00,0x00,0x00,0x8e,0xf3,0x6d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x48,0xaa,
0xe5,0xf9,0xea,0xb7,0x5f,0x05,0x00,0x00,0x00,0x00,0x00,0x0f,0xb2,0xff,0xff,0xff,0xf2,0xfe,
0xff,0xff,0xd0,0x21,0x00,0x00,0x00,0x02,0xc0,0xff,0xf3,0x77,0x13,0x00,0x09,0x55,0xdd,0xff,
0xdb,0x0c,0x00,0x00,0x63,0xff,0xf4,0x2c,0x66,0xd2,0xf5,0xdb,0x76,0x13,0xd6,0xff,0x82,0x00,
0x00,0xc5,0xff,0x71,0x88,0xff,0xff,0xf4,0xff,0xff,0x8b,0x4b,0xff,0xe1,0x00,0x00,0xf2,0xff,
0x23,0xf1,0xff,0x53,0x00,0x4a,0xff,0xee,0x0e,0xff,0xff,0x09,0x00,0xf2,0xff,0x24,0xe7,0xff,
0x68,0x08,0x67,0xff,0xff,0x51,0xff,0xf6,0x02,0x00,0xc5,0xff,0x72,0x6c,0xff,0xff,0xff,0xff,
0xe8,0xf4,0xff,0xff,0x9b,0x00,0x00,0x65,0xff,0xf0,0x2f,0x4e,0xbf,0xe3,0xbf,0x31,0x58,0xdd,
0xa0,0x0b,0x00,0x00,0x03,0xc2,0xff,0xf8,0x87,0x1c,0x00,0x03,0x24,0x60,0x02,0x00,0x00,0x00,
0x00,0x00,0x10,0xb5,0xff,0xff,0xff,0xf4,0xfb,0xff,0xef,0x27,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x49,0xaa,0xe5,0xfa,0xe8,0xa0,0x1f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xae,0xff,0xff,0x46,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x11,0xf9,0xff,0xff,0xa2,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x67,0xff,0xfb,
0xff,0xf4,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,0xc3,0xff,0x94,0xec,0xff,0x5c,0x00,0x00,0x00,
0x00,0x00,0x21,0xfe,0xff,0x38,0x9a,0xff,0xb8,0x00,0x00,0x00,0x00,0x00,0x7c,0xff,0xdf,0x00,
0x43,0xff,0xfc,0x18,0x00,0x00,0x00,0x00,0xd8,0xff,0x88,0x00,0x03,0xe8,0xff,0x72,0x00,0x00,
0x00,0x35,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xce,0x00,0x00,0x00,0x91,0xff,0xfe,0xf4,0xf4,
0xf4,0xf7,0xff,0xff,0x2b,0x00,0x04,0xe9,0xff,0xac,0x00,0x00,0x00,0x18,0xfc,0xff,0x88,0x00,
0x4a,0xff,0xff,0x4d,0x00,0x00,0x00,0x00,0xb6,0xff,0xe2,0x01,0xa6,0xff,0xea,0x04,0x00,0x00,
0x00,0x00,0x57,0xff,0xff,0x41,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0xfe,0xec,0xbe,0x56,0x00,0x00,0x00,0x00,0xff,0xff,0xed,0xee,0xfe,
0xff,0xff,0x6a,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x0b,0x93,0xff,0xe1,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0x23,0xff,0xf8,0x00,0x00,0x00,0xff,0xff,0x14,0x01,0x1c,0xa0,0xff,0xc0,
0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xee,0x2b,0x00,0x00,0x00,0xff,0xff,0xed,0xec,
0xf2,0xff,0xff,0xe3,0x27,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x13,0x97,0xff,0xbb,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0x24,0xff,0xf4,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x12,0x97,
0xff,0xd3,0x00,0x00,0xff,0xff,0xed,0xec,0xf2,0xff,0xff,0xfd,0x4d,0x00,0x00,0xff,0xff,0xff,
0xff,0xfe,0xed,0xb7,0x45,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x4f,0xc2,0xf4,0xf4,0xb8,0x3a,0x00,0x00,0x00,0x00,0x00,0x6f,
0xff,0xff,0xf6,0xfd,0xff,0xfd,0x5a,0x00,0x00,0x00,0x28,0xf8,0xfa,0x61,0x02,0x14,0xbd,0xff,
0xf6,0x1e,0x00,0x00,0x8e,0xff,0x96,0x00,0x00,0x00,0x10,0xf5,0xff,0x81,0x00,0x00,0xd7,0xff,
0x3f,0x00,0x00,0x00,0x00,0x13,0x18,0x0f,0x00,0x00,0xf4,0xff,0x1c,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xf4,0xff,0x1b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd9,0xff,
0x3e,0x00,0x00,0x00,0x00,0x0c,0x10,0x0a,0x00,0x00,0x90,0xff,0x94,0x00,0x00,0x00,0x0f,0xf3,
0xff,0x84,0x00,0x00,0x29,0xf9,0xfa,0x60,0x06,0x14,0xb7,0xff,0xf7,0x22,0x00,0x00,0x00,0x72,
0xff,0xff,0xf9,0xfe,0xff,0xfe,0x60,0x00,0x00,0x00,0x00,0x00,0x52,0xc5,0xf5,0xeb,0xb8,0x3e,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0xff,0xea,0xb1,0x36,0x00,0x00,0x00,0x00,0xff,0xff,0xed,0xf1,0xff,0xff,0xfb,0x5d,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x1d,0xa5,0xff,0xf7,0x26,0x00,0x00,0xff,0xff,0x14,0x00,
0x00,0x01,0xc5,0xff,0x94,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x4d,0xff,0xdb,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0x22,0xff,0xf7,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x2d,
0xff,0xee,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x5a,0xff,0xcd,0x00,0x00,0xff,0xff,0x14,
0x00,0x00,0x02,0xd0,0xff,0x82,0x00,0x00,0xff,0xff,0x14,0x00,0x1e,0xaa,0xff,0xf0,0x18,0x00,
0x00,0xff,0xff,0xed,0xf0,0xff,0xff,0xf8,0x4f,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xe9,0xaf,
0x31,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,
0x00,0xff,0xff,0xed,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0xff,0xff,0xed,
0xec,0xec,0xec,0xec,0x76,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xed,0xec,0xec,0xec,0xec,0xec,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xed,0xec,0xec,
0xec,0xec,0xec,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0xff,0xff,0xed,0xec,0xec,0xec,0xec,0x76,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4a,0xbb,0xf0,0xf5,0xc9,0x5a,0x00,0x00,0x00,
0x00,0x00,0x70,0xff,0xff,0xfb,0xfa,0xff,0xff,0x85,0x00,0x00,0x00,0x2f,0xfa,0xff,0x86,0x08,
0x0b,0x87,0xc2,0x7b,0x02,0x00,0x00,0x98,0xff,0xaa,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xdc,0xff,0x45,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf7,0xff,0x1e,0x00,0x00,
0xec,0xec,0xec,0xec,0xec,0x00,0x00,0xee,0xff,0x27,0x00,0x00,0xff,0xff,0xff,0xff,0xfa,0x00,
0x00,0xce,0xff,0x55,0x00,0x00,0x00,0x00,0x6f,0xff,0xe4,0x00,0x00,0x81,0xff,0xbc,0x00,0x00,
0x00,0x00,0xae,0xff,0xaa,0x00,0x00,0x1e,0xf2,0xff,0x91,0x08,0x07,0x67,0xfe,0xff,0x48,0x00,
0x00,0x00,0x5b,0xfd,0xff,0xfa,0xfb,0xff,0xff,0x9b,0x00,0x00,0x00,0x00,0x00,0x3d,0xb5,0xef,
0xf8,0xd2,0x6e,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0x14,0x00,0xff,0xff,0xed,0xec,0xec,0xec,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,
0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,
0x00,0xf5,0xff,0x1f,0x00,0x00,0x0b,0xff,0xff,0x0a,0x00,0xd9,0xff,0x46,0x00,0x00,0x37,0xff,
0xed,0x00,0x00,0x8e,0xff,0xc2,0x10,0x0a,0xbb,0xff,0xa5,0x00,0x00,0x19,0xe9,0xff,0xfd,0xf9,
0xff,0xf1,0x26,0x00,0x00,0x00,0x25,0xb0,0xf3,0xf0,0xb6,0x2e,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x0b,0xcc,0xff,0xdc,0x15,0x00,0xff,0xff,0x14,0x00,0x03,
0xb5,0xff,0xed,0x26,0x00,0x00,0xff,0xff,0x14,0x00,0x9a,0xff,0xf9,0x3e,0x00,0x00,0x00,0xff,
0xff,0x14,0x7b,0xff,0xff,0x5b,0x00,0x00,0x00,0x00,0xff,0xff,0x71,0xff,0xff,0x7d,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xfd,0xff,0xee,0x0a,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xfe,0xfb,
0xff,0x9b,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x73,0x73,0xff,0xff,0x54,0x00,0x00,0x00,0x00,
0xff,0xff,0x14,0x01,0xb7,0xff,0xef,0x1f,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x16,0xe7,0xff,
0xc5,0x04,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x45,0xfe,0xff,0x84,0x00,0x00,0xff,0xff,0x14,
0x00,0x00,0x00,0x89,0xff,0xfd,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xf8,0xf8,0xf8,0xf8,0xf8,0xf8,0x00,0x00,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xfb,0x16,0x00,0x00,0x00,0x00,0x14,0xfa,
0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x6f,0x00,0x00,0x00,0x00,0x6e,0xff,0xff,0xff,0x00,0x00,
0xff,0xff,0xff,0xcc,0x00,0x00,0x00,0x00,0xcd,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,
0x29,0x00,0x00,0x2b,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xdb,0xff,0x86,0x00,0x00,0x8a,
0xff,0xda,0xff,0xff,0x00,0x00,0xff,0xff,0x84,0xff,0xe1,0x01,0x03,0xe5,0xff,0x81,0xff,0xff,
0x00,0x00,0xff,0xff,0x2f,0xfd,0xff,0x41,0x47,0xff,0xfc,0x2b,0xff,0xff,0x00,0x00,0xff,0xff,
0x14,0xc1,0xff,0x9e,0xa6,0xff,0xbb,0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x6a,0xff,0xf2,
0xf6,0xff,0x62,0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x16,0xfc,0xff,0xff,0xf9,0x10,0x14,
0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x00,0xbb,0xff,0xff,0xb0,0x00,0x14,0xff,0xff,0x00,0x00,
0xff,0xff,0x14,0x00,0x64,0xff,0xff,0x58,0x00,0x14,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0x56,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0xe8,0x11,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0xff,0xff,0xff,0x9f,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0xff,0xff,
0x43,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0xaf,0xff,0xdd,0x0a,0x00,0xff,0xff,0x14,0x00,
0xff,0xff,0x26,0xea,0xff,0x8d,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x5e,0xff,0xfc,0x34,
0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xbf,0xff,0xd1,0xff,0xff,0x14,0x00,0xff,0xff,0x14,
0x00,0x28,0xf9,0xff,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0x84,0xff,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0x00,0x08,0xdd,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0x47,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x49,0xbd,0xf2,0xf4,0xc5,0x57,0x00,0x00,0x00,0x00,0x00,0x68,0xff,0xff,0xf9,
0xf8,0xff,0xff,0x7f,0x00,0x00,0x00,0x24,0xf6,0xfe,0x74,0x06,0x02,0x5a,0xf8,0xfc,0x33,0x00,
0x00,0x8a,0xff,0xa1,0x00,0x00,0x00,0x00,0x87,0xff,0x9d,0x00,0x00,0xd4,0xff,0x45,0x00,0x00,
0x00,0x00,0x2f,0xff,0xe7,0x00,0x00,0xf3,0xff,0x1e,0x00,0x00,0x00,0x00,0x0d,0xff,0xff,0x06,
0x00,0xf3,0xff,0x1d,0x00,0x00,0x00,0x00,0x0a,0xff,0xff,0x07,0x00,0xd5,0xff,0x43,0x00,0x00,
0x00,0x00,0x2e,0xff,0xe8,0x00,0x00,0x8c,0xff,0xa0,0x00,0x00,0x00,0x00,0x88,0xff,0x9f,0x00,
0x00,0x26,0xf7,0xfe,0x72,0x0a,0x07,0x61,0xf9,0xfc,0x35,0x00,0x00,0x00,0x6b,0xff,0xff,0xfc,
0xfb,0xff,0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x4c,0xc1,0xf3,0xf5,0xc8,0x5b,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xfc,0xe6,0xb3,0x47,0x00,0x00,
0x00,0xff,0xff,0xed,0xec,0xfb,0xff,0xff,0x69,0x00,0x00,0xff,0xff,0x14,0x00,0x04,0x73,0xff,
0xe5,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x0e,0xff,0xff,0x09,0x00,0xff,0xff,0x14,0x00,0x04,
0x73,0xff,0xe6,0x00,0x00,0xff,0xff,0xed,0xec,0xfa,0xff,0xff,0x6b,0x00,0x00,0xff,0xff,0xff,
0xfc,0xe5,0xb4,0x48,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x46,0xba,0xf1,
0xf6,0xca,0x61,0x00,0x00,0x00,0x00,0x00,0x69,0xff,0xff,0xfa,0xf9,0xff,0xff,0x8e,0x00,0x00,
0x00,0x25,0xf6,0xfb,0x65,0x05,0x03,0x61,0xfa,0xff,0x43,0x00,0x00,0x8f,0xff,0x94,0x00,0x00,
0x00,0x00,0x7f,0xff,0xad,0x00,0x00,0xda,0xff,0x3b,0x00,0x00,0x00,0x00,0x27,0xff,0xf0,0x00,
0x00,0xf7,0xff,0x1c,0x00,0x00,0x00,0x00,0x07,0xff,0xff,0x0a,0x00,0xed,0xff,0x1d,0x00,0x00,
0x00,0x00,0x11,0xff,0xff,0x09,0x00,0xcc,0xff,0x47,0x00,0x00,0x22,0x59,0x35,0xff,0xe8,0x00,
0x00,0x7a,0xff,0xa7,0x00,0x0c,0xd8,0xff,0xe1,0xff,0xa7,0x00,0x00,0x17,0xec,0xff,0x77,0x08,
0x5d,0xfa,0xff,0xff,0x53,0x00,0x00,0x00,0x50,0xfc,0xff,0xfb,0xf7,0xff,0xff,0xff,0xe4,0x07,
0x00,0x00,0x00,0x39,0xb5,0xf0,0xf9,0xd2,0x6e,0x7c,0x61,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0xfc,0xe3,0xaf,0x45,0x00,0x00,0x00,0xff,0xff,0xed,0xec,0xfb,
0xff,0xff,0x69,0x00,0x00,0xff,0xff,0x14,0x00,0x04,0x74,0xff,0xe6,0x00,0x00,0xff,0xff,0x14,
0x00,0x00,0x0e,0xff,0xff,0x0a,0x00,0xff,0xff,0x14,0x00,0x03,0x71,0xff,0xea,0x00,0x00,0xff,
0xff,0xed,0xec,0xfa,0xff,0xff,0x6c,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xec,0x4b,0x00,0x00,
0x00,0xff,0xff,0x14,0x53,0xff,0xfe,0x2f,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xc4,0xff,0xb4,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x38,0xff,0xff,0x39,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0xa9,0xff,0xbe,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x21,0xf9,0xff,0x45,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x56,0xc2,
0xef,0xed,0xb4,0x34,0x00,0x00,0x00,0x75,0xff,0xff,0xf7,0xfd,0xff,0xf7,0x3d,0x00,0x00,0xe6,
0xff,0x68,0x01,0x0c,0x93,0xae,0x5a,0x00,0x00,0xf1,0xff,0x3b,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xa5,0xff,0xf1,0x82,0x1e,0x00,0x00,0x00,0x00,0x00,0x0c,0xa0,0xfe,0xff,0xfe,0xb8,0x32,
0x00,0x00,0x00,0x00,0x00,0x26,0x89,0xe7,0xff,0xfb,0x51,0x00,0x00,0x00,0x00,0x00,0x00,0x06,
0x91,0xff,0xdf,0x00,0x2b,0x93,0x95,0x00,0x00,0x00,0x10,0xff,0xff,0x0b,0x1e,0xfe,0xff,0x79,
0x04,0x03,0x75,0xff,0xed,0x01,0x00,0x7b,0xff,0xff,0xfa,0xf9,0xff,0xff,0x6c,0x00,0x00,0x00,
0x51,0xc3,0xef,0xec,0xbe,0x48,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xec,0xec,0xec,0xec,0xff,
0xff,0xed,0xec,0xec,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,
0x00,0x00,0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x14,
0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,
0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x14,0xff,0xff,0x00,
0x00,0xf5,0xff,0x22,0x00,0x00,0x00,0x1f,0xff,0xf4,0x00,0x00,0xd3,0xff,0x5b,0x00,0x00,0x00,
0x5a,0xff,0xd3,0x00,0x00,0x82,0xff,0xe1,0x2a,0x00,0x30,0xe0,0xff,0x82,0x00,0x00,0x0c,0xd7,
0xff,0xff,0xf5,0xff,0xff,0xd6,0x0c,0x00,0x00,0x00,0x11,0x8f,0xe0,0xf8,0xe0,0x8e,0x10,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x46,0xff,0xff,0x20,0x00,0x00,0x00,0x3d,
0xff,0xfd,0x15,0x07,0xf3,0xff,0x68,0x00,0x00,0x00,0x87,0xff,0xc9,0x00,0x00,0xaf,0xff,0xb0,
0x00,0x00,0x00,0xd1,0xff,0x7f,0x00,0x00,0x63,0xff,0xf3,0x05,0x00,0x1b,0xff,0xff,0x35,0x00,
0x00,0x19,0xfe,0xff,0x41,0x00,0x65,0xff,0xe9,0x01,0x00,0x00,0x00,0xcc,0xff,0x8a,0x00,0xaf,
0xff,0xa1,0x00,0x00,0x00,0x00,0x80,0xff,0xd2,0x06,0xf2,0xff,0x57,0x00,0x00,0x00,0x00,0x35,
0xff,0xff,0x5e,0xff,0xfc,0x10,0x00,0x00,0x00,0x00,0x01,0xe7,0xff,0xe4,0xff,0xc3,0x00,0x00,
0x00,0x00,0x00,0x00,0x9d,0xff,0xff,0xff,0x79,0x00,0x00,0x00,0x00,0x00,0x00,0x52,0xff,0xff,
0xff,0x2f,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xf9,0xff,0xe4,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x4f,0xff,0xed,0x00,0x00,0xac,0xff,0xff,0x20,0x00,0x7d,0xff,0xbf,0x00,
0x25,0xff,0xff,0x0f,0x00,0xdd,0xff,0xff,0x50,0x00,0x9f,0xff,0x95,0x00,0x03,0xf7,0xff,0x31,
0x0e,0xfe,0xff,0xff,0x80,0x00,0xc1,0xff,0x6b,0x00,0x00,0xd0,0xff,0x54,0x3e,0xff,0xff,0xff,
0xb1,0x00,0xe3,0xff,0x41,0x00,0x00,0xa6,0xff,0x76,0x6f,0xff,0xce,0xff,0xe1,0x06,0xfd,0xff,
0x17,0x00,0x00,0x7b,0xff,0x99,0xa0,0xff,0x75,0xfa,0xff,0x37,0xff,0xed,0x00,0x00,0x00,0x51,
0xff,0xbb,0xd0,0xff,0x41,0xd0,0xff,0x8a,0xff,0xc3,0x00,0x00,0x00,0x27,0xff,0xe4,0xfa,0xff,
0x12,0xa1,0xff,0xdb,0xff,0x99,0x00,0x00,0x00,0x03,0xf8,0xff,0xff,0xe3,0x00,0x72,0xff,0xff,
0xff,0x6f,0x00,0x00,0x00,0x00,0xd2,0xff,0xff,0xb4,0x00,0x42,0xff,0xff,0xff,0x45,0x00,0x00,
0x00,0x00,0xa8,0xff,0xff,0x85,0x00,0x13,0xff,0xff,0xff,0x1b,0x00,0x00,0x00,0x00,0x7d,0xff,
0xff,0x57,0x00,0x00,0xe4,0xff,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x28,0xfa,0xff,0xa5,0x00,
0x00,0x00,0xa5,0xff,0xfa,0x2b,0x00,0x89,0xff,0xfe,0x31,0x00,0x32,0xfe,0xff,0x8a,0x00,0x00,
0x0b,0xe3,0xff,0xba,0x00,0xbb,0xff,0xe3,0x0c,0x00,0x00,0x00,0x54,0xff,0xff,0x8a,0xff,0xff,
0x54,0x00,0x00,0x00,0x00,0x00,0xb9,0xff,0xff,0xff,0xb9,0x00,0x00,0x00,0x00,0x00,0x00,0x25,
0xf9,0xff,0xf9,0x25,0x00,0x00,0x00,0x00,0x00,0x00,0x29,0xfa,0xff,0xfa,0x29,0x00,0x00,0x00,
0x00,0x00,0x00,0xbe,0xff,0xff,0xff,0xbd,0x00,0x00,0x00,0x00,0x00,0x5a,0xff,0xff,0x81,0xff,
0xff,0x59,0x00,0x00,0x00,0x0e,0xe6,0xff,0xb6,0x00,0xb5,0xff,0xe6,0x0e,0x00,0x00,0x90,0xff,
0xfd,0x2e,0x00,0x2c,0xfd,0xff,0x8f,0x00,0x2e,0xfc,0xff,0xa1,0x00,0x00,0x00,0x9e,0xff,0xfc,
0x2e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x8a,0xff,0xff,0x3b,0x00,0x00,0x30,0xfe,0xff,0x9a,0x0c,0xe3,0xff,
0xc1,0x00,0x00,0xb5,0xff,0xeb,0x12,0x00,0x53,0xff,0xff,0x48,0x3a,0xff,0xff,0x63,0x00,0x00,
0x00,0xb7,0xff,0xcd,0xc0,0xff,0xc7,0x01,0x00,0x00,0x00,0x23,0xf7,0xff,0xff,0xfc,0x30,0x00,
0x00,0x00,0x00,0x00,0x7f,0xff,0xff,0x91,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0xff,0xff,0x1b,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0x3c,0x00,0xec,0xec,
0xec,0xec,0xec,0xef,0xff,0xff,0x99,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8a,0xff,0xe7,0x10,
0x00,0x00,0x00,0x00,0x00,0x00,0x30,0xfb,0xff,0x56,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0xcb,
0xff,0xb4,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x71,0xff,0xf4,0x1f,0x00,0x00,0x00,0x00,0x00,
0x00,0x1f,0xf4,0xff,0x71,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb5,0xff,0xcc,0x03,0x00,0x00,
0x00,0x00,0x00,0x00,0x58,0xff,0xfc,0x32,0x00,0x00,0x00,0x00,0x00,0x00,0x11,0xe9,0xff,0x8d,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x9d,0xff,0xff,0xf0,0xec,0xec,0xec,0xec,0xec,0x76,0x3f,
0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0x00,
0x00,0xff,0xff,0xed,0xec,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0xed,0xec,0x00,0x00,0xff,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x1c,0x80,0x80,0x19,0x00,0x00,0x00,0x00,0x00,0x04,0xe5,0xff,0x84,0x00,0x00,0x00,0x00,0x00,
0x00,0x80,0xff,0xea,0x07,0x00,0x00,0x00,0x00,0x00,0x1a,0xfb,0xff,0x5e,0x00,0x00,0x00,0x00,
0x00,0x00,0xab,0xff,0xcb,0x00,0x00,0x00,0x00,0x00,0x00,0x41,0xff,0xff,0x39,0x00,0x00,0x00,
0x00,0x00,0x00,0xd6,0xff,0xa6,0x00,0x00,0x00,0x00,0x00,0x00,0x6d,0xff,0xfa,0x19,0x00,0x00,
0x00,0x00,0x00,0x0e,0xf3,0xff,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x98,0xff,0xe7,0x05,0x00,
0x00,0x00,0x00,0x00,0x2e,0xff,0xff,0x5a,0x00,0x00,0x00,0x00,0x00,0x00,0xc4,0xff,0xc7,0x00,
0x00,0x00,0x00,0x00,0x00,0x59,0xff,0xff,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0x14,0x00,0xec,0xec,0xff,0xff,0x14,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xec,0xec,
0xff,0xff,0x14,0x00,0xff,0xff,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x1b,0xe9,0xff,0xb6,0x02,0x00,0x00,0x00,0x06,0xc7,0xff,0xe0,0xff,0x81,0x00,0x00,
0x00,0x97,0xff,0xad,0x13,0xe0,0xfe,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0x00,
0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x65,0x11,
0x00,0x00,0x4a,0xff,0xd5,0x1d,0x00,0x0c,0xa6,0xff,0xe4,0x23,0x00,0x00,0x5a,0xe0,0x25,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1f,0xa6,0xed,0xf4,0xc6,
0x4b,0x00,0x00,0x00,0x1d,0xf3,0xf9,0xf6,0xff,0xfe,0x59,0x00,0x00,0x00,0x20,0x03,0x03,0x82,
0xff,0xd1,0x00,0x00,0x0a,0x97,0xed,0xde,0x59,0xff,0xfa,0x04,0x00,0x95,0xff,0xff,0xf5,0xf0,
0xff,0xff,0x13,0x00,0xea,0xff,0x5c,0x00,0x42,0xff,0xff,0x14,0x00,0xe9,0xff,0x58,0x00,0x3f,
0xff,0xff,0x14,0x00,0x92,0xff,0xff,0xf4,0xf6,0xff,0xff,0x14,0x00,0x0a,0x98,0xed,0xe0,0x5b,
0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x2f,0xad,0xf5,0xdf,
0x63,0x00,0x00,0x00,0xff,0xff,0xe4,0xfc,0xfb,0xff,0xff,0x40,0x00,0x00,0xff,0xff,0xba,0x12,
0x10,0xba,0xff,0xab,0x00,0x00,0xff,0xff,0x3a,0x00,0x00,0x3c,0xff,0xe1,0x00,0x00,0xff,0xff,
0x1c,0x00,0x00,0x1d,0xff,0xf8,0x00,0x00,0xff,0xff,0x3a,0x00,0x00,0x42,0xff,0xe2,0x00,0x00,
0xff,0xff,0xb8,0x10,0x14,0xc8,0xff,0xaf,0x00,0x00,0xff,0xff,0xe7,0xfc,0xfb,0xff,0xff,0x41,
0x00,0x00,0xff,0xff,0x30,0xae,0xf5,0xdf,0x62,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1f,
0xa9,0xef,0xf0,0xa9,0x1c,0x00,0x00,0x1a,0xe7,0xff,0xfd,0xfc,0xff,0xe4,0x14,0x00,0x9b,0xff,
0xcc,0x17,0x0e,0x98,0x55,0x02,0x00,0xe5,0xff,0x44,0x00,0x00,0x00,0x00,0x00,0x00,0xfb,0xff,
0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0xe8,0xff,0x42,0x00,0x00,0x00,0x00,0x00,0x00,0xa2,0xff,
0xce,0x1d,0x03,0x8d,0x53,0x01,0x00,0x21,0xed,0xff,0xff,0xf5,0xff,0xe1,0x13,0x00,0x00,0x25,
0xab,0xee,0xf4,0xb1,0x1d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x14,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0x00,0x63,0xdf,
0xf5,0xad,0x2e,0xff,0xff,0x00,0x00,0x41,0xff,0xff,0xfb,0xfd,0xe4,0xff,0xff,0x00,0x00,0xaf,
0xff,0xb8,0x0f,0x14,0xbc,0xff,0xff,0x00,0x00,0xe4,0xff,0x3b,0x00,0x00,0x3d,0xff,0xff,0x00,
0x00,0xf9,0xff,0x1d,0x00,0x00,0x1e,0xff,0xff,0x00,0x00,0xe3,0xff,0x41,0x00,0x00,0x3d,0xff,
0xff,0x00,0x00,0xb0,0xff,0xc6,0x12,0x11,0xba,0xff,0xff,0x00,0x00,0x41,0xff,0xff,0xfb,0xfc,
0xe2,0xff,0xff,0x00,0x00,0x00,0x63,0xe0,0xf5,0xab,0x2d,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1a,0xa3,0xee,0xf1,0xb1,0x26,0x00,0x00,
0x00,0x15,0xe1,0xff,0xf7,0xfb,0xff,0xec,0x1c,0x00,0x00,0x96,0xff,0x6d,0x01,0x08,0x9a,0xff,
0x9b,0x00,0x00,0xe4,0xff,0xed,0xec,0xec,0xf1,0xff,0xe4,0x00,0x00,0xfa,0xff,0xff,0xff,0xff,
0xff,0xff,0xfc,0x00,0x00,0xe1,0xff,0x23,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8f,0xff,0x9f,
0x07,0x09,0x50,0x0c,0x00,0x00,0x00,0x13,0xdc,0xff,0xf8,0xfb,0xff,0xef,0x1f,0x00,0x00,0x00,
0x0e,0x96,0xeb,0xf5,0xbd,0x3d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x05,0x7c,0xe5,0xf1,0xb1,0x10,0x00,0x00,0x9a,0xff,0xf8,0xf9,0x9c,0x00,0x00,0x00,0xfc,0xff,
0x22,0x06,0x03,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xec,0xff,0xff,0xed,0xec,
0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0xdf,0xf5,0xad,0x2e,0xff,0xff,0x00,
0x00,0x41,0xff,0xff,0xfb,0xfd,0xe4,0xff,0xff,0x00,0x00,0xaf,0xff,0xb8,0x0f,0x14,0xbc,0xff,
0xff,0x00,0x00,0xe4,0xff,0x3b,0x00,0x00,0x3d,0xff,0xff,0x00,0x00,0xf9,0xff,0x1d,0x00,0x00,
0x1e,0xff,0xff,0x00,0x00,0xe3,0xff,0x41,0x00,0x00,0x3d,0xff,0xff,0x00,0x00,0xb0,0xff,0xc6,
0x12,0x11,0xba,0xff,0xff,0x00,0x00,0x41,0xff,0xff,0xfb,0xfc,0xe2,0xff,0xff,0x00,0x00,0x00,
0x63,0xe0,0xf5,0xab,0x2d,0xff,0xff,0x00,0x00,0x71,0xa8,0x54,0x01,0x01,0x61,0xff,0xe1,0x00,
0x00,0x45,0xfc,0xff,0xf6,0xf6,0xff,0xff,0x5d,0x00,0x00,0x00,0x44,0xbe,0xef,0xf7,0xcb,0x5b,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x35,0xbb,0xf3,
0xac,0x0e,0x00,0x00,0xff,0xff,0xe4,0xf6,0xff,0xff,0x9c,0x00,0x00,0xff,0xff,0x90,0x00,0x6b,
0xff,0xea,0x00,0x00,0xff,0xff,0x2d,0x00,0x14,0xff,0xff,0x0e,0x00,0xff,0xff,0x17,0x00,0x01,
0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,
0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xa8,0xbd,0x00,0x00,0xf6,0xff,0x0a,0x00,0x83,0x95,0x00,0x00,
0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,
0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xa8,0xbd,0x00,0x00,0x00,0x00,
0xf6,0xff,0x0a,0x00,0x00,0x00,0x83,0x95,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0x13,0x00,0x11,0x1e,0xff,0xf3,0x02,0x50,0xf9,0xf9,
0xff,0x81,0x00,0x8b,0xee,0xeb,0x80,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0x14,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0x14,0x15,0xe2,0xff,0x9e,0x00,0x00,0xff,0xff,0x17,0xbc,0xff,0xc6,0x06,0x00,0x00,0xff,0xff,
0x9c,0xff,0xe3,0x17,0x00,0x00,0x00,0xff,0xff,0xff,0xf6,0x32,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0xf4,0x23,0x00,0x00,0x00,0x00,0xff,0xff,0xa0,0xff,0xc6,0x03,0x00,0x00,0x00,0xff,0xff,
0x1b,0xd6,0xff,0x7b,0x00,0x00,0x00,0xff,0xff,0x14,0x38,0xfd,0xfa,0x32,0x00,0x00,0xff,0xff,
0x14,0x00,0x8c,0xff,0xd6,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,
0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xfc,0xff,0x20,0x00,0xd7,0xff,0xcb,0x00,0x40,
0xcf,0xce,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x36,0xbd,0xf4,0xba,0x11,0x1d,0xc1,0xf1,0xa2,0x09,0x00,
0x00,0xff,0xff,0xe8,0xf9,0xff,0xff,0x9e,0xdf,0xf9,0xff,0xff,0x7e,0x00,0x00,0xff,0xff,0x85,
0x02,0x88,0xff,0xff,0x71,0x02,0x9b,0xff,0xcc,0x00,0x00,0xff,0xff,0x2a,0x00,0x2c,0xff,0xff,
0x16,0x00,0x40,0xff,0xe7,0x00,0x00,0xff,0xff,0x16,0x00,0x17,0xff,0xff,0x02,0x00,0x2b,0xff,
0xec,0x00,0x00,0xff,0xff,0x14,0x00,0x14,0xff,0xff,0x00,0x00,0x28,0xff,0xec,0x00,0x00,0xff,
0xff,0x14,0x00,0x14,0xff,0xff,0x00,0x00,0x28,0xff,0xec,0x00,0x00,0xff,0xff,0x14,0x00,0x14,
0xff,0xff,0x00,0x00,0x28,0xff,0xec,0x00,0x00,0xff,0xff,0x14,0x00,0x14,0xff,0xff,0x00,0x00,
0x28,0xff,0xec,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x39,0xbe,0xf3,0xac,0x0e,0x00,0x00,0xff,0xff,
0xe8,0xf6,0xff,0xff,0x9c,0x00,0x00,0xff,0xff,0x90,0x00,0x6b,0xff,0xea,0x00,0x00,0xff,0xff,
0x2d,0x00,0x14,0xff,0xff,0x0e,0x00,0xff,0xff,0x17,0x00,0x01,0xff,0xff,0x14,0x00,0xff,0xff,
0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,
0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x36,0xbc,0xf3,0xf4,0xbe,0x37,0x00,0x00,0x00,0x2e,0xf7,
0xff,0xfb,0xfc,0xff,0xf7,0x2e,0x00,0x00,0xab,0xff,0xb0,0x0a,0x0b,0xb0,0xff,0xab,0x00,0x00,
0xea,0xff,0x34,0x00,0x00,0x35,0xff,0xe9,0x00,0x00,0xfc,0xff,0x1a,0x00,0x00,0x1b,0xff,0xfb,
0x00,0x00,0xea,0xff,0x34,0x00,0x00,0x35,0xff,0xe8,0x00,0x00,0xac,0xff,0xaf,0x0a,0x0a,0xb0,
0xff,0xa9,0x00,0x00,0x2f,0xf7,0xff,0xfb,0xfb,0xff,0xf6,0x2c,0x00,0x00,0x00,0x37,0xbd,0xf5,
0xf5,0xbe,0x37,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0x32,0xb0,0xf5,0xdf,0x63,0x00,0x00,0x00,0xff,0xff,0xe8,0xfc,0xfb,0xff,0xff,0x40,
0x00,0x00,0xff,0xff,0xba,0x12,0x10,0xba,0xff,0xab,0x00,0x00,0xff,0xff,0x3a,0x00,0x00,0x3c,
0xff,0xe1,0x00,0x00,0xff,0xff,0x1c,0x00,0x00,0x1d,0xff,0xf8,0x00,0x00,0xff,0xff,0x3a,0x00,
0x00,0x42,0xff,0xe2,0x00,0x00,0xff,0xff,0xb8,0x10,0x14,0xc8,0xff,0xaf,0x00,0x00,0xff,0xff,
0xe2,0xfc,0xfb,0xff,0xff,0x41,0x00,0x00,0xff,0xff,0x2d,0xab,0xf5,0xdf,0x62,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0xdf,0xf5,0xad,0x2e,
0xff,0xff,0x00,0x00,0x41,0xff,0xff,0xfb,0xfd,0xe4,0xff,0xff,0x00,0x00,0xaf,0xff,0xb8,0x0f,
0x14,0xbc,0xff,0xff,0x00,0x00,0xe4,0xff,0x3b,0x00,0x00,0x3d,0xff,0xff,0x00,0x00,0xf9,0xff,
0x1d,0x00,0x00,0x1e,0xff,0xff,0x00,0x00,0xe3,0xff,0x41,0x00,0x00,0x3d,0xff,0xff,0x00,0x00,
0xb0,0xff,0xc6,0x12,0x11,0xba,0xff,0xff,0x00,0x00,0x41,0xff,0xff,0xfb,0xfc,0xe2,0xff,0xff,
0x00,0x00,0x00,0x63,0xe0,0xf5,0xab,0x2d,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x12,
0xff,0xff,0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xea,0xff,0xdd,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x42,0xcb,0xd9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0x3c,0xcc,0xec,0x7b,0x00,0xff,0xff,0xe8,0xf7,0xff,0x71,0x00,0xff,0xff,0xac,0x04,
0x2c,0x0d,0x00,0xff,0xff,0x35,0x00,0x00,0x00,0x00,0xff,0xff,0x16,0x00,0x00,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1e,0xab,0xeb,0xf1,0xb6,0x29,0x00,
0x00,0xc8,0xff,0xfe,0xfa,0xff,0xe7,0x12,0x00,0xf6,0xff,0x30,0x08,0x9d,0x99,0x28,0x00,0xb8,
0xff,0xd0,0x66,0x0d,0x00,0x00,0x00,0x10,0x9c,0xfb,0xff,0xf0,0x68,0x00,0x00,0x00,0x00,0x1c,
0x89,0xfc,0xff,0x3f,0x2c,0xbe,0xb4,0x0d,0x04,0xd5,0xff,0x74,0x07,0xd9,0xff,0xfa,0xf5,0xff,
0xfc,0x38,0x00,0x1c,0xa6,0xeb,0xf2,0xc2,0x46,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,
0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xec,0xff,
0xff,0xed,0xec,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,
0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0xfa,0xff,
0x34,0x00,0x00,0x00,0x00,0xcb,0xff,0xfe,0xee,0x00,0x00,0x00,0x2a,0xb9,0xee,0xfe,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xfe,0xff,0x16,0x00,0x03,0xff,0xff,0x14,
0x00,0xf2,0xff,0x2b,0x00,0x1b,0xff,0xff,0x14,0x00,0xcb,0xff,0x74,0x00,0x6f,0xff,0xff,0x14,
0x00,0x78,0xff,0xfb,0xc7,0xf0,0xfe,0xff,0x14,0x00,0x07,0xac,0xf8,0xcb,0x3b,0xfc,0xff,0x14,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x40,0xff,0xf3,0x05,0x00,0x33,0xff,0xd8,0x00,0x03,0xec,0xff,0x3b,0x00,0x7a,0xff,0x88,0x00,
0x00,0xa0,0xff,0x7e,0x00,0xc1,0xff,0x38,0x00,0x00,0x50,0xff,0xc1,0x0d,0xfa,0xe6,0x01,0x00,
0x00,0x0a,0xf6,0xf9,0x59,0xff,0x98,0x00,0x00,0x00,0x00,0xb0,0xff,0xd9,0xff,0x48,0x00,0x00,
0x00,0x00,0x60,0xff,0xff,0xf1,0x06,0x00,0x00,0x00,0x00,0x13,0xfc,0xff,0xa8,0x00,0x00,0x00,
0x00,0x00,0x00,0xc0,0xff,0x58,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0xff,0xb7,
0x00,0x0b,0xfd,0xff,0x14,0x00,0xa5,0xff,0x5a,0x19,0xfe,0xf2,0x03,0x3a,0xff,0xff,0x45,0x00,
0xe6,0xfc,0x12,0x00,0xcd,0xff,0x36,0x6b,0xff,0xff,0x77,0x27,0xff,0xc5,0x00,0x00,0x83,0xff,
0x75,0x9c,0xf0,0xfe,0xa8,0x68,0xff,0x7a,0x00,0x00,0x38,0xff,0xb5,0xce,0xb5,0xd9,0xd9,0xa9,
0xff,0x2f,0x00,0x00,0x02,0xeb,0xf1,0xf9,0x7e,0xa6,0xfe,0xed,0xe4,0x00,0x00,0x00,0x00,0xa3,
0xff,0xff,0x48,0x72,0xff,0xff,0x9a,0x00,0x00,0x00,0x00,0x58,0xff,0xff,0x12,0x3e,0xff,0xff,
0x4f,0x00,0x00,0x00,0x00,0x11,0xfc,0xdb,0x00,0x0c,0xfd,0xf9,0x0b,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0xff,0xf5,0x1b,0x00,0x60,0xff,0xfc,0x34,
0x00,0xbb,0xff,0xa3,0x0a,0xe4,0xff,0x8a,0x00,0x00,0x28,0xfa,0xfe,0xac,0xff,0xda,0x09,0x00,
0x00,0x00,0x89,0xff,0xff,0xfe,0x3e,0x00,0x00,0x00,0x00,0x19,0xfe,0xff,0xcd,0x00,0x00,0x00,
0x00,0x00,0x92,0xff,0xff,0xff,0x4d,0x00,0x00,0x00,0x36,0xfd,0xfd,0xa4,0xff,0xe0,0x0a,0x00,
0x04,0xd1,0xff,0x9b,0x09,0xe2,0xff,0x8a,0x00,0x77,0xff,0xf3,0x18,0x00,0x5c,0xff,0xfb,0x2d,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,
0x00,0xfe,0xff,0x17,0x00,0x05,0xff,0xff,0x14,0x00,0xf2,0xff,0x30,0x00,0x22,0xff,0xff,0x14,
0x00,0xcb,0xff,0x8f,0x01,0x8e,0xff,0xff,0x14,0x00,0x78,0xff,0xff,0xf6,0xe4,0xff,0xff,0x14,
0x00,0x07,0xac,0xf7,0xc5,0x2b,0xff,0xff,0x11,0x00,0x12,0x67,0x1f,0x00,0x46,0xff,0xdf,0x00,
0x00,0x70,0xff,0xff,0xf1,0xff,0xfd,0x45,0x00,0x00,0x00,0x65,0xd6,0xf8,0xcc,0x3d,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x81,0x00,0xec,0xec,0xec,0xfb,0xff,0xed,0x0f,0x00,
0x00,0x00,0x17,0xf5,0xff,0x77,0x00,0x00,0x00,0x00,0x91,0xff,0xe8,0x0a,0x00,0x00,0x00,0x1d,
0xf8,0xff,0x6d,0x00,0x00,0x00,0x00,0x9a,0xff,0xe2,0x07,0x00,0x00,0x00,0x22,0xfb,0xff,0x64,
0x00,0x00,0x00,0x00,0xa2,0xff,0xff,0xee,0xec,0xec,0x76,0x28,0xfc,0xff,0xff,0xff,0xff,0xff,
0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x4c,0xd1,0xe9,0x00,0x00,0x00,0xe2,0xff,0xff,0x00,0x00,0x00,0xf9,
0xff,0x40,0x00,0x00,0x03,0xff,0xff,0x10,0x00,0x00,0x17,0xff,0xfd,0x03,0x00,0x00,0x48,0xff,
0xd6,0x00,0x00,0x0e,0xc7,0xfe,0x64,0x00,0x00,0x19,0xd3,0xfb,0x54,0x00,0x00,0x00,0x50,0xff,
0xd0,0x00,0x00,0x00,0x1a,0xff,0xfc,0x02,0x00,0x00,0x05,0xff,0xff,0x0f,0x00,0x00,0x00,0xfa,
0xff,0x32,0x00,0x00,0x00,0xe6,0xff,0xfa,0x00,0x00,0x00,0x5e,0xe6,0xfd,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,
0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,
0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x75,0xe5,0xc7,
0x48,0x00,0x00,0x80,0xff,0xff,0xef,0x00,0x00,0x03,0x47,0xff,0xff,0x09,0x00,0x00,0x01,0xfc,
0xff,0x16,0x00,0x00,0x00,0xed,0xff,0x28,0x00,0x00,0x00,0xc2,0xff,0x65,0x00,0x00,0x00,0x52,
0xfc,0xd5,0x14,0x00,0x00,0x45,0xf7,0xdf,0x21,0x00,0x00,0xbd,0xff,0x6e,0x00,0x00,0x00,0xeb,
0xff,0x2b,0x00,0x00,0x00,0xfb,0xff,0x18,0x00,0x00,0x33,0xff,0xff,0x0a,0x00,0x78,0xfe,0xff,
0xf4,0x00,0x00,0x80,0xfd,0xe1,0x5d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x9f,0xf1,0x76,0x03,0x4e,0x00,0x00,0xff,0xf9,0xff,
0xf9,0xff,0x00,0x00,0x47,0x06,0x85,0xf0,0xac,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4d,0xc1,
0xf4,0xf4,0xba,0x3e,0x00,0x00,0x00,0x00,0x6c,0xff,0xff,0xf6,0xfd,0xff,0xfe,0x64,0x00,0x00,
0x27,0xf8,0xfa,0x61,0x02,0x12,0xb8,0xff,0xf9,0x26,0x00,0x8e,0xff,0x93,0x00,0x00,0x00,0x0c,
0xf0,0xff,0x8b,0x00,0xd7,0xff,0x3e,0x00,0x00,0x00,0x00,0x12,0x18,0x10,0x00,0xf4,0xff,0x1c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf4,0xff,0x1b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xd9,0xff,0x3d,0x00,0x00,0x00,0x00,0x0b,0x10,0x0b,0x00,0x90,0xff,0x91,0x00,0x00,0x00,
0x0b,0xed,0xff,0x8c,0x00,0x29,0xf8,0xfa,0x60,0x06,0x12,0xb1,0xff,0xfa,0x29,0x00,0x00,0x6f,
0xff,0xff,0xf9,0xfe,0xff,0xfe,0x6a,0x00,0x00,0x00,0x00,0x50,0xc3,0xf5,0xec,0xbb,0x42,0x00,
0x00,0x00,0x00,0x00,0x00,0x0f,0xa1,0x74,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0xf4,
0xeb,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x9f,0xff,0x9e,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x3a,0x5b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x6c,0xcb,0x4c,0x00,0x6c,0xcb,0x4c,0x00,0x00,0x00,0x00,0x00,
0xf3,0xff,0xca,0x00,0xf3,0xff,0xca,0x00,0x00,0x00,0x00,0x00,0x90,0xf3,0x6d,0x00,0x90,0xf3,
0x6d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x49,0xbd,0xf2,0xf4,0xc5,0x57,0x00,0x00,0x00,0x00,0x00,0x68,0xff,0xff,0xf9,0xf8,0xff,0xff,
0x7f,0x00,0x00,0x00,0x24,0xf6,0xfe,0x74,0x06,0x02,0x5a,0xf8,0xfc,0x33,0x00,0x00,0x8a,0xff,
0xa1,0x00,0x00,0x00,0x00,0x87,0xff,0x9d,0x00,0x00,0xd4,0xff,0x45,0x00,0x00,0x00,0x00,0x2f,
0xff,0xe7,0x00,0x00,0xf3,0xff,0x1e,0x00,0x00,0x00,0x00,0x0d,0xff,0xff,0x06,0x00,0xf3,0xff,
0x1d,0x00,0x00,0x00,0x00,0x0a,0xff,0xff,0x07,0x00,0xd5,0xff,0x43,0x00,0x00,0x00,0x00,0x2e,
0xff,0xe8,0x00,0x00,0x8c,0xff,0xa0,0x00,0x00,0x00,0x00,0x88,0xff,0x9f,0x00,0x00,0x26,0xf7,
0xfe,0x72,0x0a,0x07,0x61,0xf9,0xfc,0x35,0x00,0x00,0x00,0x6b,0xff,0xff,0xfc,0xfb,0xff,0xff,
0x82,0x00,0x00,0x00,0x00,0x00,0x4c,0xc1,0xf3,0xf5,0xc8,0x5b,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x4f,0xcb,0x6b,0x00,0x6d,0xcb,0x4d,0x00,0x00,0x00,0x00,0xcb,0xff,0xf3,
0x00,0xf3,0xff,0xcb,0x00,0x00,0x00,0x00,0x6b,0xf3,0x8e,0x00,0x8e,0xf3,0x6a,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x14,
0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,
0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x14,0xff,0xff,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,
0x14,0xff,0xff,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x14,0xff,0xff,0x00,0x00,0xf5,0xff,
0x22,0x00,0x00,0x00,0x1f,0xff,0xf4,0x00,0x00,0xd3,0xff,0x5b,0x00,0x00,0x00,0x5a,0xff,0xd3,
0x00,0x00,0x82,0xff,0xe1,0x2a,0x00,0x30,0xe0,0xff,0x82,0x00,0x00,0x0c,0xd7,0xff,0xff,0xf5,
0xff,0xff,0xd6,0x0c,0x00,0x00,0x00,0x11,0x8f,0xe0,0xf8,0xe0,0x8e,0x10,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1b,0xa1,0xec,0xf4,0xb9,0x32,0x00,0x00,0x17,0xe2,
0xff,0xff,0xfa,0xff,0xf8,0x36,0x00,0x98,0xff,0xd2,0x1e,0x08,0x81,0x71,0x0a,0x00,0xe4,0xff,
0x46,0x00,0x00,0x00,0x00,0x00,0x00,0xfb,0xff,0x1d,0x00,0x00,0x00,0x00,0x00,0x00,0xe6,0xff,
0x45,0x00,0x00,0x00,0x00,0x00,0x00,0x9e,0xff,0xd2,0x22,0x00,0x7c,0x6a,0x07,0x00,0x1d,0xe8,
0xff,0xff,0xf4,0xff,0xf6,0x33,0x00,0x00,0x1e,0xa2,0xe9,0xf7,0xc2,0x34,0x00,0x00,0x00,0x00,
0x20,0xb6,0x74,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0xff,0xea,0x00,0x00,0x00,0x00,0x00,0x04,
0xc6,0xff,0x8c,0x00,0x00,0x00,0x00,0x00,0x00,0x49,0x41,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x6d,0xf3,0x8f,0x00,0x6d,0xf3,
0x8f,0x00,0x00,0x00,0xcb,0xff,0xf3,0x00,0xcb,0xff,0xf3,0x00,0x00,0x00,0x4b,0xcb,0x6d,0x00,
0x4d,0xcb,0x6d,0x00,0x00,0x00,0x00,0x36,0xbc,0xf3,0xf4,0xbe,0x37,0x00,0x00,0x00,0x2e,0xf7,
0xff,0xfb,0xfc,0xff,0xf7,0x2e,0x00,0x00,0xab,0xff,0xb0,0x0a,0x0b,0xb0,0xff,0xab,0x00,0x00,
0xea,0xff,0x34,0x00,0x00,0x35,0xff,0xe9,0x00,0x00,0xfc,0xff,0x1a,0x00,0x00,0x1b,0xff,0xfb,
0x00,0x00,0xea,0xff,0x34,0x00,0x00,0x35,0xff,0xe8,0x00,0x00,0xac,0xff,0xaf,0x0a,0x0a,0xb0,
0xff,0xa9,0x00,0x00,0x2f,0xf7,0xff,0xfb,0xfb,0xff,0xf6,0x2c,0x00,0x00,0x00,0x37,0xbd,0xf5,
0xf5,0xbe,0x37,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x6d,0xf3,0x8f,0x00,0x6d,0xf3,0x8f,0x00,0x00,0xcb,0xff,0xf3,0x00,0xcb,0xff,0xf3,0x00,
0x00,0x4b,0xca,0x6b,0x00,0x4d,0xcb,0x6a,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,
0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0xfe,0xff,0x16,0x00,0x03,0xff,0xff,0x14,
0x00,0xf2,0xff,0x2b,0x00,0x1b,0xff,0xff,0x14,0x00,0xcb,0xff,0x74,0x00,0x6f,0xff,0xff,0x14,
0x00,0x78,0xff,0xfb,0xc7,0xf0,0xfe,0xff,0x14,0x00,0x07,0xac,0xf8,0xcb,0x3b,0xfc,0xff,0x14,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf2,0xc9,0x13,
0x10,0xc2,0xf1,0x00,0x00,0x00,0x00,0x00,0x00,0x9a,0xff,0xf9,0xf9,0xff,0x9a,0x00,0x00,0x00,
0x00,0x00,0x00,0x0a,0x92,0xed,0xed,0x91,0x09,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4b,0xbc,0xf1,0xf4,0xc6,0x54,0x00,0x00,0x00,
0x00,0x00,0x70,0xff,0xff,0xfa,0xfa,0xff,0xff,0x7b,0x00,0x00,0x00,0x2f,0xfa,0xff,0x82,0x07,
0x0d,0x8e,0xbb,0x72,0x00,0x00,0x00,0x98,0xff,0xaa,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xdc,0xff,0x45,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf7,0xff,0x1e,0x00,0x00,
0xf8,0xf8,0xf8,0xf8,0xf8,0x00,0x00,0xee,0xff,0x27,0x00,0x00,0xff,0xff,0xff,0xff,0xf9,0x00,
0x00,0xce,0xff,0x55,0x00,0x00,0x00,0x00,0x7f,0xff,0xe3,0x00,0x00,0x81,0xff,0xbc,0x00,0x00,
0x00,0x00,0xbe,0xff,0xa9,0x00,0x00,0x1e,0xf2,0xff,0x8c,0x07,0x08,0x73,0xff,0xff,0x48,0x00,
0x00,0x00,0x5b,0xfd,0xff,0xfa,0xfb,0xff,0xff,0x9b,0x00,0x00,0x00,0x00,0x00,0x3d,0xb5,0xef,
0xf8,0xd2,0x6e,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf6,0xdc,0x03,0x00,0x25,
0xf3,0x00,0x00,0x00,0x00,0xc8,0xff,0x7d,0x11,0xbb,0xc8,0x00,0x00,0x00,0x00,0x57,0xff,0xff,
0xff,0xff,0x57,0x00,0x00,0x00,0x00,0x00,0x60,0xd4,0xd4,0x60,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0xdf,0xf5,0xad,0x2e,0xff,0xff,0x00,
0x00,0x41,0xff,0xff,0xfb,0xfd,0xe4,0xff,0xff,0x00,0x00,0xaf,0xff,0xb8,0x0f,0x14,0xbc,0xff,
0xff,0x00,0x00,0xe4,0xff,0x3b,0x00,0x00,0x3d,0xff,0xff,0x00,0x00,0xf9,0xff,0x1d,0x00,0x00,
0x1e,0xff,0xff,0x00,0x00,0xe3,0xff,0x41,0x00,0x00,0x3d,0xff,0xff,0x00,0x00,0xb0,0xff,0xc6,
0x12,0x11,0xba,0xff,0xff,0x00,0x00,0x41,0xff,0xff,0xfb,0xfc,0xe2,0xff,0xff,0x00,0x00,0x00,
0x63,0xe0,0xf5,0xab,0x2d,0xff,0xff,0x00,0x00,0x97,0xa8,0x54,0x01,0x01,0x61,0xff,0xe1,0x00,
0x00,0x79,0xff,0xff,0xf6,0xf6,0xff,0xff,0x59,0x00,0x00,0x00,0x61,0xcb,0xf3,0xf5,0xc5,0x54,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x6c,0xcb,0x4c,0x00,0x00,0xf3,0xff,0xca,0x00,0x00,0x93,0xf3,0x6d,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0xff,
0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,
0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0xff,0xff,
0x14,0x00,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,
0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,0x14,0x00,0xff,0xff,
0x14,0x00,0xff,0xff,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x56,0xc2,0xef,0xed,0xb4,0x34,0x00,0x00,0x00,0x75,0xff,0xff,0xf7,0xfd,0xff,0xf7,0x3d,
0x00,0x00,0xe6,0xff,0x68,0x01,0x0c,0x93,0xae,0x5a,0x00,0x00,0xf1,0xff,0x3b,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xa5,0xff,0xf1,0x82,0x1e,0x00,0x00,0x00,0x00,0x00,0x0c,0xa0,0xfe,0xff,
0xfe,0xb8,0x32,0x00,0x00,0x00,0x00,0x00,0x26,0x89,0xe7,0xff,0xfb,0x51,0x00,0x00,0x00,0x00,
0x00,0x00,0x06,0x91,0xff,0xdf,0x00,0x2b,0x93,0x95,0x00,0x00,0x00,0x10,0xff,0xff,0x0b,0x1e,
0xfe,0xff,0x79,0x04,0x03,0x75,0xff,0xed,0x01,0x00,0x7b,0xff,0xff,0xfa,0xf9,0xff,0xff,0x6c,
0x00,0x00,0x00,0x51,0xca,0xff,0xff,0xbe,0x48,0x00,0x00,0x00,0x00,0x00,0x02,0xe4,0xfe,0x16,
0x00,0x00,0x00,0x00,0x00,0x00,0x7a,0xfe,0xd7,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x3d,0x85,
0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1e,0xab,0xeb,0xf1,0xb5,0x27,0x00,0x00,0xc8,0xff,
0xfc,0xfd,0xff,0xe5,0x10,0x00,0xf6,0xff,0x2d,0x23,0xb6,0x93,0x26,0x00,0xb8,0xff,0xd0,0x66,
0x0d,0x00,0x00,0x00,0x10,0x9c,0xfb,0xff,0xf0,0x68,0x00,0x00,0x00,0x00,0x1c,0x89,0xfc,0xff,
0x3f,0x2c,0xbe,0xb4,0x0d,0x04,0xd5,0xff,0x74,0x07,0xd9,0xff,0xfa,0xf5,0xff,0xfc,0x38,0x00,
0x1c,0xa6,0xff,0xff,0xc2,0x46,0x00,0x00,0x00,0x00,0xbc,0xe9,0x00,0x00,0x00,0x00,0x00,0x50,
0xf4,0xbd,0x00,0x00,0x00,0x00,0x00,0x1f,0x8a,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,
  };
  static const struct {
    uint32_t ds;
    uint16_t width;
  } glinfo[]= {

{0,5}, {105,4}, {189,7}, {336,13}, 
{609,9}, {798,12}, {1050,11}, {1281,4}, 
{1365,6}, {1491,6}, {1617,10}, {1827,9}, 
{2016,4}, {2100,8}, {2268,4}, {2352,9}, 
{2541,10}, {2751,10}, {2961,10}, {3171,10}, 
{3381,10}, {3591,10}, {3801,10}, {4011,10}, 
{4221,10}, {4431,10}, {4641,4}, {4725,4}, 
{4809,7}, {4956,9}, {5145,7}, {5292,9}, 
{5481,15}, {5796,12}, {6048,11}, {6279,12}, 
{6531,11}, {6762,10}, {6972,10}, {7182,12}, 
{7434,10}, {7644,4}, {7728,10}, {7938,11}, 
{8169,10}, {8379,14}, {8673,11}, {8904,12}, 
{9156,10}, {9366,12}, {9618,10}, {9828,10}, 
{10038,12}, {10290,11}, {10521,11}, {10752,14}, 
{11046,11}, {11277,10}, {11487,11}, {11718,6}, 
{11844,9}, {12033,6}, {12159,9}, {12348,11}, 
{12579,5}, {12684,9}, {12873,10}, {13083,9}, 
{13272,10}, {13482,10}, {13692,8}, {13860,10}, 
{14070,9}, {14259,4}, {14343,6}, {14469,9}, 
{14658,4}, {14742,14}, {15036,9}, {15225,10}, 
{15435,10}, {15645,10}, {15855,7}, {16002,8}, 
{16170,7}, {16317,9}, {16506,9}, {16695,12}, 
{16947,9}, {17136,9}, {17325,8}, {17493,6}, 
{17619,4}, {17703,6}, {17829,7}, {17976,11}, 
{18207,12}, {18459,11}, {18690,9}, {18879,10}, 
{19089,9}, {19278,12}, {19530,10}, {19740,5}, 
{19845,4}, {19929,10}, {20139,8}, 
  };
  // static const int nmap= sizeof(glmap)/sizeof(glmap[0]);
  int i,j,k,e,x,y,sX,eX;
  int W, X, *Xpos;
  char buf[20];
  int *map, *gl;
  int ngl;
  unsigned int C;
  int glyph_width;
  const unsigned char *glyph_data;
  unsigned char *data;

  map= alloca(L*sizeof(int));

  e= 0;
  ngl= 0;
  for(i=0;i<L;i++)
  {
    C= str[i];
    
    if (C&(1<<31)) { ngl+=2; e= 1; continue; }
    map[i]= font_hash(C);
    if (map[i]!=107) { ngl++; continue; }
    sprintf(buf,"%x",C); ngl+= strlen(buf); e= 1;
  }

  if (e) goto fix_encoding_errors;
  else gl= map;

encoding_errors_fixed:

  W= 0;
  for(i=0;i<ngl;i++) W+= glinfo[gl[i]].width;

  if (Rdata) data= malloc(W*21);
  X= 0;
  if (e && Rdata) Xpos= alloca(sizeof(int)*(ngl+1));
  for(i=0;i<ngl;i++)
  {
    if (e && Rdata) Xpos[i]= X;
    glyph_width= glinfo[gl[i]].width;
    if (Rdata) {
      glyph_data= font_data + glinfo[gl[i]].ds;
      for(y=0;y<21;y++)
         for(x=0;x<glyph_width;x++)
            data[y*W+X+x]= glyph_data[y*glyph_width+x];
    }
    X+= glyph_width;
  }
  if (e && Rdata) Xpos[i]= X;
  if (RW) *RW= W;
  if (RA) *RA= 17;
  if (RH) *RH= 21;
  if (Rdata) *Rdata= data;

  if (!e || !Rdata) return ;

  j= 0;
  for(i=0;i<L;i++)
  {
    if (str[i]&(1<<31))
    {
      sX= Xpos[j]; eX= Xpos[j+2];
      for(x=sX;x<eX;x++)
        for(y=0;y<21;y++)
          data[y*W + x] ^= 0xff;
      j+= 2;
      continue;
    }
    if (map[i]==107)
    {
      sprintf(buf,"%x",str[i]);
      sX= Xpos[j]; eX= Xpos[j+strlen(buf)];
      for(x=sX;x<eX;x++) {
        data[ 0*W + x]= 0xff;
        data[ (21-1)*W + x]= 0xff;
      }
      for(y=0;y<21;y++)
      {
        data[y*W+sX] = 0xff;
        data[y*W+eX-1] = 0xff;
      }
      j+= strlen(buf);
      continue;
    }
    j++;
  }
  return ;


fix_encoding_errors:
  j= 0;
  gl= alloca(sizeof(int)*ngl);
  for(i=0;i<L;i++)
  {
    if (str[i]&(1<<31))
    {
      sprintf(buf,"%02x", str[i]&0xff);
      for(k=0;buf[k];k++)
         gl[j++]= font_hash(buf[k]);
      continue;
    }
    if (map[i]==107)
    {
      sprintf(buf,"%x", str[i]);
      for(k=0;buf[k];k++)
         gl[j++]= font_hash(buf[k]);
      continue;
    }
    gl[j++]= map[i];
  }
  goto encoding_errors_fixed;
}

static 
void font3_render
        (uint32_t *str, size_t L, 
        int *RW, int *RH, int *RA,
        unsigned char **Rdata)
{
  static const uint8_t font_data[]= {

0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0xb6,0xf2,0x7c,0x00,0x00,0x49,0xff,0xff,
0xfd,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x22,0xcc,0xf1,0xa9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4b,0xe4,0xe4,
0x49,0x00,0x00,0xe5,0xff,0xff,0xe4,0x00,0x00,0xe3,0xff,0xff,0xe3,0x00,0x00,0x47,0xe3,0xe4,
0x49,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x97,0xf5,0xcb,0x1e,0x16,0xc7,0xf2,0xa4,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4b,0x4b,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x4c,0xff,0xff,0xff,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xc2,0xf5,0xd5,0x29,0x22,0xd1,0xf4,0xba,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x30,0x30,0x30,0x30,0xff,
0xff,0xff,0x6d,0x30,0x30,0xff,0xff,0xff,0x6d,0x30,0x30,0x30,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x30,0x30,0x30,0x30,0xff,0xff,0xff,0x6d,0x30,0x30,0xff,0xff,0xff,0x6d,
0x30,0x30,0x30,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x28,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xff,0xff,0x3f,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x04,0x78,0xe4,0xff,0xff,0xff,0xff,0xee,0x8f,0x17,0x00,0x00,0x00,0x06,0xb8,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0x2d,0x00,0x00,0x7a,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xe0,0x10,0x00,0xda,0xff,0xff,0xd7,0x32,0xff,0xff,0x61,0xbf,0xff,0xd8,
0x85,0x1c,0x00,0xfa,0xff,0xff,0x5c,0x00,0xff,0xff,0x28,0x07,0x2b,0x00,0x00,0x00,0x00,0xec,
0xff,0xff,0x72,0x00,0xff,0xff,0x28,0x00,0x00,0x00,0x00,0x00,0x00,0xb3,0xff,0xff,0xf0,0x5e,
0xff,0xff,0x28,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0xfe,0xff,0xff,0xff,0xff,0xff,0x45,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x73,0xfe,0xff,0xff,0xff,0xff,0xff,0xce,0x57,0x01,0x00,0x00,
0x00,0x00,0x00,0x3b,0xbb,0xfd,0xff,0xff,0xff,0xff,0xff,0xc9,0x22,0x00,0x00,0x00,0x00,0x00,
0x00,0x15,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0x54,0xac,0xff,0xff,0xff,0x8a,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0x28,0x00,0xac,0xff,
0xff,0xdc,0x00,0x18,0x74,0xb2,0x03,0x00,0xff,0xff,0x28,0x00,0x56,0xff,0xff,0xf9,0x1e,0xfb,
0xff,0xff,0x6b,0x00,0xff,0xff,0x28,0x00,0x86,0xff,0xff,0xea,0x00,0xbe,0xff,0xff,0xf6,0x64,
0xff,0xff,0x4a,0x80,0xfa,0xff,0xff,0xab,0x00,0x2d,0xf7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xfb,0x33,0x00,0x00,0x60,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0x5f,0x00,
0x00,0x00,0x00,0x47,0xd6,0xff,0xff,0xff,0xff,0xf8,0xaf,0x31,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x26,0xff,0xff,0x4f,0x09,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0x28,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1d,0x26,0x04,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0xd5,0xff,0xff,
0xed,0x6b,0x00,0x00,0x00,0x00,0x4a,0x64,0x27,0x00,0x00,0x00,0x00,0x00,0x35,0xf8,0xff,0xff,
0xff,0xff,0xff,0x73,0x00,0x00,0x20,0xf9,0xf4,0x16,0x00,0x00,0x00,0x00,0x00,0xba,0xff,0xff,
0xff,0xff,0xff,0xff,0xf5,0x0e,0x00,0x9f,0xff,0x87,0x00,0x00,0x00,0x00,0x00,0x00,0xf5,0xff,
0xff,0x7c,0x31,0xff,0xff,0xff,0x40,0x27,0xfc,0xf1,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0xf1,
0xff,0xff,0x9f,0x62,0xff,0xff,0xff,0x3c,0xa9,0xff,0x81,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xab,0xff,0xff,0xff,0xff,0xff,0xff,0xed,0x38,0xfe,0xee,0x0f,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x22,0xea,0xff,0xff,0xff,0xff,0xfd,0x57,0xb3,0xff,0x7b,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x20,0xa7,0xee,0xf7,0xc3,0x44,0x37,0xff,0xeb,0x0c,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xbc,0xff,0x74,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x41,0xff,0xe7,0x09,0x51,0xb0,0xc4,0x93,
0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xc6,0xff,0x6e,0x81,0xff,0xff,0xff,
0xff,0xeb,0x29,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4b,0xff,0xe3,0x3c,0xfe,0xff,0xff,
0xff,0xff,0xff,0xc9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xce,0xff,0x68,0x8f,0xff,0xff,
0x92,0x49,0xff,0xff,0xff,0x26,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0xff,0xdf,0x05,0xae,0xff,
0xff,0x52,0x07,0xff,0xff,0xff,0x46,0x00,0x00,0x00,0x00,0x00,0x02,0xd6,0xff,0x61,0x00,0x9d,
0xff,0xff,0xa7,0x68,0xff,0xff,0xff,0x33,0x00,0x00,0x00,0x00,0x00,0x5e,0xff,0xda,0x03,0x00,
0x52,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0x03,0x00,0x00,0x00,0x00,0x05,0xde,0xff,0x5b,0x00,
0x00,0x01,0xb5,0xff,0xff,0xff,0xff,0xfc,0x4d,0x00,0x00,0x00,0x00,0x00,0x06,0x24,0x24,0x02,
0x00,0x00,0x00,0x07,0x89,0xe6,0xf9,0xc8,0x44,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x2f,0x68,0x79,0x60,0x29,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2b,0xc6,
0xff,0xff,0xff,0xff,0xfe,0xad,0x0f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0xf1,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xd0,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0xcd,0xff,0xff,0xff,0xd8,
0xba,0xee,0xff,0xff,0xff,0x69,0x00,0x00,0x00,0x00,0x00,0x26,0xff,0xff,0xff,0x7b,0x00,0x00,
0x0d,0xb6,0xb8,0x43,0x00,0x00,0x00,0x00,0x00,0x00,0x44,0xff,0xff,0xff,0x0e,0x00,0x00,0x00,
0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x29,0xff,0xff,0xff,0x37,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xd3,0xff,0xff,0xe1,0x63,0x35,0x30,0x30,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x34,0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5f,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x6d,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x2f,0xfb,0xff,0xff,0xc9,0x39,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x94,0xff,0xff,0xe9,0x10,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0x00,0x00,0xd6,0xff,0xff,0x7f,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,
0x00,0xf0,0xff,0xff,0x57,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
0xf3,0xff,0xff,0x75,0x00,0x00,0x00,0x30,0x30,0x7e,0xff,0xff,0xf2,0x30,0x30,0x00,0x00,0xcc,
0xff,0xff,0xe2,0x0f,0x00,0x00,0x00,0x00,0xb6,0xff,0xff,0xc7,0x00,0x00,0x00,0x00,0x6a,0xff,
0xff,0xff,0xde,0x63,0x3a,0x50,0xb4,0xff,0xff,0xff,0x6a,0x00,0x00,0x00,0x00,0x07,0xc4,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc0,0x05,0x00,0x00,0x00,0x00,0x00,0x12,0xb7,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xc9,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4f,0xac,
0xe2,0xf9,0xed,0xbc,0x61,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x97,0xf5,0xcb,0x1e,0x00,0x00,
0x00,0xff,0xff,0xff,0x4b,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xc2,0xf5,0xd5,0x29,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x15,0xed,0xff,0xff,0x7e,0x00,0x00,
0x00,0x00,0xa0,0xff,0xff,0xe1,0x09,0x00,0x00,0x00,0x32,0xff,0xff,0xff,0x5a,0x00,0x00,0x00,
0x00,0x9a,0xff,0xff,0xe7,0x04,0x00,0x00,0x00,0x0b,0xf2,0xff,0xff,0x88,0x00,0x00,0x00,0x00,
0x4a,0xff,0xff,0xff,0x2b,0x00,0x00,0x00,0x00,0x82,0xff,0xff,0xec,0x00,0x00,0x00,0x00,0x00,
0xb8,0xff,0xff,0xb3,0x00,0x00,0x00,0x00,0x00,0xd7,0xff,0xff,0x81,0x00,0x00,0x00,0x00,0x00,
0xe7,0xff,0xff,0x6b,0x00,0x00,0x00,0x00,0x00,0xf8,0xff,0xff,0x58,0x00,0x00,0x00,0x00,0x00,
0xf9,0xff,0xff,0x53,0x00,0x00,0x00,0x00,0x00,0xea,0xff,0xff,0x68,0x00,0x00,0x00,0x00,0x00,
0xdb,0xff,0xff,0x7d,0x00,0x00,0x00,0x00,0x00,0xbd,0xff,0xff,0xab,0x00,0x00,0x00,0x00,0x00,
0x8c,0xff,0xff,0xe8,0x00,0x00,0x00,0x00,0x00,0x5b,0xff,0xff,0xff,0x28,0x00,0x00,0x00,0x00,
0x17,0xfd,0xff,0xff,0x86,0x00,0x00,0x00,0x00,0x00,0xb7,0xff,0xff,0xe7,0x04,0x00,0x00,0x00,
0x00,0x42,0xff,0xff,0xff,0x5a,0x00,0x00,0x00,0x00,0x00,0xb4,0xff,0xff,0xe0,0x09,0x00,0x00,
0x00,0x00,0x1d,0xef,0xff,0xff,0x7e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x32,0xfe,0xff,0xff,0x45,0x00,0x00,0x00,0x00,
0x00,0xa0,0xff,0xff,0xda,0x07,0x00,0x00,0x00,0x00,0x1a,0xf8,0xff,0xff,0x75,0x00,0x00,0x00,
0x00,0x00,0xa8,0xff,0xff,0xdd,0x01,0x00,0x00,0x00,0x00,0x46,0xff,0xff,0xff,0x45,0x00,0x00,
0x00,0x00,0x02,0xe6,0xff,0xff,0x92,0x00,0x00,0x00,0x00,0x00,0xa9,0xff,0xff,0xca,0x00,0x00,
0x00,0x00,0x00,0x6c,0xff,0xff,0xf9,0x07,0x00,0x00,0x00,0x00,0x38,0xff,0xff,0xff,0x20,0x00,
0x00,0x00,0x00,0x21,0xff,0xff,0xff,0x31,0x00,0x00,0x00,0x00,0x0a,0xff,0xff,0xff,0x43,0x00,
0x00,0x00,0x00,0x0b,0xff,0xff,0xff,0x43,0x00,0x00,0x00,0x00,0x20,0xff,0xff,0xff,0x33,0x00,
0x00,0x00,0x00,0x36,0xff,0xff,0xff,0x23,0x00,0x00,0x00,0x00,0x68,0xff,0xff,0xfa,0x07,0x00,
0x00,0x00,0x00,0xa5,0xff,0xff,0xce,0x00,0x00,0x00,0x00,0x01,0xe3,0xff,0xff,0x9a,0x00,0x00,
0x00,0x00,0x41,0xff,0xff,0xff,0x51,0x00,0x00,0x00,0x00,0xa4,0xff,0xff,0xeb,0x08,0x00,0x00,
0x00,0x18,0xf6,0xff,0xff,0x80,0x00,0x00,0x00,0x00,0x9e,0xff,0xff,0xe6,0x0c,0x00,0x00,0x00,
0x31,0xfd,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x49,0x6c,0x04,
0x00,0x02,0x6a,0x52,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x68,0xff,0xff,0x66,0x00,0x5f,0xff,
0xff,0x84,0x00,0x00,0x00,0x00,0x00,0x00,0x62,0xff,0xff,0xe5,0x0f,0xde,0xff,0xff,0x6c,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xb2,0xff,0xff,0xcd,0xff,0xff,0xbb,0x01,0x00,0x00,0x00,0x00,
0x10,0x30,0x30,0x45,0xf6,0xff,0xff,0xff,0xfa,0x4c,0x30,0x30,0x0c,0x00,0x00,0xc6,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbf,0x00,0x00,0xf6,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0x00,0x00,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xad,0x00,0x00,0x00,0x00,0x00,0x32,0xfb,0xff,0xff,0xff,0xf7,0x28,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x06,0xd2,0xff,0xff,0xb4,0xff,0xff,0xc8,0x03,0x00,0x00,0x00,0x00,
0x00,0x00,0x7d,0xff,0xff,0xd5,0x07,0xdf,0xff,0xff,0x6e,0x00,0x00,0x00,0x00,0x00,0x00,0x6f,
0xff,0xff,0x5a,0x00,0x63,0xff,0xff,0x63,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x48,0x6c,0x02,
0x00,0x03,0x6b,0x48,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x30,
0x6d,0xff,0xff,0xff,0x30,0x30,0x30,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x3b,0xe1,0xe1,0x40,0x00,0x00,0xd7,0xff,0xff,0xd8,0x00,0x00,0xf5,0xff,0xff,
0xf8,0x00,0x00,0xad,0xff,0xff,0xcc,0x00,0x00,0x0c,0xb4,0xff,0x6a,0x00,0x00,0x00,0xc5,0xda,
0x0a,0x00,0x00,0x04,0xa1,0x2f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,
0x30,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x4b,0xe4,0xe4,0x49,0x00,0x00,0xe5,0xff,0xff,0xe4,0x00,0x00,0xe4,0xff,0xff,
0xe3,0x00,0x00,0x49,0xe4,0xe4,0x49,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x22,0xfe,0xff,0xff,0x68,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x8a,0xff,0xff,0xf4,0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,
0xeb,0xff,0xff,0x9c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5c,0xff,0xff,0xff,0x36,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xc5,0xff,0xff,0xd0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x2e,0xff,0xff,0xff,0x6a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x97,0xff,0xff,0xf5,
0x0e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0xf3,0xff,0xff,0x9e,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x6a,0xff,0xff,0xff,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd3,0xff,
0xff,0xd1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3c,0xff,0xff,0xff,0x6b,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xa5,0xff,0xff,0xf5,0x0f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x15,
0xf9,0xff,0xff,0x9f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x77,0xff,0xff,0xff,0x39,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xde,0xff,0xff,0xd3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x4a,0xff,0xff,0xff,0x6d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb3,0xff,0xff,0xf6,
0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1f,0xfd,0xff,0xff,0xa1,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x85,0xff,0xff,0xff,0x3b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xe8,0xff,
0xff,0xd4,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x57,0xff,0xff,0xff,0x6f,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x22,0x93,0xdc,0xf7,
0xea,0xb9,0x51,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4d,0xf3,0xff,0xff,0xff,0xff,0xff,
0xff,0x9e,0x01,0x00,0x00,0x00,0x00,0x00,0x2f,0xf6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0x7e,0x00,0x00,0x00,0x00,0x00,0xbf,0xff,0xff,0xe6,0x67,0x38,0x49,0xb0,0xff,0xff,0xf7,0x1c,
0x00,0x00,0x00,0x2f,0xff,0xff,0xfc,0x2b,0x00,0x00,0x00,0x01,0xc5,0xff,0xff,0x80,0x00,0x00,
0x00,0x81,0xff,0xff,0xbb,0x00,0x00,0x00,0x00,0x00,0x65,0xff,0xff,0xd1,0x00,0x00,0x00,0xb6,
0xff,0xff,0x83,0x00,0x00,0x00,0x00,0x00,0x32,0xff,0xff,0xfc,0x07,0x00,0x00,0xdf,0xff,0xff,
0x64,0x00,0x00,0x00,0x00,0x00,0x17,0xff,0xff,0xff,0x2b,0x00,0x00,0xf1,0xff,0xff,0x53,0x00,
0x00,0x00,0x00,0x00,0x08,0xff,0xff,0xff,0x3c,0x00,0x00,0xfc,0xff,0xff,0x4d,0x00,0x00,0x00,
0x00,0x00,0x02,0xff,0xff,0xff,0x47,0x00,0x00,0xf2,0xff,0xff,0x4f,0x00,0x00,0x00,0x00,0x00,
0x05,0xff,0xff,0xff,0x3d,0x00,0x00,0xe1,0xff,0xff,0x58,0x00,0x00,0x00,0x00,0x00,0x0e,0xff,
0xff,0xff,0x2b,0x00,0x00,0xcb,0xff,0xff,0x70,0x00,0x00,0x00,0x00,0x00,0x23,0xff,0xff,0xff,
0x15,0x00,0x00,0x97,0xff,0xff,0x98,0x00,0x00,0x00,0x00,0x00,0x48,0xff,0xff,0xdf,0x00,0x00,
0x00,0x5c,0xff,0xff,0xce,0x00,0x00,0x00,0x00,0x00,0x88,0xff,0xff,0xa3,0x00,0x00,0x00,0x12,
0xf4,0xff,0xff,0x40,0x00,0x00,0x00,0x10,0xea,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x91,0xff,
0xff,0xf4,0x6d,0x37,0x54,0xd0,0xff,0xff,0xd5,0x01,0x00,0x00,0x00,0x00,0x15,0xdf,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xfa,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x2b,0xea,0xff,0xff,0xff,
0xff,0xff,0xfc,0x61,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x91,0xdf,0xfa,0xeb,0xac,
0x35,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x7f,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x8e,
0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x9c,0xff,0xff,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2a,0x70,0x70,0x70,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x46,0xb2,0xec,0xf7,0xdf,0xa8,0x3c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x06,0xa0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8d,0x00,0x00,0x00,0x00,0x00,0x00,0xa5,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x62,0x00,0x00,0x00,0x00,0x3f,0xff,0xff,0xff,0xda,
0x5b,0x3b,0x73,0xf3,0xff,0xff,0xdc,0x00,0x00,0x00,0x00,0x60,0xec,0xff,0xf6,0x1e,0x00,0x00,
0x00,0x60,0xff,0xff,0xff,0x22,0x00,0x00,0x00,0x00,0x03,0x3c,0x58,0x00,0x00,0x00,0x00,0x14,
0xff,0xff,0xff,0x41,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0e,0xff,0xff,
0xff,0x42,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0xff,0xff,0xff,0x29,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1b,0xe9,0xff,0xff,0xe6,0x01,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x27,0xdf,0xff,0xff,0xff,0x6e,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x5a,0xf3,0xff,0xff,0xff,0xb1,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x08,0x9d,0xff,0xff,0xff,0xff,0xb8,0x0a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x13,0xc5,0xff,
0xff,0xff,0xff,0x94,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0xcb,0xff,0xff,0xff,0xf5,
0x5b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x99,0xff,0xff,0xff,0xe9,0x33,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2d,0xfe,0xff,0xff,0xeb,0x24,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x88,0xff,0xff,0xff,0x83,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,
0x00,0x00,0x00,0xc5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
0x00,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xf8,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x55,0xbb,0xe4,0xfa,0xe8,0xb4,0x5a,0x02,0x00,0x00,
0x00,0x00,0x00,0x00,0x05,0xab,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc5,0x12,0x00,0x00,0x00,
0x00,0x00,0x89,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa7,0x00,0x00,0x00,0x00,0x0a,
0xda,0xff,0xff,0xe0,0x5d,0x39,0x6c,0xea,0xff,0xff,0xfc,0x16,0x00,0x00,0x00,0x00,0x00,0x20,
0x5c,0x22,0x00,0x00,0x00,0x47,0xff,0xff,0xff,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x0d,0xff,0xff,0xff,0x3c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x55,0xff,0xff,0xf4,0x0c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x30,0x36,0x74,0xf3,
0xff,0xff,0x6d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xff,0xff,0xff,0xff,0xff,0xbc,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xff,0xff,0xff,0xff,0xff,0xff,0x92,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x59,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x21,0x8d,0xfd,0xff,0xff,0xd7,0x01,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x82,0xff,0xff,0xff,0x2a,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x19,0xff,0xff,0xff,0x46,0x00,0x00,0x00,0x0e,0x4f,0x4d,0x00,
0x00,0x00,0x00,0x00,0x1c,0xff,0xff,0xff,0x39,0x00,0x00,0xa9,0xfb,0xff,0xf0,0x26,0x00,0x00,
0x00,0x00,0x87,0xff,0xff,0xf7,0x0c,0x00,0x00,0x6f,0xff,0xff,0xff,0xec,0x78,0x3c,0x4b,0xa3,
0xff,0xff,0xff,0x9b,0x00,0x00,0x00,0x03,0xbe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xe1,0x16,0x00,0x00,0x00,0x00,0x0f,0xb8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd5,0x25,0x00,
0x00,0x00,0x00,0x00,0x00,0x01,0x52,0xb2,0xe7,0xfa,0xed,0xbd,0x65,0x05,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xd7,0xff,
0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x76,0xff,0xff,0xff,0xff,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1b,0xf3,0xff,0xff,0xff,0xff,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xa8,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x41,0xff,0xff,0xfd,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x04,0xd5,0xff,0xff,0x98,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x73,0xff,0xff,0xb2,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x19,0xf2,0xff,
0xf6,0x21,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xa5,0xff,0xff,0x7e,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0xff,0xff,0xdc,0x07,0x00,0x4c,0xff,
0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x04,0xd3,0xff,0xff,0x4a,0x00,0x00,0x4c,0xff,0xff,0xff,
0x00,0x00,0x00,0x00,0x00,0x71,0xff,0xff,0xb0,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,
0x00,0x00,0x00,0xf1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x30,0x30,0x30,
0x30,0x30,0x30,0x30,0x6d,0xff,0xff,0xff,0x30,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,
0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xac,0x00,0x00,0x00,0x00,0x00,0x00,0x7e,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xac,0x00,0x00,0x00,0x00,0x00,0x00,0xa8,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xac,0x00,0x00,0x00,0x00,0x00,0x00,0xd1,0xff,0xff,0x98,0x30,0x30,0x30,0x30,0x30,
0x20,0x00,0x00,0x00,0x00,0x00,0x03,0xf7,0xff,0xff,0x56,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x24,0xff,0xff,0xff,0x27,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x4d,0xff,0xff,0xf5,0x14,0x2a,0x19,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x77,0xff,0xff,0xfc,0xfd,0xff,0xff,0xf2,0x9b,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0xa0,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0x48,0x00,0x00,0x00,0x00,0x00,0x8a,0xf8,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xf1,0x24,0x00,0x00,0x00,0x00,0x00,0x10,0x66,0xae,0x44,0x0d,
0x19,0x86,0xff,0xff,0xff,0xa5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x9c,0xff,0xff,0xf8,0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2c,0xff,
0xff,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0b,0xff,0xff,0xff,
0x43,0x00,0x00,0x3e,0x76,0xa3,0x9e,0x00,0x00,0x00,0x00,0x00,0x2e,0xff,0xff,0xff,0x2a,0x00,
0x00,0x9d,0xff,0xff,0xff,0x50,0x00,0x00,0x00,0x00,0xa9,0xff,0xff,0xef,0x07,0x00,0x00,0x30,
0xfd,0xff,0xff,0xf8,0x83,0x3e,0x50,0xb4,0xff,0xff,0xff,0x91,0x00,0x00,0x00,0x00,0x7a,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xdf,0x11,0x00,0x00,0x00,0x00,0x00,0x78,0xfd,0xff,
0xff,0xff,0xff,0xff,0xff,0xd6,0x29,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2f,0x9c,0xdf,0xf9,
0xea,0xc4,0x6b,0x0a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xa6,0xff,0xff,0xfc,0x2b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x26,0xfd,0xff,0xff,0x99,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xa0,
0xff,0xff,0xf3,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xfb,0xff,0xff,
0x7c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x9a,0xff,0xff,0xe3,0x09,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1d,0xf9,0xff,0xff,0x5e,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x94,0xff,0xff,0xcd,0x01,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x18,0xf7,0xff,0xff,0xfd,0xf8,0xdd,0x97,0x20,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0x4b,0x00,0x00,0x00,0x00,0x00,0x13,
0xf5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0x27,0x00,0x00,0x00,0x00,0x6c,0xff,0xff,
0xff,0xc4,0x49,0x46,0xa6,0xff,0xff,0xff,0xaa,0x00,0x00,0x00,0x00,0xb3,0xff,0xff,0xe2,0x0a,
0x00,0x00,0x00,0xa6,0xff,0xff,0xf9,0x09,0x00,0x00,0x00,0xe3,0xff,0xff,0x78,0x00,0x00,0x00,
0x00,0x2f,0xff,0xff,0xff,0x34,0x00,0x00,0x00,0xfa,0xff,0xff,0x55,0x00,0x00,0x00,0x00,0x0a,
0xff,0xff,0xff,0x46,0x00,0x00,0x00,0xeb,0xff,0xff,0x75,0x00,0x00,0x00,0x00,0x29,0xff,0xff,
0xff,0x34,0x00,0x00,0x00,0xbe,0xff,0xff,0xdd,0x08,0x00,0x00,0x00,0x9e,0xff,0xff,0xf2,0x06,
0x00,0x00,0x00,0x5f,0xff,0xff,0xff,0xcc,0x51,0x44,0xa1,0xff,0xff,0xff,0x90,0x00,0x00,0x00,
0x00,0x02,0xc0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0x11,0x00,0x00,0x00,0x00,0x00,
0x14,0xc0,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0x27,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,
0x5f,0xc2,0xec,0xf5,0xcf,0x78,0x0a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb5,0x00,
0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x39,0x00,0x00,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0x00,0x00,0x00,0x30,0x30,0x30,
0x30,0x30,0x30,0x30,0x61,0xff,0xff,0xff,0xff,0x41,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xab,0xff,0xff,0xff,0xc5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x27,
0xfd,0xff,0xff,0xff,0x48,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x9e,0xff,0xff,
0xff,0xcb,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1d,0xfa,0xff,0xff,0xff,0x50,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x91,0xff,0xff,0xff,0xd2,0x01,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xf5,0xff,0xff,0xff,0x58,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x83,0xff,0xff,0xff,0xd8,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x0d,0xef,0xff,0xff,0xff,0x5f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x76,0xff,0xff,0xff,0xde,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0xe7,0xff,
0xff,0xff,0x67,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x69,0xff,0xff,0xff,0xe4,
0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0xde,0xff,0xff,0xff,0x6e,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5b,0xff,0xff,0xff,0xe8,0x09,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x01,0xd3,0xff,0xff,0xff,0x76,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x4e,0xff,0xff,0xff,0xed,0x0c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xc7,0xff,0xff,0xff,0x7e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x34,0x9f,0xdd,0xf7,0xf6,0xd9,0x97,0x28,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7d,0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0x68,0x00,
0x00,0x00,0x00,0x00,0x56,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x49,0x00,0x00,
0x00,0x00,0xc7,0xff,0xff,0xf6,0x7b,0x3e,0x3f,0x7d,0xf7,0xff,0xff,0xc2,0x00,0x00,0x00,0x00,
0xf5,0xff,0xff,0x77,0x00,0x00,0x00,0x00,0x78,0xff,0xff,0xf4,0x00,0x00,0x00,0x00,0xe8,0xff,
0xff,0x62,0x00,0x00,0x00,0x00,0x63,0xff,0xff,0xee,0x00,0x00,0x00,0x00,0xa8,0xff,0xff,0xdf,
0x44,0x0d,0x0c,0x47,0xe2,0xff,0xff,0xaf,0x00,0x00,0x00,0x00,0x24,0xee,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xef,0x27,0x00,0x00,0x00,0x00,0x00,0x5d,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0x5a,0x00,0x00,0x00,0x00,0x00,0x52,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xfb,0x54,0x00,0x00,0x00,0x18,0xf3,0xff,0xff,0xea,0x75,0x43,0x46,0x87,0xf7,0xff,0xff,0xf4,
0x1b,0x00,0x00,0x89,0xff,0xff,0xeb,0x18,0x00,0x00,0x00,0x00,0x2b,0xef,0xff,0xff,0x8d,0x00,
0x00,0xd5,0xff,0xff,0x7d,0x00,0x00,0x00,0x00,0x00,0x00,0x7d,0xff,0xff,0xd5,0x00,0x00,0xf6,
0xff,0xff,0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x52,0xff,0xff,0xf6,0x00,0x00,0xee,0xff,0xff,
0x74,0x00,0x00,0x00,0x00,0x00,0x00,0x75,0xff,0xff,0xeb,0x00,0x00,0xbe,0xff,0xff,0xe4,0x19,
0x00,0x00,0x00,0x00,0x1a,0xe6,0xff,0xff,0xbc,0x00,0x00,0x62,0xff,0xff,0xff,0xed,0x78,0x43,
0x43,0x79,0xee,0xff,0xff,0xff,0x5f,0x00,0x00,0x03,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xc1,0x02,0x00,0x00,0x00,0x11,0xb9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xb2,0x0e,0x00,0x00,0x00,0x00,0x00,0x01,0x4e,0xab,0xe2,0xf8,0xf6,0xdb,0xa2,0x42,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x5a,0xbb,0xed,0xf4,
0xd2,0x7c,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x13,0xc0,0xff,0xff,0xff,0xff,0xff,0xff,
0xe3,0x34,0x00,0x00,0x00,0x00,0x00,0x03,0xc2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xec,
0x1b,0x00,0x00,0x00,0x00,0x61,0xff,0xff,0xff,0xc3,0x50,0x41,0x92,0xfe,0xff,0xff,0xa6,0x00,
0x00,0x00,0x00,0xc3,0xff,0xff,0xcc,0x03,0x00,0x00,0x00,0x80,0xff,0xff,0xf8,0x0f,0x00,0x00,
0x00,0xf3,0xff,0xff,0x65,0x00,0x00,0x00,0x00,0x1b,0xff,0xff,0xff,0x36,0x00,0x00,0x00,0xf6,
0xff,0xff,0x61,0x00,0x00,0x00,0x00,0x13,0xff,0xff,0xff,0x46,0x00,0x00,0x00,0xd6,0xff,0xff,
0xb5,0x00,0x00,0x00,0x00,0x67,0xff,0xff,0xff,0x2f,0x00,0x00,0x00,0x8c,0xff,0xff,0xff,0x98,
0x21,0x10,0x5f,0xf3,0xff,0xff,0xf7,0x06,0x00,0x00,0x00,0x1c,0xf2,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xb6,0x00,0x00,0x00,0x00,0x00,0x4c,0xf7,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0x51,0x00,0x00,0x00,0x00,0x00,0x00,0x2f,0xb2,0xf8,0xff,0xff,0xff,0xff,0xff,0xd5,
0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x09,0x25,0x8b,0xff,0xff,0xff,0x58,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0xe1,0xff,0xff,0xd9,0x02,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x67,0xff,0xff,0xff,0x5f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x05,0xe1,0xff,0xff,0xde,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x66,0xff,0xff,0xff,0x67,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xe0,
0xff,0xff,0xe3,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x65,0xff,0xff,0xff,
0x6e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xdf,0xff,0xff,0xe8,0x09,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x49,
0xe3,0xe3,0x47,0x00,0x00,0xe4,0xff,0xff,0xe2,0x00,0x00,0xe4,0xff,0xff,0xe4,0x00,0x00,0x4b,
0xe4,0xe4,0x4a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x4b,0xe4,0xe4,0x49,0x00,0x00,0xe5,0xff,0xff,0xe4,0x00,0x00,0xe4,
0xff,0xff,0xe3,0x00,0x00,0x49,0xe4,0xe4,0x49,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x49,0xe3,0xe3,0x47,0x00,0x00,0xe4,0xff,0xff,0xe2,0x00,0x00,0xe4,
0xff,0xff,0xe4,0x00,0x00,0x4b,0xe4,0xe4,0x4a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x03,0x61,0x61,0x04,0x00,0x00,0x99,0xff,0xff,0x9e,0x00,0x00,0xf3,
0xff,0xff,0xf4,0x00,0x00,0xde,0xff,0xff,0xe8,0x00,0x00,0x4f,0xee,0xff,0xa0,0x00,0x00,0x00,
0x99,0xfd,0x2e,0x00,0x00,0x03,0xe8,0x85,0x00,0x00,0x00,0x00,0x26,0x02,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3a,0xfd,0xff,0xff,0xfa,0x2d,0x00,
0x00,0x00,0x00,0x08,0xd7,0xff,0xff,0xff,0x80,0x00,0x00,0x00,0x00,0x00,0x87,0xff,0xff,0xff,
0xd3,0x06,0x00,0x00,0x00,0x00,0x32,0xfb,0xff,0xff,0xfc,0x35,0x00,0x00,0x00,0x00,0x05,0xd1,
0xff,0xff,0xff,0x8b,0x00,0x00,0x00,0x00,0x00,0x7e,0xff,0xff,0xff,0xda,0x09,0x00,0x00,0x00,
0x00,0x2c,0xf9,0xff,0xff,0xfe,0x3e,0x00,0x00,0x00,0x00,0x03,0xca,0xff,0xff,0xff,0x95,0x00,
0x00,0x00,0x00,0x00,0x3d,0xff,0xff,0xff,0xff,0x1e,0x00,0x00,0x00,0x00,0x00,0x00,0xb7,0xff,
0xff,0xff,0x9d,0x00,0x00,0x00,0x00,0x00,0x00,0x1e,0xf3,0xff,0xff,0xff,0x43,0x00,0x00,0x00,
0x00,0x00,0x00,0x6e,0xff,0xff,0xff,0xdd,0x0a,0x00,0x00,0x00,0x00,0x00,0x02,0xc7,0xff,0xff,
0xff,0x8c,0x00,0x00,0x00,0x00,0x00,0x00,0x2a,0xf9,0xff,0xff,0xfc,0x34,0x00,0x00,0x00,0x00,
0x00,0x00,0x80,0xff,0xff,0xff,0xd1,0x05,0x00,0x00,0x00,0x00,0x00,0x06,0xd5,0xff,0xff,0xff,
0x7b,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0xfd,0xff,0xff,0xf8,0x28,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x30,0x30,
0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xe4,0xff,0xff,0xff,0x83,
0x00,0x00,0x00,0x00,0x00,0x00,0x4b,0xff,0xff,0xff,0xfa,0x2e,0x00,0x00,0x00,0x00,0x00,0x00,
0xa2,0xff,0xff,0xff,0xcc,0x03,0x00,0x00,0x00,0x00,0x00,0x12,0xe7,0xff,0xff,0xff,0x77,0x00,
0x00,0x00,0x00,0x00,0x00,0x51,0xff,0xff,0xff,0xf7,0x26,0x00,0x00,0x00,0x00,0x00,0x00,0xa8,
0xff,0xff,0xff,0xc2,0x01,0x00,0x00,0x00,0x00,0x00,0x15,0xea,0xff,0xff,0xff,0x6b,0x00,0x00,
0x00,0x00,0x00,0x00,0x56,0xff,0xff,0xff,0xf2,0x1e,0x00,0x00,0x00,0x00,0x00,0x01,0xe1,0xff,
0xff,0xff,0x78,0x00,0x00,0x00,0x00,0x00,0x69,0xff,0xff,0xff,0xe1,0x0b,0x00,0x00,0x00,0x00,
0x1d,0xf2,0xff,0xff,0xff,0x4b,0x00,0x00,0x00,0x00,0x00,0xb5,0xff,0xff,0xff,0xaa,0x00,0x00,
0x00,0x00,0x00,0x5c,0xff,0xff,0xff,0xef,0x18,0x00,0x00,0x00,0x00,0x16,0xec,0xff,0xff,0xff,
0x66,0x00,0x00,0x00,0x00,0x00,0xa9,0xff,0xff,0xff,0xc2,0x01,0x00,0x00,0x00,0x00,0x4e,0xff,
0xff,0xff,0xf9,0x28,0x00,0x00,0x00,0x00,0x0f,0xe5,0xff,0xff,0xff,0x80,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x0f,0x52,0x75,0x71,0x52,0x11,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x73,
0xf2,0xff,0xff,0xff,0xff,0xf7,0x9c,0x0d,0x00,0x00,0x00,0x00,0x89,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xd9,0x1a,0x00,0x00,0x39,0xfe,0xff,0xff,0xff,0xd7,0xbd,0xe7,0xff,0xff,0xff,
0xb2,0x00,0x00,0xa1,0xff,0xff,0xff,0x77,0x00,0x00,0x03,0x97,0xff,0xff,0xfe,0x17,0x00,0x9f,
0xe7,0xff,0xd6,0x00,0x00,0x00,0x00,0x11,0xff,0xff,0xff,0x41,0x00,0x00,0x00,0x10,0x22,0x00,
0x00,0x00,0x00,0x15,0xff,0xff,0xff,0x3b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7a,
0xff,0xff,0xff,0x1a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x46,0xf7,0xff,0xff,0xd3,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5b,0xf9,0xff,0xff,0xfd,0x45,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x58,0xfe,0xff,0xff,0xff,0x6b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0b,0xee,0xff,
0xff,0xfc,0x63,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3b,0xff,0xff,0xff,0x76,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x11,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x29,
0x8c,0x8c,0x8c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4b,0xe4,0xe4,0x49,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xe5,0xff,0xff,0xe4,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xe3,0xff,0xff,0xe3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x47,0xe3,0xe4,
0x49,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0b,0x48,
0x66,0x79,0x6a,0x50,0x15,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x38,0xb3,0xf8,0xff,0xff,0xff,0xff,0xff,0xfe,0xc7,0x55,0x02,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x0d,0xa7,0xff,0xff,0xff,0xee,0xc1,0xb0,0xcc,0xf3,0xff,0xff,0xff,0xcc,
0x23,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0xd6,0xff,0xff,0xd7,0x50,0x05,0x00,0x00,0x00,
0x0a,0x65,0xe1,0xff,0xff,0xf1,0x37,0x00,0x00,0x00,0x00,0x00,0x08,0xc9,0xff,0xff,0xa3,0x06,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x12,0xb7,0xff,0xff,0xeb,0x21,0x00,0x00,0x00,0x00,0x81,
0xff,0xff,0xb2,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x09,0xd1,0xff,0xff,0xb3,
0x00,0x00,0x00,0x16,0xf4,0xff,0xf1,0x13,0x00,0x00,0x2b,0x9e,0xe2,0xfa,0xee,0xbc,0x58,0x00,
0x00,0x2e,0xff,0xff,0xff,0x39,0x00,0x00,0x6b,0xff,0xff,0x86,0x00,0x00,0x63,0xf9,0xff,0xff,
0xff,0xff,0xff,0xff,0xae,0x06,0x00,0xbe,0xff,0xff,0x91,0x00,0x00,0xb9,0xff,0xff,0x30,0x00,
0x44,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x85,0x00,0x79,0xff,0xff,0xd3,0x00,0x00,
0xe0,0xff,0xf6,0x00,0x00,0xc2,0xff,0xff,0xf0,0x6d,0x39,0x64,0xec,0xff,0xff,0xe6,0x00,0x58,
0xff,0xff,0xee,0x00,0x00,0xf8,0xff,0xdf,0x00,0x00,0xf5,0xff,0xff,0x6b,0x00,0x00,0x00,0x65,
0xff,0xff,0xff,0x09,0x5c,0xff,0xff,0xf7,0x00,0x00,0xed,0xff,0xeb,0x00,0x00,0xef,0xff,0xff,
0x73,0x00,0x00,0x00,0x79,0xff,0xff,0xff,0x58,0xac,0xff,0xff,0xd4,0x00,0x00,0xd3,0xff,0xfe,
0x11,0x00,0xb2,0xff,0xff,0xf5,0x72,0x3a,0x6a,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x7e,
0x00,0x00,0x95,0xff,0xff,0x5d,0x00,0x36,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xff,
0xff,0xff,0xff,0xd9,0x0b,0x00,0x00,0x43,0xff,0xff,0xcd,0x01,0x00,0x5a,0xf8,0xff,0xff,0xff,
0xff,0xff,0xb2,0x0b,0xad,0xff,0xff,0xc2,0x1d,0x00,0x00,0x00,0x00,0xc8,0xff,0xff,0x68,0x00,
0x00,0x29,0xa0,0xe4,0xf9,0xdf,0x83,0x06,0x00,0x00,0x1a,0x1e,0x00,0x00,0x00,0x00,0x00,0x00,
0x39,0xfb,0xff,0xf9,0x47,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x6a,0xff,0xff,0xfc,0x7d,0x08,0x00,0x00,0x00,0x00,0x00,0x00,
0x36,0x8d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x67,0xf8,0xff,0xff,0xe5,0x8d,
0x4b,0x30,0x38,0x65,0xb8,0xff,0xff,0x4f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x30,0xb9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xcf,0x36,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x33,0x85,0xc9,0xe7,0xfa,0xf0,0xd6,0x9e,0x47,0x01,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,
0xf6,0xff,0xff,0xff,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x5d,0xff,0xff,0xff,0xff,0xc8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xb5,0xff,0xff,0xff,0xff,0xff,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x12,0xfa,0xff,0xff,0xff,0xff,0xff,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x65,0xff,0xff,0xff,0xf8,0xff,0xff,0xd0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xbd,0xff,0xff,0xf2,0x8a,0xff,0xff,0xff,0x29,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x18,0xfd,0xff,0xff,0xa7,0x31,0xff,0xff,0xff,0x81,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x6e,0xff,0xff,0xff,0x54,0x00,0xde,0xff,0xff,0xd8,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xc6,0xff,0xff,0xf6,0x0b,0x00,0x8d,0xff,0xff,0xff,0x31,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x1f,0xfe,0xff,0xff,0xae,0x00,0x00,0x3a,0xff,0xff,0xff,0x89,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x76,0xff,0xff,0xff,0x5c,0x00,0x00,0x01,0xe6,0xff,0xff,0xe0,
0x01,0x00,0x00,0x00,0x00,0x00,0x00,0xcf,0xff,0xff,0xfe,0x3f,0x30,0x30,0x30,0xaf,0xff,0xff,
0xff,0x3a,0x00,0x00,0x00,0x00,0x00,0x27,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0x92,0x00,0x00,0x00,0x00,0x00,0x7f,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xe7,0x02,0x00,0x00,0x00,0x00,0xd6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x42,0x00,0x00,0x00,0x2f,0xff,0xff,0xff,0xf1,0x08,0x00,0x00,0x00,
0x00,0x00,0x8e,0xff,0xff,0xff,0x9b,0x00,0x00,0x00,0x88,0xff,0xff,0xff,0x9e,0x00,0x00,0x00,
0x00,0x00,0x00,0x33,0xff,0xff,0xff,0xed,0x05,0x00,0x00,0xdf,0xff,0xff,0xff,0x42,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xd8,0xff,0xff,0xff,0x4b,0x00,0x38,0xff,0xff,0xff,0xe3,0x02,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x7d,0xff,0xff,0xff,0xa3,0x00,0x90,0xff,0xff,0xff,0x8a,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x22,0xff,0xff,0xff,0xf2,0x08,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0xff,0xff,0xfd,0xea,0xd0,0x9c,0x3a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xfe,0x93,0x01,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0x7c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x31,
0x43,0x83,0xf6,0xff,0xff,0xf1,0x06,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x5e,0xff,0xff,0xff,0x36,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,
0x0e,0xff,0xff,0xff,0x44,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x40,
0xff,0xff,0xff,0x28,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x01,0x11,0x4f,0xe2,0xff,
0xff,0xcf,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xdc,
0x28,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0x53,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x7b,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,0x30,0x3d,0x68,0xcc,0xff,0xff,0xff,0x41,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x04,0xbe,0xff,0xff,0xb8,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x5c,0xff,0xff,0xf2,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x5c,0xff,0xff,0xf2,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x04,0xbe,0xff,0xff,0xd2,0x00,0x00,0x00,0xff,
0xff,0xff,0x6d,0x30,0x30,0x30,0x3c,0x67,0xcb,0xff,0xff,0xff,0x7e,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0x0e,0x00,0x00,0x00,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xda,0x29,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xf8,0xe4,0xb4,0x63,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2f,0x96,0xda,0xf6,0xf2,0xcc,0x7d,0x15,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x8a,0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xee,0x4b,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x94,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,
0x48,0x00,0x00,0x00,0x00,0x00,0x4b,0xff,0xff,0xff,0xd9,0x64,0x38,0x56,0xca,0xff,0xff,0xff,
0xf5,0x23,0x00,0x00,0x00,0x00,0xd4,0xff,0xff,0xcd,0x0c,0x00,0x00,0x00,0x05,0xb3,0xff,0xff,
0xff,0xa7,0x00,0x00,0x00,0x3d,0xff,0xff,0xff,0x33,0x00,0x00,0x00,0x00,0x00,0x15,0xf3,0xff,
0xff,0xfa,0x10,0x00,0x00,0x8b,0xff,0xff,0xcf,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8d,0xd4,
0xd4,0xd4,0x38,0x00,0x00,0xc4,0xff,0xff,0x8e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xe8,0xff,0xff,0x64,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xfa,0xff,0xff,0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xfa,0xff,0xff,0x53,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xe9,0xff,0xff,0x64,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xc6,0xff,0xff,0x8d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x8d,0xff,0xff,0xce,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x89,0xc8,
0xc8,0xc8,0x35,0x00,0x00,0x3e,0xff,0xff,0xff,0x3d,0x00,0x00,0x00,0x00,0x00,0x15,0xf5,0xff,
0xff,0xfa,0x10,0x00,0x00,0x01,0xd6,0xff,0xff,0xcd,0x0b,0x00,0x00,0x00,0x03,0xb4,0xff,0xff,
0xff,0xa9,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0xd7,0x62,0x37,0x4f,0xc0,0xff,0xff,0xff,
0xf6,0x25,0x00,0x00,0x00,0x00,0x00,0x94,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,
0x4e,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x8b,0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0x51,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2f,0x97,0xdb,0xf6,0xf3,0xce,0x82,0x18,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0xff,0xff,0xff,0xf7,0xd7,0x91,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0x6a,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x78,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,
0x30,0x37,0x5a,0xb2,0xff,0xff,0xff,0xfe,0x3b,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x59,0xfb,0xff,0xff,0xd0,0x02,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x78,0xff,0xff,0xff,0x3c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x05,0xe6,0xff,0xff,0x8e,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,
0x00,0x00,0x94,0xff,0xff,0xc9,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,
0x00,0x62,0xff,0xff,0xe2,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x51,0xff,0xff,0xf8,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x59,
0xff,0xff,0xec,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x73,0xff,
0xff,0xd5,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xaa,0xff,0xff,
0xb7,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x0e,0xf5,0xff,0xff,0x70,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x8e,0xff,0xff,0xff,0x23,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x67,0xfe,0xff,0xff,0xb1,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0x6d,0x30,0x37,0x5b,0xb8,0xff,0xff,0xff,0xf9,0x24,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0x5b,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0x57,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0xf7,0xd6,0x8c,0x1e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x6d,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,0x30,0x30,0x30,0x30,
0x30,0x30,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,0x30,0x30,0x30,0x30,
0x30,0x30,0x30,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,0x30,0x30,
0x30,0x30,0x30,0x30,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x4a,0x6c,0x7a,0x63,0x25,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x12,0x8f,0xf6,0xff,0xff,0xff,0xff,0xff,0xbb,0x23,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x24,0xdc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xec,0x3a,0x00,
0x00,0x00,0x00,0x00,0x00,0x0d,0xd9,0xff,0xff,0xff,0xf6,0xc4,0xc8,0xfe,0xff,0xff,0xff,0xe6,
0x0e,0x00,0x00,0x00,0x00,0x00,0x8c,0xff,0xff,0xff,0xa7,0x14,0x00,0x00,0x31,0xd5,0xff,0xff,
0xd2,0x3f,0x00,0x00,0x00,0x00,0x13,0xf7,0xff,0xff,0xb8,0x01,0x00,0x00,0x00,0x00,0x1f,0x7a,
0x25,0x00,0x00,0x00,0x00,0x00,0x00,0x6c,0xff,0xff,0xfd,0x27,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb1,0xff,0xff,0xc0,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xdd,0xff,0xff,0x7e,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf6,0xff,0xff,0x59,0x00,0x00,0x00,0x00,
0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x00,0x00,0x00,0xfd,0xff,0xff,0x4f,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xf3,0xff,0xff,0x5c,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xda,0xff,0xff,0x7f,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xee,0x00,0x00,0x00,0xb1,0xff,0xff,0xba,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe3,0xff,0xff,0xd7,0x00,0x00,0x00,0x72,0xff,0xff,
0xf9,0x16,0x00,0x00,0x00,0x00,0x00,0x00,0x0e,0xfb,0xff,0xff,0xb8,0x00,0x00,0x00,0x1c,0xfd,
0xff,0xff,0x88,0x00,0x00,0x00,0x00,0x00,0x00,0x5e,0xff,0xff,0xff,0x77,0x00,0x00,0x00,0x00,
0xa9,0xff,0xff,0xfb,0x47,0x00,0x00,0x00,0x00,0x14,0xe4,0xff,0xff,0xfc,0x1c,0x00,0x00,0x00,
0x00,0x23,0xf5,0xff,0xff,0xfa,0x90,0x44,0x3a,0x70,0xe3,0xff,0xff,0xff,0x98,0x00,0x00,0x00,
0x00,0x00,0x00,0x5a,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xda,0x10,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x54,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0x1c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x12,0x7a,0xc8,0xec,0xfa,0xe6,0xb2,0x59,0x02,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,
0x30,0x30,0x30,0x30,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x4b,
0x4c,0x4c,0x17,0x00,0x00,0x00,0x00,0x00,0x01,0xff,0xff,0xff,0x4a,0x00,0x00,0xee,0xff,0xff,
0x54,0x00,0x00,0x00,0x00,0x00,0x13,0xff,0xff,0xff,0x3b,0x00,0x00,0xd8,0xff,0xff,0x70,0x00,
0x00,0x00,0x00,0x00,0x31,0xff,0xff,0xff,0x26,0x00,0x00,0xb7,0xff,0xff,0xad,0x00,0x00,0x00,
0x00,0x00,0x7b,0xff,0xff,0xf3,0x05,0x00,0x00,0x72,0xff,0xff,0xfb,0x2c,0x00,0x00,0x00,0x0b,
0xde,0xff,0xff,0xb3,0x00,0x00,0x00,0x15,0xf5,0xff,0xff,0xe8,0x67,0x37,0x52,0xcc,0xff,0xff,
0xff,0x48,0x00,0x00,0x00,0x00,0x74,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb2,0x00,
0x00,0x00,0x00,0x00,0x00,0x8c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb9,0x09,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x3d,0xa9,0xe5,0xfa,0xf0,0xc0,0x61,0x02,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x16,0xe0,0xff,0xff,0xff,
0x76,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x07,0xc5,0xff,0xff,0xff,0xa3,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0xa2,0xff,0xff,0xff,0xc9,0x08,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x79,0xff,0xff,0xff,0xe5,0x19,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x50,0xfe,0xff,0xff,0xf6,0x34,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x30,0xf4,0xff,0xff,0xff,0x59,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x18,0xe2,0xff,0xff,0xff,0x86,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x54,0xc8,0xff,0xff,0xff,0xb1,0x02,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xde,0xff,0xff,0xff,0xd5,0x0d,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0x11,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa7,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xc0,0xf1,0xff,0xff,0xff,0x58,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xd3,0x0e,0x60,0xff,0xff,0xff,0xee,0x1b,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4f,0x00,0x00,0xae,0xff,0xff,0xff,0xba,0x01,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x14,0xe8,0xff,0xff,0xff,0x6e,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0xf6,0x28,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x9a,0xff,0xff,0xff,0xcb,0x04,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x0c,0xdc,0xff,0xff,0xff,0x83,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x3a,0xfc,0xff,0xff,0xfc,
0x38,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x85,0xff,0xff,0xff,
0xdb,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,
0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xc0,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xbd,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xfd,0x1a,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x19,0xfd,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0xff,0xff,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x71,0xff,0xff,0xff,0xff,0xff,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xc9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xcb,
0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x21,0x00,0x00,
0x00,0x00,0x00,0x24,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
0xff,0xff,0x79,0x00,0x00,0x00,0x00,0x00,0x7e,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0xfe,0xff,0xff,0xd2,0x00,0x00,0x00,0x00,0x00,0xd7,0xff,0xff,0xfd,0xff,
0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xc4,0xff,0xff,0xff,0x29,0x00,0x00,0x00,0x32,
0xff,0xff,0xff,0xc3,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x72,0xff,0xff,0xff,
0x82,0x00,0x00,0x00,0x8b,0xff,0xff,0xff,0x6e,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0xd3,0xff,0xff,0xda,0x00,0x00,0x01,0xe3,0xff,0xff,0xce,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x80,0xff,0xff,0xff,0x33,0x00,0x3f,0xff,0xff,0xff,0x79,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x2d,0xff,0xff,0xff,0x8b,0x00,
0x99,0xff,0xff,0xff,0x25,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0xda,0xff,0xff,0xe2,0x06,0xed,0xff,0xff,0xd0,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x87,0xff,0xff,0xff,0x88,0xff,0xff,0xff,0x7c,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x34,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,
0x28,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0xe0,0xff,
0xff,0xff,0xff,0xff,0xd3,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x8e,0xff,0xff,0xff,0xff,0xff,0x7f,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x3b,0xff,0xff,0xff,0xff,0xff,0x2a,0x00,0x00,0x4c,
0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x01,0xe6,0xff,0xff,0xff,
0xd6,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x95,0xff,0xff,0xff,0x82,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x81,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0xf7,0x23,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xb5,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0x50,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xe0,0x0a,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0x86,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0xf6,0xff,0xff,0xf9,0x26,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x82,0xfe,0xff,0xff,0xba,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0xa0,0xff,0xff,0xff,0x56,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x19,0xf3,0xff,0xff,0xe4,0x0c,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x77,0xff,0xff,0xff,0x8b,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x06,0xdc,0xff,0xff,0xfb,0x2a,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x4e,0xff,0xff,0xff,0xc0,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0xb9,0xff,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x2a,0xfb,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x91,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x10,0xeb,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x68,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x03,0xd0,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x29,0x90,0xd6,0xf3,0xf4,0xd7,0x92,0x2b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,
0x82,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0x86,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x8c,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8f,0x00,0x00,0x00,0x00,0x00,0x44,0xff,
0xff,0xff,0xea,0x77,0x3d,0x40,0x79,0xe9,0xff,0xff,0xff,0x47,0x00,0x00,0x00,0x00,0xcd,0xff,
0xff,0xdc,0x19,0x00,0x00,0x00,0x00,0x1b,0xdc,0xff,0xff,0xd0,0x00,0x00,0x00,0x37,0xff,0xff,
0xff,0x43,0x00,0x00,0x00,0x00,0x00,0x00,0x43,0xff,0xff,0xff,0x3c,0x00,0x00,0x89,0xff,0xff,
0xd9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd5,0xff,0xff,0x8b,0x00,0x00,0xc4,0xff,0xff,
0x95,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x91,0xff,0xff,0xc4,0x00,0x00,0xe8,0xff,0xff,
0x67,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x71,0xff,0xff,0xe8,0x00,0x00,0xfa,0xff,0xff,
0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x59,0xff,0xff,0xf9,0x00,0x00,0xfa,0xff,0xff,
0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0xff,0xff,0xfa,0x00,0x00,0xe9,0xff,0xff,
0x67,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x67,0xff,0xff,0xe9,0x00,0x00,0xc6,0xff,0xff,
0x93,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x93,0xff,0xff,0xc6,0x00,0x00,0x8b,0xff,0xff,
0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd6,0xff,0xff,0x8d,0x00,0x00,0x39,0xff,0xff,
0xff,0x4e,0x00,0x00,0x00,0x00,0x00,0x00,0x4d,0xff,0xff,0xff,0x3e,0x00,0x00,0x00,0xcf,0xff,
0xff,0xdc,0x17,0x00,0x00,0x00,0x00,0x16,0xdc,0xff,0xff,0xd2,0x00,0x00,0x00,0x00,0x45,0xff,
0xff,0xff,0xe8,0x75,0x3c,0x3c,0x74,0xe7,0xff,0xff,0xff,0x48,0x00,0x00,0x00,0x00,0x00,0x8c,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x01,
0x82,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0x86,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x29,0x91,0xd7,0xf4,0xf4,0xd8,0x93,0x2b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xfc,0xec,0xce,0x9c,0x40,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xcb,0x28,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0x13,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,
0x30,0x30,0x39,0x59,0xb0,0xff,0xff,0xff,0x81,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0xb5,0xff,0xff,0xd1,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,
0x00,0x59,0xff,0xff,0xf6,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x59,
0xff,0xff,0xf6,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xb6,0xff,0xff,
0xd1,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,0x39,0x59,0xb0,0xff,0xff,0xff,0x80,0x00,
0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0x10,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc0,0x21,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xfb,0xeb,0xcc,0x97,0x37,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x0e,0x51,0x70,0x75,0x59,0x17,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x12,0x96,0xf8,0xff,0xff,0xff,0xff,0xfd,0xaa,0x1f,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x21,0xdc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0x32,0x00,0x00,0x00,
0x00,0x00,0x09,0xd3,0xff,0xff,0xff,0xef,0xc1,0xc4,0xf1,0xff,0xff,0xff,0xe3,0x13,0x00,0x00,
0x00,0x00,0x81,0xff,0xff,0xfe,0x83,0x0a,0x00,0x00,0x0d,0x95,0xff,0xff,0xff,0x96,0x00,0x00,
0x00,0x0c,0xf2,0xff,0xff,0x8f,0x00,0x00,0x00,0x00,0x00,0x00,0x9d,0xff,0xff,0xf9,0x16,0x00,
0x00,0x5f,0xff,0xff,0xf4,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xf6,0xff,0xff,0x6d,0x00,
0x00,0xa7,0xff,0xff,0xab,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xad,0xff,0xff,0xae,0x00,
0x00,0xd8,0xff,0xff,0x7b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x76,0xff,0xff,0xda,0x00,
0x00,0xf4,0xff,0xff,0x62,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5c,0xff,0xff,0xf3,0x00,
0x00,0xfc,0xff,0xff,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x50,0xff,0xff,0xfc,0x00,
0x00,0xf2,0xff,0xff,0x5b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5c,0xff,0xff,0xf1,0x00,
0x00,0xd6,0xff,0xff,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7b,0xff,0xff,0xdf,0x00,
0x00,0xa8,0xff,0xff,0xa8,0x00,0x00,0x00,0x00,0x00,0x9d,0x48,0x00,0xaa,0xff,0xff,0xb6,0x00,
0x00,0x64,0xff,0xff,0xf0,0x08,0x00,0x00,0x00,0x78,0xff,0xfd,0x80,0xed,0xff,0xff,0x81,0x00,
0x00,0x12,0xf7,0xff,0xff,0x69,0x00,0x00,0x2d,0xfc,0xff,0xff,0xff,0xff,0xff,0xff,0x34,0x00,
0x00,0x00,0x97,0xff,0xff,0xf1,0x26,0x00,0x00,0x5f,0xf1,0xff,0xff,0xff,0xff,0xcb,0x00,0x00,
0x00,0x00,0x16,0xed,0xff,0xff,0xee,0x79,0x3b,0x3f,0xa1,0xff,0xff,0xff,0xff,0xdc,0x28,0x00,
0x00,0x00,0x00,0x49,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0x05,
0x00,0x00,0x00,0x00,0x47,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb8,0xf2,0xff,0x5d,0x00,
0x00,0x00,0x00,0x00,0x00,0x0f,0x79,0xca,0xef,0xf9,0xe2,0xaa,0x49,0x00,0x33,0x91,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
0xfd,0xed,0xd1,0x9f,0x42,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xcb,0x28,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xe7,0x13,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,0x3a,0x5d,
0xb4,0xff,0xff,0xff,0x83,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x01,
0xb9,0xff,0xff,0xd2,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x5a,
0xff,0xff,0xf6,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x59,0xff,
0xff,0xef,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xb5,0xff,0xff,
0xcf,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,0x39,0x59,0xb0,0xff,0xff,0xff,0x7e,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0x14,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd2,0x25,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0x87,0x01,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x43,0xff,0xff,0xff,0xd2,0x01,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0xb2,0xff,0xff,0xff,0x59,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x26,0xfa,0xff,0xff,0xda,0x03,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x90,0xff,0xff,0xff,0x62,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x12,0xed,0xff,0xff,0xe0,0x06,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x6e,0xff,0xff,0xff,0x6c,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x05,0xd8,0xff,0xff,0xe6,0x09,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x75,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x15,0x7b,0xc6,0xef,0xfa,0xe6,0xad,0x46,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4e,0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaa,0x0a,
0x00,0x00,0x00,0x00,0x2e,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb4,0x00,0x00,
0x00,0x00,0xaa,0xff,0xff,0xfd,0x90,0x47,0x37,0x5c,0xd3,0xff,0xff,0xff,0x52,0x00,0x00,0x00,
0xea,0xff,0xff,0x85,0x00,0x00,0x00,0x00,0x10,0x9d,0x73,0x41,0x0e,0x00,0x00,0x00,0xf9,0xff,
0xff,0x5a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xda,0xff,0xff,0xd2,
0x1a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7d,0xff,0xff,0xff,0xf2,0x8d,
0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xb0,0xff,0xff,0xff,0xff,0xff,0xc3,
0x60,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x72,0xed,0xff,0xff,0xff,0xff,0xff,0xdc,
0x47,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0x69,0xd0,0xff,0xff,0xff,0xff,0xfe,0x71,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x33,0xa1,0xfc,0xff,0xff,0xfe,0x40,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x35,0xf3,0xff,0xff,0xb1,0x00,0x00,0x00,
0x00,0x12,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x77,0xff,0xff,0xe4,0x00,0x00,0x75,0xe5,0xff,
0xe4,0x04,0x00,0x00,0x00,0x00,0x00,0x5a,0xff,0xff,0xf8,0x00,0x00,0x61,0xff,0xff,0xff,0x8a,
0x00,0x00,0x00,0x00,0x00,0xaf,0xff,0xff,0xdb,0x00,0x00,0x0a,0xe7,0xff,0xff,0xff,0xb2,0x53,
0x38,0x53,0xb5,0xff,0xff,0xff,0x8d,0x00,0x00,0x00,0x48,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xe4,0x13,0x00,0x00,0x00,0x00,0x4a,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xd3,0x25,0x00,0x00,0x00,0x00,0x00,0x00,0x12,0x78,0xc6,0xf0,0xfb,0xea,0xb8,0x62,0x06,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x30,0x30,0x30,0x30,0x30,0x30,0x6d,0xff,0xff,0xff,0x30,
0x30,0x30,0x30,0x30,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xfe,0xff,0xff,0x4d,0x00,0x00,0x00,0x00,0x00,0x00,
0x4c,0xff,0xff,0xfe,0x00,0x00,0x00,0x00,0xee,0xff,0xff,0x60,0x00,0x00,0x00,0x00,0x00,0x00,
0x5b,0xff,0xff,0xed,0x00,0x00,0x00,0x00,0xd8,0xff,0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x00,
0x80,0xff,0xff,0xd7,0x00,0x00,0x00,0x00,0xa7,0xff,0xff,0xd5,0x00,0x00,0x00,0x00,0x00,0x00,
0xce,0xff,0xff,0xa5,0x00,0x00,0x00,0x00,0x5c,0xff,0xff,0xff,0x55,0x00,0x00,0x00,0x00,0x63,
0xff,0xff,0xff,0x58,0x00,0x00,0x00,0x00,0x09,0xe0,0xff,0xff,0xfa,0x87,0x3e,0x3d,0x90,0xfe,
0xff,0xff,0xdc,0x07,0x00,0x00,0x00,0x00,0x00,0x4b,0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xfc,0x47,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x52,0xf7,0xff,0xff,0xff,0xff,0xff,0xff,
0xf6,0x4f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x22,0x93,0xd9,0xf8,0xf8,0xd8,0x91,
0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,
0xed,0xff,0xff,0xde,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x93,0xff,0xff,0xff,0x19,0x00,0xa8,
0xff,0xff,0xff,0x23,0x00,0x00,0x00,0x00,0x00,0x00,0xda,0xff,0xff,0xd2,0x00,0x00,0x60,0xff,
0xff,0xff,0x69,0x00,0x00,0x00,0x00,0x00,0x20,0xff,0xff,0xff,0x8b,0x00,0x00,0x19,0xff,0xff,
0xff,0xae,0x00,0x00,0x00,0x00,0x00,0x66,0xff,0xff,0xff,0x45,0x00,0x00,0x00,0xd0,0xff,0xff,
0xef,0x03,0x00,0x00,0x00,0x00,0xac,0xff,0xff,0xf6,0x08,0x00,0x00,0x00,0x88,0xff,0xff,0xff,
0x39,0x00,0x00,0x00,0x03,0xef,0xff,0xff,0xb8,0x00,0x00,0x00,0x00,0x40,0xff,0xff,0xff,0x7e,
0x00,0x00,0x00,0x39,0xff,0xff,0xff,0x72,0x00,0x00,0x00,0x00,0x05,0xf2,0xff,0xff,0xc3,0x00,
0x00,0x00,0x7f,0xff,0xff,0xff,0x2b,0x00,0x00,0x00,0x00,0x00,0xb0,0xff,0xff,0xfb,0x0d,0x00,
0x00,0xc5,0xff,0xff,0xe4,0x00,0x00,0x00,0x00,0x00,0x00,0x68,0xff,0xff,0xff,0x4e,0x00,0x0f,
0xfb,0xff,0xff,0x9f,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0xff,0xff,0xff,0x93,0x00,0x51,0xff,
0xff,0xff,0x58,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd8,0xff,0xff,0xd9,0x00,0x98,0xff,0xff,
0xfd,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x90,0xff,0xff,0xff,0x1e,0xdd,0xff,0xff,0xcb,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x48,0xff,0xff,0xff,0x88,0xff,0xff,0xff,0x85,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x09,0xf7,0xff,0xff,0xf7,0xff,0xff,0xff,0x3f,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb8,0xff,0xff,0xff,0xff,0xff,0xf2,0x05,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x70,0xff,0xff,0xff,0xff,0xff,0xb2,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x28,0xff,0xff,0xff,0xff,0xff,0x6b,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xdf,0xff,0xff,0xff,0xff,0x25,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x98,0xff,0xff,0xff,0xdf,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf3,0xff,0xff,0x9c,0x00,0x00,0x00,0x7b,0xff,0xff,
0xff,0xd7,0x00,0x00,0x00,0x40,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xcc,0xff,0xff,0xbc,0x00,
0x00,0x00,0xaa,0xff,0xff,0xff,0xfc,0x08,0x00,0x00,0x61,0xff,0xff,0xff,0x24,0x00,0x00,0x00,
0xa4,0xff,0xff,0xdd,0x00,0x00,0x00,0xd9,0xff,0xff,0xff,0xff,0x34,0x00,0x00,0x82,0xff,0xff,
0xf8,0x03,0x00,0x00,0x00,0x7c,0xff,0xff,0xfb,0x03,0x00,0x0a,0xfd,0xff,0xff,0xff,0xff,0x63,
0x00,0x00,0xa2,0xff,0xff,0xd4,0x00,0x00,0x00,0x00,0x53,0xff,0xff,0xff,0x1f,0x00,0x37,0xff,
0xff,0xff,0xff,0xff,0x91,0x00,0x00,0xc3,0xff,0xff,0xac,0x00,0x00,0x00,0x00,0x2b,0xff,0xff,
0xff,0x40,0x00,0x66,0xff,0xff,0xff,0xff,0xff,0xc0,0x00,0x00,0xe4,0xff,0xff,0x84,0x00,0x00,
0x00,0x00,0x06,0xfc,0xff,0xff,0x61,0x00,0x95,0xff,0xff,0xed,0xff,0xff,0xee,0x00,0x06,0xfd,
0xff,0xff,0x5c,0x00,0x00,0x00,0x00,0x00,0xdb,0xff,0xff,0x81,0x00,0xc4,0xff,0xff,0x9b,0xff,
0xff,0xff,0x1d,0x25,0xff,0xff,0xff,0x34,0x00,0x00,0x00,0x00,0x00,0xb3,0xff,0xff,0xa2,0x01,
0xf2,0xff,0xff,0x4e,0xf4,0xff,0xff,0x4c,0x46,0xff,0xff,0xff,0x0c,0x00,0x00,0x00,0x00,0x00,
0x8a,0xff,0xff,0xc3,0x22,0xff,0xff,0xff,0x21,0xc9,0xff,0xff,0x7b,0x66,0xff,0xff,0xe4,0x00,
0x00,0x00,0x00,0x00,0x00,0x62,0xff,0xff,0xe4,0x51,0xff,0xff,0xf3,0x01,0x9d,0xff,0xff,0xa9,
0x87,0xff,0xff,0xbc,0x00,0x00,0x00,0x00,0x00,0x00,0x3a,0xff,0xff,0xfe,0x87,0xff,0xff,0xc9,
0x00,0x71,0xff,0xff,0xd8,0xa8,0xff,0xff,0x94,0x00,0x00,0x00,0x00,0x00,0x00,0x12,0xff,0xff,
0xff,0xd5,0xff,0xff,0x9d,0x00,0x44,0xff,0xff,0xfd,0xd2,0xff,0xff,0x6c,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xea,0xff,0xff,0xff,0xff,0xff,0x71,0x00,0x18,0xff,0xff,0xff,0xff,0xff,0xff,
0x44,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xc1,0xff,0xff,0xff,0xff,0xff,0x45,0x00,0x00,0xeb,
0xff,0xff,0xff,0xff,0xff,0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x99,0xff,0xff,0xff,0xff,
0xff,0x19,0x00,0x00,0xc0,0xff,0xff,0xff,0xff,0xf3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x71,0xff,0xff,0xff,0xff,0xec,0x00,0x00,0x00,0x93,0xff,0xff,0xff,0xff,0xcc,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x49,0xff,0xff,0xff,0xff,0xc1,0x00,0x00,0x00,0x67,0xff,0xff,0xff,
0xff,0xa4,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xff,0xff,0xff,0xff,0x95,0x00,0x00,
0x00,0x3b,0xff,0xff,0xff,0xff,0x7c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xf6,0xff,
0xff,0xff,0x69,0x00,0x00,0x00,0x0f,0xff,0xff,0xff,0xff,0x54,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xcf,0xff,0xff,0xff,0x9e,0x00,0x00,0x00,
0x00,0x00,0x29,0xfd,0xff,0xff,0xff,0x4e,0x00,0x3f,0xff,0xff,0xff,0xfc,0x25,0x00,0x00,0x00,
0x00,0xab,0xff,0xff,0xff,0xb9,0x00,0x00,0x00,0xac,0xff,0xff,0xff,0xa6,0x00,0x00,0x00,0x30,
0xfe,0xff,0xff,0xfb,0x2a,0x00,0x00,0x00,0x21,0xf7,0xff,0xff,0xfd,0x2c,0x00,0x00,0xb3,0xff,
0xff,0xff,0x93,0x00,0x00,0x00,0x00,0x00,0x85,0xff,0xff,0xff,0xaf,0x00,0x38,0xff,0xff,0xff,
0xed,0x12,0x00,0x00,0x00,0x00,0x00,0x0c,0xe6,0xff,0xff,0xff,0x33,0xbc,0xff,0xff,0xff,0x6b,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5f,0xff,0xff,0xff,0xda,0xff,0xff,0xff,0xd4,0x03,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0x44,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x39,0xff,0xff,0xff,0xff,0xff,0xb0,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xa5,0xff,0xff,0xff,0xf9,0x24,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xaf,0xff,0xff,0xff,0xfb,0x2a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x42,0xff,0xff,0xff,0xff,0xff,0xb9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x03,0xd2,0xff,0xff,0xff,0xff,0xff,0xff,0x4e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x69,0xff,0xff,0xff,0xd1,0xff,0xff,0xff,0xdb,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0xeb,
0xff,0xff,0xfe,0x2d,0xb3,0xff,0xff,0xff,0x76,0x00,0x00,0x00,0x00,0x00,0x00,0x90,0xff,0xff,
0xff,0xa6,0x00,0x30,0xfe,0xff,0xff,0xf2,0x17,0x00,0x00,0x00,0x00,0x28,0xfb,0xff,0xff,0xfc,
0x25,0x00,0x00,0xa9,0xff,0xff,0xff,0x9e,0x00,0x00,0x00,0x00,0xb7,0xff,0xff,0xff,0x9e,0x00,
0x00,0x00,0x28,0xfc,0xff,0xff,0xfe,0x33,0x00,0x00,0x4b,0xff,0xff,0xff,0xf9,0x20,0x00,0x00,
0x00,0x00,0x9f,0xff,0xff,0xff,0xc4,0x00,0x05,0xd8,0xff,0xff,0xff,0x96,0x00,0x00,0x00,0x00,
0x00,0x20,0xfa,0xff,0xff,0xff,0x5a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xcc,0xff,0xff,0xff,0xa0,0x00,
0x00,0x00,0x00,0x00,0x5c,0xff,0xff,0xff,0xf6,0x1f,0x00,0x3b,0xff,0xff,0xff,0xfc,0x24,0x00,
0x00,0x00,0x02,0xd8,0xff,0xff,0xff,0x82,0x00,0x00,0x00,0xa5,0xff,0xff,0xff,0xa0,0x00,0x00,
0x00,0x5b,0xff,0xff,0xff,0xe3,0x0a,0x00,0x00,0x00,0x1c,0xf5,0xff,0xff,0xfc,0x24,0x00,0x02,
0xd7,0xff,0xff,0xff,0x5a,0x00,0x00,0x00,0x00,0x00,0x7d,0xff,0xff,0xff,0xa0,0x00,0x59,0xff,
0xff,0xff,0xc5,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0xdf,0xff,0xff,0xfc,0x26,0xd6,0xff,0xff,
0xfe,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0xff,0xff,0xff,0xdc,0xff,0xff,0xff,0x9e,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xbf,0xff,0xff,0xff,0xff,0xff,0xf2,0x17,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2e,0xfc,0xff,0xff,0xff,0xff,0x76,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x97,0xff,0xff,0xff,0xdb,0x06,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x13,0xff,0xff,0xff,0x5f,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x45,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa6,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0x17,0x00,0x00,0x00,0x00,0x30,0x30,0x30,0x30,
0x30,0x30,0x30,0x30,0x30,0xc9,0xff,0xff,0xff,0x6a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x4b,0xff,0xff,0xff,0xc9,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x0a,0xdf,0xff,0xff,0xfc,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x88,0xff,0xff,0xff,0x8f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x2c,0xfa,0xff,0xff,0xe4,0x0d,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xc3,0xff,0xff,0xff,0x53,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0xff,0xff,0xff,0xb4,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xec,0xff,0xff,0xf5,0x20,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xa0,0xff,0xff,0xff,0x78,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0xff,0xff,0xff,0xd4,0x05,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xd5,0xff,0xff,0xfe,0x3c,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7a,0xff,0xff,0xff,0x9d,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf6,0xff,0xff,0xeb,0x13,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb6,0xff,0xff,0xff,0x8b,0x30,0x30,0x30,
0x30,0x30,0x30,0x30,0x30,0x30,0x18,0x00,0x00,0x00,0x55,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x0d,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x91,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x30,
0x30,0x30,0x30,0x30,0x18,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x6d,0x30,0x30,0x18,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,
0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x52,0xff,0xff,0xff,0x87,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,
0xe8,0xff,0xff,0xe8,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x89,0xff,0xff,0xff,0x55,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x25,0xff,0xff,0xff,0xbc,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xc1,0xff,0xff,0xfe,0x25,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5c,
0xff,0xff,0xff,0x8b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0xef,0xff,0xff,0xeb,0x06,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x93,0xff,0xff,0xff,0x59,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x2f,0xff,0xff,0xff,0xc1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xcb,
0xff,0xff,0xff,0x28,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x66,0xff,0xff,0xff,0x8f,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0xf4,0xff,0xff,0xed,0x08,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x9e,0xff,0xff,0xff,0x5e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x39,0xff,
0xff,0xff,0xc5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd4,0xff,0xff,0xff,0x2c,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x70,0xff,0xff,0xff,0x93,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x13,0xf8,0xff,0xff,0xf0,0x0a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xa8,0xff,
0xff,0xff,0x62,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x43,0xff,0xff,0xff,0xc9,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xdd,0xff,0xff,0xff,0x30,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x7a,0xff,0xff,0xff,0x98,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x30,0x30,0x30,0x30,0x0e,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x30,0x30,
0x30,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x22,0xec,0xff,0xff,0xd4,0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0e,0xd5,0xff,0xff,0xff,
0xff,0xb3,0x02,0x00,0x00,0x00,0x00,0x00,0x03,0xb6,0xff,0xff,0xcb,0xe7,0xff,0xff,0x8a,0x00,
0x00,0x00,0x00,0x00,0x8f,0xff,0xff,0xeb,0x1d,0x38,0xf8,0xff,0xff,0x5f,0x00,0x00,0x00,0x17,
0x7f,0x80,0x80,0x3a,0x00,0x00,0x48,0x80,0x80,0x78,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x30,
0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x0c,0x69,0x17,0x00,0x00,0x00,0x00,0xb5,0xff,0xde,0x28,0x00,0x00,0x00,0xef,0xff,0xff,
0xed,0x3d,0x00,0x00,0x41,0xe3,0xff,0xff,0xf8,0x57,0x00,0x00,0x10,0xa6,0xff,0xff,0xd6,0x00,
0x00,0x00,0x00,0x57,0xe4,0x56,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0x6c,0xbd,0xee,0xf6,0xdb,
0xb2,0x57,0x02,0x00,0x00,0x00,0x00,0x05,0xdc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc5,0x16,
0x00,0x00,0x00,0x00,0x6b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xcc,0x04,0x00,0x00,0x00,
0x00,0x60,0x4c,0x3a,0x35,0x54,0xac,0xff,0xff,0xff,0x60,0x00,0x00,0x00,0x00,0x47,0xbf,0xf3,
0xec,0xad,0x29,0xa4,0xff,0xff,0xb6,0x00,0x00,0x00,0x72,0xff,0xff,0xff,0xff,0xff,0xf1,0x86,
0xff,0xff,0xe8,0x00,0x00,0x39,0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xff,0xff,0xfb,0x00,
0x00,0xac,0xff,0xff,0xff,0xa5,0x48,0x42,0x94,0xfe,0xff,0xff,0xff,0x00,0x00,0xe9,0xff,0xff,
0xa4,0x00,0x00,0x00,0x00,0x9b,0xff,0xff,0xff,0x00,0x00,0xfb,0xff,0xff,0x5a,0x00,0x00,0x00,
0x00,0x59,0xff,0xff,0xff,0x00,0x00,0xe8,0xff,0xff,0x9e,0x00,0x00,0x00,0x00,0x95,0xff,0xff,
0xff,0x00,0x00,0xaa,0xff,0xff,0xff,0x9f,0x46,0x40,0x90,0xfe,0xff,0xff,0xff,0x00,0x00,0x36,
0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xff,0xff,0xff,0x00,0x00,0x00,0x6f,0xff,0xff,0xff,
0xff,0xff,0xf9,0x8c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x45,0xbd,0xf3,0xef,0xb8,0x38,0x4c,
0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x0a,0x81,0xde,0xf6,0xcf,0x5f,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x5c,0xce,0xff,0xff,
0xff,0xff,0xff,0x88,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,
0xfe,0x36,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xd7,0x5a,0x44,0xa7,0xff,0xff,0xff,0xa4,0x00,
0x00,0x00,0xff,0xff,0xff,0xec,0x16,0x00,0x00,0x00,0xb5,0xff,0xff,0xee,0x02,0x00,0x00,0xff,
0xff,0xff,0x91,0x00,0x00,0x00,0x00,0x47,0xff,0xff,0xff,0x21,0x00,0x00,0xff,0xff,0xff,0x5d,
0x00,0x00,0x00,0x00,0x12,0xff,0xff,0xff,0x3e,0x00,0x00,0xff,0xff,0xff,0x4f,0x00,0x00,0x00,
0x00,0x04,0xff,0xff,0xff,0x48,0x00,0x00,0xff,0xff,0xff,0x5d,0x00,0x00,0x00,0x00,0x16,0xff,
0xff,0xff,0x3f,0x00,0x00,0xff,0xff,0xff,0x90,0x00,0x00,0x00,0x00,0x52,0xff,0xff,0xff,0x24,
0x00,0x00,0xff,0xff,0xff,0xec,0x15,0x00,0x00,0x05,0xc6,0xff,0xff,0xf1,0x02,0x00,0x00,0xff,
0xff,0xff,0xff,0xdc,0x5a,0x3f,0xbd,0xff,0xff,0xff,0xa8,0x00,0x00,0x00,0xff,0xff,0xff,0xed,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x3a,0x00,0x00,0x00,0xff,0xff,0xff,0x60,0xd3,0xff,0xff,
0xff,0xff,0xff,0x8d,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x0b,0x83,0xde,0xf6,0xd1,0x64,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3a,0xa9,0xe6,0xf7,0xd9,0x88,0x18,0x00,
0x00,0x00,0x00,0x00,0x00,0x84,0xfe,0xff,0xff,0xff,0xff,0xff,0xeb,0x41,0x00,0x00,0x00,0x00,
0x73,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0x25,0x00,0x00,0x1c,0xf9,0xff,0xff,0xeb,
0x65,0x38,0x73,0xf6,0xff,0xc8,0x36,0x00,0x00,0x83,0xff,0xff,0xfe,0x35,0x00,0x00,0x00,0x48,
0x4c,0x00,0x00,0x00,0x00,0xc7,0xff,0xff,0xae,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xed,0xff,0xff,0x6e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfb,0xff,0xff,
0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xef,0xff,0xff,0x6d,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xcd,0xff,0xff,0xad,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x8c,0xff,0xff,0xfe,0x3c,0x00,0x00,0x00,0x4d,0x50,0x00,0x00,0x00,0x00,0x25,
0xfd,0xff,0xff,0xf1,0x74,0x36,0x56,0xeb,0xff,0xc7,0x31,0x00,0x00,0x00,0x83,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xf1,0x21,0x00,0x00,0x00,0x02,0x91,0xff,0xff,0xff,0xff,0xff,0xff,
0xf1,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x3c,0xa6,0xe2,0xf8,0xe3,0x9f,0x21,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x36,0xb7,0xf0,0xeb,0xa5,0x23,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x45,0xf9,0xff,0xff,0xff,0xff,0xf0,0x3a,0xff,0xff,0xff,
0x4c,0x00,0x00,0x07,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xff,0xff,0xff,0x4c,0x00,0x00,
0x5a,0xff,0xff,0xff,0xd2,0x54,0x48,0xaf,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0xa5,0xff,0xff,
0xea,0x13,0x00,0x00,0x01,0xb8,0xff,0xff,0xff,0x4c,0x00,0x00,0xd6,0xff,0xff,0x8f,0x00,0x00,
0x00,0x00,0x48,0xff,0xff,0xff,0x4c,0x00,0x00,0xf2,0xff,0xff,0x5d,0x00,0x00,0x00,0x00,0x13,
0xff,0xff,0xff,0x4c,0x00,0x00,0xfc,0xff,0xff,0x4f,0x00,0x00,0x00,0x00,0x04,0xff,0xff,0xff,
0x4c,0x00,0x00,0xf4,0xff,0xff,0x61,0x00,0x00,0x00,0x00,0x13,0xff,0xff,0xff,0x4c,0x00,0x00,
0xdb,0xff,0xff,0x9a,0x00,0x00,0x00,0x00,0x48,0xff,0xff,0xff,0x4c,0x00,0x00,0xac,0xff,0xff,
0xf4,0x1f,0x00,0x00,0x01,0xb7,0xff,0xff,0xff,0x4c,0x00,0x00,0x5f,0xff,0xff,0xff,0xdd,0x52,
0x47,0xb6,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x08,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xed,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x4a,0xfa,0xff,0xff,0xff,0xff,0xf2,0x41,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x3a,0xba,0xf1,0xec,0xa5,0x25,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x35,0xa7,0xe6,0xf7,0xd8,0x90,0x1d,0x00,0x00,0x00,0x00,0x00,
0x00,0x76,0xfd,0xff,0xff,0xff,0xff,0xff,0xf0,0x42,0x00,0x00,0x00,0x00,0x65,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xf6,0x22,0x00,0x00,0x17,0xf5,0xff,0xff,0xb3,0x4b,0x42,0x90,0xfd,
0xff,0xff,0xb2,0x00,0x00,0x7f,0xff,0xff,0xba,0x00,0x00,0x00,0x00,0x7c,0xff,0xff,0xfc,0x15,
0x00,0xc6,0xff,0xff,0x79,0x28,0x28,0x28,0x28,0x3c,0xff,0xff,0xff,0x54,0x00,0xee,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x6f,0x00,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0x7f,0x00,0xea,0xff,0xff,0x5d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xbe,0xff,0xff,0x83,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,
0xff,0xff,0xde,0x0c,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x00,0x00,0x10,0xf1,0xff,0xff,0xd0,
0x52,0x3d,0x7f,0xf4,0xa5,0x37,0x00,0x00,0x00,0x00,0x5c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0x45,0x00,0x00,0x00,0x00,0x6b,0xfa,0xff,0xff,0xff,0xff,0xff,0xfa,0x73,0x00,0x00,
0x00,0x00,0x00,0x00,0x2a,0x9d,0xe2,0xf8,0xdf,0x99,0x2b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x80,0xe3,0xf7,0xc6,
0x3d,0x00,0x00,0x00,0x00,0x05,0xc0,0xff,0xff,0xff,0xff,0xc0,0x00,0x00,0x00,0x00,0x80,0xff,
0xff,0xff,0xff,0xdf,0x16,0x00,0x00,0x00,0x00,0xe8,0xff,0xff,0x93,0x4d,0x2a,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x30,0x30,0xff,0xff,0xff,0x6d,0x30,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x36,0xb7,0xf0,0xeb,0xa6,0x26,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x45,
0xf9,0xff,0xff,0xff,0xff,0xf3,0x44,0xff,0xff,0xff,0x4c,0x00,0x00,0x07,0xe5,0xff,0xff,0xff,
0xff,0xff,0xff,0xee,0xff,0xff,0xff,0x4c,0x00,0x00,0x5a,0xff,0xff,0xff,0xd2,0x54,0x48,0xaf,
0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0xa5,0xff,0xff,0xea,0x13,0x00,0x00,0x01,0xb8,0xff,0xff,
0xff,0x4c,0x00,0x00,0xd6,0xff,0xff,0x8f,0x00,0x00,0x00,0x00,0x48,0xff,0xff,0xff,0x4c,0x00,
0x00,0xf2,0xff,0xff,0x5d,0x00,0x00,0x00,0x00,0x13,0xff,0xff,0xff,0x4c,0x00,0x00,0xfc,0xff,
0xff,0x4f,0x00,0x00,0x00,0x00,0x04,0xff,0xff,0xff,0x4c,0x00,0x00,0xf4,0xff,0xff,0x61,0x00,
0x00,0x00,0x00,0x13,0xff,0xff,0xff,0x4c,0x00,0x00,0xdb,0xff,0xff,0x9a,0x00,0x00,0x00,0x00,
0x48,0xff,0xff,0xff,0x4c,0x00,0x00,0xac,0xff,0xff,0xf4,0x1f,0x00,0x00,0x01,0xb7,0xff,0xff,
0xff,0x4c,0x00,0x00,0x5f,0xff,0xff,0xff,0xdd,0x52,0x47,0xb6,0xff,0xff,0xff,0xff,0x4c,0x00,
0x00,0x08,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x4a,
0xfa,0xff,0xff,0xff,0xff,0xee,0x38,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x3a,0xba,0xf1,
0xeb,0xa3,0x22,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x11,0x1c,0x1c,0x0d,0x00,0x00,0x00,0x00,
0x05,0xff,0xff,0xff,0x47,0x00,0x00,0x7d,0xff,0xff,0xb2,0x00,0x00,0x00,0x00,0x46,0xff,0xff,
0xff,0x2a,0x00,0x00,0x2f,0xfe,0xff,0xff,0xa0,0x45,0x3a,0x70,0xeb,0xff,0xff,0xde,0x00,0x00,
0x00,0x00,0xa5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x5a,0x00,0x00,0x00,0x00,0x0a,
0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x6a,0xc5,
0xf2,0xf9,0xe1,0xa4,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x05,0x6f,0xd0,0xf5,0xdb,0x68,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x59,0xc5,0xff,0xff,0xff,0xff,0xff,0x88,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0xe0,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0x35,0x00,0x00,0x00,0xff,
0xff,0xff,0xff,0xbd,0x47,0x41,0x9a,0xff,0xff,0xff,0x9c,0x00,0x00,0x00,0xff,0xff,0xff,0xdb,
0x09,0x00,0x00,0x00,0xc8,0xff,0xff,0xdc,0x00,0x00,0x00,0xff,0xff,0xff,0x83,0x00,0x00,0x00,
0x00,0x7c,0xff,0xff,0xf7,0x00,0x00,0x00,0xff,0xff,0xff,0x5b,0x00,0x00,0x00,0x00,0x5a,0xff,
0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x4e,0x00,0x00,0x00,0x00,0x4e,0xff,0xff,0xff,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,
0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x10,0x6f,0x2e,0x00,0x00,0x00,0x00,0xba,0xff,0xf2,0x13,0x00,0x00,0x00,0xf6,0xff,
0xff,0x41,0x00,0x00,0x00,0xba,0xff,0xf2,0x14,0x00,0x00,0x00,0x11,0x6f,0x2e,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x10,0x6f,0x2e,0x00,0x00,0x00,0x00,0x00,0x00,0xba,0xff,0xf2,0x13,0x00,0x00,
0x00,0x00,0x00,0xf6,0xff,0xff,0x41,0x00,0x00,0x00,0x00,0x00,0xba,0xff,0xf2,0x14,0x00,0x00,
0x00,0x00,0x00,0x11,0x6f,0x2e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x47,0x00,0x00,
0x00,0x00,0x0b,0xff,0xff,0xff,0x28,0x00,0x05,0x87,0x40,0x9a,0xff,0xff,0xd9,0x00,0x02,0xa8,
0xff,0xff,0xff,0xff,0xff,0x59,0x00,0x74,0xff,0xff,0xff,0xff,0xff,0x8c,0x00,0x00,0x0e,0x9b,
0xea,0xf7,0xcb,0x56,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x5f,0xff,0xff,0xff,0xbf,0x03,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x29,0xf4,0xff,0xff,0xe4,0x16,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x09,0xd2,0xff,0xff,0xfa,0x39,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x9a,0xff,
0xff,0xff,0x6b,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xa3,0xff,0xff,0xff,0xa2,0x00,0x00,
0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xd1,0x09,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0xff,0xff,0xff,0x29,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
0xff,0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xd9,0xff,0xff,0xfa,0x2f,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x53,0xd9,0xff,0xff,0xce,0x04,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x41,0xff,0xff,0xff,0x7c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x9f,0xff,0xff,0xf9,0x2b,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x13,
0xeb,0xff,0xff,0xc9,0x03,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x5d,0xff,0xff,0xff,
0x76,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xbc,0xff,0xff,0xf7,0x27,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0x4e,0x00,0x00,0x00,0xf6,0xff,0xff,0xaa,0x1d,0x00,0x00,0xcf,0xff,0xff,0xff,
0x84,0x00,0x00,0x63,0xff,0xff,0xff,0x84,0x00,0x00,0x00,0x5c,0xc3,0xf0,0x82,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x1b,0xab,
0xf5,0xd5,0x50,0x00,0x00,0x16,0xaa,0xf4,0xd6,0x51,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
0x66,0xe1,0xff,0xff,0xff,0xfb,0x33,0x15,0xdc,0xff,0xff,0xff,0xfc,0x3d,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0xed,0xff,0xff,0xff,0xff,0xff,0xa6,0xa7,0xff,0xff,0xff,0xff,0xff,0xc0,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0xfa,0x61,0x45,0xdd,0xff,0xff,0xf8,0xfa,0x64,0x46,0xdf,0xff,
0xff,0xfd,0x0f,0x00,0x00,0x00,0xff,0xff,0xff,0xa5,0x00,0x00,0x5e,0xff,0xff,0xff,0xaa,0x00,
0x00,0x62,0xff,0xff,0xff,0x36,0x00,0x00,0x00,0xff,0xff,0xff,0x6e,0x00,0x00,0x24,0xff,0xff,
0xff,0x6f,0x00,0x00,0x26,0xff,0xff,0xff,0x47,0x00,0x00,0x00,0xff,0xff,0xff,0x55,0x00,0x00,
0x0b,0xff,0xff,0xff,0x55,0x00,0x00,0x0b,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0x4d,0x00,0x00,0x01,0xff,0xff,0xff,0x4d,0x00,0x00,0x01,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x06,0x72,0xd1,0xf5,0xdb,0x68,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x5c,0xcb,0xff,0xff,0xff,0xff,0xff,0x88,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0x35,0x00,0x00,0x00,0xff,0xff,0xff,
0xff,0xbd,0x47,0x41,0x9a,0xff,0xff,0xff,0x9c,0x00,0x00,0x00,0xff,0xff,0xff,0xdb,0x09,0x00,
0x00,0x00,0xc8,0xff,0xff,0xdc,0x00,0x00,0x00,0xff,0xff,0xff,0x83,0x00,0x00,0x00,0x00,0x7c,
0xff,0xff,0xf7,0x00,0x00,0x00,0xff,0xff,0xff,0x5b,0x00,0x00,0x00,0x00,0x5a,0xff,0xff,0xff,
0x00,0x00,0x00,0xff,0xff,0xff,0x4e,0x00,0x00,0x00,0x00,0x4e,0xff,0xff,0xff,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,
0xff,0xff,0xff,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x05,0x6e,0xc7,0xf0,0xf7,0xda,0x91,0x1a,0x00,0x00,0x00,0x00,0x00,0x11,0xca,0xff,0xff,
0xff,0xff,0xff,0xff,0xf1,0x3b,0x00,0x00,0x00,0x00,0xb3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xec,0x15,0x00,0x00,0x40,0xff,0xff,0xff,0xbd,0x48,0x3b,0x90,0xff,0xff,0xff,0x8c,0x00,
0x00,0x9b,0xff,0xff,0xdd,0x07,0x00,0x00,0x00,0x9b,0xff,0xff,0xe7,0x00,0x00,0xd4,0xff,0xff,
0x83,0x00,0x00,0x00,0x00,0x39,0xff,0xff,0xff,0x1f,0x00,0xf1,0xff,0xff,0x59,0x00,0x00,0x00,
0x00,0x0f,0xff,0xff,0xff,0x3c,0x00,0xfc,0xff,0xff,0x4e,0x00,0x00,0x00,0x00,0x03,0xff,0xff,
0xff,0x47,0x00,0xf1,0xff,0xff,0x5a,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0xff,0x3b,0x00,0xd4,
0xff,0xff,0x84,0x00,0x00,0x00,0x00,0x3a,0xff,0xff,0xff,0x1e,0x00,0x9c,0xff,0xff,0xde,0x08,
0x00,0x00,0x00,0x9c,0xff,0xff,0xe5,0x00,0x00,0x41,0xff,0xff,0xff,0xbd,0x47,0x3a,0x8f,0xff,
0xff,0xff,0x89,0x00,0x00,0x00,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0x13,0x00,
0x00,0x00,0x13,0xcd,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0x39,0x00,0x00,0x00,0x00,0x00,0x06,
0x71,0xc9,0xf1,0xf7,0xdb,0x91,0x1a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x0b,0x84,0xde,0xf6,0xcf,0x5f,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x61,0xd6,0xff,0xff,0xff,0xff,0xff,0x88,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xee,
0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0x36,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xd7,0x5a,0x44,
0xa7,0xff,0xff,0xff,0xa4,0x00,0x00,0x00,0xff,0xff,0xff,0xec,0x16,0x00,0x00,0x00,0xb5,0xff,
0xff,0xee,0x02,0x00,0x00,0xff,0xff,0xff,0x91,0x00,0x00,0x00,0x00,0x47,0xff,0xff,0xff,0x21,
0x00,0x00,0xff,0xff,0xff,0x5d,0x00,0x00,0x00,0x00,0x12,0xff,0xff,0xff,0x3e,0x00,0x00,0xff,
0xff,0xff,0x4f,0x00,0x00,0x00,0x00,0x04,0xff,0xff,0xff,0x48,0x00,0x00,0xff,0xff,0xff,0x5d,
0x00,0x00,0x00,0x00,0x16,0xff,0xff,0xff,0x3f,0x00,0x00,0xff,0xff,0xff,0x90,0x00,0x00,0x00,
0x00,0x52,0xff,0xff,0xff,0x24,0x00,0x00,0xff,0xff,0xff,0xec,0x15,0x00,0x00,0x05,0xc6,0xff,
0xff,0xf1,0x02,0x00,0x00,0xff,0xff,0xff,0xff,0xdc,0x5a,0x3f,0xbd,0xff,0xff,0xff,0xa8,0x00,
0x00,0x00,0xff,0xff,0xff,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x3a,0x00,0x00,0x00,0xff,
0xff,0xff,0x5b,0xcc,0xff,0xff,0xff,0xff,0xff,0x8d,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x09,0x80,0xde,0xf6,0xd1,0x64,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x36,0xb7,0xf0,0xeb,0xa6,0x26,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,
0x45,0xf9,0xff,0xff,0xff,0xff,0xf3,0x44,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x07,0xe5,0xff,
0xff,0xff,0xff,0xff,0xff,0xee,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x5a,0xff,0xff,0xff,0xd2,
0x54,0x48,0xaf,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xa5,0xff,0xff,0xea,0x13,0x00,0x00,
0x01,0xb8,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xd6,0xff,0xff,0x8f,0x00,0x00,0x00,0x00,0x48,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xf2,0xff,0xff,0x5d,0x00,0x00,0x00,0x00,0x13,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0xfc,0xff,0xff,0x4f,0x00,0x00,0x00,0x00,0x04,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0xf4,0xff,0xff,0x61,0x00,0x00,0x00,0x00,0x13,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0xdb,0xff,0xff,0x9a,0x00,0x00,0x00,0x00,0x48,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xac,
0xff,0xff,0xf4,0x1f,0x00,0x00,0x01,0xb7,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x5f,0xff,0xff,
0xff,0xdd,0x52,0x47,0xb6,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x08,0xe8,0xff,0xff,0xff,
0xff,0xff,0xff,0xe5,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4a,0xfa,0xff,0xff,0xff,0xff,
0xee,0x38,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x3a,0xba,0xf1,0xeb,0xa3,0x22,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xee,0xff,0xff,0xa1,0x38,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb3,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x31,0xf5,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x28,0x9f,0xdf,0xf9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x1a,0xaa,
0xf4,0xd9,0x6a,0x03,0x00,0x00,0xff,0xff,0xff,0x65,0xe0,0xff,0xff,0xff,0xf8,0x16,0x00,0x00,
0xff,0xff,0xff,0xea,0xff,0xff,0xff,0xff,0xa1,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xbe,0x43,
0x56,0xc5,0x34,0x00,0x00,0x00,0xff,0xff,0xff,0xe7,0x0a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x94,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x61,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x20,0x99,0xdd,0xf9,0xf2,0xbf,0x4c,0x00,0x00,0x00,0x00,0x00,0x31,0xf0,0xff,0xff,0xff,
0xff,0xff,0xff,0x81,0x00,0x00,0x00,0x00,0xbf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x46,
0x00,0x00,0x00,0xf7,0xff,0xff,0xb0,0x3b,0x49,0xd8,0xff,0xff,0xb4,0x00,0x00,0x00,0xf5,0xff,
0xff,0x5f,0x00,0x00,0x41,0x7c,0x50,0x21,0x00,0x00,0x00,0xbb,0xff,0xff,0xec,0x68,0x0d,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x36,0xf5,0xff,0xff,0xff,0xf3,0x94,0x1e,0x00,0x00,0x00,0x00,
0x00,0x00,0x32,0xca,0xff,0xff,0xff,0xff,0xf5,0x64,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,
0xc2,0xff,0xff,0xff,0xfe,0x4e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0xdc,0xff,0xff,
0xc3,0x00,0x00,0x36,0x83,0xae,0x85,0x00,0x00,0x00,0x5f,0xff,0xff,0xef,0x00,0x00,0x4e,0xff,
0xff,0xfe,0x85,0x39,0x43,0xbe,0xff,0xff,0xf1,0x00,0x00,0x04,0xd8,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xa2,0x00,0x00,0x00,0x26,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xcc,0x13,0x00,
0x00,0x00,0x00,0x1c,0x95,0xdf,0xfb,0xf2,0xc5,0x6b,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,
0x80,0x80,0x26,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0x00,0x00,0x30,0x30,0xff,0xff,0xff,0x6d,0x30,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0xff,
0xff,0xff,0x4f,0x00,0x00,0x00,0x00,0x00,0xf6,0xff,0xff,0xb1,0x35,0x00,0x00,0x00,0x00,0xce,
0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x60,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
0x5a,0xc5,0xf0,0xfe,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,
0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xfd,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x4f,0xff,0xff,0xff,0x00,0x00,0xf5,0xff,0xff,0x57,0x00,0x00,0x00,0x00,0x65,0xff,0xff,
0xff,0x00,0x00,0xdf,0xff,0xff,0x79,0x00,0x00,0x00,0x00,0x84,0xff,0xff,0xff,0x00,0x00,0xb8,
0xff,0xff,0xc2,0x00,0x00,0x00,0x02,0xd5,0xff,0xff,0xff,0x00,0x00,0x79,0xff,0xff,0xff,0x70,
0x00,0x09,0x99,0xff,0xff,0xff,0xff,0x00,0x00,0x1d,0xfa,0xff,0xff,0xff,0xed,0xf6,0xff,0xf9,
0xff,0xff,0xff,0x00,0x00,0x00,0x7f,0xff,0xff,0xff,0xff,0xff,0xe2,0x70,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x6e,0xdd,0xf7,0xd9,0x84,0x10,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe5,0xff,0xff,0xa3,0x00,0x00,0x00,
0x00,0xb2,0xff,0xff,0x8d,0x00,0x00,0x9a,0xff,0xff,0xe2,0x00,0x00,0x00,0x04,0xf1,0xff,0xff,
0x41,0x00,0x00,0x4e,0xff,0xff,0xff,0x22,0x00,0x00,0x3a,0xff,0xff,0xf0,0x04,0x00,0x00,0x0a,
0xf7,0xff,0xff,0x61,0x00,0x00,0x7e,0xff,0xff,0xa8,0x00,0x00,0x00,0x00,0xb5,0xff,0xff,0xa0,
0x00,0x00,0xc2,0xff,0xff,0x5c,0x00,0x00,0x00,0x00,0x69,0xff,0xff,0xe0,0x00,0x0b,0xfa,0xff,
0xfc,0x13,0x00,0x00,0x00,0x00,0x1d,0xff,0xff,0xff,0x1f,0x4a,0xff,0xff,0xc4,0x00,0x00,0x00,
0x00,0x00,0x00,0xd0,0xff,0xff,0x5e,0x8e,0xff,0xff,0x77,0x00,0x00,0x00,0x00,0x00,0x00,0x84,
0xff,0xff,0x9e,0xd2,0xff,0xff,0x2b,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0xff,0xff,0xe9,0xfe,
0xff,0xde,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xe9,0xff,0xff,0xff,0xff,0x93,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x9f,0xff,0xff,0xff,0xff,0x46,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x53,0xff,0xff,0xff,0xf3,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0xf9,
0xff,0xff,0xae,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xba,0xff,0xff,0x62,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x1d,0xff,0xff,0xff,0x3a,0x00,0x00,0x0d,0xfe,0xff,0xff,0x2f,
0x00,0x00,0x10,0xfd,0xff,0xff,0x20,0x00,0xd6,0xff,0xff,0x76,0x00,0x00,0x3b,0xff,0xff,0xff,
0x5e,0x00,0x00,0x4d,0xff,0xff,0xd9,0x00,0x00,0x8f,0xff,0xff,0xb3,0x00,0x00,0x6a,0xff,0xff,
0xff,0x8d,0x00,0x00,0x8b,0xff,0xff,0x92,0x00,0x00,0x48,0xff,0xff,0xee,0x01,0x00,0x99,0xff,
0xff,0xff,0xbc,0x00,0x00,0xc9,0xff,0xff,0x4b,0x00,0x00,0x09,0xf7,0xff,0xff,0x2c,0x00,0xc8,
0xff,0xff,0xff,0xeb,0x00,0x0b,0xfb,0xff,0xf9,0x0b,0x00,0x00,0x00,0xba,0xff,0xff,0x69,0x02,
0xf5,0xff,0xfc,0xff,0xff,0x1b,0x45,0xff,0xff,0xbd,0x00,0x00,0x00,0x00,0x73,0xff,0xff,0xa6,
0x26,0xff,0xff,0xb1,0xff,0xff,0x4a,0x82,0xff,0xff,0x76,0x00,0x00,0x00,0x00,0x2c,0xff,0xff,
0xe2,0x55,0xff,0xff,0x4c,0xff,0xff,0x79,0xc0,0xff,0xff,0x2f,0x00,0x00,0x00,0x00,0x00,0xe4,
0xff,0xff,0xa4,0xff,0xdc,0x0c,0xfe,0xff,0xaf,0xf7,0xff,0xe7,0x01,0x00,0x00,0x00,0x00,0x00,
0x9e,0xff,0xff,0xf9,0xff,0xa8,0x00,0xd9,0xff,0xfa,0xff,0xff,0xa1,0x00,0x00,0x00,0x00,0x00,
0x00,0x57,0xff,0xff,0xff,0xff,0x74,0x00,0xa7,0xff,0xff,0xff,0xff,0x5a,0x00,0x00,0x00,0x00,
0x00,0x00,0x13,0xfd,0xff,0xff,0xff,0x40,0x00,0x76,0xff,0xff,0xff,0xfe,0x15,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xc9,0xff,0xff,0xfe,0x0e,0x00,0x44,0xff,0xff,0xff,0xcc,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x82,0xff,0xff,0xd9,0x00,0x00,0x13,0xff,0xff,0xff,0x85,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x3b,0xff,0xff,0xa5,0x00,0x00,0x00,0xe1,0xff,0xff,0x3f,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x25,0xfa,
0xff,0xff,0xb1,0x00,0x00,0x00,0x05,0xdb,0xff,0xff,0xfe,0x3a,0x00,0x8e,0xff,0xff,0xff,0x3b,
0x00,0x00,0x6a,0xff,0xff,0xff,0x9a,0x00,0x00,0x10,0xec,0xff,0xff,0xc5,0x00,0x0b,0xe7,0xff,
0xff,0xe9,0x11,0x00,0x00,0x00,0x6b,0xff,0xff,0xff,0x51,0x7c,0xff,0xff,0xff,0x5d,0x00,0x00,
0x00,0x00,0x03,0xd5,0xff,0xff,0xdb,0xf1,0xff,0xff,0xbe,0x00,0x00,0x00,0x00,0x00,0x00,0x47,
0xff,0xff,0xff,0xff,0xff,0xf9,0x27,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb6,0xff,0xff,0xff,
0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x49,0xff,0xff,0xff,0xff,0x20,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0xbd,0xff,0xff,0xff,0xff,0xa1,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x57,0xff,0xff,0xff,0xff,0xff,0xfe,0x38,0x00,0x00,0x00,0x00,0x00,0x0c,0xe4,0xff,0xff,
0xd1,0xee,0xff,0xff,0xcb,0x02,0x00,0x00,0x00,0x00,0x8b,0xff,0xff,0xff,0x45,0x76,0xff,0xff,
0xff,0x64,0x00,0x00,0x00,0x29,0xfa,0xff,0xff,0xbd,0x00,0x09,0xe4,0xff,0xff,0xea,0x10,0x00,
0x00,0xbd,0xff,0xff,0xff,0x35,0x00,0x00,0x65,0xff,0xff,0xff,0x91,0x00,0x57,0xff,0xff,0xff,
0xac,0x00,0x00,0x00,0x03,0xd8,0xff,0xff,0xfb,0x2c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,
0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xfd,0xff,0xff,
0x4e,0x00,0x00,0x00,0x00,0x4f,0xff,0xff,0xff,0x00,0x00,0xf5,0xff,0xff,0x5c,0x00,0x00,0x00,
0x00,0x61,0xff,0xff,0xff,0x00,0x00,0xdf,0xff,0xff,0x87,0x00,0x00,0x00,0x00,0x93,0xff,0xff,
0xff,0x00,0x00,0xb8,0xff,0xff,0xde,0x09,0x00,0x00,0x18,0xec,0xff,0xff,0xff,0x00,0x00,0x79,
0xff,0xff,0xff,0xb9,0x42,0x53,0xd5,0xff,0xff,0xff,0xff,0x00,0x00,0x1d,0xfa,0xff,0xff,0xff,
0xff,0xff,0xff,0xde,0xff,0xff,0xff,0x00,0x00,0x00,0x7f,0xff,0xff,0xff,0xff,0xff,0xc3,0x58,
0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x6e,0xdd,0xf6,0xd3,0x72,0x05,0x4c,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0xff,0xff,0xf1,0x00,0x00,0x00,0x0e,0x8d,
0x3c,0x00,0x00,0x00,0x00,0x9d,0xff,0xff,0xba,0x00,0x00,0x42,0xe7,0xff,0xf0,0x76,0x3c,0x45,
0x9c,0xff,0xff,0xff,0x4b,0x00,0x00,0x1d,0xf1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa0,
0x00,0x00,0x00,0x00,0x38,0xe9,0xff,0xff,0xff,0xff,0xff,0xff,0x9d,0x05,0x00,0x00,0x00,0x00,
0x00,0x13,0x7d,0xcf,0xf0,0xf0,0xb6,0x46,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2f,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xb3,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x38,
0x00,0x00,0x00,0x00,0x30,0x30,0x30,0x30,0x58,0xff,0xff,0xff,0xbd,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xa1,0xff,0xff,0xff,0x42,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1f,
0xfb,0xff,0xff,0xc7,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x95,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0xf7,0xff,0xff,0xd0,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x8a,0xff,0xff,0xff,0x56,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x11,
0xf2,0xff,0xff,0xd9,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7e,0xff,0xff,0xff,0x60,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0b,0xed,0xff,0xff,0xef,0x35,0x30,0x30,0x30,0x30,
0x00,0x00,0x00,0x00,0x73,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x07,
0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x67,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x7b,0xda,0xf7,0xfe,0x00,0x00,0x00,
0x00,0x85,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0xd9,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
0x00,0xef,0xff,0xff,0xd1,0x44,0x00,0x00,0x00,0x00,0xf8,0xff,0xff,0x60,0x00,0x00,0x00,0x00,
0x03,0xff,0xff,0xff,0x4a,0x00,0x00,0x00,0x00,0x14,0xff,0xff,0xff,0x42,0x00,0x00,0x00,0x00,
0x27,0xff,0xff,0xff,0x2e,0x00,0x00,0x00,0x00,0x50,0xff,0xff,0xfd,0x0c,0x00,0x00,0x00,0x00,
0xa0,0xff,0xff,0xcb,0x00,0x00,0x00,0x00,0x35,0xf9,0xff,0xff,0x5b,0x00,0x00,0x00,0x1b,0xe2,
0xff,0xff,0xb4,0x00,0x00,0x00,0x00,0x00,0x33,0xf9,0xff,0xff,0x5c,0x00,0x00,0x00,0x00,0x00,
0x9f,0xff,0xff,0xcc,0x00,0x00,0x00,0x00,0x00,0x50,0xff,0xff,0xfe,0x0c,0x00,0x00,0x00,0x00,
0x27,0xff,0xff,0xff,0x2e,0x00,0x00,0x00,0x00,0x14,0xff,0xff,0xff,0x42,0x00,0x00,0x00,0x00,
0x03,0xff,0xff,0xff,0x4a,0x00,0x00,0x00,0x00,0x00,0xf8,0xff,0xff,0x60,0x00,0x00,0x00,0x00,
0x00,0xef,0xff,0xff,0xcf,0x43,0x00,0x00,0x00,0x00,0xda,0xff,0xff,0xff,0xff,0x00,0x00,0x00,
0x00,0x88,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x03,0x7f,0xde,0xf8,0xfe,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xfe,0xf7,0xe3,0x92,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
0xff,0xff,0xc5,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x1f,0x00,0x00,0x00,
0x00,0x00,0x3f,0xad,0xff,0xff,0xff,0x37,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0xff,0xff,0xff,
0x42,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfe,0xff,0xff,0x4d,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xf7,0xff,0xff,0x5f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe4,0xff,0xff,0x72,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0xbf,0xff,0xff,0x9d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xff,
0xff,0xe8,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x1d,0xf3,0xff,0xff,0x7e,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x69,0xff,0xff,0xfd,0x51,0x00,0x00,0x00,0x00,0x00,0x1d,0xf4,0xff,0xff,0x7c,
0x00,0x00,0x00,0x00,0x00,0x00,0x81,0xff,0xff,0xe7,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0xbf,
0xff,0xff,0x9c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe4,0xff,0xff,0x72,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xf7,0xff,0xff,0x5f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfe,0xff,0xff,0x4d,
0x00,0x00,0x00,0x00,0x00,0x00,0x18,0xff,0xff,0xff,0x42,0x00,0x00,0x00,0x00,0x00,0x3e,0xaa,
0xff,0xff,0xff,0x37,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0x20,0x00,0x00,0x00,
0x00,0x00,0xff,0xff,0xff,0xff,0xca,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xfd,0xe7,0x9e,0x17,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x09,0x1b,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x4b,0xeb,0xff,0x9a,0x04,0x00,0x00,0x2d,0x00,0x00,0xfc,0xff,0xff,
0xff,0xbf,0x45,0x69,0xee,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0xd0,
0x36,0x17,0xb2,0xff,0xff,0xff,0xf8,0x00,0x00,0x0b,0x00,0x00,0x02,0x7d,0xf0,0xce,0x39,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x88,0xd0,0xef,0xf8,0xdd,
0x9e,0x39,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x75,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xa1,0x06,0x00,0x00,0x00,0x00,0x00,0x81,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xb1,0x05,0x00,0x00,0x00,0x3f,0xfe,0xff,0xff,0xe7,0x74,0x3c,0x44,0x95,0xfc,0xff,0xff,
0xff,0x93,0x00,0x00,0x00,0xcc,0xff,0xff,0xda,0x16,0x00,0x00,0x00,0x00,0x4a,0xfc,0xff,0xff,
0xfe,0x2b,0x00,0x39,0xff,0xff,0xff,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x89,0xff,0xff,0xff,
0x90,0x00,0x8a,0xff,0xff,0xd6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1c,0xd4,0xd4,0xd4,0xaa,
0x00,0xc4,0xff,0xff,0x91,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xe8,0xff,0xff,0x64,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfa,
0xff,0xff,0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfa,0xff,
0xff,0x53,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe9,0xff,0xff,
0x64,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xc6,0xff,0xff,0x8f,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8c,0xff,0xff,0xd4,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x1e,0xc8,0xc8,0xc8,0x9e,0x00,0x3b,0xff,0xff,0xff,0x4a,0x00,
0x00,0x00,0x00,0x00,0x00,0x89,0xff,0xff,0xff,0x90,0x00,0x00,0xce,0xff,0xff,0xda,0x15,0x00,
0x00,0x00,0x00,0x45,0xfc,0xff,0xff,0xfe,0x2c,0x00,0x00,0x40,0xfe,0xff,0xff,0xe6,0x73,0x3b,
0x3e,0x88,0xf8,0xff,0xff,0xff,0x9a,0x00,0x00,0x00,0x00,0x81,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xb8,0x08,0x00,0x00,0x00,0x00,0x00,0x76,0xf8,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xa6,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x89,0xd1,0xf0,0xf9,0xde,0xa0,
0x3c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x67,0xb4,0x0f,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x98,0xff,0xff,0x9a,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4b,0xff,0xff,0xf0,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0xb4,0xff,0xff,0xd0,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xbc,0xff,0xff,0xe8,0x32,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x1d,0xdd,0x93,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x49,0xe3,0xe3,0x47,0x00,0x00,0x49,0xe3,0xe3,0x47,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xe4,0xff,0xff,0xe3,0x00,0x00,0xe4,0xff,0xff,0xe3,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xe3,0xff,0xff,0xe3,0x00,0x00,0xe4,0xff,0xff,0xe3,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x47,0xe3,0xe4,0x49,0x00,0x00,0x49,0xe3,0xe4,0x49,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x29,0x90,0xd6,0xf3,0xf4,0xd7,0x92,0x2b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x01,0x82,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0x86,0x01,0x00,0x00,0x00,0x00,0x00,
0x00,0x8c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8f,0x00,0x00,0x00,0x00,0x00,
0x44,0xff,0xff,0xff,0xea,0x77,0x3d,0x40,0x79,0xe9,0xff,0xff,0xff,0x47,0x00,0x00,0x00,0x00,
0xcd,0xff,0xff,0xdc,0x19,0x00,0x00,0x00,0x00,0x1b,0xdc,0xff,0xff,0xd0,0x00,0x00,0x00,0x37,
0xff,0xff,0xff,0x43,0x00,0x00,0x00,0x00,0x00,0x00,0x43,0xff,0xff,0xff,0x3c,0x00,0x00,0x89,
0xff,0xff,0xd9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd5,0xff,0xff,0x8b,0x00,0x00,0xc4,
0xff,0xff,0x95,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x91,0xff,0xff,0xc4,0x00,0x00,0xe8,
0xff,0xff,0x67,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x71,0xff,0xff,0xe8,0x00,0x00,0xfa,
0xff,0xff,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x59,0xff,0xff,0xf9,0x00,0x00,0xfa,
0xff,0xff,0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0xff,0xff,0xfa,0x00,0x00,0xe9,
0xff,0xff,0x67,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x67,0xff,0xff,0xe9,0x00,0x00,0xc6,
0xff,0xff,0x93,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x93,0xff,0xff,0xc6,0x00,0x00,0x8b,
0xff,0xff,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xd6,0xff,0xff,0x8d,0x00,0x00,0x39,
0xff,0xff,0xff,0x4e,0x00,0x00,0x00,0x00,0x00,0x00,0x4d,0xff,0xff,0xff,0x3e,0x00,0x00,0x00,
0xcf,0xff,0xff,0xdc,0x17,0x00,0x00,0x00,0x00,0x16,0xdc,0xff,0xff,0xd2,0x00,0x00,0x00,0x00,
0x45,0xff,0xff,0xff,0xe8,0x75,0x3c,0x3c,0x74,0xe7,0xff,0xff,0xff,0x48,0x00,0x00,0x00,0x00,
0x00,0x8c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x90,0x00,0x00,0x00,0x00,0x00,
0x00,0x01,0x82,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0x86,0x01,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x29,0x91,0xd7,0xf4,0xf4,0xd8,0x93,0x2b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x49,0xe3,0xe3,0x47,0x00,0x00,0x49,0xe3,0xe3,0x47,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xe4,0xff,0xff,0xe2,0x00,0x00,0xe4,0xff,0xff,0xe2,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xe4,0xff,0xff,0xe4,0x00,0x00,0xe3,0xff,0xff,0xe4,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x49,0xe3,0xe4,0x4a,0x00,0x00,0x47,0xe3,0xe4,0x4a,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
0xfe,0xff,0xff,0x4d,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xfe,0x00,0x00,0x00,0x00,
0xee,0xff,0xff,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x5b,0xff,0xff,0xed,0x00,0x00,0x00,0x00,
0xd8,0xff,0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xff,0xff,0xd7,0x00,0x00,0x00,0x00,
0xa7,0xff,0xff,0xd5,0x00,0x00,0x00,0x00,0x00,0x00,0xce,0xff,0xff,0xa5,0x00,0x00,0x00,0x00,
0x5c,0xff,0xff,0xff,0x55,0x00,0x00,0x00,0x00,0x63,0xff,0xff,0xff,0x58,0x00,0x00,0x00,0x00,
0x09,0xe0,0xff,0xff,0xfa,0x87,0x3e,0x3d,0x90,0xfe,0xff,0xff,0xdc,0x07,0x00,0x00,0x00,0x00,
0x00,0x4b,0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0x47,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x52,0xf7,0xff,0xff,0xff,0xff,0xff,0xff,0xf6,0x4f,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x22,0x93,0xd9,0xf8,0xf8,0xd8,0x91,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3d,0xab,0xe7,0xf7,
0xd6,0x82,0x13,0x00,0x00,0x00,0x00,0x00,0x88,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0x35,0x00,
0x00,0x00,0x76,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xec,0x19,0x00,0x1d,0xf9,0xff,0xff,
0xe9,0x62,0x39,0x7b,0xfa,0xff,0xbf,0x2b,0x00,0x84,0xff,0xff,0xfd,0x31,0x00,0x00,0x00,0x52,
0x42,0x00,0x00,0x00,0xc7,0xff,0xff,0xad,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xed,
0xff,0xff,0x6e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfb,0xff,0xff,0x54,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xef,0xff,0xff,0x6d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xce,0xff,0xff,0xad,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8e,0xff,0xff,
0xfe,0x3b,0x00,0x00,0x00,0x54,0x48,0x00,0x00,0x00,0x27,0xfd,0xff,0xff,0xf0,0x72,0x36,0x5a,
0xf0,0xff,0xbe,0x27,0x00,0x00,0x87,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe8,0x16,0x00,
0x00,0x02,0x96,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0x32,0x00,0x00,0x00,0x00,0x00,0x3f,0xa8,
0xe4,0xf9,0xe0,0x97,0x1a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x9f,0x86,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xbf,0xff,0xfe,0x2e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x76,0xff,0xff,0x73,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x25,0xd7,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0xd5,0xff,0xff,0x9a,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x28,0xcf,0x58,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4b,0xe4,0xe4,0x49,0x00,0x00,0x4b,0xe4,0xe4,0x49,0x00,
0x00,0x00,0x00,0x00,0xe5,0xff,0xff,0xe4,0x00,0x00,0xe5,0xff,0xff,0xe4,0x00,0x00,0x00,0x00,
0x00,0xe3,0xff,0xff,0xe3,0x00,0x00,0xe4,0xff,0xff,0xe3,0x00,0x00,0x00,0x00,0x00,0x47,0xe3,
0xe4,0x49,0x00,0x00,0x49,0xe4,0xe4,0x49,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x6e,0xc7,0xf0,0xf7,0xda,0x91,
0x1a,0x00,0x00,0x00,0x00,0x00,0x00,0x11,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xf1,0x3b,0x00,
0x00,0x00,0x00,0x00,0xb3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xec,0x15,0x00,0x00,0x00,
0x40,0xff,0xff,0xff,0xbd,0x48,0x3b,0x90,0xff,0xff,0xff,0x8c,0x00,0x00,0x00,0x9b,0xff,0xff,
0xdd,0x07,0x00,0x00,0x00,0x9b,0xff,0xff,0xe7,0x00,0x00,0x00,0xd4,0xff,0xff,0x83,0x00,0x00,
0x00,0x00,0x39,0xff,0xff,0xff,0x1f,0x00,0x00,0xf1,0xff,0xff,0x59,0x00,0x00,0x00,0x00,0x0f,
0xff,0xff,0xff,0x3c,0x00,0x00,0xfc,0xff,0xff,0x4e,0x00,0x00,0x00,0x00,0x03,0xff,0xff,0xff,
0x47,0x00,0x00,0xf1,0xff,0xff,0x5a,0x00,0x00,0x00,0x00,0x0f,0xff,0xff,0xff,0x3b,0x00,0x00,
0xd4,0xff,0xff,0x84,0x00,0x00,0x00,0x00,0x3a,0xff,0xff,0xff,0x1e,0x00,0x00,0x9c,0xff,0xff,
0xde,0x08,0x00,0x00,0x00,0x9c,0xff,0xff,0xe5,0x00,0x00,0x00,0x41,0xff,0xff,0xff,0xbd,0x47,
0x3a,0x8f,0xff,0xff,0xff,0x89,0x00,0x00,0x00,0x00,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xeb,0x13,0x00,0x00,0x00,0x00,0x13,0xcd,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0x39,0x00,
0x00,0x00,0x00,0x00,0x00,0x06,0x71,0xc9,0xf1,0xf7,0xdb,0x91,0x1a,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x49,0xe3,0xe3,0x47,0x00,0x00,0x49,0xe3,0xe3,0x47,0x00,0x00,
0x00,0x00,0xe4,0xff,0xff,0xe2,0x00,0x00,0xe4,0xff,0xff,0xe2,0x00,0x00,0x00,0x00,0xe3,0xff,
0xff,0xe4,0x00,0x00,0xe4,0xff,0xff,0xe4,0x00,0x00,0x00,0x00,0x47,0xe3,0xe4,0x4a,0x00,0x00,
0x49,0xe3,0xe4,0x4a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,
0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0xfd,0xff,0xff,0x4c,0x00,0x00,0x00,
0x00,0x4f,0xff,0xff,0xff,0x00,0x00,0xf5,0xff,0xff,0x57,0x00,0x00,0x00,0x00,0x65,0xff,0xff,
0xff,0x00,0x00,0xdf,0xff,0xff,0x79,0x00,0x00,0x00,0x00,0x84,0xff,0xff,0xff,0x00,0x00,0xb8,
0xff,0xff,0xc2,0x00,0x00,0x00,0x02,0xd5,0xff,0xff,0xff,0x00,0x00,0x79,0xff,0xff,0xff,0x70,
0x00,0x09,0x99,0xff,0xff,0xff,0xff,0x00,0x00,0x1d,0xfa,0xff,0xff,0xff,0xed,0xf6,0xff,0xf9,
0xff,0xff,0xff,0x00,0x00,0x00,0x7f,0xff,0xff,0xff,0xff,0xff,0xe2,0x70,0xff,0xff,0xff,0x00,
0x00,0x00,0x00,0x6e,0xdd,0xf7,0xd9,0x84,0x10,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfa,0xf6,0x12,0x00,0x00,0x00,0x7a,0xff,0xf9,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xdb,0xff,0x87,0x00,0x00,0x17,0xf1,0xff,
0xda,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x9b,0xff,0xfc,0x6a,0x39,0xbe,0xff,
0xff,0x9b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x32,0xfe,0xff,0xff,0xff,0xff,
0xff,0xfe,0x32,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x81,0xff,0xff,0xff,
0xff,0xff,0x81,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x56,0xce,
0xf6,0xce,0x56,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x09,0x49,
0x6c,0x7a,0x63,0x27,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x8d,0xf5,
0xff,0xff,0xff,0xff,0xff,0xbe,0x25,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x22,0xdb,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xed,0x3e,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0xd7,0xff,
0xff,0xff,0xff,0xd5,0xbe,0xf7,0xff,0xff,0xff,0xe9,0x11,0x00,0x00,0x00,0x00,0x00,0x8b,0xff,
0xff,0xff,0xf1,0x4b,0x00,0x00,0x1d,0xc6,0xff,0xff,0xd5,0x44,0x00,0x00,0x00,0x00,0x13,0xf7,
0xff,0xff,0xff,0x4e,0x00,0x00,0x00,0x00,0x15,0x7d,0x29,0x00,0x00,0x00,0x00,0x00,0x00,0x6c,
0xff,0xff,0xff,0xc6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xb1,0xff,0xff,0xff,0x6b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xdd,0xff,0xff,0xff,0x2f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0xf6,0xff,0xff,0xff,0x0c,0x00,0x00,0x00,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,
0x00,0x00,0x00,0xfd,0xff,0xff,0xff,0x03,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0x00,0x00,0x00,0xf3,0xff,0xff,0xff,0x0f,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0x00,0x00,0x00,0xda,0xff,0xff,0xff,0x30,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xee,0x00,0x00,0x00,0xb1,0xff,0xff,0xff,0x66,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xd2,0xff,0xff,0xd7,0x00,0x00,0x00,0x72,0xff,0xff,0xff,0xb4,0x00,0x00,0x00,0x00,0x00,0x00,
0x03,0xf0,0xff,0xff,0xb8,0x00,0x00,0x00,0x1c,0xfd,0xff,0xff,0xfc,0x27,0x00,0x00,0x00,0x00,
0x00,0x42,0xff,0xff,0xff,0x77,0x00,0x00,0x00,0x00,0xa9,0xff,0xff,0xff,0xc7,0x0b,0x00,0x00,
0x00,0x07,0xca,0xff,0xff,0xfc,0x1c,0x00,0x00,0x00,0x00,0x23,0xf5,0xff,0xff,0xff,0xd3,0x5b,
0x37,0x5b,0xcd,0xff,0xff,0xff,0x98,0x00,0x00,0x00,0x00,0x00,0x00,0x5a,0xfe,0xff,0xff,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xda,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0xee,0xff,
0xff,0xff,0xff,0xff,0xff,0xff,0xca,0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x12,
0x7a,0xc8,0xec,0xfa,0xe6,0xb2,0x59,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf9,0xf5,0x11,0x00,0x00,0x00,0x6f,0xff,0xf9,0x00,0x00,
0x00,0x00,0x00,0x00,0xdb,0xff,0x88,0x00,0x00,0x13,0xec,0xff,0xda,0x00,0x00,0x00,0x00,0x00,
0x00,0x9b,0xff,0xfc,0x6c,0x38,0xbb,0xff,0xff,0x9b,0x00,0x00,0x00,0x00,0x00,0x00,0x32,0xfe,
0xff,0xff,0xff,0xff,0xff,0xfe,0x32,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x81,0xff,0xff,0xff,
0xff,0xff,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0xcd,0xf6,0xce,0x54,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x36,0xb7,0xf0,0xeb,0xa6,0x26,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x45,0xf9,0xff,0xff,0xff,0xff,0xf3,0x44,0xff,0xff,0xff,0x4c,0x00,0x00,0x07,0xe5,0xff,
0xff,0xff,0xff,0xff,0xff,0xee,0xff,0xff,0xff,0x4c,0x00,0x00,0x5a,0xff,0xff,0xff,0xd2,0x54,
0x48,0xaf,0xff,0xff,0xff,0xff,0x4c,0x00,0x00,0xa5,0xff,0xff,0xea,0x13,0x00,0x00,0x01,0xb8,
0xff,0xff,0xff,0x4c,0x00,0x00,0xd6,0xff,0xff,0x8f,0x00,0x00,0x00,0x00,0x48,0xff,0xff,0xff,
0x4c,0x00,0x00,0xf2,0xff,0xff,0x5d,0x00,0x00,0x00,0x00,0x13,0xff,0xff,0xff,0x4c,0x00,0x00,
0xfc,0xff,0xff,0x4f,0x00,0x00,0x00,0x00,0x04,0xff,0xff,0xff,0x4c,0x00,0x00,0xf4,0xff,0xff,
0x61,0x00,0x00,0x00,0x00,0x13,0xff,0xff,0xff,0x4c,0x00,0x00,0xdb,0xff,0xff,0x9a,0x00,0x00,
0x00,0x00,0x48,0xff,0xff,0xff,0x4c,0x00,0x00,0xac,0xff,0xff,0xf4,0x1f,0x00,0x00,0x01,0xb7,
0xff,0xff,0xff,0x4c,0x00,0x00,0x5f,0xff,0xff,0xff,0xdd,0x52,0x47,0xb6,0xff,0xff,0xff,0xff,
0x4c,0x00,0x00,0x08,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0x4a,0xfa,0xff,0xff,0xff,0xff,0xee,0x38,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x3a,
0xba,0xf1,0xeb,0xa3,0x22,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x0b,0x1c,0x1c,0x0d,0x00,0x00,
0x00,0x00,0x05,0xff,0xff,0xff,0x47,0x00,0x00,0x4f,0xff,0xff,0xb2,0x00,0x00,0x00,0x00,0x46,
0xff,0xff,0xff,0x2a,0x00,0x00,0x10,0xf1,0xff,0xff,0xa0,0x45,0x3a,0x70,0xeb,0xff,0xff,0xde,
0x00,0x00,0x00,0x00,0x79,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x5c,0x00,0x00,0x00,
0x00,0x01,0xa3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x87,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x58,0xbd,0xef,0xfa,0xe4,0xa8,0x3d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x49,0xe3,0xe3,
0x47,0x00,0x00,0xe4,0xff,0xff,0xe3,0x00,0x00,0xe3,0xff,0xff,0xe3,0x00,0x00,0x47,0xe3,0xe4,
0x49,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x4c,0xff,0xff,
0xff,0x00,0x00,0x4c,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,
0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,
0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,
0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,
0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,
0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0xff,0xff,0xff,0x4c,0x00,0x00,0x00,
0xff,0xff,0xff,0x4c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x15,0x7b,0xc6,0xef,0xfa,0xe6,0xad,0x46,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x4e,0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaa,0x0a,0x00,0x00,
0x00,0x00,0x2e,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb4,0x00,0x00,0x00,0x00,
0xaa,0xff,0xff,0xfc,0x8e,0x46,0x37,0x5d,0xd4,0xff,0xff,0xff,0x52,0x00,0x00,0x00,0xea,0xff,
0xff,0x84,0x00,0x00,0x00,0x00,0x11,0x9d,0x72,0x41,0x0e,0x00,0x00,0x00,0xf9,0xff,0xff,0x5a,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xda,0xff,0xff,0xd2,0x1a,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7d,0xff,0xff,0xff,0xf2,0x8d,0x24,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xb0,0xff,0xff,0xff,0xff,0xff,0xc3,0x60,0x05,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x72,0xed,0xff,0xff,0xff,0xff,0xff,0xdc,0x47,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0c,0x69,0xd0,0xff,0xff,0xff,0xff,0xfe,0x71,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x33,0xa1,0xfc,0xff,0xff,0xfe,0x40,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x35,0xf3,0xff,0xff,0xb1,0x00,0x00,0x00,0x00,0x12,
0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x77,0xff,0xff,0xe4,0x00,0x00,0x75,0xe5,0xff,0xe4,0x04,
0x00,0x00,0x00,0x00,0x00,0x5a,0xff,0xff,0xf8,0x00,0x00,0x61,0xff,0xff,0xff,0x8a,0x00,0x00,
0x00,0x00,0x00,0xaf,0xff,0xff,0xdb,0x00,0x00,0x0a,0xe7,0xff,0xff,0xff,0xb2,0x53,0x38,0x53,
0xb5,0xff,0xff,0xff,0x8d,0x00,0x00,0x00,0x48,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
0xff,0xe4,0x13,0x00,0x00,0x00,0x00,0x4a,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd3,0x25,
0x00,0x00,0x00,0x00,0x00,0x00,0x12,0x78,0xed,0xff,0xff,0xff,0xb8,0x62,0x06,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x1d,0xfc,0xff,0xff,0x2c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x0f,0xf4,0xff,0xff,0x51,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x38,0xd8,0xff,0xff,0xdc,0x0c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2e,
0xf1,0xff,0xb5,0x1b,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0x40,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x20,0x99,0xdd,0xf9,0xf2,0xbf,0x4c,0x00,0x00,0x00,0x00,0x00,0x31,0xf0,0xff,0xff,0xff,
0xff,0xff,0xff,0x81,0x00,0x00,0x00,0x00,0xbf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x46,
0x00,0x00,0x00,0xf7,0xff,0xff,0xb0,0x3b,0x49,0xd8,0xff,0xff,0xb4,0x00,0x00,0x00,0xf5,0xff,
0xff,0x5f,0x00,0x00,0x41,0x7c,0x50,0x21,0x00,0x00,0x00,0xbb,0xff,0xff,0xec,0x68,0x0d,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x36,0xf5,0xff,0xff,0xff,0xf3,0x94,0x1e,0x00,0x00,0x00,0x00,
0x00,0x00,0x32,0xca,0xff,0xff,0xff,0xff,0xf5,0x64,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,
0xc2,0xff,0xff,0xff,0xfe,0x4e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0xdc,0xff,0xff,
0xc3,0x00,0x00,0x36,0x83,0xae,0x85,0x00,0x00,0x00,0x5f,0xff,0xff,0xef,0x00,0x00,0x4e,0xff,
0xff,0xfe,0x85,0x39,0x43,0xbe,0xff,0xff,0xf1,0x00,0x00,0x04,0xd8,0xff,0xff,0xff,0xff,0xff,
0xff,0xff,0xff,0xa2,0x00,0x00,0x00,0x26,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xcc,0x13,0x00,
0x00,0x00,0x00,0x1c,0x95,0xff,0xff,0xff,0xe2,0x6b,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x67,0xff,0xff,0x86,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x59,0xff,0xff,0xa1,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x68,0xf4,0xff,0xff,0x47,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x64,0xff,0xef,0x57,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x59,0x15,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  };
  static const struct {
    uint32_t ds;
    uint16_t width;
  } glinfo[]= {

{0,8}, {272,6}, {476,12}, {884,21}, 
{1598,14}, {2074,19}, {2720,17}, {3298,7}, 
{3536,9}, {3842,9}, {4148,15}, {4658,12}, 
{5066,6}, {5270,12}, {5678,6}, {5882,13}, 
{6324,16}, {6868,16}, {7412,16}, {7956,16}, 
{8500,16}, {9044,16}, {9588,16}, {10132,16}, 
{10676,16}, {11220,16}, {11764,6}, {11968,6}, 
{12172,11}, {12546,13}, {12988,11}, {13362,14}, 
{13838,23}, {14620,19}, {15266,17}, {15844,18}, 
{16456,17}, {17034,17}, {17612,16}, {18156,19}, 
{18802,17}, {19380,7}, {19618,16}, {20162,18}, 
{20774,16}, {21318,23}, {22100,18}, {22712,18}, 
{23324,16}, {23868,18}, {24480,17}, {25058,16}, 
{25602,18}, {26214,18}, {26826,17}, {27404,23}, 
{28186,17}, {28764,17}, {29342,19}, {29988,10}, 
{30328,13}, {30770,10}, {31110,14}, {31586,16}, 
{32130,7}, {32368,14}, {32844,15}, {33354,14}, 
{33830,15}, {34340,14}, {34816,11}, {35190,15}, 
{35700,15}, {36210,7}, {36448,9}, {36754,14}, 
{37230,7}, {37468,21}, {38182,15}, {38692,14}, 
{39168,15}, {39678,16}, {40222,12}, {40630,13}, 
{41072,9}, {41378,14}, {41854,14}, {42330,19}, 
{42976,14}, {43452,14}, {43928,14}, {44404,9}, 
{44710,7}, {44948,11}, {45322,10}, {45662,17}, 
{46240,18}, {46852,18}, {47464,13}, {47906,15}, 
{48416,14}, {48892,19}, {49538,15}, {50048,6}, 
{50252,7}, {50490,16}, {51034,13}, 
  };
  // static const int nmap= sizeof(glmap)/sizeof(glmap[0]);
  int i,j,k,e,x,y,sX,eX;
  int W, X, *Xpos;
  char buf[20];
  int *map, *gl;
  int ngl;
  unsigned int C;
  int glyph_width;
  const unsigned char *glyph_data;
  unsigned char *data;

  map= alloca(L*sizeof(int));

  e= 0;
  ngl= 0;
  for(i=0;i<L;i++)
  {
    C= str[i];
    
    if (C&(1<<31)) { ngl+=2; e= 1; continue; }
    map[i]= font_hash(C);
    if (map[i]!=107) { ngl++; continue; }
    sprintf(buf,"%x",C); ngl+= strlen(buf); e= 1;
  }

  if (e) goto fix_encoding_errors;
  else gl= map;

encoding_errors_fixed:

  W= 0;
  for(i=0;i<ngl;i++) W+= glinfo[gl[i]].width;

  if (Rdata) data= malloc(W*34);
  X= 0;
  if (e && Rdata) Xpos= alloca(sizeof(int)*(ngl+1));
  for(i=0;i<ngl;i++)
  {
    if (e && Rdata) Xpos[i]= X;
    glyph_width= glinfo[gl[i]].width;
    if (Rdata) {
      glyph_data= font_data + glinfo[gl[i]].ds;
      for(y=0;y<34;y++)
         for(x=0;x<glyph_width;x++)
            data[y*W+X+x]= glyph_data[y*glyph_width+x];
    }
    X+= glyph_width;
  }
  if (e && Rdata) Xpos[i]= X;
  if (RW) *RW= W;
  if (RA) *RA= 27;
  if (RH) *RH= 34;
  if (Rdata) *Rdata= data;

  if (!e || !Rdata) return ;

  j= 0;
  for(i=0;i<L;i++)
  {
    if (str[i]&(1<<31))
    {
      sX= Xpos[j]; eX= Xpos[j+2];
      for(x=sX;x<eX;x++)
        for(y=0;y<34;y++)
          data[y*W + x] ^= 0xff;
      j+= 2;
      continue;
    }
    if (map[i]==107)
    {
      sprintf(buf,"%x",str[i]);
      sX= Xpos[j]; eX= Xpos[j+strlen(buf)];
      for(x=sX;x<eX;x++) {
        data[ 0*W + x]= 0xff;
        data[ (34-1)*W + x]= 0xff;
      }
      for(y=0;y<34;y++)
      {
        data[y*W+sX] = 0xff;
        data[y*W+eX-1] = 0xff;
      }
      j+= strlen(buf);
      continue;
    }
    j++;
  }
  return ;


fix_encoding_errors:
  j= 0;
  gl= alloca(sizeof(int)*ngl);
  for(i=0;i<L;i++)
  {
    if (str[i]&(1<<31))
    {
      sprintf(buf,"%02x", str[i]&0xff);
      for(k=0;buf[k];k++)
         gl[j++]= font_hash(buf[k]);
      continue;
    }
    if (map[i]==107)
    {
      sprintf(buf,"%x", str[i]);
      for(k=0;buf[k];k++)
         gl[j++]= font_hash(buf[k]);
      continue;
    }
    gl[j++]= map[i];
  }
  goto encoding_errors_fixed;
}

#line 23 "/home/dodo/tmp/gfx/android/../src/event.u"

static click_handler_t *click_handlers;
static int n_click_handler, s_click_handler;
static click_handler_t release_handler;
static click_handler_t motion_handler;

#line 25 "/home/dodo/tmp/gfx/android/../src/widget.u"

static gui_widget_t **gui_widgets;
static int n_gui_widget, s_gui_widget;

#line 25 "/home/dodo/tmp/gfx/android/../src/button.u"

static uint32_t button_border_s= 0xaaaaa3, button_border,
                button_light_s=  0xffffff, button_light,
                button_dark_s=   0xd5d5d1, button_dark,
                button_grad1_s=  0xfcfcfb, button_grad1,
                button_grad2_s=  0xdbdbd7, button_grad2,
                button_grad3_s=  0xcccac9, button_grad3,
                button_grad4_s=  0xe9e8e7, button_grad4,
                button_fg_s=     0,        button_fg; 

#line 20 "/home/dodo/tmp/gfx/android/../src/menu-fw.u"

static uint32_t  fwmenu_bg_color= 0xffffff,
                 fwmenu_sep_color= 0x888888,
                 fwmenu_fg_color= 0x000000,
                 fwmenu_scrpad= 20,
                 fwmenu_intpad= 5;

#line 15 "/home/dodo/tmp/gfx/android/../src/label.u"

static int n_labels, s_labels;
static gui_label_t *labels;


#line 24 "/home/dodo/tmp/gfx/android/../src/screen.u"

static screen_t *screens;
static int n_screen, s_screen;
static int click_handler_start; /// from now on, the click handler
                                //  will only look at handlers beyond
                                //  this entry.
#line 149 "/home/dodo/tmp/gfx/android/basics.u"

/***********************************************************************
 *#M.2                                                                 *
 *                             Prototypes                              *
 *                                                                     *
 ***********************************************************************/
typedef struct req request_t;
typedef struct app app_t;

static void* worker_entry(void* arg);
static void onStop(ANativeActivity* activity);
static void onPause(ANativeActivity* activity);
static void onResume(ANativeActivity* activity);
static void onStart(ANativeActivity* activity);
static void onNativeWindowCreated
              (ANativeActivity* activity, ANativeWindow* window);
static void onNativeWindowDestroyed
              (ANativeActivity* activity, ANativeWindow* window);
static void onNativeWindowRedrawNeeded
              (ANativeActivity* activity, ANativeWindow* window);
static void onInputQueueCreated
              (ANativeActivity* activity, AInputQueue* queue);
static void onInputQueueDestroyed
              (ANativeActivity* activity, AInputQueue* queue);
static void* onSaveInstanceState
              (ANativeActivity* activity, size_t* outLen);
static void onWindowFocusChanged
              (ANativeActivity* activity, int focused);
static void onLowMemory(ANativeActivity* activity);
static void onConfigurationChanged(ANativeActivity* activity);

static void make_request(app_t *A, request_t *R);
static void complete_request(app_t *A, request_t *R);
static int get_request(app_t *A,request_t **R);
static int execute_commands(app_t *app);

static void graphics_window_destroyed(app_t *A, ANativeWindow *window);
static void graphics_window_created(app_t *A, ANativeWindow *window);

static int handle_input(app_t *A, gfx_event_t *ev);

#line 5792 "gfx.c"

static void *grow_array(int N, int *S, void *ptr, size_t eltsiz);
static int range_test(int v, int s, int w) ;
static int imin2(int a,int b) ;
static int imax2(int a,int b) ;
static int imin3(int a,int b,int c) ;
static int imax3(int a,int b,int c) ;
static double dmin2(double a,double b) ;
static double dmax2(double a,double b) ;
static int utf8_encode_string
    (const uint32_t* wstr,size_t wlen, uint8_t *dst, size_t *dlen);
static uint32_t utf8_decode_single_char(char *src);
static void utf8_encode_single_char(uint32_t wide, char *dst);
static void utf8_decode_alloc(char *str, uint32_t **Rwstr, uint32_t *Rwlen);
static void clip_for_rect(int *start, int *amount, int max);
static void get_text_data(int font, char *str, 
                          int *Rw, int *Rh,int *Ra,unsigned char **Rd);
static void gfx_invert_hline(int x1,int x2, int y);
static void gfx_invert_vline(int x,int y1, int y2);
static uint32_t blend_usr_usr(uint32_t alpha, uint32_t fg, uint32_t bg);
static uint32_t blend_usr_usr_usr(uint32_t alpha, uint32_t fg, uint32_t bg);
static uint32_t blend_usr_svr(uint32_t alpha, uint32_t fg, uint32_t bg);
static void blend_graymap(int flags, int fg, int bg,
                          unsigned char *src, int src_w, int src_h,
                          int src_x, int src_y,
                          int dst_x, int dst_y, int width, int height);
static uint32_t color_s(uint32_t uR, uint32_t uG, uint32_t uB);
static uint32_t u_red(uint32_t C) ;
static uint32_t u_green(uint32_t C) ;
static uint32_t u_blue(uint32_t C) ;
static uint32_t color_u_to_s(uint32_t U);
static unsigned int blend_chan(unsigned int F, double A, unsigned int B);
static unsigned int blend(uint32_t F, double A, uint32_t B);
static void grad_fill(int x,int y, int w, int h, uint32_t start, uint32_t end);
static void hline(int x1,int x2, int y,uint32_t color);
static void vline(int x,int y1, int y2,uint32_t color);
static uint32_t *get_server_pix(int src_x,int src_y,int w,int h);
static void put_server_pix(uint32_t *D,int dst_x,int dst_y,int w, int h);
static void gradfill_left(uint32_t *dst, int dst_w, int dst_h,
                          uint32_t start, uint32_t end,
                          int fill_x, int fill_y,
                          int fill_w, int fill_h);
static void hline_buffer(uint32_t *dst, int dst_w, int dst_h,
                         int x0, int x1, int y, uint32_t color);
static void vline_buffer(uint32_t *dst, int dst_w, int dst_h,
                         int x, int y0, int y1, uint32_t color);
static void shadow_box(uint32_t *dst, int dst_w, int dst_h,
                       uint32_t light, uint32_t dark, uint32_t border,
                       int box_x, int box_y, int box_w, int box_h);
static void blend_graymap_buffer
        (unsigned int fg, unsigned char *src, int src_w, int src_h,
                          int src_x, int src_y,
                          unsigned int *dst, int dst_w, int dst_h,
                          int dst_x, int dst_y, int width, int height);
static void install_click_handler
        (int x,int y,int w,int h,int *enabled,
         void *arg, int (*func)(void*,gfx_event_t*));
static void install_release_handler
        (void *arg, int (*func)(void*,gfx_event_t*));
static void install_motion_handler(void *arg,int (*func)(void *,gfx_event_t*));
static void uninstall_motion_handler();
static int handle_click_event(gfx_event_t *event);
static int handle_release_event(gfx_event_t *event);
static int handle_motion_event(gfx_event_t *event);
static int register_widget(void *A,int type,int x,int y,int w,int h);
static void *get_widget(int number, int type);
static void widget_destroy(int number);
static void init_button_colors();
static void button_render(int x,int y,int w,int h,
                              int tw, int th, unsigned char *data,
                              uint32_t grad1, uint32_t grad2,
                              uint32_t light, uint32_t dark);
static void button_set_state(gui_button_t *B, int state);
static int gui_button_click(void *arg, gfx_event_t *event);
static int gui_button_release(void *arg, gfx_event_t *event);
static void draw_label(int flags, gui_label_t *G, char *str);
#line 193 "/home/dodo/tmp/gfx/android/basics.u"

/***********************************************************************
 *##B                                                                  *
 *                         Application Object                          *
 *                                                                     *
 ***********************************************************************
  The application object is called an 'instance' in the android documents.
  It contains the necessary data to properly handle and serialize events.
  Window handle etc. are also contained in this object.
 ***********************************************************************/


struct app {
      // command pipe. This is used for reporting system events such
      // as Pause, Resume, etc.
  int cmdpipe_r, cmdpipe_w;

      // The worker thread is actually the 'main' thread for our
      // application. It initializes the input system and then calls
      // android_main. When that function is done, it rips up the
      // input/output objects and quits.
  pthread_t worker_thread;

      // These two are used for signalling various conditions. 
  pthread_mutex_t mutex;
  pthread_cond_t cond;

      // This is one of the conditions. When this is set to 1, 
      // the thread which fired off the worker thread can continue
      // in its Java world.
  int running;

  void *init_state; size_t init_state_size;

  ANativeActivity *activity;
  ALooper *looper;
  AConfiguration *config;
  AInputQueue *inputq;
  ANativeWindow *window;

  request_t *quit_request;
  int last_pointer_id;
};

static app_t *my_app;


/***********************************************************************
 *##C                                                                  *
 *                          Application Init                           *
 *                                                                     *
 ***********************************************************************
  During the initialization, we need to do the following:
   - create a pipe for the asynchronous messages coming in
   - create a mutex and a condition variable for protecting shared memory
   - create a worker thread

  The onCreate function is the entry point from the Java world. We 
  initialize the 'activity' object with the system event handler functions.
 ***********************************************************************/

void ANativeActivity_onCreate
      (ANativeActivity* activity, void* state, size_t state_size)
{
  int cmdpipe[2];
  pthread_attr_t attr; 
  app_t *app;

  LOGV("Creating: %p\n", activity);

  // #C.1: Activity callback installation
  // activity->callbacks->onDestroy = onDestroy;
  activity->callbacks->onStart = onStart;
  activity->callbacks->onResume = onResume;
  activity->callbacks->onPause = onPause;
  activity->callbacks->onStop = onStop;
  activity->callbacks->onNativeWindowCreated = onNativeWindowCreated;
  activity->callbacks->onNativeWindowDestroyed = onNativeWindowDestroyed;
  activity->callbacks->onInputQueueCreated = onInputQueueCreated;
  activity->callbacks->onInputQueueDestroyed = onInputQueueDestroyed;
  activity->callbacks->onSaveInstanceState = onSaveInstanceState;
  activity->callbacks->onWindowFocusChanged = onWindowFocusChanged;
  activity->callbacks->onLowMemory = onLowMemory;
  activity->callbacks->onConfigurationChanged = onConfigurationChanged;

  ANativeActivity_setWindowFormat(activity, WINDOW_FORMAT_RGBX_8888);

  activity->instance= my_app= app= calloc(1,sizeof(*app));
  app->activity= activity;
  app->last_pointer_id= -1;

  if (state && state_size)
  {
    app->init_state= malloc(state_size);
    memcpy(app->init_state, state, state_size);
    app->init_state_size= state_size;
  }

  pthread_mutex_init(&app->mutex, NULL);
  pthread_cond_init(&app->cond,NULL);

  if (pipe(cmdpipe))
  {
    LOGE("Couldn't create pipe: %s\n",strerror(errno));
    return ;
  }
  app->cmdpipe_r= cmdpipe[0];
  app->cmdpipe_w= cmdpipe[1];

  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
  pthread_create(&app->worker_thread, &attr, worker_entry, app);

  
  pthread_mutex_lock(&app->mutex); 
  while(!app->running) pthread_cond_wait(&app->cond, &app->mutex); 
  pthread_mutex_unlock(&app->mutex);
}

  // #C.2 Worker Function
static void* worker_entry(void* arg)
{
  app_t *app;

  app= arg;

  app->config= AConfiguration_new();
  AConfiguration_fromAssetManager(app->config, app->activity->assetManager);

  app->looper= ALooper_prepare(ALOOPER_PREPARE_ALLOW_NON_CALLBACKS);
  ALooper_addFd(app->looper, app->cmdpipe_r,
                PIPE_CMD, ALOOPER_EVENT_INPUT, NULL, app);
  
  pthread_mutex_lock(&app->mutex); 
  app->running=1; 
  pthread_cond_broadcast(&app->cond);
  pthread_mutex_unlock(&app->mutex);

  while(!app->inputq || !app->window)
    if (execute_commands(app)) goto done;

  init_button_colors();
  android_main(app->init_state, app->init_state_size);

  while(!app->quit_request)
     if (execute_commands(app))
       break;

done:
  //free_saved_state(A);
  if (app->inputq) 
      { AInputQueue_detachLooper(app->inputq); app->inputq= NULL; }
  AConfiguration_delete(app->config);
  app->config= NULL;

  if (app->quit_request)
     complete_request(app, app->quit_request);
  return NULL;
}


/***********************************************************************
 *##D                                                                  *
 *                          System Callbacks                           *
 *                                                                     *
 ***********************************************************************
  System callbacks are things like onResume, onWindowCreated etc. These
  are supposed to be handled in a separate thread but that makes things
  complicated. So, I'm going to serialize them into a pipe and let the
  main thread do everything. The google guys made sure that these events
  are acknowledged by the main thread before returning from the callback.
  So, I'm going to do the same thing and include an 'acknowledgement'
  field in each request. This might be important for adding event queues
  to loopers etc. 

  Requests are represented by the following struct. Only a pointer to 
  this struct is passed around since everything is in the same address
  space.
 ***********************************************************************/

struct req {
  int type;
  int complete;
  union {
    ANativeWindow *window;
    AInputQueue   *inputq;
    struct {
       void* ptr;
       size_t size;
    }             buffer; 
    int           intv;
  } d;
};

/***********************************************************************
  The following functions send, receive and acknowledge requests.
 ***********************************************************************/

static void make_request(app_t *A, request_t *R)
{
   R->complete= 0;
   if (write(A->cmdpipe_w, &R, sizeof(R))!=sizeof(R))
   {
     LOGE("Couldn't write to command pipe: %s\n", strerror(errno));
     return ;
   }
   
   pthread_mutex_lock(&A->mutex); 
   while(!R->complete) pthread_cond_wait(&A->cond, &A->mutex); 
   pthread_mutex_unlock(&A->mutex); 
}

static int get_request(app_t *A,request_t **R)
{
   request_t *T;
   if (read(A->cmdpipe_r, &T, sizeof(T))!=sizeof(T))
   {
     LOGE("Couldn't read from command pipe: %s\n", strerror(errno));
     return 1;
   }
   *R= T;
   return 0;
}

static void complete_request(app_t *A, request_t *R)
{
   pthread_mutex_lock(&A->mutex); 
   R->complete= 1;
   pthread_cond_broadcast(&A->cond);
   pthread_mutex_unlock(&A->mutex); 
}


/***********************************************************************
 * #D.1                                                                *
 *                          Application State                          *
 *                                                                     *
 ***********************************************************************/

static void simple_command(ANativeActivity *ac, int cmd)
{
   request_t R;
   R.type= cmd;
   make_request(ac->instance, &R);
}

static void onStart(ANativeActivity* activity)
{
   LOGV("Start: %p\n", activity);
   simple_command(activity, CMD_APP_START);
}

static void onResume(ANativeActivity* activity)
{
   LOGV("Resume: %p\n", activity);
   simple_command(activity, CMD_APP_RESUME);
}

static void onPause(ANativeActivity* activity)
{
   LOGV("Pause: %p\n", activity);
   simple_command(activity, CMD_APP_PAUSE);
}

static void onStop(ANativeActivity* activity)
{
   LOGV("Stop: %p\n", activity);
   simple_command(activity, CMD_APP_STOP);
}

/***********************************************************************
 *#D.2                                                                 *
 *                       System Object Handling                        *
 *                                                                     *
 ***********************************************************************
  The system notifies the availability of various objects such as the
  native window, input queue etc. thru callbacks. It's quite stupid, but
  what can you do? The destruction of these objects are also reported
  in the same way.
 
  It looks like the default glue code doesn't handle all of the callbacks.
  These are: 

  void (*onNativeWindowResized)
        (ANativeActivity* activity, ANativeWindow* window);
  void (*onContentRectChanged)
        (ANativeActivity* activity, const ARect* rect);

  WindowResized and ContentRectChanged are called when the device is
  rotated, as far as I understand. WindowRedrawNeeded is called when
  the application is brought back to foreground. I will do these after 
  I have the graphics system.
 ***********************************************************************/

static void onNativeWindowCreated
   (ANativeActivity* activity, ANativeWindow* window)
{
  request_t R;
  LOGV("NativeWindowCreated: %p -- %p\n", activity, window);
  R.type= CMD_WINDOW_CREATED;
  R.d.window= window;
  make_request(activity->instance, &R);
}

static void onNativeWindowDestroyed
   (ANativeActivity* activity, ANativeWindow* window)
{
  request_t R;
  LOGV("NativeWindowDestroyed: %p -- %p\n", activity, window);
  R.type= CMD_WINDOW_DESTROYED;
  R.d.window= window;
  make_request(activity->instance, &R);
}

static void onNativeWindowRedrawNeeded
              (ANativeActivity* activity, ANativeWindow* window)
{
  request_t R;
  LOGV("NativeWindowRedraw: %p -- %p\n", activity, window);
  R.type= CMD_WINDOW_REDRAW;
  R.d.window= window;
  make_request(activity->instance, &R);
}

static void onInputQueueCreated
   (ANativeActivity* activity, AInputQueue* queue)
{
  request_t R;
  LOGV("InputQueueCreated: %p -- %p\n", activity, queue);
  R.type= CMD_INPUTQ_CREATED;
  R.d.inputq= queue;
  make_request(activity->instance, &R);
}

static void onInputQueueDestroyed
   (ANativeActivity* activity, AInputQueue* queue)
{
  request_t R;
  LOGV("InputQueueDestroyed: %p -- %p\n", activity, queue);
  R.type= CMD_INPUTQ_DESTROYED;
  R.d.inputq= queue;
  make_request(activity->instance, &R);
}

static void* onSaveInstanceState
   (ANativeActivity* activity, size_t* outLen)
{
  request_t R;
  LOGV("SaveInstanceState: %p\n", activity);
  R.type= CMD_SAVE_STATE;
  R.d.buffer.ptr= NULL;
  R.d.buffer.size= 0;
  make_request(activity->instance, &R);
  if (R.d.buffer.ptr)
    *outLen= R.d.buffer.size;
  return R.d.buffer.ptr;
}

static void onWindowFocusChanged
   (ANativeActivity* activity, int focused)
{
  request_t R;
  LOGV("WindowFocusChanged: %p -- %d\n", activity, focused);
  R.type= CMD_FOCUS_CHANGE;
  R.d.intv= focused;
  make_request(activity->instance, &R);
}

static void onLowMemory(ANativeActivity* activity)
{
  request_t R;
  LOGV("LowMemory: %p\n", activity);
  R.type= CMD_LOW_MEMORY;
  make_request(activity->instance, &R);
}

static void onConfigurationChanged(ANativeActivity* activity)
{
  request_t R;
  LOGV("ConfigurationChanged: %p\n", activity);
  R.type= CMD_CONFIG_CHANGE;
  make_request(activity->instance, &R);
}


/***********************************************************************
 *#D.3                                                                 *
 *                           Command Handler                           *
 *                                                                     *
 ***********************************************************************
  The following function handles the various commands generated by the
  callbacks. Some things are still missing, such as reporting the 
  application state etc. I will do these later on.

  The function returns 1 if the received command causes us to quit the
  application. This happens only if we get a destroy callback. This 
  callback isn't implemented yet.
 ***********************************************************************/

static int handle_command(app_t *A)
{

  request_t *R;
  if (get_request(A, &R)) return 0;
  switch(R->type)
  {
  case CMD_APP_STOP:
  case CMD_APP_START:
  case CMD_APP_PAUSE:
  case CMD_APP_RESUME:
     // FIXME: inform the client, maybe??
     complete_request(A,R);
     break;
  case CMD_WINDOW_DESTROYED:
     graphics_window_destroyed(A, R->d.window);
     complete_request(A,R);
     break;
  case CMD_WINDOW_CREATED:
     graphics_window_created(A, R->d.window);
     complete_request(A,R);
     gfx_update_all();
     break;
  case CMD_WINDOW_REDRAW:
     gfx_update_all();
     break;
  case CMD_SAVE_STATE:
  case CMD_FOCUS_CHANGE:
  case CMD_LOW_MEMORY:
     complete_request(A,R);
     break;
  case CMD_CONFIG_CHANGE:
     LOGV("APP_CMD_CONFIG_CHANGED\n");
     AConfiguration_fromAssetManager(A->config, A->activity->assetManager);
     complete_request(A,R);
     break;
  case CMD_INPUTQ_DESTROYED:
     if (A->inputq!=R->d.inputq)
     {
       complete_request(A,R);
       break;
     }
     AInputQueue_detachLooper(A->inputq);
     A->inputq= NULL;
     complete_request(A,R);
     break;
  case CMD_INPUTQ_CREATED:
     if (A->inputq!=NULL)
        AInputQueue_detachLooper(A->inputq);
     A->inputq= R->d.inputq;
     AInputQueue_attachLooper(A->inputq, A->looper, PIPE_EVENT, NULL,A);
     complete_request(A,R);
     break;
  case CMD_QUIT:
     // FIXME: when do we get this?
     LOGV("APP_CMD_QUIT\n");
     A->quit_request= R;
     return 1;
  }
  return 0;
}

/***********************************************************************
 *#D.4                                                                 *
 *                            Command Loop                             *
 *                                                                     *
 ***********************************************************************
  We need a function which simply waits for commands and ignores events.
  This is necessary when we're initalizing or destroying the application
  object. The function returns 1 if a request to quit the application was
  received. Otherwise, it returns 0.
 ***********************************************************************/

static int execute_commands(app_t *app)
{
   int ident;
   int dFD, dEV;
   void *dPTR;

again:
   ident= ALooper_pollAll(-1,  &dFD, &dEV,  &dPTR);

   switch(ident)
   {
     // somebody might have woke us up because the looper
     // has changed and we need to poll again.
   case ALOOPER_POLL_WAKE:
      goto again;
   case PIPE_CMD:
      if (handle_command(app)) return 1;
      return 0;
   case PIPE_EVENT:
     handle_input(app, NULL);
     goto again;
   }

   // other things are not possible. maybe I can add some
   // sort of a time-out later.
   // so, unhandled stuff corresponds to an error, which
   // makes the application quit. I think it's OK.
   return 1;
}

/***********************************************************************
 *#M.3                                                                 *
 *                            Unimplemented                            *
 *                                                                     *
 ***********************************************************************
  The following stuff isn't used yet. I will incorporate them later,
  when I have most of the stuff working properly.

  The destruction function is supposedly executed by the 'main' thread,
  which is the thread that called onCreate initially.
 ***********************************************************************/

#if 0
static void onDestroy(ANativeActivity* activity)
{
  request_t R;
  app_t *A;

  LOGV("Destroy: %p\n", activity);

  A= activity->instance;
  R.type= CMD_QUIT;
  make_request(A, &R);

  close(A->cmdpipe_r);
  close(A->cmdpipe_w);
  pthread_cond_destroy(&A->cond);
  pthread_mutex_destroy(&A->mutex);
  free(A);
}

static void free_saved_state(struct android_app* android_app) {
    pthread_mutex_lock(&android_app->mutex);
    if (android_app->savedState != NULL) {
        free(android_app->savedState);
        android_app->savedState = NULL;
        android_app->savedStateSize = 0;
    }
    pthread_mutex_unlock(&android_app->mutex);
}

#endif

/***********************************************************************
 *##E                                                                  *
 *                           Basic Graphics                            *
 *                                                                     *
 ***********************************************************************
  In contrast to the input system, the output system is quite simple.
  When you want to put something on the screen, you
   - lock the corresponding rectangle
   - fill the returned buffer
   - unlock and post the buffer
  Since android doesn't keep the contents of a window, I need to do it
  myself, just like I did it for the X11 version, using an X11 image
  and pixmap combo. I will store the image as an array of uint32_t pixels,
  but I'm planning to update the screen using other formats as well.
 
  So basicly, the image will be stored in RGBX_8888 format but if the
  unlock stuff returns a different format, I will convert the pixels
  to that format when I'm writing to the pixel array of the buffer.
 ***********************************************************************/

static uint32_t *Gpixels, x_window_width, x_window_height, **x_image_data;

#define x_red_mask    0x0000ff
#define x_green_mask  0x00ff00
#define x_blue_mask   0xff0000
#define x_red_shift   0
#define x_green_shift 8
#define x_blue_shift  16

static void graphics_window_destroyed(app_t *A, ANativeWindow *window)
{
  if (A->window!=window) return ;
  A->window= NULL;
}

static void graphics_window_created(app_t *A, ANativeWindow *window)
{
   int W,H;
   A->window= window;
   W= ANativeWindow_getWidth(A->window);
   H= ANativeWindow_getHeight(A->window);
   if (W<=0) W= 1;
   if (H<=0) H= 1;

   if (W!=x_window_width || H!=x_window_height || !Gpixels)
   {
     int y;
     if (Gpixels) free(Gpixels);
     Gpixels= malloc(sizeof(uint32_t)*W*H);
     if (x_image_data) free(x_image_data);
     x_image_data= malloc(sizeof(x_image_data[0])*H);
     for(y=0;y<H;y++)
       x_image_data[y]= y*W + Gpixels;
     x_window_width= W;
     x_window_height= H;
   }
}


void gfx_update(int x,int y,int w,int h)
{
   ARect rect;
   ANativeWindow_Buffer buffer;
   int R;
   uint32_t *spixels;
   uint16_t *wpixels;
   int eX, eY;

   if (!Gpixels) return ; // display not initialized yet

   eX= x+w; eY= y+h;

   if (x>=x_window_width || eX<=0 || y>=x_window_height || eY<=0) return ;

   eX= imin2(eX, x_window_width); eY= imin2(eY, x_window_height);
   x= imax2(x, 0); y= imax2(y,0);

   rect.left= x; rect.top= y;
   rect.right= eX; rect.bottom= eY;

   R= ANativeWindow_lock(my_app->window, &buffer, &rect);
   if (R<0)
   {
      LOGV("Unable to lock buffer: %d\n", R);
      return ;
   }

   // LOGV("GFX::Buffer width=%d height=%d stride=%d format=%d\n", buffer.width, buffer.height,  buffer.stride, buffer.format);

   // LOGV("GFX::Rect left=%d top=%d right=%d bottom=%d\n", rect.left,
           // rect.top, rect.right, rect.bottom);
   
  switch(buffer.format)
  {
   case WINDOW_FORMAT_RGBX_8888:
     spixels= buffer.bits;
     for(y=rect.top;y<rect.bottom;y++)
       for(x=rect.left;x<rect.right;x++)
         spixels[y*buffer.stride+x]= Gpixels[x+ y*x_window_width];
     break;
   case WINDOW_FORMAT_RGBA_8888:
     spixels= buffer.bits;
     for(y=rect.top;y<rect.bottom;y++)
       for(x=rect.left;x<rect.right;x++)
         spixels[y*buffer.stride+x]= Gpixels[x+ y*x_window_width] | 0xff000000;
     break;
   case WINDOW_FORMAT_RGB_565:
     wpixels= buffer.bits;
     for(y=rect.top;y<rect.bottom;y++)
       for(x=rect.left;x<rect.right;x++)
       {
         uint32_t P,R,G,B;
         P= x_image_data[y][x];
         R= (P>>16)&0xff;  R>>= 3;
         G= (P>>8) &0xff;  G>>= 2;
         B= P&0xff;        B>>= 3;
         wpixels[y*buffer.stride+x]= (R<<11) | (G<<5) | B;
       }
      break;
    default:
      LOGV("unknown buffer format %d",buffer.format);
      break;
  }
  ANativeWindow_unlockAndPost(my_app->window);
}

void gfx_update_all()
{
  gfx_update(0,0,x_window_width,x_window_height);
}



/***********************************************************************
 *##M                                                                  *
 *                            Miscellaneous                            *
 *                                                                     *
 ***********************************************************************
  This section defines several utility functions. I wonder if these get
  inlined when compiled with optimization. It looks like they do, for
  gcc -O3.
 ***********************************************************************/


/***********************************************************************
 *##F                                                                  *
 *                               Events                                *
 *                                                                     *
 ***********************************************************************
  The following event code is just a place holder for now. I'll look
  further into it later.
 ***********************************************************************/

static int handle_input(app_t *A, gfx_event_t *ev)
{
  AInputEvent* event;
  int type;
  uint32_t action,index, i, npointers;
  gfx_event_t dummy;
  event = NULL;
  if (!ev) ev= &dummy;
again:
     // Is this an error? should I log it or somehow
     // return an indication? FIXME

  if (AInputQueue_getEvent(A->inputq, &event)<0) return 1;

  type= AInputEvent_getType(event);
  // LOGV("New input event: type=%d\n", type);

  if (AInputQueue_preDispatchEvent(A->inputq, event)) goto again;

  switch(type)
  {
  case AINPUT_EVENT_TYPE_KEY: goto next_event;
  case AINPUT_EVENT_TYPE_MOTION: goto touch_event;
  }

next_event:
  AInputQueue_finishEvent(A->inputq, event, 1);
  goto again;

touch_event:
  action= AMotionEvent_getAction(event);
  index= (action & AMOTION_EVENT_ACTION_POINTER_INDEX_MASK)
              >>AMOTION_EVENT_ACTION_POINTER_INDEX_SHIFT;
  action= (action & AMOTION_EVENT_ACTION_MASK);

  switch(action)
  {
  case AMOTION_EVENT_ACTION_UP:
     ev->type= GFXEV_BUTTON_RELEASE;
     ev->button= 1;
     ev->x= AMotionEvent_getX(event,index);
     ev->y= AMotionEvent_getY(event,index);
     // LOGV("Mouse up: %d %d\n", ev->mouse.x, ev->mouse.y);
     A->last_pointer_id= -1;
     break;
  case AMOTION_EVENT_ACTION_DOWN:
     ev->type= GFXEV_BUTTON_PRESS;
     ev->x= AMotionEvent_getX(event,index);
     ev->y= AMotionEvent_getY(event,index);
     ev->button= 1;
     A->last_pointer_id= AMotionEvent_getPointerId(event, index);    
     // LOGV("Mouse down: %d %d\n", ev->mouse.x, ev->mouse.y);
     break;
  case AMOTION_EVENT_ACTION_MOVE:
     npointers= AMotionEvent_getPointerCount(event);
     for(i=0;i<npointers;i++)
        if (AMotionEvent_getPointerId(event, i)==A->last_pointer_id)
        { 
          ev->type= GFXEV_POINTER_MOVE;
          ev->x= AMotionEvent_getX(event,i);
          ev->y= AMotionEvent_getY(event,i);
          // LOGV("Mouse move: %d %d [%d %d %d]\n", ev->mouse.x, ev->mouse.y,
                             // npointers, i, A->last_pointer_id);
          goto done;
        }
  default:
     goto next_event;
  }

done:
  AInputQueue_finishEvent(A->inputq, event, 1);
  return 0;
}


static int get_event(unsigned int *msecs, gfx_event_t *E)
{
   int ident;
   int dFD, dEV;
   app_t *app;
again:
   ident= ALooper_pollAll(-1,  &dFD, &dEV, (void**) &app);

   switch(ident)
   {
     // somebody might have woke us up because the looper
     // has changed and we need to poll again.
   case ALOOPER_POLL_WAKE:
      goto again;
   case PIPE_CMD:
      if (handle_command(app)) // quit was requested.
      {
        E->type= GFXEV_QUIT;
        return 1;
      }
      goto again;
   case PIPE_EVENT:
     if (handle_input(app, E) || !app->window)
       goto again;
     return 1;
   }

   // other things are not possible. maybe I can add some
   // sort of a time-out later.
   return 0;
}

int gfx_event (unsigned int *msecs, gfx_event_t *E)
{
retry:
  if (!get_event(msecs,E)) return 0;
  switch(E->type)
  {
  case GFXEV_POINTER_MOVE:
     if (handle_motion_event(E)==2) goto retry;
     break;
  case GFXEV_BUTTON_PRESS:
     if (handle_click_event(E)==2) goto retry;
     break;
  case GFXEV_BUTTON_RELEASE:
     if (handle_release_event(E)==2) goto retry;
     break;
  }
  return 1;
}

static int event_noprocess(unsigned int *msecs,gfx_event_t *E)
{
  return get_event(msecs,E);
}

#line 5 "/home/dodo/tmp/gfx/android/../src/util.u"

static void *grow_array(int N, int *S, void *ptr, size_t eltsiz)
{
  if (N<*S) return ptr;
  if (!*S)
  {
    *S= 20;
    ptr= malloc(*S*eltsiz);
  }
  else
  {
    *S+= *S/2;
    ptr= realloc(ptr, *S*eltsiz);
  }
  return ptr;
}

static int range_test(int v, int s, int w) { return v>=s && v<(s+w); }


static int imin2(int a,int b) { return a < b ? a : b; }
static int imax2(int a,int b) { return a > b ? a : b; }

static int imin3(int a,int b,int c) { return imin2(imin2(a,b),c); }
static int imax3(int a,int b,int c) { return imax2(imax2(a,b),c); }

static double dmin2(double a,double b) { return a<b?a:b; }
static double dmax2(double a,double b) { return a>b?a:b; }

static int utf8_encode_string
    (const uint32_t* wstr,size_t wlen, uint8_t *dst, size_t *dlen)
{
/*
  This is all from
  http://www.cl.cam.ac.uk/~mgk25/unicode.html

  TODO: what to do if the input has invalid
  unicode chars such as unassigned chars etc?
  pass them thru?
*/

/* Encoding Table
                  Start-Byte    
 Min Value    8421 8421   hex     Length
0x04000000    1111 110x   fc        6
0x00200000    1111 10xx   f8        5
0x00010000    1111 0xxx   f0        4
0x00000800    1110 xxxx   e0        3
0x00000080    110x xxxx   c0        2 */
  int i,k,len;
  uint32_t chr;
  unsigned char *odst;

  static const unsigned int lowerlimit[]=
    { 0x04000000, 0x00200000, 0x00010000, 0x00000800, 0x00000080 };
  static const unsigned char topbits []= { 0xfc, 0xf8, 0xf0, 0xe0, 0xc0 };
  const unsigned char lower6=  0x3f; // binary 0011 1111
  const unsigned char seventh= 0x80; // binary 1000 0000
  const unsigned int bit31=    0x80000000; 

  odst= dst;

  while(wlen)
  {
    chr= *(wstr++); wlen--;
    if (chr<=0x7f)   { *(dst++)= chr; continue; }
    if (chr & bit31) { *(dst++)= chr & ~bit31; continue; }

    for(i=0;i<4;i++) if (chr >= lowerlimit[i]) break;
    len= 4-i+2;

    for(k=len-1;k>0;k--)
    {
      dst[k]= seventh | (chr & lower6);
      chr >>= 6;
    }
    dst[k]= chr | topbits[i];
    dst+= len;
  }

  *dlen= dst-odst;
  return 0;
}

static uint32_t utf8_decode_single_char(char *src)
{
  uint32_t *dst;
  size_t L;
  L= strlen(src);
  dst= alloca(sizeof(*dst)*(L+1));
  utf8_decode_string((uint8_t*) src, L, dst, &L);
  if (L==0) return ' ';
  return dst[0];
}


static void utf8_encode_single_char(uint32_t wide, char *dst)
{
  /* This assumes that dst is big enough, at least 6 bytes. */
  size_t L;
  L= 6;
  utf8_encode_string(&wide,1, (uint8_t*) dst, &L);
}

static void utf8_decode_alloc(char *str, uint32_t **Rwstr, uint32_t *Rwlen)
{
  uint32_t *S;
  size_t L,SL;
  L= strlen(str);
  S= malloc((L+1)*sizeof(uint32_t));
  utf8_decode_string((uint8_t*)str, L, S, &SL);
  *Rwstr= S;
  *Rwlen= SL;
}


static void clip_for_rect(int *start, int *amount, int max)
{
  int end;
  if (*amount<=0) return ;
  end= *start + *amount;
  if (end>max) end= max;
  if (*start<0) *start= 0;
  *amount= end - *start;
}

#line 10 "/home/dodo/tmp/gfx/android/../src/text.u"

static void get_text_data(int font, char *str, 
                          int *Rw, int *Rh,int *Ra,unsigned char **Rd)
{
  uint32_t *wtr;
  size_t L= strlen(str);
  wtr= alloca((L+1)*sizeof(uint32_t));
  utf8_decode_string((uint8_t*) str, L, wtr, &L);

  if (font<1) font=1;
  if (font>3) font=3;
  switch(font)
  {
  case 1: font1_render(wtr, L, Rw, Rh, Ra, Rd); break;
  case 2: font2_render(wtr, L, Rw, Rh, Ra, Rd); break;
  case 3: font3_render(wtr, L, Rw, Rh, Ra, Rd); break;
  }
}

#line 127 "/home/dodo/tmp/gfx/android/../src/graphics.u"

static void gfx_invert_hline(int x1,int x2, int y)
{
  uint32_t all_mask;

  if (y<0 || y>=x_window_height || x1>x2 || x1>=x_window_width) return ;
  if (x2>=x_window_width) x2= x_window_width-1;

  all_mask= x_red_mask | x_green_mask | x_blue_mask;
  for(;x1<=x2;x1++)
    x_image_data[y][x1] ^= all_mask;
}

static void gfx_invert_vline(int x,int y1, int y2)
{
  uint32_t all_mask;

  if (x<0 || x>=x_window_width || y1>y2 || y1>=x_window_height) return ;
  if (y2>=x_window_height) y2= x_window_height-1;

  all_mask= x_red_mask | x_green_mask | x_blue_mask;
  for(;y1<=y2;y1++)
    x_image_data[y1][x] ^= all_mask;
}

static uint32_t blend_usr_usr(uint32_t alpha, uint32_t fg, uint32_t bg)
{
  uint32_t fR, fG, fB; 
  uint32_t bR, bG, bB; 
  uint32_t pR, pG, pB; 
  double A,B;

  fR= (fg>>16)&0xff; fG= (fg>>8)&0xff; fB= fg&0xff;

  if (alpha==0xff) { pR= fR; pG= fG; pB= fB; goto OK;}

  bR= (bg>>16)&0xff; bG= (bg>>8)&0xff; bB= bg&0xff;
  if (alpha==0) { pR= bR; pG= bG; pB= bB; goto OK; }

  A= (double) alpha / 255.0;
  B= 1.0-A;

  pR= A*fR + B*bR;
  pG= A*fG + B*bG;
  pB= A*fB + B*bB;

OK:
  return   ((pR<<x_red_shift)&x_red_mask)
         | ((pG<<x_green_shift)&x_green_mask)
         | ((pB<<x_blue_shift)&x_blue_mask);
}

static uint32_t blend_usr_usr_usr(uint32_t alpha, uint32_t fg, uint32_t bg)
{
  uint32_t fR, fG, fB; 
  uint32_t bR, bG, bB; 
  uint32_t pR, pG, pB; 
  double A,B;

  if (alpha==0xff)  return fg; 
  if (alpha==0)    return bg;  

  fR= (fg>>16)&0xff; fG= (fg>>8)&0xff; fB= fg&0xff;


  bR= (bg>>16)&0xff; bG= (bg>>8)&0xff; bB= bg&0xff;

  A= (double) alpha / 255.0;
  B= 1.0-A;

  pR= A*fR + B*bR;
  pG= A*fG + B*bG;
  pB= A*fB + B*bB;

  return  (pR<<16) | (pG<<8) | (pB<<0);
}

static uint32_t blend_usr_svr(uint32_t alpha, uint32_t fg, uint32_t bg)
{
  uint32_t fR, fG, fB; 
  uint32_t bR, bG, bB; 
  uint32_t pR, pG, pB; 
  double A,B;

  if (alpha==0) return bg;

  fR= (fg>>16)&0xff; fG= (fg>>8)&0xff; fB= fg&0xff;

  if (alpha==0xff) { pR= fR; pG= fG; pB= fB; goto OK;}

  bR= (bg&x_red_mask)>>x_red_shift; 
  bG= (bg&x_green_mask)>>x_green_shift;
  bB= (bg&x_blue_mask)>>x_blue_shift;

  A= (double) alpha / 255.0;
  B= 1.0-A;

  pR= A*fR + B*bR;
  pG= A*fG + B*bG;
  pB= A*fB + B*bB;

OK:
  return   ((pR<<x_red_shift)&x_red_mask)
         | ((pG<<x_green_shift)&x_green_mask)
         | ((pB<<x_blue_shift)&x_blue_mask);
}

static void blend_graymap(int flags, int fg, int bg,
                          unsigned char *src, int src_w, int src_h,
                          int src_x, int src_y,
                          int dst_x, int dst_y, int width, int height)
{
  int x,y;
  uint32_t A;

  clip_for_rect( &src_x, &width, src_w);
  clip_for_rect( &src_y, &height, src_h);
  clip_for_rect( &dst_x, &width, x_window_width);
  clip_for_rect( &dst_y, &height, x_window_height);
  if (width<=0 || height<=0) return ;

  if (flags&GFX_BACKGND) // blend with fg,bg pair
  {
    for(y=0;y<height;y++)
      for(x=0;x<width;x++)   
      {
        A= src[(x+src_x) + (y+src_y)*src_w];
        x_image_data[y+dst_y][x+dst_x]= blend_usr_usr(A,fg, bg);
      }
  } 
  else
  {
    for(y=0;y<height;y++)
      for(x=0;x<width;x++)   
      {
        A= src[(x+src_x) + (y+src_y)*src_w];
        x_image_data[y+dst_y][x+dst_x]= 
             blend_usr_svr(A,fg, x_image_data[y+dst_y][x+dst_x]);
      }
  } 

  if (flags&GFX_UPDATE)
    gfx_update(dst_x, dst_y, width, height);
}

static uint32_t color_s(uint32_t uR, uint32_t uG, uint32_t uB)
{
  return ((uR<<x_red_shift)&x_red_mask)
        |((uG<<x_green_shift)&x_green_mask)
        |((uB<<x_blue_shift)&x_blue_mask);
}

static uint32_t u_red(uint32_t C) { return (C>>16)&0xff; }
static uint32_t u_green(uint32_t C) { return (C>>8)&0xff; }
static uint32_t u_blue(uint32_t C) { return (C)&0xff; }

static uint32_t color_u_to_s(uint32_t U)
{
  return color_s(u_red(U), u_green(U), u_blue(U));
}

static unsigned int blend_chan(unsigned int F, double A, unsigned int B)
{
  return  (unsigned int) ((F&0xff)*A + (B&0xff)*(1-A));
}


static unsigned int blend(uint32_t F, double A, uint32_t B)
{
  return color_s(
            blend_chan( u_red(F), A, u_red(B)),
            blend_chan( u_green(F), A, u_green(B)),
            blend_chan( u_blue(F), A, u_blue(B))
                );
}

static void grad_fill(int x,int y, int w, int h, uint32_t start, uint32_t end)
{
  int fx, fy, fw, fh,x0,y0,dx,dy;
  fx= x; fy= y; fw= w; fh= h;

  clip_for_rect(&x, &w, x_window_width);
  clip_for_rect(&y, &h, x_window_height);
  if (w<=0 || h<=0) return;

  for(y0=0;y0<h;y0++)
    for(x0=0;x0<w;x0++)
    {
      dx= (x0+x)-fx; dy= (y0+y)-fy;
      x_image_data[y0+y][x0+x]= blend(start, 1.0-(double)(dx+dy)/(fw+fh), end);
    }
}

static void hline(int x1,int x2, int y,uint32_t color)
{
  if (y<0 || y>=x_window_height || x1>x2 || x1>=x_window_width) return ;
  if (x2>=x_window_width) x2= x_window_width-1;

  for(;x1<=x2;x1++)
    x_image_data[y][x1]= color;
}

static void vline(int x,int y1, int y2,uint32_t color)
{
  if (x<0 || x>=x_window_width || y1>y2 || y1>=x_window_height) return ;
  if (y2>=x_window_height) y2= x_window_height-1;

  for(;y1<=y2;y1++)
    x_image_data[y1][x]= color;
}

static uint32_t *get_server_pix(int src_x,int src_y,int w,int h)
{
  uint32_t *D; int dw;
  int x,y;
  D= calloc(1,sizeof(uint32_t*)*w*h);
  dw= w;
  clip_for_rect(&src_x, &w, x_window_width);
  clip_for_rect(&src_y, &h, x_window_height);

  for(y=0;y<h;y++)
    for(x=0;x<w;x++)
      D[x+y*dw]= x_image_data[y+src_y][x+src_x];
  return D;
}

static void put_server_pix(uint32_t *D,int dst_x,int dst_y,int w, int h)
{
  int dw;
  int x,y;
  dw= w;
  clip_for_rect(&dst_x, &w, x_window_width);
  clip_for_rect(&dst_y, &h, x_window_height);

  for(y=0;y<h;y++)
    for(x=0;x<w;x++)
      x_image_data[y+dst_y][x+dst_x]= D[x+y*dw];
}



#line 402 "/home/dodo/tmp/gfx/android/../src/graphics.u"

static void gradfill_left(uint32_t *dst, int dst_w, int dst_h,
                          uint32_t start, uint32_t end,
                          int fill_x, int fill_y,
                          int fill_w, int fill_h)
{
   int x,y;
   uint32_t *row;
   clip_for_rect(&fill_x, &fill_w, dst_w);
   clip_for_rect(&fill_y, &fill_h, dst_h);
   if (fill_w<=0 || fill_h<=0) return ;
   row= alloca(fill_w*sizeof(uint32_t));
   for(x=0;x<fill_w;x++)
     row[x]= blend(start, 1.0-(double)x/(fill_w-1), end);
   for(y=0;y<fill_h;y++)
     for(x=0;x<fill_w;x++)
        dst[ (x+fill_x) + (y+fill_y)*dst_w ]= row[x];
}


static void hline_buffer(uint32_t *dst, int dst_w, int dst_h,
                         int x0, int x1, int y, uint32_t color)
{
  int x;
  if (x0<0) x0=0;
  if (x1>=dst_w) x1= dst_w-1;
  if (x0>x1 || y<0 || y>=dst_h) return ;
  for(x=x0;x<=x1;x++) dst[x+y*dst_w]= color;
}

static void vline_buffer(uint32_t *dst, int dst_w, int dst_h,
                         int x, int y0, int y1, uint32_t color)
{
  int y;
  if (y0<0) y0=0;
  if (y1>=dst_h) y1= dst_h-1;
  if (y0>y1 || x<0 || x>=dst_w) return ;
  for(y=y0;y<=y1;y++) dst[x+y*dst_w]= color;
}

static void shadow_box(uint32_t *dst, int dst_w, int dst_h,
                       uint32_t light, uint32_t dark, uint32_t border,
                       int box_x, int box_y, int box_w, int box_h)
{
  int sX,sY,eX,eY;

  light= color_u_to_s(light);
  dark= color_u_to_s(dark);
  border= color_u_to_s(border);

  sX= box_x; eX= box_x + box_w - 1;
  sY= box_y; eY= box_y + box_h - 1;


  hline_buffer(dst, dst_w, dst_h, sX, eX, sY,     border);
  hline_buffer(dst, dst_w, dst_h, sX, eX, eY,     border);
  vline_buffer(dst, dst_w, dst_h, sX, sY+1, eY-1, border);
  vline_buffer(dst, dst_w, dst_h, eX, sY+1, eY-1, border);

  hline_buffer(dst, dst_w, dst_h, sX+1, eX-2, sY+1, light);
  vline_buffer(dst, dst_w, dst_h, sX+1, sY+1, eY-2, light);
  hline_buffer(dst, dst_w, dst_h, sX+1, eX-1, eY-1, dark);
  vline_buffer(dst, dst_w, dst_h, eX-1, sY+1, eY-1, dark);
}

static void blend_graymap_buffer
        (unsigned int fg, unsigned char *src, int src_w, int src_h,
                          int src_x, int src_y,
                          unsigned int *dst, int dst_w, int dst_h,
                          int dst_x, int dst_y, int width, int height)
{
  int x,y;
  uint32_t A,D;

  clip_for_rect( &src_x, &width, src_w);
  clip_for_rect( &src_y, &height, src_h);
  clip_for_rect( &dst_x, &width, dst_w);
  clip_for_rect( &dst_y, &height, dst_h);
  if (width<=0 || height<=0) return ;

  for(y=0;y<height;y++)
  {
    for(x=0;x<width;x++)   
    {
       A= src[(x+src_x) + (y+src_y)*src_w];
       D= (y+dst_y)*dst_w + (x+dst_x);
       dst[D]= blend_usr_usr_usr(A,fg, dst[D]);
    }
  }
}

#line 30 "/home/dodo/tmp/gfx/android/../src/event.u"

static void install_click_handler
        (int x,int y,int w,int h,int *enabled,
         void *arg, int (*func)(void*,gfx_event_t*))
{
  click_handler_t *C;
  click_handlers= grow_array(n_click_handler, &s_click_handler,
                             click_handlers, sizeof(click_handlers[0]));
  C= click_handlers + n_click_handler++;
  C->x= x;
  C->y= y;
  C->w= w;
  C->h= h;
  C->enabled= enabled;
  C->arg= arg;
  C->func= func;
}

static void install_release_handler
        (void *arg, int (*func)(void*,gfx_event_t*))
{
  release_handler.func= func;
  release_handler.arg= arg;
}

static void install_motion_handler(void *arg,int (*func)(void *,gfx_event_t*))
{
  motion_handler.func= func;
  motion_handler.arg= arg;
}

static void uninstall_motion_handler()
{
  motion_handler.func= NULL;
  motion_handler.arg= NULL;
}

#line 75 "/home/dodo/tmp/gfx/android/../src/event.u"

static int handle_click_event(gfx_event_t *event)
{
  int i;
  click_handler_t *C;

  if (event->type!=GFXEV_BUTTON_PRESS || event->button!=1) return 0;
  for(i=click_handler_start;i<n_click_handler;i++)
  {
    C= click_handlers+i;
    if (range_test(event->x,C->x, C->w) && range_test(event->y,C->y, C->h))
    {
             // if the widget was disabled, look for something else
      if ((!C->enabled || C->enabled[0])&& C->func)
           return (*C->func)(C->arg, event);
    }
  }
  return 0;
}

static int handle_release_event(gfx_event_t *event)
{
  void *A; int (*F)(void *,gfx_event_t*);
  if (event->type!=GFXEV_BUTTON_RELEASE || event->button!=1) return 0;
  if (release_handler.func)
  {
    A= release_handler.arg;
    F= release_handler.func;
    release_handler.arg= NULL;
    release_handler.func= NULL;
    return (*F)(A, event);
  }
  return 0;
}

static int handle_motion_event(gfx_event_t *event)
{
  if (event->type!=GFXEV_POINTER_MOVE) return 0;
  if (motion_handler.func)
     return (*motion_handler.func)(motion_handler.arg,event);
  return 0;
}

#line 30 "/home/dodo/tmp/gfx/android/../src/widget.u"

static int register_widget(void *A,int type,int x,int y,int w,int h)
{
  int R;
  gui_widget_t *W;
  W= A;
  gui_widgets= grow_array(n_gui_widget, &s_gui_widget,
                          gui_widgets, sizeof(gui_widgets[0]));
  R= n_gui_widget++;
  gui_widgets[R]= W;
  W->type= type;
  W->x= x;
  W->y= y;
  W->w= w;
  W->h= h;
  return R;   
}

static void *get_widget(int number, int type)
{
  gui_widget_t *W;
  if (number<0 || number>=n_gui_widget) return NULL;
  W= gui_widgets[number];
  if (W->type!=type) return NULL;
  return W;
}

static void widget_destroy(int number)
{
  gui_widget_t *W;
  if (number<0 || number>=n_gui_widget) return ;
  W= gui_widgets[number];
  if (W && W->destroy)
  {
    (*W->destroy)(W);
    gui_widgets[number]= NULL;
  }
}

#line 36 "/home/dodo/tmp/gfx/android/../src/button.u"

static void init_button_colors()
{
  button_border= color_u_to_s(button_border_s);
  button_light= color_u_to_s(button_light_s);
  button_dark= color_u_to_s(button_dark_s);
  button_grad1= color_u_to_s(button_grad1_s);
  button_grad2= color_u_to_s(button_grad2_s);
  button_grad3= color_u_to_s(button_grad3_s);
  button_grad4= color_u_to_s(button_grad4_s);
  button_fg= color_u_to_s(button_fg_s);
}

static void button_render(int x,int y,int w,int h,
                              int tw, int th, unsigned char *data,
                              uint32_t grad1, uint32_t grad2,
                              uint32_t light, uint32_t dark)
{
  int copy_w, copy_h,src_x,src_y, text_x, text_y;
  int aw, ah;

  aw= w-4; ah= h-4;
  grad_fill(x+2,y+2,aw,ah, grad1, grad2);

  if (tw>aw) { src_x= (tw-aw)/2; copy_w= aw; text_x= x+2; }
        else { src_x= 0; copy_w= tw; text_x= x+2+(aw-tw)/2; }
  if (th>ah) { src_y= (th-ah)/2; copy_h= ah; text_y= y+2; }
        else { src_y= 0; copy_h= th; text_y= y+2+(ah-th)/2; }

  blend_graymap(0, button_fg, 0, data, tw,th, src_x,src_y,
                                 text_x,text_y,copy_w,copy_h);

  hline(x, x+w-1, y, button_border);
  hline(x, x+w-1, y+h-1, button_border);
  vline(x, y, y+h-1, button_border);
  vline(x+w-1, y, y+h-1, button_border);

  hline(x+1, x+w-3, y+1, light); 
  vline(x+1, y+1, y+h-3, light);

  hline(x+1, x+w-2, y+h-2, dark);
  vline(x+w-2, y+1, y+h-2, dark);
}

#line 121 "/home/dodo/tmp/gfx/android/../src/button.u"

static void button_set_state(gui_button_t *B, int state)
{
  if (state<0 || state>2) state= 0;
  B->state= state;
  put_server_pix( B->pic[B->state], B->widget.x, B->widget.y,
                                    B->widget.w, B->widget.h);
  gfx_update(B->widget.x, B->widget.y, B->widget.w, B->widget.h);
  if (B->state==2) B->enabled= 0;
              else B->enabled= 1;
}

#line 146 "/home/dodo/tmp/gfx/android/../src/button.u"

static int gui_button_click(void *arg, gfx_event_t *event)
{
  gui_button_t *B;
  B= arg;
  if (B->state==2) return 0; // shouldn't happen, but you never know
  button_set_state(arg, 1);
  install_release_handler(arg, gui_button_release);
  return 2;
}

static int gui_button_release(void *arg, gfx_event_t *event)
{
  gui_button_t *B;
  B= arg;
  event->type= B->action;
  button_set_state(B,0);
  return 1;
}

#line 21 "/home/dodo/tmp/gfx/android/../src/label.u"

static void draw_label(int flags, gui_label_t *G, char *str)
{
  int tw, th; unsigned char *td;
  int dst_x, dst_y, copy_w, copy_h, src_x, src_y;
  
  get_text_data(G->font, str, &tw, &th, NULL, &td);
  gfx_fill_rect(0, G->x, G->y, G->w, G->h, G->bg);

  if (th<=G->h)
  {
    src_y= 0; dst_y= G->y + (G->h-th)/2; copy_h= th;
  }
  else
  {
    src_y= (th-G->h)/2; dst_y= G->y; copy_h= G->h;
  }

  if (tw<G->w)
  {
    src_x= 0; copy_w= tw;
    if (G->flags&GFX_JUST_CENTER)
      dst_x= G->x+(G->w-tw)/2;
    else if (G->flags&GFX_JUST_RIGHT)
      dst_x= G->x+G->w-tw;
    else
      dst_x= G->x;
  }
  else
  {
    dst_x= G->x; copy_w= G->w;
    if (G->flags&GFX_CUT_LEFT)
      src_x= tw - G->h;
    else
      src_x= 0;
  }

  blend_graymap(GFX_BACKGND, G->fg, G->bg, td, tw, th,
                src_x, src_y, dst_x, dst_y, copy_w, copy_h);

  if (flags&GFX_UPDATE) gfx_update(G->x,G->y,G->w,G->h);
}

#line 1035 "/home/dodo/tmp/gfx/android/basics.u"

void gfx_geometry(unsigned int *W, unsigned int *H)
{
  if (W) *W= x_window_width;
  if (H) *H= x_window_height;
}

#line 30 "/home/dodo/tmp/gfx/android/../src/text.u"

void gfx_text_size(int font, char *str, int *W, int *H)
{
  get_text_data(font, str, W, H, NULL, NULL);
}

void gfx_print(int flags, int font, int fg, int bg, int x, int y, 
               char *str,int *Rw, int *Rh)
{
  int w,h; unsigned char *data;
  get_text_data(font, str, &w, &h, NULL, &data);

  blend_graymap(flags, fg, bg, data, w,h, 0,0, x,y,w,h);
  if (data) free(data);
  if (Rw) *Rw= w;
  if (Rh) *Rh= h;
}

#line 95 "/home/dodo/tmp/gfx/android/../src/graphics.u"

void gfx_get(unsigned int *buffer, int dst_w, int dst_h,
             int dst_x, int dst_y,
             int src_x, int src_y,
             int width, int height)
{
   int x,y;

   clip_for_rect( &src_x, &width, x_window_width);
   clip_for_rect( &src_y, &height, x_window_height);
   clip_for_rect( &dst_x, &width, dst_w);
   clip_for_rect( &dst_y, &height, dst_h);
   if (width<=0 || height<=0) return ;
   
   for(y=0;y<height;y++)
     for(x=0;x<width;x++)
     {
         uint32_t sP;
         uint32_t dR, dG, dB, dP;

         sP= x_image_data[y+src_y][x+src_x];

         dR= (sP & x_red_mask)>>x_red_shift;
         dG= (sP & x_green_mask)>>x_green_shift;
         dB= (sP & x_blue_mask)>>x_blue_shift;
 
         dP= (0xff<<24) | (dR<<16) | (dG<<8) | (dB);
         buffer[ (x+dst_x) + (y+dst_y)*dst_w ]= dP;
     }
}

#line 369 "/home/dodo/tmp/gfx/android/../src/graphics.u"

void gfx_fill_rect(int flags, int dst_x,int dst_y,
                   int w,int h, unsigned int color)
{
  int x,y;
  uint32_t sv;
  clip_for_rect(&dst_x, &w, x_window_width);
  clip_for_rect(&dst_y, &h, x_window_height);
  if (w<=0 || h<=0) return ;
  sv= color_u_to_s(color);
  for(y=0;y<h;y++)
   for(x=0;x<w;x++)
     x_image_data[y+dst_y][x+dst_x]= sv;
  if (flags&GFX_UPDATE) gfx_update(dst_x, dst_y, w,h);
}


void gfx_draw_rect(int flags, int dst_x,int dst_y,
                   int w,int h, unsigned int color)
{
  uint32_t sv;
  sv= color_u_to_s(color);
  hline(dst_x, dst_x+w-1, dst_y, sv);
  hline(dst_x, dst_x+w-1, dst_y+h-1, sv);
  vline(dst_x, dst_y, dst_y+h-1, sv);
  vline(dst_x+w-1, dst_y, dst_y+h-1, sv);
  if (flags&GFX_UPDATE) gfx_update(dst_x, dst_y, w,h);
}
#line 494 "/home/dodo/tmp/gfx/android/../src/graphics.u"

void gfx_put_graymap(int flags,
             unsigned char *buffer, int src_w, int src_h,
             int src_x, int src_y,
             int dst_x, int dst_y,
             int width, int height)
{
   int x,y;

   clip_for_rect( &src_x, &width, src_w);
   clip_for_rect( &src_y, &height, src_h);
   clip_for_rect( &dst_x, &width, x_window_width);
   clip_for_rect( &dst_y, &height, x_window_height);
   if (width<=0 || height<=0) return ;

   for(y=0;y<height;y++)
     for(x=0;x<width;x++)
     {
       uint8_t sP;
       uint32_t dP;
       sP= buffer[ (x+src_x) + (y+src_y)*src_w ];

       dP=  (sP << x_red_shift)&x_red_mask;
       dP|= (sP << x_green_shift)&x_green_mask;
       dP|= (sP << x_blue_shift)&x_blue_mask;

       x_image_data[y+dst_y][x+dst_x]= dP;
     }

   if (flags & GFX_UPDATE)
      gfx_update(dst_x,dst_y,width,height);
}

#line 528 "/home/dodo/tmp/gfx/android/../src/graphics.u"

void gfx_put(int flags,
             unsigned int *buffer, int src_w, int src_h,
             int src_x, int src_y,
             int dst_x, int dst_y,
             int width, int height)
{
  int x,y;

  clip_for_rect( &src_x, &width, src_w);
  clip_for_rect( &src_y, &height, src_h);
  clip_for_rect( &dst_x, &width, x_window_width);
  clip_for_rect( &dst_y, &height, x_window_height);
  if (width<=0 || height<=0) return ;

  if (!(flags&GFX_ALPHA)) 
  {
    for(y=0;y<height;y++)
      for(x=0;x<width;x++)
      {
         uint32_t sR, sG, sB, sP;
         uint32_t dP;
         sP= buffer[ (x+src_x) + (y+src_y)*src_w ];
         sR= (sP >> 16)&0xff;
         sG= (sP >> 8 )&0xff;
         sB= (sP      )&0xff;

         dP= (sR << x_red_shift)&x_red_mask;
         dP|= (sG << x_green_shift)&x_green_mask;
         dP|= (sB << x_blue_shift)&x_blue_mask;

         x_image_data[y+dst_y][x+dst_x]= dP;
       }
  }
  else
  {
    for(y=0;y<height;y++)
      for(x=0;x<width;x++)
    {
        uint32_t sA, sR, sG, sB, sP;
        uint32_t     dR, dG, dB, dP;

        sP= buffer[ (x+src_x) + (y+src_y)*src_w ];
        sA= (sP >> 24)&0xff;
        if (!sA) continue;

        sR= (sP >> 16)&0xff;
        sG= (sP >> 8 )&0xff;
        sB= (sP      )&0xff;

        if (sA==0xff)
        {
           dR= sR;
           dG= sG;
           dB= sB;
        }
        else 
        {
            double R,V;

            dP= x_image_data[y+dst_y][x+dst_x];

            dR= (dP & x_red_mask)>>x_red_shift;
            dG= (dP & x_green_mask)>>x_green_shift;
            dB= (dP & x_blue_mask)>>x_blue_shift;

            V= sA/255.0; R= 1.0 - V;

            dR=  dR*R + sR*V;
            dG=  dG*R + sG*V;
            dB=  dB*R + sB*V;
         }

         dP= (dR << x_red_shift)&x_red_mask;
         dP|= (dG << x_green_shift)&x_green_mask;
         dP|= (dB << x_blue_shift)&x_blue_mask;

         x_image_data[y+dst_y][x+dst_x]= dP;
     }
  }

  if (flags & GFX_UPDATE)
      gfx_update(dst_x,dst_y,width,height);
}

#line 81 "/home/dodo/tmp/gfx/android/../src/button.u"

int gfx_button(int flags,int x,int y, int w, int h, char *str, int action)
{
  int tw,th; unsigned char *data;
  int bno;
  gui_button_t *B;

  B= calloc(1,sizeof(*B));
  bno= register_widget(B,GFXCLASS_BUTTON,x,y,w,h);
  B->action= action;
  B->state= 0;

  B->pic[2]= get_server_pix(x,y,w,h);

  get_text_data(1, str, &tw, &th, NULL, &data);

  button_render(x,y,w,h, tw, th, data, 
                button_grad3, button_grad4, 
                button_dark, button_light);

  B->pic[1]= get_server_pix(x,y,w,h);
  
  button_render(x,y,w,h, tw, th, data, 
                button_grad1, button_grad2, 
                button_light, button_dark);

  B->pic[0]= get_server_pix(x,y,w,h);
  B->enabled= 1;

  free(data);

  if (flags&GFX_UPDATE)
    gfx_update(x,y,w,h);

  install_click_handler(x,y,w,h, &B->enabled, B, gui_button_click);
  return bno;
}


#line 134 "/home/dodo/tmp/gfx/android/../src/button.u"

void gfx_button_enable(int btn,int a)
{
  gui_button_t *B;
  if ((B= get_widget(btn, GFXCLASS_BUTTON)))
     button_set_state(B, a ? 0 : 2);
}

#line 28 "/home/dodo/tmp/gfx/android/../src/menu-fw.u"

int gfx_fwmenu(int font,int nentries, TUPiis_t *entries)
{
  int i;
  int R;
  int maxtextw;
  int texth,fonth;

  int maxwindoww, maxwindowh,my,dy,window_x,window_y,window_w,window_h;

  if (nentries<=0) return -1;

#line 7694 "gfx.c"
  static const unsigned char GMfwmenu_sub[]= {
    0x61,0x3c,0x00,0x00,0x00,0x00,0x4b,0x80,0x48,0x00,
    0x00,0x00,0x00,0x41,0x80,0x51,0x02,0x00,0x00,0x00,
    0x36,0x7f,0x59,0x04,0x00,0x00,0x00,0x2c,0x7d,0x56,
    0x00,0x00,0x00,0x2c,0x7d,0x5c,0x00,0x00,0x36,0x7f,
    0x60,0x07,0x00,0x41,0x80,0x56,0x03,0x00,0x48,0x80,
    0x4b,0x01,0x00,0x00,0x60,0x3e,0x00,0x00,0x00,0x00,
  };
  static const unsigned int GMWfwmenu_sub= 6, GMHfwmenu_sub=10;
#line 42 "/home/dodo/tmp/gfx/android/../src/menu-fw.u"

  maxtextw= 0;
  texth= 0;
  for(i=0;i<nentries;i++)
  {
    int w,h;
    gfx_text_size(font,entries[i].sv,&w,&h);
    maxtextw= imax2(w, maxtextw);
    texth+= h;
    fonth= h;
  }

  maxwindoww= x_window_width - 2*fwmenu_scrpad;
  maxwindowh= x_window_height - 2*fwmenu_scrpad;

  unsigned int *img, img_w, img_h;

  img_w= imin2(maxwindoww, maxtextw+GMWfwmenu_sub+1+2*fwmenu_intpad);
  img_h= texth+nentries-1;
  img= malloc(img_w*img_h*sizeof(uint32_t));

  for(i=0;i<img_w*img_h;i++) img[i]= fwmenu_bg_color;

  int y;
  y= 0;
  for(i=0;i<nentries;i++)
  {
    unsigned char *line; int linew, lineh;
    int cp_w;

    get_text_data(font, entries[i].sv, &linew,&lineh,NULL,&line);

    cp_w= imin2(img_w - 2*fwmenu_intpad- GMWfwmenu_sub-1, linew);

    blend_graymap_buffer(fwmenu_fg_color, line, linew,lineh, 0,0,
                         img, img_w, img_h, fwmenu_intpad, y, cp_w, lineh);

    free(line);
    if (entries[i].iv2)
      blend_graymap_buffer(fwmenu_fg_color, 
                    (unsigned char*) GMfwmenu_sub, GMWfwmenu_sub,
                                            GMHfwmenu_sub, 0,0,
                         img, img_w, img_h,
                         img_w - fwmenu_intpad - GMWfwmenu_sub, 
                         y + (lineh-GMHfwmenu_sub)/2, 
                            GMWfwmenu_sub, GMHfwmenu_sub);
    y+= lineh;
    hline_buffer(img, img_w,img_h, fwmenu_intpad, img_w-fwmenu_intpad,y,
                            fwmenu_sep_color);
    y+= 1;
  }

  window_w= img_w;
  window_h= imin2(maxwindowh, img_h);

  window_x= (x_window_width - window_w)/2;
  window_y= (x_window_height - window_h)/2;

  int pos, dirty, dragging, down;
  gfx_event_t event;
  uint32_t *backstore;

  R= -1;
  down= dragging= 0; dirty= 1;
  pos= 0;

  backstore= get_server_pix(window_x-2,window_y-2,window_w+4,window_h+4);

  gfx_draw_rect(0,window_x-1,window_y-1,window_w+2,window_h+2, 0x888888);
  gfx_draw_rect(GFX_UPDATE,window_x-2,window_y-2,window_w+4,window_h+4, 0x00);

  while(1)
  {
    if (dirty)
    {
     gfx_put(GFX_UPDATE, img, img_w, img_h,
                         0, pos, window_x, window_y,
                        img_w, window_h);
                         
     dirty= 0;
    }
    if (!event_noprocess(NULL,&event)) continue;
#if 0
   char buf[2000];
#endif
    switch(event.type)
    {
    case GFXEV_BUTTON_PRESS:
#if 0
    sprintf(buf,"event: %d,%d window: %d,%d - %dx%d",event.x, event.y,
                           window_x,window_y, window_w,window_h);
     gfx_print(GFX_UPDATE,2, 0x00, 0xffffff, 20,300, buf,NULL,NULL);
#endif
        if (event.button!=1) continue;
        if (event.x<window_x || event.x>=(window_x+window_w) ||
            event.y<window_y || event.y>=(window_y+window_h)) goto cancelled;
        my= event.y;
        down= 1;
        dragging= 0;
        break;
    case GFXEV_BUTTON_RELEASE:
        if (event.button!=1) continue;
        down= 0;
        if (dragging)
        {
          dragging= 0;
          break;
        }
        R= (my - window_y + pos)/(fonth+1);
        goto done;
    case GFXEV_POINTER_MOVE:
        if (!down || window_h>=img_h) break;
        if (!dragging)
        {
          if (abs(event.y-my)<=20) break;
          dragging= 1;
        }
        dy= event.y - my;
        if (!dy) break;
        my= event.y;
        if (dy<0) {  // dragging finger upwards
          if (pos+window_h>=img_h) break;
          pos -= dy; 
          if (pos+window_h>img_h) pos= img_h-window_h;
        } else
        {
          if (pos==0) break;
          pos-= dy;
          if (pos<0) pos= 0;
        }
        dirty= 1;
        break;
    }
  }

cancelled:

  R= -1;
  while(1)
  {
    if (!event_noprocess(NULL,&event)) continue;
    switch(event.type)
    {
    case GFXEV_BUTTON_RELEASE:
        if (event.button==1) goto done;
    }
  }

done:
  free(img);
  put_server_pix(backstore, window_x-2,window_y-2,window_w+4,window_h+4);
  free(backstore);
  gfx_update(window_x-2,window_y-2,window_w+4,window_h+4);
  if (R>=0 && R<nentries)
     return entries[R].iv1; 
  else return -1;
}
#line 65 "/home/dodo/tmp/gfx/android/../src/label.u"

int gfx_label(int flags, int font, int fg, int bg,
              int x,int y, int w, int h, char *str)
{
  gui_label_t *G;
  int R;
  labels= grow_array(n_labels, &s_labels, labels, sizeof(labels[0]));
  G= labels + n_labels;
  R= n_labels;
  n_labels++;
  G->font= font;
  G->fg= fg;
  G->bg= bg;
  G->x= x;
  G->y= y;
  G->w= w;
  G->h= h;
  G->flags= (GFX_JUST_RIGHT | GFX_JUST_CENTER | GFX_CUT_LEFT) & flags;
  draw_label(flags,G, str);
  return R;
}

void gfx_label_set_text(int flags,int labno, char *str)
{
  if (labno<0 || labno>=n_labels) return ;
  draw_label(flags, labels + labno,str);
}
#line 31 "/home/dodo/tmp/gfx/android/../src/screen.u"

int gfx_push_screen()
{
  screen_t *SC;
  int R;
  screens= grow_array(n_screen, &s_screen, screens, sizeof(screens[0]));
  SC= screens+n_screen;
  R= n_screen;
  n_screen++;

  SC->n_gui_widget= n_gui_widget;
  SC->n_click_handler= n_click_handler;
  SC->click_handler_start= click_handler_start;
  SC->release_handler= release_handler;
  SC->motion_handler= motion_handler;

  SC->backstore= get_server_pix(0,0,x_window_width,x_window_height);
  SC->backw= x_window_width;
  SC->backh= x_window_height;

  click_handler_start= n_click_handler;
  memset(&release_handler,0,sizeof(release_handler));
  memset(&motion_handler,0,sizeof(motion_handler));

  return R;
}

void gfx_restore_screen(int sn)
{
  int i;
  screen_t *SC;
  if (sn<0 || sn>=n_screen) return ;
  
  SC= screens+sn;

  for(i=SC->n_gui_widget;i<n_gui_widget;i++)
    widget_destroy(i);
  n_gui_widget= SC->n_gui_widget;
  n_click_handler= SC->n_click_handler;
  click_handler_start= SC->click_handler_start;
  release_handler= SC->release_handler;
  motion_handler= SC->motion_handler;
  
  put_server_pix(SC->backstore, 0,0,SC->backw,SC->backh);
  gfx_update(0,0,x_window_width,x_window_height);
  for(i=sn;i<n_screen;i++)
  {
    free(screens[i].backstore);
    memset(screens+i, 0, sizeof(screens[i]));
  }
  n_screen= sn;
}
#line 1024 "/home/dodo/tmp/gfx/android/basics.u"

